/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

/**
 * Manages user alerts
 */
EbayCompanion.Alerts = {
  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      this._panel = document.getElementById("gs-ebay-alert-panel");
      this._alertbox = document.getElementById("gs-ebay-alert-box");

      EbayCompanion.Observers.add(this,
        EbayCompanion.Topics.ALERT_SHOW_PRIMARY);
      EbayCompanion.Observers.add(this,
        EbayCompanion.Topics.ALERT_SHOW_SECONDARY);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.ALERT_SHOW_ITEM);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.ALERT_CLOSE_ITEM);

      // import the module when we actually need it
      if (typeof(EbayCompanion.AlertsService) == "undefined") {
        EbayCompanion._importModule("services/alertsService.js");
      }
      // reload all alerts for new windows.
      EbayCompanion.AlertsService.showCurrentAlerts();
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Uninitializes the resource.
   */
  uninit : function() {
    EbayCompanion.Observers.removeAll(this);
  },

  /**
   * Shows the alert box with the param alert.
   * @param aAlert the alert object.
   */
  _showAlertBox : function(aAlert) {
    this._alertbox.currentAlertIndex =
      EbayCompanion.AlertsService.getPrimaryAlertsIndex();
    this._alertbox.alertsQueueLength =
      EbayCompanion.AlertsService.getPrimaryAlertsSize();
    this._alertbox.alert = aAlert;
  },

  /**
   * Displays the alert in the panel or updates the panel if it is already
   * displayed
   * @param aAlert the alert to be displayed.
   */
  _showPrimaryAlert : function(aAlert) {
    let bottomBox = document.getElementById("browser-bottombox");
    let currentAlert = EbayCompanion.AlertsService.getCurrentPrimaryAlert();

    this._showAlertBox(currentAlert);

    if (this._panel.state == "closed") {
      this._panel.openPopup(bottomBox, "before_end", -20, -2, false, false);
      EbayCompanion.EventAnalytics.push(
        ['_trackEvent', "201-load-notification-toast-primary", "clicked"]);
    }
  },

  /**
   * Displays a secondary alert by notifying the secondary alert observer
   * (listened by the sidebar if opened) and makes the toolbar button glow
   */
  _showSecondaryAlert : function(aAlert) {
    let toolbarButton = document.getElementById("gs-ebay-toolbar-button");

    if (toolbarButton) {
      toolbarButton.glowing = true;
    }
    EbayCompanion.updateButtonTooltip();

    // show secondary alerts only if the user is in the my ebay tab
    let selectedTab = EbayCompanion.StateHelper.getSelectedTab();
    if (selectedTab == EbayCompanion.StateHelper.BUYING_TAB ||
        selectedTab == EbayCompanion.StateHelper.SELLING_TAB ||
        selectedTab == EbayCompanion.StateHelper.ACTIVE_TAB ||
        selectedTab == EbayCompanion.StateHelper.ENDED_TAB) {
      // give it some "air" so we prevent secondary alerts not being shown when
      // the browser starts up
      window.setTimeout(function() {
        EbayCompanion.Observers.notify(
          aAlert.object, EbayCompanion.Topics.ITEM_HIGHTLIGHT, null);
      }, 2000);
    }
  },

  /**
   * Closes the current alert being shown.
   */
  closeCurrentAlert : function() {
    let currentAlert = EbayCompanion.AlertsService.getCurrentPrimaryAlert();

    EbayCompanion.AlertsService.dismissAlert(currentAlert);
  },

  /**
   * Closes the alert panel, if it's open.
   */
  closePanel : function() {
    let isAlertsPanelOpen = (this._panel.state != "closed") ? true : false;

    this._panel.hidePopup();
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case EbayCompanion.Topics.ALERT_SHOW_PRIMARY:
        this._showPrimaryAlert(aSubject);
        break;
      case EbayCompanion.Topics.ALERT_SHOW_SECONDARY:
        this._showSecondaryAlert(aSubject);
        break;
      case EbayCompanion.Topics.ALERT_SHOW_ITEM:
        this._showAlertBox(aSubject);
        break;
      case EbayCompanion.Topics.ALERT_CLOSE_ITEM:
        this.closePanel();
        break;
    }
  }
};
