/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

/**
 * Main overlay object for eBay Companion
 */
var EbayCompanion = {

  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      // import required modules
      this._importModule("common.js");
      this._importModule("helpers/logger.js");
      this._importModule("helpers/observers.js");
      this._importModule("helpers/utilityHelper.js");
      this._importModule("helpers/progressListener.js");
      this._importModule("datasource.js");
      this._importModule("constants.js");
      this._importModule("eventAnalytics.js");

      //recent items modules
      this._importModule("services/recentAlertService.js");
      this._importModule("services/recentBidService.js");
      this._importModule("services/recentSearchService.js");
      this._importModule("services/recentViewService.js");
      this._importModule("services/recentWatchService.js");
      this._importModule("objects/alert.js");
      this._importModule("services/itemService.js");

      // set the flag to know this is a San Disk build. User-set preferences are
      // not overwritten by upgrades. This is why we don't set this in the
      // preferences.js file.
      // XXX: standard version of the add-on should not have this line
      //EbayCompanion.Constants.prefBranch.set("SanDiskBuild", true);

      let that = this;
      // assume the user is present until proven otherwise
      this.userIsAway = false;

      EbayCompanion.Observers.add(this, EbayCompanion.Topics.ACCOUNT_LOGGED_IN);
      EbayCompanion.Observers.add(
        this, EbayCompanion.Topics.ACCOUNT_LOGGED_OUT);
      EbayCompanion.Observers.add(
        this, EbayCompanion.Topics.ACCOUNT_FEEDBACK_CHANGED);
      EbayCompanion.Observers.add(
        this, EbayCompanion.Topics.ACCOUNT_UNREAD_MESSAGES_CHANGED);

      let connected = EbayCompanion.Datasource.activeAccount() != null;
      this._setUpdateLoginBroadcaster(connected);

      // Used to catch Auth & Auth login
      let content = document.getElementById("content");
      this.addManagedEventListener(
        content, "DOMContentLoaded",
        function(event) that._onContentLoaded(event),
        false);

      // implement use of idle service instead of listening for mouse move event
      this._idleService =
        Cc["@mozilla.org/widget/idleservice;1"]
          .getService(Ci.nsIIdleService)

      // 20 secs idle observer
      this._idleService.addIdleObserver(EbayCompanion.idleObserver, 20);

      // register a focus listener to display any primary alert that might
      // have come up while the browser was out of focus or minimized
      this.addManagedEventListener(
        window, "focus",
        function(event) that._onFocus(event),
        true);

      // register a blur listener to set the user as away
      this._blurListener = this.addManagedEventListener(
        window, "blur",
        function(event) that._onBlur(event),
        true);

      // initializes the ebay button.
      this._initeBayButton();

      // Initialise sub-objects
      EbayCompanion.Alerts.init();
      EbayCompanion.EventAnalytics.init();
      EbayCompanion.EventAnalytics.pageView();
      EbayCompanion.BrowserContextMenu.init();

      // here we perform the actions related to the installed version of the
      // extension
      EbayCompanion.UtilityHelper.getVersion(that._startupChecks);

      // if the user hasn't signed in for the first time, we keep showing the
      // terms and conditions notification.
      if (EbayCompanion.Constants.prefBranch.get("firstSignIn")) {
        this._showTermsNotification();
      }

      let ebayArgumentsHandler =
        Cc["@mozilla.org/commandlinehandler/general-startup;1?type=ebayarghandler"].
          getService(Ci.ecIEbayArgumentsHandler);
      let shortcut = ebayArgumentsHandler.startedUsingShortcut;

      // check command line arguments
      if (0 != shortcut &&
          !EbayCompanion.Constants.shortcutParameterRead) {
        // give the window a chance to initialize
        window.setTimeout(function() {
          // open the homepage
          if (1 == shortcut) {
            that.openPage(null, 'desktopShortcut', 'homePage');
          } else if (2 == shortcut) {
            that.openPage(null, 'installerShortcut', 'homePage');
          }

          // and the sidebar
          // if this is a San Disk build, we don't have to force open the
          // sidebar
          if (!EbayCompanion.Constants.prefBranch.get("SanDiskBuild")) {
            that.openSidebar();
          }

          EbayCompanion.Constants.shortcutParameterRead = true;
        }, 500);
      }

      this._addSidebarSplitterListener();

      // we can delay the import of uninstall manager to improve startup
      // performance
      window.setTimeout(function() {
        that._importModule("services/uninstallService.js");
      }, 1000);
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Uninitializes the resource.
   */
  uninit : function() {
    try {
      // Uninitialise sub-objects
      EbayCompanion.Alerts.uninit();
      this._idleService.removeIdleObserver(EbayCompanion.idleObserver, 20);
      EbayCompanion.Observers.removeAll(this);
      this.removeAllManagedEventListeners();
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * This method performs the necessary checks related to the installed version
   * of the extension at browser startup
   * @param aVersion the version of the extension
   */
  _startupChecks : function(aVersion) {
    // used when the upgrade is from 1.6
    let upgrade = false;
    // normal upgrade from 2.* versions and we need to display the terms &
    // conditions notification again
    let termsChanged = false;
    // if reportUpgrade is true, we keep that value until the report callback
    // sets it to false
    // used to ping the upgrade API when there's any change in the version
    // number
    let reportUpgrade =
      EbayCompanion.Constants.prefBranch.get("reportUpgrade");
    let currentVersion = EbayCompanion.Constants.prefBranch.get("version");
    if (currentVersion && currentVersion.match(/^1\.(.)*$/g)) {
      upgrade = true;
      termsChanged = true;
      reportUpgrade = true;
    }

    //For use case where EITHER/BOTH 3.0.0.2 & V2* versions are previously
    //installed make active/ended the default menu style
    if (currentVersion && currentVersion.match(/^2\.(.)*$/g) ||
       currentVersion == "3.0.0.0" || currentVersion == "3.0.0.1"
       || currentVersion == "3.0.0.2") {
      EbayCompanion.Constants.prefBranch.set("default.tab", 3);
      EbayCompanion.Constants.prefBranch.set("default.view", 0);
    }

    // if this is not an upgrade from 1.6.11, check if it is an upgrade
    // from any other 2.X version
    if (!reportUpgrade && currentVersion && currentVersion != aVersion) {
      // only for 2.0.3 upgrade as agrement changed. Any future version that
      // requires the terms and conditions notification to be displayed,
      // should be added here
      if (aVersion == "2.0.3") {
        termsChanged = true;
      }
      reportUpgrade = true;
      // force sidebar to be opened after the window has had a chance to
      // initialise
      window.setTimeout(function() {
        EbayCompanion.openSidebar();
      }, 2000);
    }
    // set the flag to send the install API "upgrade" call
    EbayCompanion.Constants.prefBranch.set("reportUpgrade", reportUpgrade);
    // set the version in the preferences so in case it isn't set, it gets
    // set
    EbayCompanion.Constants.prefBranch.set("version", aVersion);

    // First run
    if (EbayCompanion.Constants.prefBranch.get("firstRun") ||
        upgrade || termsChanged) {
      // call the first-run method after the window has had a chance to
      // initialise
      window.setTimeout(function() {
        EbayCompanion._firstRun(upgrade, termsChanged);
        EbayCompanion.Constants.prefBranch.set("firstRun", false);
      }, 2000);
    }
  },

  /**
   * Tasks to perform the first time the Companion is loaded
   * @param aUpgrade whether this is a 1.6 upgrade
   * @param aTermsChanged whether the terms and conditions chanced so we have
   * to display the notification to the user
   */
  _firstRun : function(aUpgrade, aTermsChanged) {
    try {
      // if this is an upgrade from 1.6, extract the signed account info (if
      // any) and use it to sign in the user automatically
      if (aUpgrade) {
        this._extractAccountInfoFromPasswordManager();
      }
      this.showLicence(aTermsChanged);
      this.openSidebar();
      // set default web site
      this._setDefaultWebSite();
      // sends the installation tracking request
      EbayCompanion.Datasource.sendTrackingRequest("install");
      // creates a Firefox shortcut in the desktop
      // no need to import this class in the overlay's init as we only
      // use it here, and only once!
      if (EbayCompanion.Constants.prefBranch.get("common.icon")) {
        this._importModule("helpers/shortcutHelper.js");
        EbayCompanion.ShortcutHelper.createShortcut();
      }
      // open first run page
      this._openFirstRunPage();
    }
    catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Extract the signed account information from the login manager when the
   * extension is upgraded from 1.6.11
   */
  _extractAccountInfoFromPasswordManager : function() {
    // get the token from the password manager
    let tokenContainer =
      this._getTokenFromPasswordManager();
    /* Number of days to fetch a new token before it expires. */
    const EBAYCOMP_DAYS_BEOFRE_TOKEN_EXPIRE = 7;

    if (tokenContainer) {
      let tokenArray =  tokenContainer.tokenString.split("&");
      if (tokenArray.length >= 3) {
        let dateObject =
          EbayCompanion.Constants.dateFromIso8601(tokenArray[1]);
        if (dateObject) {
          let currDateObject = new Date();
          // get the time difference
          let timeDiffObject = EbayCompanion.Constants.timeDifference(
            currDateObject.getTime(), dateObject.getTime())
          if (EBAYCOMP_DAYS_BEOFRE_TOKEN_EXPIRE >= timeDiffObject.days) {
            // in this case the token is about to expire, so we better
            // request a new one and prevent the automatic login
            this._removeTokenFromPasswordManager();
          } else {
            // The account is not stored locally
            let activeAccount =
              new EbayCompanion.Account(tokenContainer.username, false);
            activeAccount.set("token", tokenArray[0]);
            activeAccount.set("registrationSite",   // temporary value
              EbayCompanion.Constants.prefBranch.get("chosenSite"));
            activeAccount.set("feedbackRating", 0);
            activeAccount.set("numUnreadMessages", 0);
            // set the preference to prevent first display of feedback score
            // change notification when the user is signing in for the first
            // time
            EbayCompanion.Constants.prefBranch.set(
              "prevent.first.feedback.score.notification", true);
            // set the active account and then try an automatic login
            EbayCompanion.Datasource.setActiveAccount(activeAccount);
            EbayCompanion.Datasource.tryAutomaticConnect();
            return true;
          }
        }
      }
    }
    return false;
  },

  /**
   * Gets the token for the last logged in user from
   * in the password manager.
   * @return the entry if found.
   */
  _getTokenFromPasswordManager : function() {

    let tokenContainer = {
      username: null,
      tokenString: null
    };
    let ret;
    let loginManagerService;
    let passwordManagerService;

    // Firefox 2 and 3 use different services for storing passwords.
    {
      let loginManagerClass = Cc["@mozilla.org/login-manager;1"];
      let passwordManagerClass = Cc["@mozilla.org/passwordmanager;1"];

      if (loginManagerClass) {
        // Firefox 3
        loginManagerService =
          loginManagerClass.getService(Ci.nsILoginManager);
      } else {
        //Firefox 2
        passwordManagerService =
          passwordManagerClass.getService(Ci.nsIPasswordManager);
      }
    }

    const EBAY_HOST_FOR_PASSWORD_MANAGER = "eBay.companion";
    if (loginManagerService) {
      // Firefox 3
      const hostName = EBAY_HOST_FOR_PASSWORD_MANAGER;
      const formSubmitURL = EBAY_HOST_FOR_PASSWORD_MANAGER;
      const httpRealm = null;
      let logins;

      logins = loginManagerService.
                 findLogins({}, hostName, formSubmitURL, httpRealm);

      if (logins && logins[0]) {
        tokenContainer.username = logins[0].username;
        tokenContainer.tokenString = logins[0].password;
        ret = tokenContainer;
      }
    } else {
      // Firefox 2
      let passwords = passwordManagerService.enumerator;

      while (!ret && passwords.hasMoreElements()) {
        let entry = passwords.getNext().
          QueryInterface(Ci.nsIPassword);

        if (entry.host == EBAY_HOST_FOR_PASSWORD_MANAGER) {
          tokenContainer.username = entry.user;
          tokenContainer.tokenString = entry.password;
          ret = tokenContainer;
        }
      }
    }

    return ret;
  },

  /**
   * Removes the token from the password manager
   */
  _removeTokenFromPasswordManager : function() {
    let loginManagerService;
    let passwordManagerService;
    // Firefox 2 and 3 use different services for storing passwords.
    {
      let loginManagerClass = Cc["@mozilla.org/login-manager;1"];
      let passwordManagerClass = Cc["@mozilla.org/passwordmanager;1"];

      if (loginManagerClass) {
        // Firefox 3
        loginManagerService =
          loginManagerClass.getService(Ci.nsILoginManager);
      } else {
        //Firefox 2
        passwordManagerService =
          passwordManagerClass.getService(Ci.nsIPasswordManager);
      }
    }

    const EBAY_HOST_FOR_PASSWORD_MANAGER = "eBay.companion";
    if (loginManagerService) {
      // Firefox 3
      const hostName = EBAY_HOST_FOR_PASSWORD_MANAGER;
      const formSubmitURL = EBAY_HOST_FOR_PASSWORD_MANAGER;
      const httpRealm = null;
      let logins;

      logins = loginManagerService.
                 findLogins({}, hostName, formSubmitURL, httpRealm);

      if (logins && logins[0]) {
        loginManagerService.removeLogin(logins[0]);
      }
    } else {
      // Firefox 2
      let passwords = passwordManagerService.enumerator;

      while (passwords.hasMoreElements()) {
        let pass = passwords.getNext().QueryInterface(Ci.nsIPassword);

        if (pass.host == EBAY_HOST_FOR_PASSWORD_MANAGER) {
          passwordManagerService.
            removeUser(EBAY_HOST_FOR_PASSWORD_MANAGER, pass.user);
        }
      }
    }
  },

  /**
   * Returns whether the ebay sidebar is opened or not.
   */
  isSidebarOpen : function() {
    let sidebar = document.getElementById("gs-ebay-toggleSidebar-broadcaster");

    return sidebar.getAttribute("checked");
  },

  /**
   * Opens the sidebar.
   */
  openSidebar : function() {
    window.setTimeout(function() {
      toggleSidebar("gs-ebay-toggleSidebar-broadcaster", true);
    }, 0);
  },

  /**
   * Closes the sidebar.
   */
  closeSidebar : function() {
    toggleSidebar();
  },

  /**
   * Opens or closes the sidebar.
   */
  openCloseSidebar : function() {
    if (this.isSidebarOpen()) {
      this.closeSidebar();
      this._sidebarOpened = false;
    } else {
      this.openSidebar();
      this._sidebarOpened = true;
    }
    this.updateButtonTooltip();
  },

  /**
   * Updates the toolbar (or status bar) button's tooltip after checking the
   * current state of the extension
   */
  updateButtonTooltip : function() {
    try {
      let buttonBroadcaster =
        document.getElementById("gs-ebay-ebaybutton-broadcaster");
      let tooltipStringID;
      let tooltip;
      let toolbarButtonBinding = document.getElementById("gs-ebay-toolbar-button");

      if (this._sidebarOpened) {
        tooltipStringID = "ecBrowser.toolbarbutton.sidebarclose.tooltip";
      } else {
        if ("true" == buttonBroadcaster.getAttribute("connected")) {
          tooltipStringID = "ecBrowser.toolbarbutton.sidebaropen.tooltip";
          //alert(buttonBroadcaster.getAttribute("glowing"));
          if ("true" == buttonBroadcaster.getAttribute("glowing")) {
            tooltipStringID = "ecBrowser.toolbarbutton.sidebarupdate.tooltip";
          }
        } else {
          tooltipStringID = "ecBrowser.toolbarbutton.sidebarconnect.tooltip";
        }
      }

      tooltip = EbayCompanion.Constants.stringBundle.getString(tooltipStringID);
      buttonBroadcaster.setAttribute("tooltiptext", tooltip);
    }
    catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Called when the button is clicked on an gsebayitem element or in a desktop
   * alert
   * @param aEvent the event over the button
   * @param aAlertButton whether this is an alert button being clicked or not
   */
  itemButtonClick : function(aEvent, aArea, aButtonAction, aItem) {
    try {
      // ignore right clicks
      let button = aEvent.button;
      if (button != 2) {
        let itemId = null;
        let activeAccount = EbayCompanion.Datasource.activeAccount();
        let userId = activeAccount ? activeAccount.get("userId") : "";
        let redirectURL;
        let url;
        let args;

        if (aItem) {
          itemId = aItem.get("itemId");
        }
        switch (aButtonAction) {
          case "raise":
            args = {itemid: itemId};
            url = EbayCompanion.Constants.getUrl(aArea, "bid", args);
            break;
          case "pay":
            args = {itemid: itemId};
            url = EbayCompanion.Constants.getUrl(aArea, aButtonAction, args);
            // make sure there is a transaction
            if (aItem.transaction) {
              url += "&transactionId=" + aItem.transaction.get("transactionId");
            }
            break;
          case "relist":
          case "similar":
          case "makeOffer":
          case "reviewOffers":
          case "reviewOffer":
            args = {itemid: itemId};
            url = EbayCompanion.Constants.getUrl(aArea, aButtonAction, args);
            break;
          case "view":
          case "bid":
          case "bin":
            let viewItemUrl = aItem.get("viewItemUrl");
            args = {itemurl: viewItemUrl};
            url = EbayCompanion.Constants.getUrl(aArea, "listing", args);
            break;
          case "paid":
            viewItemUrl = aItem.get("viewItemUrl");
            redirectURL = EbayCompanion.Constants.getUrl(
              "postLogin", "listing", {itemurl: viewItemUrl});

            args = {itemid: itemId,
                    ru: encodeURIComponent(redirectURL)};

            url = EbayCompanion.Constants.getUrl(aArea, aButtonAction, args);
            url += "&ssPageName=MPR";
            // make sure there is a transaction
            if (aItem.transaction) {
              url += "&transId=" + aItem.transaction.get("transactionId");
            }
            break;
          case "sent":
            viewItemUrl = aItem.get("viewItemUrl");
            redirectURL = EbayCompanion.Constants.getUrl(
              "postLogin", "listing", {itemurl: viewItemUrl});

            args = {itemid: itemId,
                    ru: encodeURIComponent(redirectURL)};

            url = EbayCompanion.Constants.getUrl(aArea, aButtonAction, args);
            url += "&ssPageName=SHP";
            // make sure there is a transaction
            if (aItem.transaction) {
              url += "&transId=" + aItem.transaction.get("transactionId");
            }
            break;
          case "feedback":
            // if the item is mine, give feedback to the buyer
            if (aItem.get("sellerUserId").toLowerCase() ==
                  userId.toLowerCase()) {
              args = {itemid: itemId,
                      useridto: aItem.transaction.get("buyerUserId"),
                      useridfrom: userId};
            } else {
            // if the item is not mine, give feedback to the seller
              args = {itemid: itemId,
                      useridto: aItem.get("sellerUserId"),
                      useridfrom: userId};
            }
            url = EbayCompanion.Constants.getUrl(aArea, aButtonAction, args);
            break;
          case "find":
            args = {itemid: itemId,
                    query: escape(aItem.get("title"))};
            url = EbayCompanion.Constants.getUrl(aArea, aButtonAction, args);
            break;
          case "viewFeedback":
            args = {userid: userId};
            url = EbayCompanion.Constants.getUrl(aArea, "userFeedback", args);
            break;
        }
        EbayCompanion.openRawURL(url, aEvent);
      }
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Requests that the user accept the licence for this software.  If he
   * refuses, the software will be uninstalled.
   * @param aUpgrade whether this is an extension upgrade or a fresh install
   */
  showLicence : function(aUpgrade) {
    try {
      if (aUpgrade) {
        this._showUpgradeNotification();
      }
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Determines whether we should display a sidebar notification about extension
   * upgrade to the user. It is only displayed until the user signs in for the
   * first time
   */
  _showUpgradeNotification : function() {
    // import the class if not already imported
    if (typeof(EbayCompanion.Notification) == "undefined") {
      this._importModule("objects/notification.js");
    }

    if (typeof(EbayCompanion.InformationNotificationHelper) == "undefined") {
      this._importModule("helpers/informationNotificationHelper.js");
    }

    let notification =
      new EbayCompanion.Notification(
        EbayCompanion.InformationNotificationHelper.
          TERMS_CONDITIONS_CHANGED_NOTIFICATION);
    notification.addLinkCallback(0, function(aEvent) {
      EbayCompanion.showOptionsPanel(true);
      EbayCompanion.InformationNotificationHelper.dismissNotification(
        EbayCompanion.InformationNotificationHelper.
          TERMS_CONDITIONS_CHANGED_NOTIFICATION);
    });
    notification.set("content",
      EbayCompanion.Constants.stringBundle.getString(
        "notification.message.extensionUpdated"));
    notification.set("priority", 1);
    EbayCompanion.InformationNotificationHelper.queueNotification(
      notification);
  },

  /**
   * Determines whether we should display a sidebar notification about extension
   * terms and conditions to the user. It is only displayed when the sign in
   * sidebar is shown for the first time
   */
  _showTermsNotification : function() {
    // import the class if not already imported
    if (typeof(EbayCompanion.Notification) == "undefined") {
      this._importModule("objects/notification.js");
    }

    if (typeof(EbayCompanion.InformationNotificationHelper) == "undefined") {
      this._importModule("helpers/informationNotificationHelper.js");
    }

    let notification =
      new EbayCompanion.Notification(
        EbayCompanion.InformationNotificationHelper.
          TERMS_CONDITIONS_NOTIFICATION);
    notification.addLinkCallback(0, function(aEvent) {
      EbayCompanion.showOptionsPanel(true, "terms");
    });
    notification.addLinkCallback(1, function(aEvent) {
      EbayCompanion.showOptionsPanel(true, "privacy");
    });
    notification.set("content",
      EbayCompanion.Constants.stringBundle.getString(
        "notification.message.acceptTermsConditions"));
    notification.set("priority", 1);
    EbayCompanion.InformationNotificationHelper.queueNotification(
      notification);
  },

  /**
   * Opens the options panel
   * @param showNewLegalTerms set to true if the user is to be presented with
   *                          the Legal tab as soon as the panel opens
   *                          (used after terms have changed)
   * @param goTo the id of the section in the t&c page that we want to
   * display
   */
  showOptionsPanel : function(showNewLegalTerms, goTo) {
    // Look for a pre-existing instance of the options panel
    var windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
                           getService(Ci.nsIWindowMediator);
    var optionsPanel = windowMediator.getMostRecentWindow("ebayComp:options");
    if (optionsPanel) {
      optionsPanel.focus();
      if (showNewLegalTerms) {
        optionsPanel.Options.showNewLegalTerms(goTo);
      }
    } else {
      let params = {};
      if (showNewLegalTerms) {
        params.showNewLegalTerms = true;
        if (goTo) {
          params.goTo = goTo;
        }
      }
      window.openDialog("chrome://ebaycompanion/content/options/options.xul",
                        "eBay-Options",
                        "chrome,titlebar,toolbar,centerscreen," +
                          "dialog=no,resizable=yes",
                        params);
    }
  },

  /**
   * Creates a new messages notification with the number sent as parameter
   * @param aMessagesNumber the number of messages to be displayed
   */
  showNewMessagesNotification : function(aMessagesNumber) {
    let stringBundle = EbayCompanion.Constants.stringBundle;
    if (aMessagesNumber > 0) {
      let message =
        (aMessagesNumber == 1) ?
          stringBundle.getString(
            "notification.message.email", [aMessagesNumber]) :
          stringBundle.getString(
            "notification.message.email.plural", [aMessagesNumber]);
      // import the class if not already imported
      if (typeof(EbayCompanion.Notification) == "undefined") {
        this._importModule("objects/notification.js");
      }

      if (typeof(EbayCompanion.InformationNotificationHelper) == "undefined") {
        this._importModule("helpers/informationNotificationHelper.js");
      }

      let notification =
        new EbayCompanion.Notification(
          EbayCompanion.InformationNotificationHelper.
            NEW_MESSAGES_NOTIFICATION);
      notification.addLinkCallback(0, function(aEvent){
          EbayCompanion.openPage(aEvent, "sidebarNotification", "myMessages", {});
          EbayCompanion.InformationNotificationHelper.dismissNotification(
            EbayCompanion.InformationNotificationHelper.
              NEW_MESSAGES_NOTIFICATION);
        });
      notification.set("content", message);
      notification.set("priority", 3);
      EbayCompanion.InformationNotificationHelper.queueNotification(notification);
    }
  },

  /**
   * Shows a feedback score change notification
   * @param aDelta the difference in the feedback score to be displayed in the
   * notification
   */
  _showFeedbackChangedNotification : function(aDelta) {
    let stringBundle = EbayCompanion.Constants.stringBundle;
    let deltaString = aDelta > 0 ? "+" + aDelta : aDelta;
    let message = stringBundle.getString(
      "notification.message.feedback", [deltaString]);
    // import the class if not already imported
    if (typeof(EbayCompanion.Notification) == "undefined") {
      this._importModule("objects/notification.js");
    }

    if (typeof(EbayCompanion.InformationNotificationHelper) == "undefined") {
      this._importModule("helpers/informationNotificationHelper.js");
    }

    let notification =
      new EbayCompanion.Notification(
        EbayCompanion.InformationNotificationHelper.FEEDBACK_NOTIFICATION);
    notification.addLinkCallback(0, function(aEvent){
      EbayCompanion.openPage(aEvent, "sidebarNotification", "userFeedback",
        {userid: EbayCompanion.Datasource.activeAccount().get('userId')});
      EbayCompanion.InformationNotificationHelper.dismissNotification(
        EbayCompanion.InformationNotificationHelper.FEEDBACK_NOTIFICATION);
    });
    notification.set("content", message);
    notification.set("priority", 3);
    EbayCompanion.InformationNotificationHelper.queueNotification(notification);
  },

  /**
   * Opens the feedback page (GS widget)
   * @param aEvent the event object.
   * @param aStyle the style for GS i.e. praise, question.
   */
  openFeedbackPage : function(aEvent, aStyle) {
    let feedbackUrl =
      "chrome://ebaycompanion/content/options/feedback/feedbackPane.html";
    if (aStyle) {
      feedbackUrl += "#" + aStyle;
    }
    this.openRawURL(feedbackUrl, aEvent);
  },

  /**
   * Opens a web page given the template name and required arguments.  The
   * area name is used for rover tracking.
   * @param aEvent the event object.
   * @param areaName the mfe value for the request
   * @param pageTemplateName the url template to be used
   * @param args the arguments to be set to the url
   * of what the openPagesInNewTab says
   */
  openPage : function(aEvent, areaName, pageTemplateName, args) {
    try {
      let url = null;
      let that = this;

      // rover wants different area names for toolbar and statusbar buttons.
      if (areaName == "ebayButton") {
        let toolbarbuttonActive =
          document.getElementById("ebayCompanionToolbarItem") != undefined;
        areaName = toolbarbuttonActive ? "toolbarButton" : "statusbarButton";
      }

      let reuseLoginTab = true;
      switch (pageTemplateName) {
        // the login page needs a specially-generated URL
        case "loginPage":
        case "sandboxLoginPage":
          let useSandbox = pageTemplateName.indexOf("sandboxLoginPage") != -1;
          let generateUrlCallback = function(aUrl) {
              that.openAndReuseLoginTab(aUrl);
            };
          EbayCompanion.Datasource.generateLoginUrl(
            areaName, useSandbox, generateUrlCallback);
          break;
        default:
          url =
            EbayCompanion.Constants.getUrl(areaName, pageTemplateName, args);
          reuseLoginTab = false;
          break;
      }

      // try to reuse the same tab if the login page is already opened
      if (url) {
        if (!reuseLoginTab) {
          this.openRawURL(url, aEvent);
        }
      }

    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Method used to reuse the tab where the sign in page was opened (if any).
   * If no tab is already showing it, a new one is opened
   * @param url the login page url
   */
  openAndReuseLoginTab : function(url) {
    var wm =
      Cc["@mozilla.org/appshell/window-mediator;1"]
        .getService(Ci.nsIWindowMediator);
    var browserEnumerator = wm.getEnumerator("navigator:browser");

    // Check each browser instance for our URL
    var found = false;
    while (!found && browserEnumerator.hasMoreElements()) {
      var browserWin = browserEnumerator.getNext();
      var tabbrowser = browserWin.gBrowser;

      // Check each tab of this browser instance
      var numTabs = tabbrowser.browsers.length;
      for (var index = 0; index < numTabs; index++) {
        var currentBrowser = tabbrowser.getBrowserAtIndex(index);
        var checkUrl =
          EbayCompanion.Constants.isAuthAuthURL(currentBrowser.currentURI.spec);
        // a 0 means this is the sign in page
        // a 2 means we are in the successful login page, but if we are there
        // it is because we tried to sign in using an old sign in page from a
        // previous browsing session
        if (checkUrl == 0 || checkUrl == 2) {
          // The sign in page is already opened. Select this tab.
          tabbrowser.selectedTab = tabbrowser.tabContainer.childNodes[index];

          // Focus *this* browser-window
          browserWin.focus();
          currentBrowser.loadURI(url);

          found = true;
          break;
        }
      }
    }

    // Our URL isn't open. Open it now.
    if (!found) {
      var recentWindow = wm.getMostRecentWindow("navigator:browser");
      if (recentWindow) {
        // Use an existing browser window
        recentWindow.delayedOpenTab(url, null, null, null, null);
      }
      else {
        // No browser windows are open, so open a new one.
        window.open(url);
      }
    }
  },

  /**
   * Opens the given URL.
   * @param aURL the url to be opened.
   * @param aEvent the event object.
   * @param aForceNewTab true if force to open in a new tab.
   */
  openRawURL : function(aURL, aEvent, aForceNewTab) {
    if (!aURL) {
      EbayCompanion.Logger.error(
        "Empty URL passed to openRawURL", EbayCompanion.Logger.DUMP_STACK);
      return;
    }

    if (EbayCompanion.Constants.prefBranch.get("openPagesInNewTab") ||
        aForceNewTab) {
      window.openUILinkIn(aURL, "tab");
    } else {
      window.openUILink(aURL, aEvent);
    }

    if (null != aEvent) {
      aEvent.stopPropagation();
    }
  },

  /**
   * Opens the search request.
   * @param aEvent the event object to get the button pressed.
   */
  openSearchQuery : function(aEvent) {
    let url = EbayCompanion.Constants.getUrl(
      "searchBox", "search", {query: getBrowserSelection()});

    if (aEvent.button == 1) {
      window.openUILinkIn(url, "tab");
    } else if (aEvent.button != 2) {
      window.openUILinkIn(url, "tabshifted");
    }
  },

  /**
   * Sets the default web site depending on the locale of the browser
   */
  _setDefaultWebSite : function() {
    let set = false;
    let chosenSite;
    try {
      chosenSite = EbayCompanion.Constants.prefBranch.get("chosenSite");
      if (chosenSite) {
        set = true;
      }
    } catch (e) {
      set = false;
    }
    // get browser's locale and then determine the default site to be used.
    if (!set) {
      try {
        let language = EbayCompanion.UtilityHelper.getSelectedLocale().toUpperCase();
        // first try using the full locale string
        chosenSite = EbayCompanion.Constants.localeSiteDataSelector[language];
        if (!chosenSite) {
          // if that doesn't work, try using the first part of it
          let languageArray = language.split("-");
          language = languageArray[0];
          chosenSite = EbayCompanion.Constants.localeSiteDataSelector[language];
        }
        if (!chosenSite) {
          chosenSite = "US";
        }
        EbayCompanion.Constants.prefBranch.set("chosenSite", chosenSite);
      } catch (e) {
        EbayCompanion.Logger.error(e);
        EbayCompanion.Constants.prefBranch.set("chosenSite", "US");
      }
    }
  },

  /**
   * Opens the first run page when the extension is run for the first time
   */
  _openFirstRunPage : function() {
    this.openPage(null, 'firstRun', 'firstRun', null);
  },

  /**
   * Initializes the eBay button.
   */
  _initeBayButton : function() {
    this._overwriteCustomizeDone();
    this._installToolbarButton();
  },

  /**
   * Installs the toolbar button on the first run.
   */
  _installToolbarButton: function() {
    let buttonInstalled =
      EbayCompanion.Constants.prefBranch.get("buttonAddedToFF4");

    if (!buttonInstalled) {
      let id = "ebayCompanionToolbarItem";

      if (!document.getElementById(id)) {
        let toolbar = document.getElementById("nav-bar");

        // If no afterId is given, then append the item to the toolbar
        let before = null;
        let elem = document.getElementById("urlbar-container");
        if (elem && elem.parentNode == toolbar)
          before = elem;

        //XXX: let's add the button at the end of the toolbar
        toolbar.insertItem(id, null, null, null);
        toolbar.setAttribute("currentset", toolbar.currentSet);
        document.persist(toolbar.id, "currentset");
      }
      EbayCompanion.Constants.prefBranch.set("buttonAddedToFF4", true);
    }

    try {
      BrowserToolboxCustomizeDone(true);
    } catch (e) { }
  },

  /**
   * Overwrites the BrowserToolboxCustomizeDone method.
   */
  _overwriteCustomizeDone : function() {
    let that = this;
    let originalCustomizeDone = BrowserToolboxCustomizeDone;

    BrowserToolboxCustomizeDone = function(toolboxChanged) {
      originalCustomizeDone(toolboxChanged);
      that._afterCustomizeDone();
    };
  },

  /**
   * Processes after the BrowserToolboxCustomizeDone is called.
   */
  _afterCustomizeDone : function() {
    EbayCompanion.Constants.prefBranch.set("buttonInToolbar",
      this._isButtonInMainToolbar());
  },

  /**
   * Checks if the button is placed in the main toolbar.
   * @return true if placed in the main toolbar, false otherwise.
   */
  _isButtonInMainToolbar : function() {
    let toolbar = document.getElementById("nav-bar");

    return (-1 != toolbar.currentSet.indexOf("ebayCompanionToolbarItem"));
  },

  /**
   * Called when a new content document is loaded in any of the opened tabs.
   * Specially used for non-AJAX calls
   */
  _onContentLoaded : function(event) {
    /* Identifies the login auth & auth url. */
    const AUTH_URL_LOGIN = 0;
    /* Identifies the intermediate auth & auth url. */
    const AUTH_URL_INTERMEDIATE = 1;
    /* Identifies the accept auth & auth url. */
    const AUTH_URL_ACCEPT = 2;
    /* Identifies the cancel auth & auth url. */
    const AUTH_URL_CANCEL = 3;
    /* Identifies the normal post sign in url. */
    const REGULAR_URL_LOGIN = 4;
    /* Identifies page loaded when a new item is listed, or marked for
       watching */
    const ITEM_LISTED_URL = 5;
    /* Identifies page loaded when a user purchases a BIN item. */
    const ITEM_PURCHASED_URL = 6;
    /* Identifies the ebay homepage */
    const EBAY_HOMEPAGE = 7;
    /* Identifies page loaded when the places a best offer */
    const ITEM_OFFER_PLACED_URL = 8;
    try {
      let url = String(event.target.location);
      var doc = event.originalTarget; // doc is document that triggered the event
      var win = doc.defaultView; // win is the window for the doc

      let resultPageNum =
        EbayCompanion.Constants.isAuthAuthURL(url);
      let loginBrowser;

      switch (resultPageNum) {
        case AUTH_URL_LOGIN :
          // display notification to user explaining that he must log in
          // to the website for his own security
          if (EbayCompanion.Datasource.activeAccount() &&
              this.isSidebarOpen()) {
            if (url.indexOf("runame=") == -1 ||
                  url.indexOf("runame=&") != -1) {
              let sidebar = document.getElementById("sidebar");
              let stringBundle = EbayCompanion.Constants.stringBundle;

              if (sidebar.contentDocument.
                    getElementById("ebayCompanionSidebar")) {
                // import the class if not already imported
                if (typeof(EbayCompanion.Notification) == "undefined") {
                  this._importModule("objects/notification.js");
                }

                if (typeof(EbayCompanion.InformationNotificationHelper) ==
                      "undefined") {
                  this._importModule(
                    "helpers/informationNotificationHelper.js");
                }

                let notification =
                  new EbayCompanion.Notification(
                    EbayCompanion.InformationNotificationHelper.
                      LOGIN_EXPLAIN_NOTIFICATION);
                notification.set(
                  "content",
                  stringBundle.getString(
                    "notification.message.security"));
                notification.set("priority", 1);
                EbayCompanion.InformationNotificationHelper.queueNotification(
                  notification);

                // dismiss the notification automatically after a certain
                //   length of time (if it wasn't already)
                window.setTimeout(function() {
                  EbayCompanion.InformationNotificationHelper.
                    dismissNotification(notification.get("type"));
                }, 10000);
              }
            }
          } else if (this.isSidebarOpen() && url.indexOf("errmsg") != -1) {
            // the sign in process failed, so we have to stop the throbber
            EbayCompanion.Observers.notify(
              null, EbayCompanion.Topics.SIGN_IN_STOP, null);
          }
          break;
        // auth & auth intermediate url
        case AUTH_URL_INTERMEDIATE :
        break;
        // auth & auth accept url
        case AUTH_URL_ACCEPT :
          /*let authAuthSuccessRoot = "isAuthSuccessful=true";
          let pageLength = url.indexOf(authAuthSuccessRoot);
          pageLength += authAuthSuccessRoot.length;
          let getParam =
            function(paramName, url) {
              let re = RegExp("&" + paramName + "=(.*?)(?:&|$)");
              let matches = re.exec(url);
              let value = null;
              if (matches && matches[1]) {
                value = matches[1];
              }
              return value;
            }
          let params = url.slice(pageLength);*/
          // XXX: this parameter is known to return null with certain frequency,
          // that's why we implemented the sign in form interceptor in
          // Datasource.js to catch and store the latest username used to sign
          // in. We use that stored username here when the success page is
          // returned.
          /*let username = getParam("username", params);*/
          let username = EbayCompanion.Datasource.latestUsername();
          EbayCompanion.Logger.log(
            "Signing in with the username: " + username + "\n");
          if (!username) {
            EbayCompanion.Logger.
              error("Can't find username in Auth & Auth success URL: " + url);
            return;
          } else if (unescape(username).indexOf("@") != -1) {
            username = this.getURLParameters(url, "username");

            if (!username) {
              EbayCompanion.Logger.
                error("Can't find username in Auth & Auth success URL: " + url);
              return;
            }
          }

          // This is the browser that the login took place in; remember that
          // user may have switched tabs, so we can't use the current
          // window.content.
          loginBrowser = gBrowser.getBrowserForDocument(event.target);

          // This is done after login is complete
          let postLogin = function() {
            // open the home page (in the same tab as the login occured)
            // (remember that the tab may have been closed)
            if (loginBrowser.contentDocument) {
              let url = EbayCompanion.Constants.getUrl("postLogin", "homePage");
              loginBrowser.contentDocument.location = url;
            }
          }

          let isSandboxAccount = url.indexOf("sandbox") != -1;
          let success = EbayCompanion.Datasource.
            loginUser(username, isSandboxAccount, postLogin);

          if (success) {
            // notify the observer to change the sidebar status and display the
            // throbber
            EbayCompanion.Observers.notify(
              null, EbayCompanion.Topics.SIGN_IN_START, null);

            // Display a blank page until the login is complete
            loginBrowser.contentDocument.location = "about:blank";
          } else {
            // we tried to sign in using an onld sign in page from an old
            // browsing session, so we re-open the sign in page to generate a
            // "fresh" one
            EbayCompanion.openPage(null, 'ebayButton', 'loginPage', {});
          }
        break;
        // auth & auth cancel url
        case AUTH_URL_CANCEL :
          window.getWebNavigation().stop(nsIWebNavigation.STOP_ALL);
          // This is the browser that the login took place in; remember that
          // user may have switched tabs, so we can't use the current
          // window.content.
          loginBrowser = gBrowser.getBrowserForDocument(event.target);
          // open the home page (in the same tab as the login occured)
          // (remember that the tab may have been closed)
          if (loginBrowser.contentDocument) {
            let url = EbayCompanion.Constants.getUrl("postLogin", "homePage");
            loginBrowser.contentDocument.location = url;
          }
          // notify the observer to change the sidebar status and display the
          // sign in button
          EbayCompanion.Observers.notify(
            null, EbayCompanion.Topics.SIGN_IN_STOP, null);
          break;
        case REGULAR_URL_LOGIN :
          if (typeof(EbayCompanion.InformationNotificationHelper) ==
                "undefined") {
            this._importModule("helpers/informationNotificationHelper.js");
          }
          EbayCompanion.InformationNotificationHelper.dismissNotification(
            EbayCompanion.InformationNotificationHelper.
              LOGIN_EXPLAIN_NOTIFICATION);
          break;
        case ITEM_LISTED_URL :
          if (EbayCompanion.Datasource.activeAccount()) {
            // This is the browser that the login took place in; remember that
            // user may have switched tabs, so we can't use the current
            // window.content.
            loginBrowser = gBrowser.getBrowserForDocument(event.target);
            // open the home page (in the same tab as the login occured)
            // (remember that the tab may have been closed)
            if (loginBrowser && loginBrowser.contentDocument) {
              let loadedDoc = loginBrowser.contentDocument;
              // use XPATH to get the congratulations node from the document
              let itemListedCongratsNodes =
                loadedDoc.evaluate(
                  "//form[@name='ListItemForSale']",
                  loadedDoc,
                  null,
                  XPathResult.ANY_TYPE,
                  null);
              let congratsNode = itemListedCongratsNodes.iterateNext();

              if (congratsNode) {
                // give 3 secs before triggering a hard update
                window.setTimeout(
                  function() {
                    // import the ApiCoordinator only if it is not already there
                    if (typeof(EbayCompanion.ApiCoordinator) == "undefined") {
                      EbayCompanion._importModule("apiCoordinator.js");
                    }
                    EbayCompanion.ApiCoordinator.hardUpdate();
                  }, 3000);
              }
            }

          }
          break;
        case ITEM_PURCHASED_URL :
          if (EbayCompanion.Datasource.activeAccount()) {
            // This is the browser that the login took place in; remember that
            // user may have switched tabs, so we can't use the current
            // window.content.
            loginBrowser = gBrowser.getBrowserForDocument(event.target);
            // open the home page (in the same tab as the login occured)
            // (remember that the tab may have been closed)
            if (loginBrowser && loginBrowser.contentDocument) {
              let loadedDoc = loginBrowser.contentDocument;
              // use XPATH to get the checkout form from the document
              let itemPurchasedCheckoutFormNodes =
                loadedDoc.evaluate(
                  "//form[@name='CheckoutForm2']",
                  loadedDoc,
                  null,
                  XPathResult.ANY_TYPE,
                  null);
              let checkoutForm = itemPurchasedCheckoutFormNodes.iterateNext();

              if (checkoutForm) {
                // give 3 secs before triggering a hard update
                window.setTimeout(
                  function() {
                    // import the ApiCoordinator only if it is not already there
                    if (typeof(EbayCompanion.ApiCoordinator) == "undefined") {
                      EbayCompanion._importModule("apiCoordinator.js");
                    }
                    EbayCompanion.ApiCoordinator.hardUpdate();
                  }, 3000);
              }

              // use XPATH to get the outbid form from the document
              // this way we trigger a client alerts update after the user
              // places a bid
              let bidPlacedOutbidFormNodes =
                loadedDoc.evaluate(
                  "//form[@name='OutbidForm']",
                  loadedDoc,
                  null,
                  XPathResult.ANY_TYPE,
                  null);
              let outbidForm = bidPlacedOutbidFormNodes.iterateNext();

              if (outbidForm) {
                // give 5 secs before triggering a client alerts update
                window.setTimeout(
                  function() {
                    // import the ApiCoordinator only if it is not already there
                    if (typeof(EbayCompanion.ApiCoordinator) == "undefined") {
                      EbayCompanion._importModule("apiCoordinator.js");
                    }
                    EbayCompanion.ApiCoordinator.clientAlertsUpdate();
                  }, 10000);
              }

            }
          }
          break;
        case EBAY_HOMEPAGE:
            if (win == win.top && EbayCompanion.Datasource.activeAccount()) {
              // give 5 secs before triggering a client alerts update
              window.setTimeout(
                function() {
                  // import the ApiCoordinator only if it is not already there
                  if (typeof(EbayCompanion.ApiCoordinator) == "undefined") {
                    EbayCompanion._importModule("apiCoordinator.js");
                  }
                  EbayCompanion.ApiCoordinator.clientAlertsUpdate();
                }, 5000);
            }
          break;
        case ITEM_OFFER_PLACED_URL:
          // the user just placed an offer
          // give 3 secs before triggering a hard update
          window.setTimeout(
            function() {
              // import the ApiCoordinator only if it is not already there
              if (typeof(EbayCompanion.ApiCoordinator) == "undefined") {
                EbayCompanion._importModule("apiCoordinator.js");
              }
              EbayCompanion.ApiCoordinator.hardUpdate();
            }, 3000);
          break;
      }

    }
    catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  getURLParameters : function (aUrl, aParamName) {
    if (aUrl.indexOf("?") > 0) {
      let arrParams = aUrl.split("?");
      let arrURLParams = arrParams[1].split("&");
      let arrParamNames = new Array(arrURLParams.length);
      let arrParamValues = new Array(arrURLParams.length);
      let i = 0;
      for (i = 0; i < arrURLParams.length; i++) {
       let sParam =  arrURLParams[i].split("=");
       arrParamNames[i] = sParam[0];
       if (sParam[1] != "") {
         arrParamValues[i] = unescape(sParam[1]);
       } else {
         arrParamValues[i] = null;
       }
      }

      for (i = 0; i < arrURLParams.length; i++) {
       if (arrParamNames[i] == aParamName){
          return arrParamValues[i];
       }
      }
    }
    return null;
  },

  /**
   * Helps displaying primary alerts that might have come up while the window
   * was minimized
   */
  _onFocus : function(aEvent) {
    try {
      let that = this;
      if (aEvent.originalTarget instanceof Window) {
        let mainWindow = window.QueryInterface(Ci.nsIInterfaceRequestor)
                     .getInterface(Ci.nsIWebNavigation)
                     .QueryInterface(Ci.nsIDocShellTreeItem)
                     .rootTreeItem
                     .QueryInterface(Ci.nsIInterfaceRequestor)
                     .getInterface(Ci.nsIDOMWindow);
        let windowMediator =
          Cc["@mozilla.org/appshell/window-mediator;1"].
            getService(Ci.nsIWindowMediator);
        let mostRecentWindow =
          windowMediator.getMostRecentWindow("navigator:browser");

        if (mainWindow == mostRecentWindow) {
          // import the module when we actually need it
          if (typeof(EbayCompanion.AlertsService) == "undefined") {
            this._importModule("services/alertsService.js");
          }
          EbayCompanion.AlertsService.showCurrentAlerts();

          if (this._blurListener == null) {
            this._blurListener = this.addManagedEventListener(
              window, "blur",
              function(event) that._onBlur(event),
              true);
          }
          EbayCompanion.userIsAway = false;
        }
      }
    } catch(e) {
      EbayCompanion.Logger.error(e);
    }
  },

  /**
   * Helps setting the user as away
   */
  _onBlur : function(aEvent) {
    try {
      if (aEvent.originalTarget instanceof Window) {
        let mainWindow = window.QueryInterface(Ci.nsIInterfaceRequestor)
                     .getInterface(Ci.nsIWebNavigation)
                     .QueryInterface(Ci.nsIDocShellTreeItem)
                     .rootTreeItem
                     .QueryInterface(Ci.nsIInterfaceRequestor)
                     .getInterface(Ci.nsIDOMWindow);
        let windowMediator =
          Cc["@mozilla.org/appshell/window-mediator;1"].
            getService(Ci.nsIWindowMediator);
        let mostRecentWindow =
          windowMediator.getMostRecentWindow("navigator:browser");

        if (mainWindow == mostRecentWindow) {
          // remove the onblur listener
          this.removeManagedEventListener(this._blurListener);
          this._blurListener = null;
          delete this._blurListener;
          EbayCompanion.userIsAway = true;
        }
      }
    } catch(e) {
      EbayCompanion.Logger.error(e);
    }
  },

  /**
   * Registers an event listener, keeping a reference for later easy removal.
   * The returned index can be used to remove the listener later.
   * @param target The element to which the event listener will be attached
   * @param event (as for addEventListener)
   * @param handler (as for addEventListener)
   * @param useCapture (as for addEventListener)
   * @returns index for easy later removal of listener
   */
  addManagedEventListener : function(target, event, handler, useCapture) {
    if (!this._latestManagedEventListenerId) {
      this._latestManagedEventListenerId = 0;
    }
    if (!this._managedEventListeners) {
      this._managedEventListeners = [];
    }

    // Remember for later easy removal
    this._managedEventListeners[++this._latestManagedEventListenerId] =
      {
        target : target,
        event : event,
        handler : handler,
        useCapture : useCapture
      };
    target.addEventListener(event, handler, useCapture);

    return this._latestManagedEventListenerId;
  },

  /**
   * Removes a managed event listener using the index provided by this method's
   * counterpart.
   * @param index The managed event listener's index
   */
  removeManagedEventListener : function(index) {
    if (!this._managedEventListeners || !this._managedEventListeners[index]) {
      EbayCompanion.Logger.error("Invalid managed event listener index",
                                 EbayCompanion.Logger.DUMP_STACK);
      return;
    }

    let eventListener = this._managedEventListeners[index];
    eventListener.target.
      removeEventListener(eventListener.event, eventListener.handler,
                          eventListener.useCapture);

    delete this._managedEventListeners[index];
  },

  /**
   * Removes all managed event listeners
   */
  removeAllManagedEventListeners : function() {
    if (!this._managedEventListeners) {
      return;
    }

    for each (let [index,] in Iterator(this._managedEventListeners)) {
      this.removeManagedEventListener(index);
    }
  },

  /**
   * Imports a module into this object's namespace
   * @param the local path of the module to load
   */
  _importModule : function(moduleName) {
    try {
      Cu.import("resource://ebaycompanion/" + moduleName, EbayCompanion);
    }
    catch (e) {
      EbayCompanion.Logger.
        error("Unable to load module \"" + moduleName + "\"" + e);
    }
  },

  /**
   * Listens to login and logout notifications to update UI
   * @param aConnected
   */
  _setUpdateLoginBroadcaster : function(aConnected) {
    document.getElementById("gs-ebay-ebaybutton-broadcaster").
      setAttribute("connected", aConnected);
    this.updateButtonTooltip();
  },

  /**
   * Adds listener to sidebar spliter to restore the sidebar width.
   */
  _addSidebarSplitterListener : function() {
    let splitter = document.getElementById("sidebar-splitter");
    let sidebarbox = document.getElementById("sidebar-box");
    if (splitter && sidebarbox) {
      let that = this;

      splitter.addEventListener("dblclick", function() {
        if (that.isSidebarOpen()) {
          sidebarbox.width = "300px";
        }
      }, false);
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    let account = null;
    let currentAccount = null;

    switch (aTopic) {
      case EbayCompanion.Topics.ACCOUNT_LOGGED_IN:
        this._setUpdateLoginBroadcaster(true);
        break;
      case EbayCompanion.Topics.ACCOUNT_LOGGED_OUT:
        this._setUpdateLoginBroadcaster(false);
        break;
      case EbayCompanion.Topics.ACCOUNT_FEEDBACK_CHANGED:
        // we do this here and not in the sidebar.js file because we want
        // this kind of notifications be available when the sidebar is closed and
        // the user opens it. If we handled this in sidebar.js, these
        // notifications wouldn't be created if the sidebar is closed
        account = aSubject.object;
        currentAccount = EbayCompanion.Datasource.activeAccount();
        // make sure this is the current account the one
        // being updated
        if (currentAccount && currentAccount.get("userId").toLowerCase() ==
            account.get("userId").toLowerCase()) {
          let notifyFeedbackEnabled =
            EbayCompanion.Constants.prefBranch.get("notifyFeedback");
          let preventNotificationDisplay =
            EbayCompanion.Constants.prefBranch.get(
              "prevent.first.feedback.score.notification");
          if (notifyFeedbackEnabled && !preventNotificationDisplay) {
            let delta = account.get("feedbackRating") -
              aSubject.originalObject.get("feedbackRating");
            this._showFeedbackChangedNotification(delta);
          }
        }
        break;
      case EbayCompanion.Topics.ACCOUNT_UNREAD_MESSAGES_CHANGED:
        account = aSubject.object;
        currentAccount = EbayCompanion.Datasource.activeAccount();
        // make sure this is the current account the one
        // being updated
        let notifyNewMessagesEnabled =
          EbayCompanion.Constants.prefBranch.get("notifyNewMessages");
        if (notifyNewMessagesEnabled && currentAccount &&
            currentAccount.get("userId").toLowerCase() ==
            account.get("userId").toLowerCase()) {
          this.showNewMessagesNotification(
            currentAccount.get("numUnreadMessages"));
        }
        break;
    }
  }
};

EbayCompanion.idleObserver = {
  observe: function(subject, topic, data) {
    if (topic == "idle") {
      EbayCompanion.Logger.log("Browser is idling");
      EbayCompanion.userIsAway = true;
      // check if it is time to flush tracking info to eBay
      if (EbayCompanion.Constants.prefBranch.get("tracking.submitData")) {
        EbayCompanion.Logger.log("Tracked Urls flush taking place during " +
                                 "browser idling.");
        EbayCompanion.submitTrackingReport();
      }
    } else {
      EbayCompanion.Logger.log("Back from idling");
      EbayCompanion.userIsAway = false;
    }
  }
};


window.addEventListener(
  "load", function(){ EbayCompanion.init(); }, false);
window.addEventListener(
  "unload", function(){ EbayCompanion.uninit(); }, false);
