/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

/**
 * Main object for the eBay Companion Options Panel
 */
var Options = {
  _managedPrefObservers: [],

  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      Cu.import("resource://ebaycompanion/helpers/logger.js");
      Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
      Cu.import("resource://ebaycompanion/constants.js");
      Cu.import("resource://ebaycompanion/datasource.js");

      this._populateSites();
      this._populateDefaultTab();
      this._populatePreferenceNames();
      this._populateRegistrationSiteLabel();
      this._initLinkLabels();
      this._syncAlertEnabledBroadcasters();

      // Nasty hack required to keep the window a sane size in MacOS X
      let platformIsMac = navigator.platform.match(/mac/i);
      if (platformIsMac) {
        let prefWindow = document.getElementById("ebayOptions");
        let height = prefWindow.currentPane.contentHeight + 100;
        window.resizeTo(window.innerWidth, height);
      }

      this._managedPrefObservers[0] = function(aSubject, aTopic, aData) {
        Options._populateDefaultTab();
      };
      Constants.prefBranch.addObserver(
        this._managedPrefObservers[0], "default.view");

      if (window.arguments && window.arguments[0].showNewLegalTerms) {
        let goTo = window.arguments && window.arguments[0].goTo;
        this.showNewLegalTerms(goTo);
      }
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  uninit : function() {
    Constants.prefBranch.removeObserver(
      this._managedPrefObservers[0], "default.view");
  },

  /**
   * Resets all alerts preferences to their default values.
   */
  resetAlerts : function() {
    try {
      let alertPreferences = document.getElementById("alert-preferences");
      let alertPrefCount = alertPreferences.childNodes.length;

      // Reset all preferences to their default value
      for (let i = 0; i < alertPrefCount; i++) {
        let pref = alertPreferences.childNodes[i];
        pref.value = pref.defaultValue;
      }

      // Make sure that rows that rows are enabled / disabled as required
      this._syncAlertEnabledBroadcasters();
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Ensures that the broadcasters used to enable / disable the alert rows in
   * the UI are synchronised with the current value of their corresponding
   * preferences.  Basically, this should be called whenever an alert is enabled
   * or disabled without the user clicking on the checkbox.
   */
  _syncAlertEnabledBroadcasters : function() {
    let alertPrefs = document.getElementById("alert-preferences");
    for (let i = 0; i < alertPrefs.childNodes.length; i++) {
      let prefElement = alertPrefs.childNodes[i];
      // All the preferences ending in "isEnabled" have a corresponding
      // broadcaster for disabling the row in the UI
      if (prefElement.id.indexOf("isEnabled") != -1) {
        this.setAlertRowEnabled(prefElement.value, prefElement.id);
      }
    }
  },

  /**
   * Click handler for checkboxes that enable / disable alerts
   * @param event Mouse event
   */
  alertEnabledClick : function(event) {
    try {
      let checkbox = event.target;
      let alertEnabled = checkbox.checked;
      let prefName = checkbox.getAttribute("preference");
      this.setAlertRowEnabled(alertEnabled, prefName);
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Enables or disables an alert row in the options window UI
   * @param isEnabled true to enable, false to disable
   * @param pref Name of the preference corresponding to the alert being
   *             enabled/disabled
   */
  setAlertRowEnabled : function(isEnabled, prefName) {
    let broadcasterName =
      prefName.replace("alerts.", "").
               replace(".isEnabled", "").
               replace(".", "-") + "-broadcaster";
    let broadcaster = document.getElementById(broadcasterName);
    if (!broadcaster) {
      Logger.error("Preference '" + prefName + "' has no corresponding " +
                   "broadcaster (should be '" + broadcasterName + "')");
    }
    broadcaster.setAttribute("disabled", !isEnabled);
  },

  /**
   * Switches to the "Legal" pane and shows the "new terms" message.
   * (Used after an upgrade when the terms have changed.)
   * This method may be called from the browser overlay if the options panel is
   * already open.
   * @param goTo the id of the section in the t&c page that we want to
   * display
   */
  showNewLegalTerms : function(goTo) {
    let prefWindow = document.getElementById("ebayOptions");
    let licencePane = document.getElementById("licence-pane");
    prefWindow.showPane(licencePane);
    let newTermsBox = document.getElementById("licence-newTerms-box");
    newTermsBox.style.visibility = "visible";
    // We need to resize the window to the content, and a weird hack is needed
    // to achieve that reliably in MacOS X

    let goToSection =
      let (that = this) function(id) {
        let licenceFrame = document.getElementById("licence-frame");
        let elem = licenceFrame.contentDocument.getElementById(id);
        elem.parentNode.parentNode.scrollTop = elem.offsetTop;
      }
    if (goTo) {
      goToSection(goTo);
    }
  },

  /**
   * Adds a "name" attribute to every preference element, inferred from its id
   */
  _populatePreferenceNames : function() {
    let prefElements = document.getElementsByTagName("preference");
    let prefRoot = Constants.prefBranch.branchName();
    for (let i = 0; i < prefElements.length; i++) {
      let prefElement = prefElements[i];
      let shortName = prefElement.id;
      let fullName = prefRoot + shortName;
      prefElement.name = fullName;
      // Since this preference element had no name at load time, it doesn't yet
      // have its value set, and none of the widgets will have been initialised
      // with the preference value.  This forces a refresh.
      prefElement.value = prefElement.valueFromPreferences;
      prefElement.updateElements();
    }
  },

  /**
   * Replaces the placeholders in the label below the "use registration site"
   * radio label with the name of the registration site and username.
   */
  _populateRegistrationSiteLabel : function() {
    let siteNameLabel = document.getElementById("preferredsite-regsitename");
    if (Datasource.activeAccount()) {
      let label = siteNameLabel.value;
      let userId = Datasource.activeAccount().get("userId");
      let regSite = Datasource.activeAccount().get("registrationSite");
      if (regSite.indexOf("eBayMotors") != -1) {
        regSite = "US";
      }
      let regSiteName =
        Constants.stringBundle.getString(
          "general.country." + regSite);
      let newLabel = label.replace("#1", userId).
                           replace("#2", regSiteName);
      siteNameLabel.value = newLabel;
      siteNameLabel.removeAttribute("hidden");
    } else {
      siteNameLabel.setAttribute("hidden", true);
    }
  },

  /**
   * Populates the preferred-site dropdown box
   */
  _populateSites : function() {
    let menulist = document.getElementById("preferredsite-menulist");
    let supportedSites = Constants.supportedSites();
    for (let i = 0; i < supportedSites.length; i++) {
      let siteName =
        Constants.stringBundle.getString(
          "general.country." + supportedSites[i]);
      menulist.appendItem(siteName, supportedSites[i]);
    }
  },

  _populateDefaultTab : function() {
    let buying = document.getElementById("default-tab-buying");
    let selling = document.getElementById("default-tab-selling");
    let active = document.getElementById("default-tab-active");
    let ended = document.getElementById("default-tab-ended");

     if (Constants.prefBranch.get("default.view") == 0) {
      active.collapsed = false;
      ended.collapsed = false;
      buying.collapsed = true;
      selling.collapsed = true;
    } else {
      active.collapsed = true;
      ended.collapsed = true;
      buying.collapsed = false;
      selling.collapsed = false;
    }
  },

  /**
   * Initialises labels that contain hyperlinks
   */
  _initLinkLabels : function() {
    // Label at the top of the Licence section
    let label = document.getElementById("licence-info-label");
    let goToSection =
      let (that = this) function(id) {
        let licenceFrame = document.getElementById("licence-frame");
        let elem = licenceFrame.contentDocument.getElementById(id);
        if (id == 'uninstall') {
          elem.parentNode.parentNode.parentNode.scrollTop = elem.offsetTop;
        } else {
          elem.parentNode.parentNode.scrollTop = elem.offsetTop;
        }
      }
    let termsAction =
      let (that = this) function(event) goToSection("terms");
    let privacyAction =
      let (that = this) function(event) goToSection("privacy");
    UtilityHelper.explodeLabelWithLinks(label, [termsAction, privacyAction]);

    // Label at the bottom of the licence section (when terms have changed)
    label = document.getElementById("licence-newTerms-label");
    let uninstallAction =
      let (that = this) function(event) goToSection("uninstall");
    UtilityHelper.explodeLabelWithLinks(label, [uninstallAction]);
  },

  /**
   * Prints the licence
   */
  printLicence : function() {
    let licenceFrame = document.getElementById("licence-frame");
    licenceFrame.contentWindow.print();
  },

  /**
   * Copies the currently-selected licence text to the clipboard
   */
  copySelectedLicenceText : function(event) {
    try {
      let frame = document.getElementById("licence-frame");
      let text = frame.contentWindow.getSelection().toString();
      if (text == "") {
        text = frame.contentDocument.body.textContent;
      }
      let clipboard = Cc["@mozilla.org/widget/clipboardhelper;1"].
                        getService(Ci.nsIClipboardHelper);
      clipboard.copyString(text);
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  }
}

window.addEventListener("load", function(){ Options.init(); }, false);
window.addEventListener("unload", function(){ Options.uninit(); }, false);