/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/eventAnalytics.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
Cu.import("resource://ebaycompanion/services/autocompleteService.js");
Cu.import("resource://ebaycompanion/services/recentSearchService.js");
Cu.import("resource://ebaycompanion/objects/recentSearch.js");

/**
 * Object for the eBay Companion search manager.
 */
Sidebar.SearchManager = {
  /* Search textbox. */
  _searchTextbox : null,
  /* Search panel. */
  _searchPanel : null,
  /* Search timer. */
  _searchTimer : null,
  /* Search query. */
  _searchQuery : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    this._searchPanel = document.getElementById("gs-ebay-search-panel");
    this._searchTextbox = document.getElementById("gs-ebay-search-textbox");

    EbayCompanion.Observers.add(
      this, EbayCompanion.Topics.AUTOCOMPLETE_RESULTS);
  },

  /**
   * Uninitializes the resource.
   */
  uninit : function() {
    this._searchTimer = null;
    this._searchPanel = null;
    this._searchQuery = null;
    this._searchTextbox = null;

    EbayCompanion.Observers.removeAll(this);
  },

  /**
   * Checks whether the search panel is open or not.
   * @return boolean to indicate whether the panel is open or not.
   */
  _isSearchPanelOpen : function() {
    return (this._searchPanel.state == "open");
  },

  /**
   * Closes the search panel.
   */
  _closeSearchPanel : function() {
    this._searchPanel.hidePopup();
  },

  /**
   * Opens the search panel.
   */
  _openSearchPanel : function() {
    let searchBoxCount = this._searchPanel.firstChild.childNodes.length;

    if ("" != this._searchTextbox.value && 0 < searchBoxCount) {
      let searchContainer = document.getElementById("gs-ebay-search-container");
      let searchContainerWidth = searchContainer.boxObject.width;

      this._searchPanel.style.width = searchContainerWidth + "px";
      this._searchPanel.firstChild.selectedIndex = -1;
      this._searchPanel.openPopup(searchContainer, "after_start");
    }
  },

  /**
   * Clears search result items.
   */
  _clearResultItems : function() {
    let box = this._searchPanel.firstChild;

    for (let i = box.itemCount; i >= 0; i--) {
      box.removeItemAt(i);
    }
  },

  /**
   * Selects the next result item.
   */
  _selectNextResultItem : function() {
    let box = this._searchPanel.firstChild;
    let itemCount = box.itemCount;
    let selectedIndex = box.selectedIndex;
    let nextIndex = null;
    let targetElement;

    if (itemCount == 0) {
      box.selectedIndex = -1;
    } else if (selectedIndex == (itemCount - 1)) {
      nextIndex = 0;
      targetElement = box.getItemAtIndex(nextIndex);
      if (targetElement.nodeName == "menuseparator") {
        nextIndex = 1;
      }
    } else {
      nextIndex = (selectedIndex + 1);
      targetElement = box.getItemAtIndex(nextIndex);
      if (targetElement.nodeName == "menuseparator") {
        nextIndex = (selectedIndex + 2);
      }
    }
    if (nextIndex) {
      box.selectedIndex = nextIndex;
      box.ensureIndexIsVisible(nextIndex);
    }
  },

  /**
   * Selects the previous result item.
   */
  _selectPreviousResultItem : function() {
    let box = this._searchPanel.firstChild;
    let selectedIndex = box.selectedIndex;
    let itemCount = box.itemCount;
    let previousIndex = null;
    let targetElement;

    if (box.itemCount == 0) {
      box.selectedIndex = -1;
    } else if (selectedIndex == 0 || selectedIndex == -1) {
      previousIndex = (itemCount - 1);
    } else if (selectedIndex == 1) {
      targetElement = box.getItemAtIndex(0);
      if (targetElement.nodeName == "menuseparator") {
        previousIndex = (itemCount - 1);
      }
    } else {
      previousIndex = (selectedIndex - 1);
      targetElement = box.getItemAtIndex(previousIndex);
      if (targetElement.nodeName == "menuseparator") {
        previousIndex = (selectedIndex - 2);
      }
    }
    if (previousIndex) {
      box.selectedIndex = previousIndex;
      box.ensureIndexIsVisible(previousIndex);
    }
  },

  /**
   * Fills the search panel.
   * @param aQuery The query to be searched.
   */
  _fillSearchPanel : function(aQuery) {
    if (aQuery && aQuery.length) {
      this._searchQuery = aQuery;
      AutocompleteService.search(aQuery);
    }
  },

  /**
   * Executes when the result box is moused over.
   * @param aEvent The event triggered by user.
   */
  handleResultMouseOver : function(aEvent) {
    let item = aEvent.target;

    while (item && item.localName != "richlistitem") {
      item = item.parent;
    }
    if (item) {
      let box = this._searchPanel.firstChild;
      box.selectedItem = item;
    }
  },

  /**
   * Executes when an action is performed on an item.
   * @param aEvent The event triggered by user.
   */
  handleResultSelected : function(aEvent) {
    let selectedItem = this._searchPanel.firstChild.selectedItem;

    if ((aEvent.target.localName != "richlistbox") && selectedItem) {
      let itemStyle = selectedItem.getAttribute("itemStyle");
      let itemTitle = selectedItem.getAttribute("title");
      itemTitle = UtilityHelper.trim(itemTitle);

      this._searchTextbox.value = itemTitle;
      if ("popular-product" == itemStyle) {
        let queryArgs = { itemId : selectedItem.getAttribute("itemId") };
        let url = EbayCompanion.Constants.getUrl(
          "searchSuggestion", "catalog", queryArgs);

        // enforces the "open in new tab" preference value
        //EbayCompanion.openRawURL(url, aEvent, true);
        EbayCompanion.openPage(aEvent, 'searchSuggestion', 'catalog', queryArgs);
      } else {
        let queryArgs = { query: itemTitle };
        EbayCompanion.openPage(aEvent, 'searchBox', 'search', queryArgs);
        AutocompleteService.storeSearch(itemTitle);
        EbayCompanion.Observers.notify(null,
          EbayCompanion.Topics.SEARCH_EXECUTED, encodeURIComponent(itemTitle));
      }
      EventAnalytics.push(
          ['_trackEvent', "308-search-box-search", "searched"]);

      this._closeSearchPanel();
    } else {
      let length = this._searchTextbox.value.length;
      // set the focus back to the textbox.
      this._searchTextbox.focus();
      // unselect the text.
      this._searchTextbox.setSelectionRange(length, length);
    }
  },

  /**
   * Loads the search page.
   * @param aEvent the event object.
   */
  loadSearchPage : function(aEvent) {
    let searchTerm = this._searchTextbox.value;
    searchTerm = UtilityHelper.trim(searchTerm);

    if ("" != searchTerm) {
      searchTerm = encodeURIComponent(searchTerm);
      let queryArgs = { query: searchTerm };

      EventAnalytics.push(
        ['_trackEvent', "308-search-box-search", "searched"]);

      EbayCompanion.openPage(aEvent, 'searchBox', 'search', queryArgs);
      AutocompleteService.storeSearch(searchTerm);
      EbayCompanion.Observers.notify(
        null, EbayCompanion.Topics.SEARCH_EXECUTED, searchTerm);

      this._closeSearchPanel();
    }
  },

  openSavedSearch : function(aEvent, aSearchQuery) {
    let searchQuery =
      aSearchQuery.substring(
        aSearchQuery.lastIndexOf("/") + 1, aSearchQuery.length);
    EventAnalytics.push(
      ['_trackEvent', "311-search-box-saved-search", "clicked"]);
    EbayCompanion.openPage(
      aEvent, 'favoriteSearchClick', 'savedSearch',
      {searchquery: searchQuery});
  },

  /**
   * Shows the favorite searches
   */
  showFavoriteSearches: function(aElement, aEvent) {
    let favoriteSearches = EbayCompanion.Datasource.favoriteSearches();
    let favoriteSearch;
    let favoriteSearchElement;
    let searchPopup = document.getElementById("savedSearchPopup");
    while (searchPopup.firstChild) {
      searchPopup.removeChild(searchPopup.firstChild);
    }

    if (favoriteSearches.length > 0) {
      for (let i = 0; i < favoriteSearches.length; i++) {
        favoriteSearch = favoriteSearches[i];
        favoriteSearchElement = document.createElement("menuitem");
        favoriteSearchElement.setAttribute("oncommand",
          "Sidebar.SearchManager.openSavedSearch(event, '"
          + favoriteSearch.get('searchQuery') + "');");
        favoriteSearchElement.setAttribute("label",
          EbayCompanion.Constants.getUTF8(favoriteSearch.get("searchName")));
        searchPopup.appendChild(favoriteSearchElement);
      }
    } else {
      favoriteSearchElement = document.
        createElementNS("http://www.w3.org/1999/xhtml","div");
      favoriteSearchElement.setAttribute("id", "gs-ebay-menuitem-div");
      let divElement = document.
        createElementNS("http://www.w3.org/1999/xhtml","div");
      let txtnode = document.createTextNode(
        EbayCompanion.Constants.stringBundle.getString(
          "sidebar.saved.searches.empty"));
      divElement.setAttribute("id", "gs-ebay-saved-searches-empty");
      divElement.appendChild(txtnode);
      favoriteSearchElement.appendChild(divElement);
      searchPopup.appendChild(favoriteSearchElement);
    }

    searchPopup.appendChild(document.createElement("menuseparator"));

    favoriteSearchElement =
      document.createElement("menuitem");
    favoriteSearchElement.setAttribute("oncommand",
      "Sidebar.SearchManager.openEmptyListLinkPage(event)");
    favoriteSearchElement.setAttribute("label",
      EbayCompanion.Constants.stringBundle.getString(
        "home.tab.searches.label.edit"));
    searchPopup.appendChild(favoriteSearchElement);
    searchPopup.openPopup(aElement, "after_end", 0, 0, false, true);
    EventAnalytics.push(
      ['_trackEvent', "309-search-box-dropdown", "searched"]);
    aEvent.stopPropagation();
  },

  openEmptyListLinkPage : function(event) {
    EventAnalytics.push(
      ['_trackEvent', "310-search-box-saved-searches", "clicked"]);

    EbayCompanion.openPage(event, "favoriteSearchClick", "savedSearches", {});
  },

  /**
   * Handles the search text input event.
   * @param aEvent the event object.
   */
  handleSearchTextInput : function(aEvent) {
    let query = this._searchTextbox.value;

    // trim query
    query = query.replace(/^\s+|\s+$/g, '');

    if (this._searchTimer) {
      clearTimeout(this._searchTimer);
    }

    if ("" == query) {
      this._closeSearchPanel();
      this._clearResultItems();
    } else {
      let that = this;

      this._searchTimer =
        window.setTimeout(function() {
          that._fillSearchPanel(query);
        }, this._searchTextbox.getAttribute("timeout"));
    }
  },

  /**
   * Handles the search text key press event.
   * @param aEvent the event object.
   */
  handleSearchTextKeyPress : function(aEvent) {
    switch (aEvent.keyCode) {
      case KeyEvent.DOM_VK_TAB:
        if (this._isSearchPanelOpen()) {
          this._selectNextResultItem();
          aEvent.stopPropagation();
          aEvent.preventDefault();
        }
        break;
      case KeyEvent.DOM_VK_DOWN:
        if (this._isSearchPanelOpen()) {
          this._selectNextResultItem();
          aEvent.stopPropagation();
          aEvent.preventDefault();
        } else {
          // minor changes to fix a problem with the suggestions panel not
          // being displayed on Firefox 15 and higher
          //this._openSearchPanel();
          this._showResultItems(this._results, this._searchQuery);
        }
        break;
      case KeyEvent.DOM_VK_UP:
        if (this._isSearchPanelOpen()) {
          this._selectPreviousResultItem();
          aEvent.stopPropagation();
          aEvent.preventDefault();
        } else {
          // minor changes to fix a problem with the suggestions panel not
          // being displayed on Firefox 15 and higher
          //this._openSearchPanel();
          this._showResultItems(this._results, this._searchQuery);
        }
        break;
      case KeyEvent.DOM_VK_ENTER:
      case KeyEvent.DOM_VK_RETURN:
        if (this._isSearchPanelOpen() &&
            this._searchPanel.firstChild.selectedIndex > -1) {
          this.handleResultSelected(aEvent);
          aEvent.stopPropagation();
          aEvent.preventDefault();
        } else {
          this.loadSearchPage(aEvent);
          this._closeSearchPanel();
        }
        break;
      case KeyEvent.DOM_VK_ESCAPE:
        this._searchTextbox.value = "";
        this._searchQuery = null;
        this._results = null;
        this._closeSearchPanel();
        this._clearResultItems();
        break;
    }
  },

  /**
   * Shows search result items.
   * @param aResults The array of results.
   * @param aQuery The query key.
   */
  _showResultItems : function(aResults, aQuery) {
    if (this._searchQuery == aQuery) {
      this._results = aResults;
      let box = this._searchPanel.firstChild;
      let resultCount = aResults.length;
      let result = null;
      let item = null;
      let firstHistoryRecord = true;
      let firstSuggestion = true;
      let firstProduct = true;
      let hideProducts;

      this._clearResultItems();

      for (let i = 0; i < resultCount; i++) {
        hideProducts = true;
        result = aResults[i];

        if ("Suggestion" == result.type) {
          if (firstSuggestion) {
            firstSuggestion = false;
            // adds search suggestion separator
            item = document.createElement("menuseparator");
            item.setAttribute(
              "type", "gs-ebay-search-popup-separator");
            item.setAttribute(
              "value",
              EbayCompanion.Constants.stringBundle.getString(
                "sidebar.search.popup.suggestions.label"));
            box.appendChild(item);
          }
          if ("popular-product" == result.style) {
            hideProducts = false;
            if (firstProduct) {
              firstProduct = false;
              // adds popular products separator
              item = document.createElement("menuseparator");
              item.setAttribute(
                "type", "gs-ebay-search-popup-separator");
              item.setAttribute(
                "value",
                EbayCompanion.Constants.stringBundle.getString(
                  "sidebar.search.popup.products.label"));
              box.appendChild(item);
            }
          }
        } else {
          if (firstHistoryRecord) {
            firstHistoryRecord = false;
            // adds search history separator
            item = document.createElement("menuseparator");
            item.setAttribute(
              "type", "gs-ebay-search-popup-separator");
            item.setAttribute(
              "value",
              EbayCompanion.Constants.stringBundle.getString(
                  "sidebar.search.popup.history.label"));
            box.appendChild(item);
          }
        }

        let title = result.title;
        item = document.createElement("richlistitem");
        item.setAttribute("type", "gs-ebay-search-result-item");
        item.setAttribute("class", "gs-ebay-search-result-item");
        item.setAttribute("query", aQuery);
        item.setAttribute("title", title);
        item.setAttribute("tooltiptext", title);
        item.setAttribute("hideProducts",  hideProducts);

        item.setAttribute("image", result.image);
        item.setAttribute("itemId", result.id);
        item.setAttribute("itemStyle", result.style);

        box.appendChild(item);
      }

      if (0 < resultCount) {
        if (!this._isSearchPanelOpen()) {
          this._openSearchPanel();
        }
      } else {
        this._closeSearchPanel();
      }
    }
  },

  /**
   * Clears the search history
   */
  clearSearchHistory : function() {
    AutocompleteService.clearSearchHistory();
  },

  /**
   * Observes some topics.
   * @param aSubject The subject.
   * @param aTopic The topic.
   * @param aData The data.
   */
  observe : function(aSubject, aTopic, aData) {
    if (EbayCompanion.Topics.AUTOCOMPLETE_RESULTS == aTopic) {
      this._showResultItems(aSubject, aData);
    }
  }
};

window.addEventListener(
  "load", function() { Sidebar.SearchManager.init(); }, false);
window.addEventListener(
  "unload", function() { Sidebar.SearchManager.uninit(); }, false);
