/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");

/**
 * Proxy for main EbayCompanion object in browser overlay
 */
var EbayCompanion = parent.EbayCompanion;

/**
 * Main object for the eBay Companion sidebar
 */
var Sidebar = {
  _signingInUpdating : false,
  _managedPrefObservers: [],

  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      let that = this;

      if (typeof(EbayCompanion.InformationNotificationHelper) == "undefined") {
        EbayCompanion._importModule("helpers/informationNotificationHelper.js");
      }
      if (typeof(EbayCompanion.WarningNotificationHelper) == "undefined") {
        EbayCompanion._importModule("helpers/warningNotificationHelper.js");
      }
      if (typeof(EbayCompanion.StateHelper) == "undefined") {
        EbayCompanion._importModule("helpers/stateHelper.js");
      }

      EbayCompanion.Observers.add(this, EbayCompanion.Topics.ACCOUNT_LOGGED_IN);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.ACCOUNT_LOGGED_OUT);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.LIST_RESET_CURRENT_FILTER);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.SIDEBAR_APPLY_TAB);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.SIDEBAR_ENFORCE_TAB);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.LIST_APPLY_FILTER);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.LIST_FORCE_APPLY_FILTER);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.ITEM_SHOW_DETAILS);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.ITEM_HIDE_DETAILS);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.DEAL_SHOW_DETAILS);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.DEAL_HIDE_DETAILS);
      EbayCompanion.Observers.add(this, EbayCompanion.Topics.UPDATE_FINISHED);

      // listen to dealsTabEnabled preference changes
      this._managedPrefObservers[0] = function(aSubject, aTopic, aData) {
        that._enableDealsUI();
      };
      EbayCompanion.Constants.prefBranch.addObserver(
        this._managedPrefObservers[0], "dealsTabEnabled");

      // listen to chosenSite and useRegistrationSite preferences changes
      this._managedPrefObservers[1] = function(aSubject, aTopic, aData) {
        that._enableDealsUI(true);
      };
      EbayCompanion.Constants.prefBranch.addObserver(
        this._managedPrefObservers[1], "chosenSite");

      this._managedPrefObservers[2] = function(aSubject, aTopic, aData) {
        that._enableDealsUI(true);
      };
      EbayCompanion.Constants.prefBranch.addObserver(
        this._managedPrefObservers[2], "useRegistrationSite");

      this._managedPrefObservers[3] = function(aSubject, aTopic, aData) {
        that.enforceViewCustomization();
        that.changeViewCustomization(
          EbayCompanion.Constants.prefBranch.get("default.view"));
      };
      EbayCompanion.Constants.prefBranch.addObserver(
        this._managedPrefObservers[3], "default.view");

      this._managedPrefObservers[4] = function(aSubject, aTopic, aData) {
        let tab = EbayCompanion.Constants.prefBranch.get("default.tab");
        that.selectTab(tab);
      };
      EbayCompanion.Constants.prefBranch.addObserver(
        this._managedPrefObservers[4], "default.tab");

      // Stop the state helper timer.
      EbayCompanion.StateHelper.stopKeepTimer();

      // restores the sidebar state
      this.restoreState();

      this._applyStylingForLocale();

      // Initialise the tab recent
      this.TabRecent.init();

      this._addFocusHandler();

      // Initialise the Notifications mechanism
      this._initNotifications();

      // Set the eBay button tooltip
      EbayCompanion._sidebarOpened = true;
      EbayCompanion.updateButtonTooltip();
      EbayCompanion.EventAnalytics.push(
        ['_trackEvent', "301-load-extension-popup", "opened"]);
    }
    catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Uninitializes the resource.
   */
  uninit : function() {
    try {
      // Set the state helper timer.
      this._setStateTimer();
      let that = this;
      EbayCompanion.Constants.prefBranch.removeObserver(
        this._managedPrefObservers[0], "dealsTabEnabled");

      EbayCompanion.Constants.prefBranch.removeObserver(
        this._managedPrefObservers[1], "chosenSite");

      EbayCompanion.Constants.prefBranch.removeObserver(
        this._managedPrefObservers[2], "useRegistrationSite");

      EbayCompanion.Constants.prefBranch.removeObserver(
        this._managedPrefObservers[3], "default.view");

      EbayCompanion.Constants.prefBranch.removeObserver(
        this._managedPrefObservers[4], "default.tab");

      EbayCompanion.Observers.removeAll(this);
      this._removeFocusHandler();
      EbayCompanion.EventAnalytics.push(
        ['_trackEvent', "extension-popup-closed", "closed"]);
    }
    catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Set the state timer if it's the last sidebar instance.
   */
  _setStateTimer : function() {
    let windowMediator =
      Cc["@mozilla.org/appshell/window-mediator;1"].
        getService(Ci.nsIWindowMediator);
    let windowEnumerator = windowMediator.getEnumerator("navigator:browser");
    let sidebarInstance = 0;
    let win = null;

    while (windowEnumerator.hasMoreElements()) {
      win = windowEnumerator.getNext();
      if (win.EbayCompanion.isSidebarOpen()) {
        sidebarInstance++;
      }
    }

    if (0 == sidebarInstance) {
      EbayCompanion.StateHelper.startKeepTimer();
    }
  },

  /**
   * Sets locale flag for styling.
   */
  _applyStylingForLocale : function() {
    let language = UtilityHelper.getSelectedLocale().toUpperCase();
    let languageArray = language.split("-");
    let sidebar = document.getElementById("ebayCompanionSidebar");

    sidebar.className = languageArray[0];
  },

  /**
   * Restores the sidebar state
   */
  restoreState : function () {
    // selects the persisted tab
    let connected = EbayCompanion.Datasource.activeAccount() != null;
    this.setConnected(connected);

    this.enforceViewCustomization();
    let selectedTab = EbayCompanion.StateHelper.getSelectedTab();
    let selectedFilter = EbayCompanion.StateHelper.getSelectedFilter();
    let itemDetails = EbayCompanion.StateHelper.getItemDetails();
    let dealDetails = EbayCompanion.StateHelper.getDealDetails();

    // restore the current state.
    let elements = document.getElementsByClassName("menu-filter");
    let className = connected ? "" : "disabled";
    for (let i = 0; i < elements.length; i++) {
      elements[i].parentNode.className = className;
    }
    document.getElementById("menu").setAttribute("connected", connected);

    if (connected)
      this._setFilterSelection(selectedFilter[0], selectedFilter[1]);
    else
      this._clearFilterMenus();
    this.switchToTab(selectedTab);
    this.restoreDetails(itemDetails, dealDetails);
    // check if the daily deals section should be displayed based on the
    // browser's locale and the default site selected by the user
    this._enableDealsUI();
  },

  /**
   * Updates the UI to reflect the given connected state
   */
  setConnected : function(aConnected) {
    // set user's account details
    if (aConnected) {
      // dismiss notifications
      EbayCompanion.WarningNotificationHelper.dismissNotification(
        EbayCompanion.WarningNotificationHelper.SIGNED_OUT_WARNING);
      EbayCompanion.WarningNotificationHelper.dismissNotification(
        EbayCompanion.WarningNotificationHelper.LOGIN_FAILED_WARNING);
      EbayCompanion.WarningNotificationHelper.dismissNotification(
        EbayCompanion.WarningNotificationHelper.SECURITY_SIGN_OUT_WARNING);
      this.clearNotificationBox("gs-ebay-signed-out-warning-notificationbox");
      // also dismiss the terms and conditions notification
      EbayCompanion.InformationNotificationHelper.dismissNotification(
        EbayCompanion.InformationNotificationHelper.
          TERMS_CONDITIONS_NOTIFICATION);
    }
  },

  /**
   * Shows a notification when the user is signed out
   */
  _showSignedOutNotification : function () {
    if (typeof(EbayCompanion.Notification) == "undefined") {
      EbayCompanion._importModule("objects/notification.js");
    }
    let notification =
      new EbayCompanion.Notification(
        EbayCompanion.WarningNotificationHelper.SIGNED_OUT_WARNING);
    notification.set("content",
      EbayCompanion.Constants.stringBundle.getString(
        "notification.warning.signedOut"));
    notification.set("priority", 1);
    EbayCompanion.WarningNotificationHelper.queueNotification(
      notification);
  },

  /**
   * Resets the currently selected tab filter and sets it back to "all".
   */
  _resetCurrentTabFilter : function() {
    let buyingTab = document.getElementById("tab-buying");
    let sellingTab = document.getElementById("tab-selling");
    let activeTab = document.getElementById("tab-active");
    let endedTab = document.getElementById("tab-ended");

    if (buyingTab.hasAttribute("selected")) {
      this._setFilterSelection("buying", "all");
    } else if (sellingTab.hasAttribute("selected")) {
      this._setFilterSelection("selling", "all");
    } else if (activeTab.hasAttribute("selected")) {
      this._setFilterSelection("active", "all");
    } else if (endedTab.hasAttribute("selected")) {
      this._setFilterSelection("ended", "all");
    }
    this.applyFilter(true);
  },

  /**
   * Opens login page.
   */
  connectCommand : function(aEvent) {
    try {
      let useSandbox = false;
      let debuggingEnabled =
        EbayCompanion.Constants.prefBranch.get("debugging.enabled");
      if (debuggingEnabled && aEvent.button == 2) {
        useSandbox = true;
      }
      let pageName = useSandbox ? "sandboxLoginPage" : "loginPage";

      EbayCompanion.EventAnalytics.push(
        ['_trackEvent', "104-signed-out-click-sign-in-button", "clicked"]);

      EbayCompanion.openPage(aEvent, "connectButton", pageName, {});
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Opens support link.
   */
  openSupportLink : function(aEvent) {
    EbayCompanion.EventAnalytics.push(
      ['_trackEvent', "recent-tab-support-link", "clicked"]);
    EbayCompanion.openPage(aEvent, "supportClick", "support", {});
  },

  /**
   * Opens register link.
   */
  openRegisterLink : function(aEvent) {
    EbayCompanion.EventAnalytics.push(
      ['_trackEvent', "106-signed-out-click-register", "clicked"]);
    EbayCompanion.openPage(aEvent, "register", "register", null);
  },

  /**
   * Opens more information page.
   */
  openMoreInformation : function(aEvent) {
    EbayCompanion.EventAnalytics.push(
      ['_trackEvent', "recent-tab-more-information-link", "clicked"]);
    EbayCompanion.openPage(aEvent, "moreInformationClick", "firstRun", {});
  },

  /**
   * Loads a page from the Home button.
   * @param aEvent the event.
   * @param aPage the page name.
   */
  loadHomeButtonPage : function(aEvent, aPage) {
    var buttonClicked = false;
    var gaTargetCode = "305";
    var shouldLoad = true;
    var listClick = false;
    switch (aPage) {
      case "homePage":
        gaTargetCode = "305";
        break;
      case "myeBay":
        gaTargetCode = "312";
        break;
      case "sell":
        gaTargetCode = "307";
        shouldLoad = EbayCompanion.Datasource.activeAccount() != null;
        break;
      case "buy":
        gaTargetCode = "306";
        break;
      case "homeButton":
        gaTargetCode = "313";
        buttonClicked = true;
        aPage = "homePage";
        break;
      case "list":
        listClick = true;
        aPage = "sell";
        break;
    }

    var gaTarget = gaTargetCode + "-homebutton-click-" + aPage;
    if (buttonClicked) {
      gaTarget = gaTargetCode + "-homebutton-click";
    }
    if (listClick) {
      gaTarget = "817-click-selling-sell-an-item";
    }

    if (shouldLoad) {
      EbayCompanion.EventAnalytics.push(
        ['_trackEvent', gaTarget, "clicked"]);
      EbayCompanion.openPage(aEvent, 'sidebarButton', aPage, {});
    }
    // XXX : dropdown menu doesn't get closed automatically, so need to
    // do it with the below.
    let elements = document.getElementsByClassName("dropdown");
    for (let i = 0; i < elements.length; i++) {
      elements[i].className = elements[i].className.replace( /(?:^|\s)open(?!\S)/ , '');
    }
  },

  /**
   * Handles the sign in enter key press event.
   * @param aEvent the event object.
   */
  handleSignInKeyPress : function(aEvent) {
    switch (aEvent.keyCode) {
      case KeyEvent.DOM_VK_ENTER:
      case KeyEvent.DOM_VK_RETURN:
        this.signInCommand(aEvent);
        break;
    }
  },

  /**
   * Called when the button is clicked on an gsebayitem element
   * @param aEvent the event over the button
   */
  itemContextClick : function(aEvent, aAction) {
    let selectedOption = aEvent.target;
    let bindingParent = document.getBindingParent(selectedOption);
    let item = bindingParent.item;
    let itemId = item.get("itemId");
    let itemTransaction = item.transaction;
    let userId = EbayCompanion.Datasource.activeAccount().get("userId");
    let userIsSeller =
      (item.get("sellerUserId").toLowerCase() == userId.toLowerCase());
    let counterpartId = null;
    let url = null;
    let forceNewTab = false;

    //let item = EbayCompanion.Datasource.items()[itemId];
    let args;
    let button = aEvent.button;
    switch (aAction) {
      case 'similar':
      case 'revise':
      case 'promote':
      case 'makeOffer':
      case 'reviewOffers':
        args = {itemid: itemId};
        break;
      case "orderDetails":
        args = {itemid: itemId};
        url = EbayCompanion.Constants.getUrl("itemContext", aAction, args);
        // make sure there is a transaction and its id is not 0
        if (item.transaction) {
          url += "&transid=" + item.transaction.get("transactionId");
        }
        break;
      case 'viewOther':
      case 'userFeedback':
        // Parameters required by the URL template
        if (userIsSeller) {
          counterpartId = itemTransaction.get("buyerUserId");
        } else {
          counterpartId = item.get("sellerUserId");
        }
        args = {userid: counterpartId};
        break;
      case 'askQuestion':
        // Parameters required by the URL template
        args = {
          requested : item.get("sellerUserId"),
          iid :       itemId
        };
        break;
      case 'contactBuyer':
        // Parameters required by the URL template
        args = {
          requested : itemTransaction.get("buyerUserId"),
          iid :       itemId
        };
        break;
      case 'find':
        // Parameters required by the URL template
        var title = escape(item.get("title"));
        args = {
          itemid : itemId,
          query :  title
        };
        break;
      case 'listing': // this is open in new tab
        // parameters required by the URL template
        let viewItemUrl = item.get("viewItemUrl");
        args = {itemurl : viewItemUrl};
        forceNewTab = true;
        break;
      case 'removeItem':
        // here we want to remove the item from the list, so we do it
        // and return since we don't want to open a page here
        // check whether this is a regular watching item or if it is a sold
        // or lost item.
        // Watching items are removed through removeFromWatchList
        // Sold items are removed through removeFromWonList
        // Didn't win items are removed through removeFromDidntWinList
        // import the ApiCoordinator only if it is not already there
        if (typeof(EbayCompanion.ApiCoordinator) == "undefined") {
          EbayCompanion._importModule("apiCoordinator.js");
        }
        // when new windows are opened, the item class might not be defined
        if (typeof(EbayCompanion.Item) == "undefined") {
          EbayCompanion._importModule("objects/item.js");
        }

        let itemState = item.getCurrentState(userId);

        switch (itemState) {
          case EbayCompanion.Item.ITEM_STATE_BEST_OFFER_ITEM_WON:
          case EbayCompanion.Item.ITEM_STATE_BUYING_ITEM_WON:
            EbayCompanion.ApiCoordinator.removeFromWonList(item);
            break;
          case EbayCompanion.Item.ITEM_STATE_BUYING_ITEM_LOST:
            EbayCompanion.ApiCoordinator.removeFromDidntWinList(itemId);
            break;
          case EbayCompanion.Item.ITEM_STATE_WATCHING:
          case EbayCompanion.Item.ITEM_STATE_WATCHING_BEST_OFFER:
          case EbayCompanion.Item.ITEM_STATE_WATCHING_CLASSIFIED_AD:
            EbayCompanion.ApiCoordinator.removeFromWatchList(itemId);
            break;
        }
        return;
        break;
    }

    if (!url) {
      url = EbayCompanion.Constants.getUrl("itemContext", aAction, args);
    }

    EbayCompanion.EventAnalytics.push(
        ['_trackEvent', "303-contextmenu-click-" + aAction, "clicked"]);
    // we are good to go, open the page
    EbayCompanion.openRawURL(url, aEvent, forceNewTab);
    aEvent.stopPropagation();
  },

  /**
   * Called when the button is clicked on an gsebaydeal element
   * @param aEvent the event over the button
   */
  dealContextClick : function(aEvent, aAction) {
    let selectedOption = aEvent.target;
    let bindingParent = document.getBindingParent(selectedOption);
    let deal = bindingParent.deal;
    let itemId = deal.get("itemId");
    let itemViewUrl = deal.get("viewItemUrl");

    // parameters required by the URL template
    let args;
    args = {itemid : itemId,
            itemurl : itemViewUrl};
    let url = EbayCompanion.Constants.getUrl("dealClick", aAction, args);

    // we are good to go, open the page
    EbayCompanion.openRawURL(url, aEvent, true);
    aEvent.stopPropagation();
  },

  /**
   * Forces to show the item details
   * @param aItem the item
   * @param aSkip skip flag
   */
  forceShowItemDetails : function(aItem, aSkip) {
    let itemList = document.getElementById("gs-ebay-sidebar-item-list");

    itemList.doShowItemDetails(aItem, aSkip);
  },

  /**
   * Forces to hide the item details
   * @param aParams the params
   */
  forceHideItemDetails : function(aParams) {
    let itemList = document.getElementById("gs-ebay-sidebar-item-list");

    itemList.doHideItemDetails(
      aParams.callback, aParams.animateListDisplay, aParams.skipAnimation);
  },

  /**
   * Forces to show the deal details
   * @param aDeal the deal
   * @param aSkip skip flag
   */
  forceShowDealDetails : function(aDeal, aSkip) {
    let dealList = document.getElementById("gs-ebay-sidebar-deal-list");

    dealList.doShowDealDetails(aDeal, aSkip);
  },

  /**
   * Forces to hide the deal details
   * @param aParams the params
   */
  forceHideDealDetails : function(aParams) {
    let dealList = document.getElementById("gs-ebay-sidebar-deal-list");

    dealList.doHideDealDetails(
      aParams.callback, aParams.animateListDisplay, aParams.skipAnimation);
  },

  /**
   * Restore the details panel.
   */
  restoreDetails : function(aItemDetails, aDealDetails) {
    if (aItemDetails) {
      let itemList = document.getElementById("gs-ebay-sidebar-item-list");
      let itemNode = itemList.findItem(aItemDetails);

      if (itemNode) {
        EbayCompanion.ItemService.showItemDetails(itemNode, true);
      }
    }
    if (aDealDetails) {
      let dealList = document.getElementById("gs-ebay-sidebar-deal-list");
      let dealNode = dealList.findDeal(aDealDetails);

      if (dealNode) {
        dealList.showDealDetails(dealNode, true);
      }
    }
  },

  /**
   * Executes when mouse over a tab.
   * @param aEvent the event.
   */
  tabMouseOver : function(aEvent) {
    // XXX: hide them to avoid overlapping the tab drop down.
    let searchPopup = document.getElementById("savedSearchPopup");
    if (searchPopup.state == "open") {
      searchPopup.hidePopup();
    }
    let searchPanel = document.getElementById("gs-ebay-search-panel");
    if (searchPanel.state == "open") {
      searchPanel.hidePopup();
    }
  },

  /**
   * Selects a tab by user's action.
   * @param aTabIndex the tab index.
   * @param aSkipAnimation whether to skip filter apply animation
   */
  selectTab : function(aTabIndex, aSkipAnimation) {
    // set default filter
    let connected = EbayCompanion.Datasource.activeAccount() != null;
    if (connected) {
      EbayCompanion.StateHelper.resetSelectedFilter(aTabIndex);
      let selectedFilter = EbayCompanion.StateHelper.getSelectedFilter();
      this._setFilterSelection(selectedFilter[0], selectedFilter[1]);
    }
    this._selectTab(aTabIndex, aSkipAnimation);
  },

  /**
   * Switches to the tab passed as parameter. Filter should be set before using
   * this method.  Otherwise, please use selectTab() instead.
   * @param aTabIndex the index of the tab to switch to
   */
  switchToTab : function(aTabIndex, aSkipAnimation) {
    this._selectTab(aTabIndex, aSkipAnimation);
  },

  /**
   * Selects a tab based on settings.
   * @param aTabIndex the tab index.
   * @param aSkipAnimation whether to skip filter apply animation
   */
  _selectTab : function(aTabIndex, aSkipAnimation) {
    try {
      let gaTarget = "";
      let clickTarget = "";
      // we parse the parameter because when the user changes the view,
      // the value is sent as a string, not a number
      var index = parseInt(aTabIndex);
      switch (index) {
        case 0:
          gaTarget = "901-load-summary";
          clickTarget = "951-click-summary-menu-title";
          break;
        case 1:
          gaTarget = "902-load-buying-view";
          clickTarget = "951-click-summary-menu-title";
          break;
        case 2:
          gaTarget = "904-load-selling-view";
          clickTarget = "951-click-summary-menu-title";
          break;
        case 3:
          gaTarget = "903-load-active-view";
          clickTarget = "951-click-summary-menu-title";
          break;
        case 4:
          gaTarget = "905-load-ended-view";
          clickTarget = "951-click-summary-menu-title";
          break;
        case 5:
          gaTarget = "906-load-deals-view";
          clickTarget = "951-click-summary-menu-title";
          break;
      }

      EbayCompanion.EventAnalytics.push(['_trackEvent', clickTarget, "clicked"]);
      EbayCompanion.StateHelper.setCurrentTab(aTabIndex);
      this._applyTab(aTabIndex, false, aSkipAnimation);
      EbayCompanion.EventAnalytics.push(['_trackEvent', gaTarget, "loaded"]);
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Selects a tab based on the user's selection
   * @param aTab the selected tab
   * @param aPreventPropagation whether to prevent propagation to other windows
   * or not
   * @param aSkipAnimation whether to skip filter apply animation
   */
  _applyTab: function(aTab, aPreventPropagation, aSkipAnimation) {
    this._setTabSelection(aTab);
    let connected = EbayCompanion.Datasource.activeAccount() != null;
    let mainDeck = document.getElementById("gs-ebay-sidebar-main-deck");
    if (aTab == EbayCompanion.StateHelper.HOME_TAB) {
      mainDeck.selectedIndex = 0;
      // home tab has signed in and signed out states, so handle them
      // correctly
      let homeTabDeck =
        document.getElementById("gs-ebay-sidebar-home-tab-deck");
      homeTabDeck.selectedIndex = connected ? 1 : 0;
    } else if (aTab == EbayCompanion.StateHelper.BUYING_TAB ||
               aTab == EbayCompanion.StateHelper.SELLING_TAB ||
               aTab == EbayCompanion.StateHelper.ACTIVE_TAB ||
               aTab == EbayCompanion.StateHelper.ENDED_TAB) {
      // item list
      mainDeck.selectedIndex = 1;
      // home tab has signed in and signed out states, so handle them
      // correctly
      let myEbayTabDeck =
        document.getElementById("gs-ebay-sidebar-myebay-tab-deck");
      if (connected) {
        let firstSignIn =
          EbayCompanion.Constants.prefBranch.get("firstSignIn");
        if (this._signingInUpdating && firstSignIn) {
          myEbayTabDeck.selectedIndex = 2;
        } else {
          myEbayTabDeck.selectedIndex = 1;
          if (connected)
            this.applyFilter(aPreventPropagation, aSkipAnimation);
        }
      } else {
        myEbayTabDeck.selectedIndex = 0;
      }
    } else if (aTab == EbayCompanion.StateHelper.DEALS_TAB) {
      mainDeck.selectedIndex = 2;
    }
    if (!aPreventPropagation) {
      // notify the observer so other windows apply the same filter
      EbayCompanion.Observers.notify(
        null, EbayCompanion.Topics.SIDEBAR_APPLY_TAB, aTab);
    }
  },

  /**
   * Tracks filter selection.
   * @param aTabName the tab name.
   * @param aFilter the filter.
   */
  filterSelected: function(aTabName, aFilter) {
    let connected = EbayCompanion.Datasource.activeAccount() != null;
    if (!connected) {
      return;
    }

    let filterName = aTabName + "." + aFilter;
    let gaTarget = "";
    switch (filterName) {
      case "buying.all":
        gaTarget = "801-click-buying-all";
        break;
      case "buying.watching":
        gaTarget = "802-click-buying-watching-active";
        break;
      case "buying.outbid":
        gaTarget = "803-click-buying-outbid";
        break;
      case "buying.winning":
        gaTarget = "804-click-buying-winning";
        break;
      case "buying.won":
        gaTarget = "805-click-buying-won";
        break;
      case "buying.lost":
        gaTarget = "806-click-buying-didnt-win";
        break;
      case "buying.ended":
        gaTarget = "807-click-buying-ended";
        break;
      case "selling.all":
        gaTarget = "812-click-selling-all";
        break;
      case "selling.active":
        gaTarget = "813-click-selling-active";
        break;
      case "selling.sold":
        gaTarget = "814-click-selling-sold";
        break;
      case "selling.unsold":
        gaTarget = "815-click-selling-unsold";
        break;
      case "selling.offers":
        gaTarget = "816-click-selling-with-offers";
        break;
      case "active.all":
        gaTarget = "808-click-active-all";
        break;
      case "active.buying":
        gaTarget = "809-click-active-buying";
        break;
      case "active.watching":
        gaTarget = "810-click-active-watching";
        break;
      case "active.selling":
        gaTarget = "811-click-active-selling";
        break;
      case "ended.all":
        gaTarget = "818-click-ended-all";
        break;
      case "ended.watching":
        gaTarget = "819-click-ended-watching";
        break;
      case "ended.won":
        gaTarget = "820-click-ended-won";
        break;
      case "ended.lost":
        gaTarget = "821-click-ended-didnt-win";
        break;
      case "ended.sold":
        gaTarget = "822-click-ended-sold";
        break;
      case "ended.unsold":
        gaTarget = "823-click-ended-unsold";
        break;
    }
    EbayCompanion.EventAnalytics.push(['_trackEvent', gaTarget, "clicked"]);

    this._setFilterSelection(aTabName, aFilter);

    let targetTabIndex = this._getTabIndex(aTabName);
    if (this._getTabSelection() != targetTabIndex) {
      this._selectTab(targetTabIndex);
    } else {
      this.applyFilter();
    }
  },

  /**
   * Applies a filter to the item list according to the choices selected by the
   * user in the UI.
   * @param aPreventPropagation whether to prevent the propagation to other tabs
   * or not
   */
  applyFilter : function(aPreventPropagation, aSkipAnimation) {
    try {
      let itemList = document.getElementById("gs-ebay-sidebar-item-list");
      let filterSelection = this._getFilterSelection();
      let filterPrefix = filterSelection[0];
      let filterSuffix = filterSelection[1];
      let filterName = filterPrefix + "." + filterSuffix;
      let filterFunc;
      let filterNotificationKey = filterSuffix;

      if (filterSuffix == "all") {
        filterNotificationKey = null;
      }

      EbayCompanion.StateHelper.setCurrentFilter(filterPrefix, filterSuffix);
      EbayCompanion.WarningNotificationHelper.dismissFilterNotifications();
      if (filterPrefix == "ended") {
        EbayCompanion.WarningNotificationHelper.dismissNotification(
          EbayCompanion.WarningNotificationHelper.
            ITEM_MOVED_TO_ENDED_TAB_WARNING);
      }

      // set reverse sort order before applying the filter, as items are
      // sorted immediately after the filter is applied
      let needsReverseSort = false;
      if (filterPrefix == "ended") {
        needsReverseSort = true;
      } else {
        // we are using buying/selling filters
        let filtersWithReverse = ["won", "lost", "ended", "sold", "unsold"];
        for (let i = 0; i < filtersWithReverse.length; i++) {
          if (filterSuffix == filtersWithReverse[i]) {
            needsReverseSort = true;
            break;
          }
        }
      }

      filterFunc = this._getFilterFunction(filterName);
      itemList.reverseSort = needsReverseSort;
      itemList.applyFilter(filterFunc, aSkipAnimation);
      itemList.updateEmptyListUI(filterName);

      if (!aPreventPropagation) {
        // notify the observer so other windows apply the same filter
        EbayCompanion.Observers.notify(
          null, EbayCompanion.Topics.LIST_APPLY_FILTER, filterName);
      }

      let myEbayDeck =
        document.getElementById("gs-ebay-sidebar-myebay-tab-deck");

      if (filterNotificationKey && 1 == myEbayDeck.selectedIndex) {
        let type;
        let numberElements = itemList.itemList.filteredItems.length;

        switch (filterNotificationKey) {
          case "buying":
            type =
              EbayCompanion.WarningNotificationHelper.BIDDING_FILTER_WARNING;
            break;
          case "watching":
            type =
              EbayCompanion.WarningNotificationHelper.WATCHING_FILTER_WARNING;
            break;
          case "selling":
            filterNotificationKey = "active";
          case "active":
            type =
              EbayCompanion.WarningNotificationHelper.SELLING_FILTER_WARNING;
            break;
          case "ended":
            type =
              EbayCompanion.WarningNotificationHelper.WATCHED_FILTER_WARNING;
            break;
          case "won":
            type =
              EbayCompanion.WarningNotificationHelper.WON_FILTER_WARNING;
            break;
          case "lost":
            type =
              EbayCompanion.WarningNotificationHelper.LOST_FILTER_WARNING;
            break;
          case "sold":
            type =
              EbayCompanion.WarningNotificationHelper.SOLD_FILTER_WARNING;
            break;
          case "unsold":
            type =
              EbayCompanion.WarningNotificationHelper.UNSOLD_FILTER_WARNING;
            break;
          case "outbid":
            type =
              EbayCompanion.WarningNotificationHelper.OUTBID_FILTER_WARNING;
            break;
          case "winning":
            type =
              EbayCompanion.WarningNotificationHelper.WINNING_FILTER_WARNING;
            break;
          case "offers":
            type =
              EbayCompanion.WarningNotificationHelper.OFFERS_FILTER_WARNING;
            break;
        }

        if (typeof(EbayCompanion.Notification) == "undefined") {
          EbayCompanion._importModule("objects/notification.js");
        }

        let notification = new EbayCompanion.Notification(type);
        let notificationText = EbayCompanion.Constants.stringBundle.getString(
            "notification.filter." + filterNotificationKey, [numberElements]);
        notification.set(
          "content", notificationText);
        notification.set("priority", 1);
        EbayCompanion.WarningNotificationHelper.queueNotification(
          notification);
      }

      // request the alerts manager to highlight items (in secondary alerts)
      EbayCompanion.AlertsService.applySecondaryAlerts();
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Forces the specified tab application
   * @param aFilter the filter to be applied
   */
  _forceTab : function (aTab) {
    let tabSelection = this._getTabSelection();

    // apply the filter only if it's different that the one currently applied.
    if (tabSelection != aTab) {
      this._applyTab(aTab, true, false);
    }
  },

  /**
   * Forces the specified filter application
   * @param aFilter the filter to be applied
   */
  _forceFilter : function (aFilter) {
    let filterSelection = this._getFilterSelection();
    let filterPrefix = filterSelection[0];
    let filterSuffix = filterSelection[1];
    let filterName = filterPrefix + "." + filterSuffix;

    // apply the filter only if it's different that the one currently applied.
    if (filterName != aFilter) {
      if (aFilter == null) {
        aFilter = filterName;
      } else {
        filterName = aFilter;
      }
      filterSelection = aFilter.split(".");
      filterPrefix = filterSelection[0];
      filterSuffix = filterSelection[1];
      // force tab selection
      this._setFilterSelection(filterPrefix, filterSuffix);
      // get the filter function here so that in case the "more" tab is selected
      // both the active and ended tab selections are reset.
      let filterFunc = this._getFilterFunction(filterName);

      // item list
      let itemList = document.getElementById("gs-ebay-sidebar-item-list");
      // set reverse sort order before applying the filter, as items are
      // sorted immediately after the filter is applied
      let needsReverseSort = false;
      if (filterPrefix == "ended") {
        needsReverseSort = true;
      } else {
         // we are using buying/selling filters
        let filtersWithReverse = ["won", "lost", "ended", "sold", "unsold"];
        for (let i = 0; i < filtersWithReverse.length; i++) {
          if (filterSuffix == filtersWithReverse[i]) {
            needsReverseSort = true;
            break;
          }
        }
      }

      itemList.reverseSort = needsReverseSort;
      itemList.applyFilter(filterFunc);
      itemList.updateEmptyListUI(filterName);
    }
  },

  /**
   * Gets tab index.
   * @param aTabName the tab name
   *
   * @returns the tab index
   */
  _getTabIndex : function(aTabName) {
    let tab = document.getElementById("tab-" + aTabName);
    let index = EbayCompanion.StateHelper.HOME_TAB;

    if (tab == document.getElementById("tab-recent")) {
      index = EbayCompanion.StateHelper.HOME_TAB;
    } else if (tab == document.getElementById("tab-buying")) {
      index = EbayCompanion.StateHelper.BUYING_TAB;
    } else if (tab == document.getElementById("tab-selling")) {
      index = EbayCompanion.StateHelper.SELLING_TAB;
    } else if (tab == document.getElementById("tab-active")) {
      index = EbayCompanion.StateHelper.ACTIVE_TAB;
    } else if (tab == document.getElementById("tab-ended")) {
      index = EbayCompanion.StateHelper.ENDED_TAB;
    } else if (tab == document.getElementById("tab-deals")) {
      index = EbayCompanion.StateHelper.DEALS_TAB;
    }

    return index;
  },

  /**
   * Returns the current tab selection
   * @returns the current tab selection
   */
  _getTabSelection : function () {
    let homeTab = document.getElementById("tab-recent");
    let buyingTab = document.getElementById("tab-buying");
    let sellingTab = document.getElementById("tab-selling");
    let activeTab = document.getElementById("tab-active");
    let endedTab = document.getElementById("tab-ended");
    let dealsTab = document.getElementById("tab-deals");

    let selected = EbayCompanion.StateHelper.HOME_TAB;
    if (homeTab.hasAttribute("selected")) {
      selected = EbayCompanion.StateHelper.HOME_TAB;
    } else if (buyingTab.hasAttribute("selected")) {
      selected = EbayCompanion.StateHelper.BUYING_TAB;
    } else if (sellingTab.hasAttribute("selected")) {
      selected = EbayCompanion.StateHelper.SELLING_TAB;
    } else if (activeTab.hasAttribute("selected")) {
      selected = EbayCompanion.StateHelper.ACTIVE_TAB;
    } else if (endedTab.hasAttribute("selected")) {
      selected = EbayCompanion.StateHelper.ENDED_TAB;
    } else if (dealsTab.hasAttribute("selected")) {
      selected = EbayCompanion.StateHelper.DEALS_TAB;
    }
    return selected;
  },

  /**
   * Returns an array with the prefix and the suffix of the current filter
   * selection
   * @return an array with the prefix and suffix of the current filter selection
   */
  _getFilterSelection : function () {
    let buyingTab = document.getElementById("tab-buying");
    let sellingTab = document.getElementById("tab-selling");
    let activeTab = document.getElementById("tab-active");
    let endedTab = document.getElementById("tab-ended");

    let filterPrefix;
    let filterSuffix = EbayCompanion.StateHelper.DEFAULT_FILTER;
    if (buyingTab.hasAttribute("selected")) {
      filterPrefix = "buying";
      filterSuffix = EbayCompanion.StateHelper.BUYING_DEFAULT_FILTER;
    } else if (sellingTab.hasAttribute("selected")) {
      filterPrefix = "selling";
      filterSuffix = EbayCompanion.StateHelper.SELLING_DEFAULT_FILTER;
    } else if (activeTab.hasAttribute("selected")) {
      filterPrefix = "active";
    } else if (endedTab.hasAttribute("selected")) {
      filterPrefix = "ended";
    }

    let menu = document.getElementById("tab-" + filterPrefix + "-dropdown");
    if (menu) {
      let menuItems = menu.childNodes;
      for (let i = 0; i < menuItems.length; i++) {
        if (menuItems[i].nodeName == "html:li") {
          let target = menuItems[i].firstChild;
          if (target.firstChild.nodeName == "i") {
            filterSuffix = target.getAttribute("value");
            break;
          }
        }
      }
    }

    return [filterPrefix, filterSuffix];
  },

  /**
   * Set tab selection
   * @param aTab the tab to set
   */
  _setTabSelection : function(aTab) {
    let homeTab = document.getElementById("tab-recent");
    let buyingTab = document.getElementById("tab-buying");
    let sellingTab = document.getElementById("tab-selling");
    let activeTab = document.getElementById("tab-active");
    let endedTab = document.getElementById("tab-ended");
    let dealsTab = document.getElementById("tab-deals");
    let tabToSelect;
    let tabsToUnselect;

    let dealsEnabled = dealsTab.style.display == "";

    if (!aTab || (aTab == EbayCompanion.StateHelper.DEALS_TAB && !dealsEnabled)) {
      aTab = EbayCompanion.StateHelper.HOME_TAB;
    }
    if (aTab == EbayCompanion.StateHelper.HOME_TAB) {
      tabToSelect = homeTab;
    } else if (aTab == EbayCompanion.StateHelper.BUYING_TAB) {
      tabToSelect = buyingTab;
    } else if (aTab == EbayCompanion.StateHelper.SELLING_TAB) {
      tabToSelect = sellingTab;
    } else if (aTab == EbayCompanion.StateHelper.ACTIVE_TAB) {
      tabToSelect = activeTab;
    } else if (aTab == EbayCompanion.StateHelper.ENDED_TAB) {
      tabToSelect = endedTab;
    } else if (aTab == EbayCompanion.StateHelper.DEALS_TAB) {
      tabToSelect = dealsTab;
    }
    [homeTab, buyingTab, sellingTab, activeTab, endedTab, dealsTab].forEach(
      function(aElement) {
        if (aElement == tabToSelect) {
          aElement.setAttribute("selected", true);
        } else {
          aElement.removeAttribute("selected");
        }
      });
  },

  /**
   * Clear filter menus.
   */
  _clearFilterMenus : function() {
    let oldElements = document.getElementsByClassName("list-filter");
    for (let i = 0; i < oldElements.length; i++) {
      let parentNode = oldElements[i].parentNode;
      parentNode.removeChild(oldElements[i]);
    }
  }
,
  /**
   * Set the filter selection according to the values sent as parameter
   * @param aPrefix the prefix of the filter to be set
   * @param aFilter the filter to be applied to the given tab
   */
  _setFilterSelection : function (aPrefix, aFilter) {
    this._clearFilterMenus();

    let menu = document.getElementById("tab-" + aPrefix + "-dropdown");
    if (menu) {
      let menuItems = menu.childNodes;
      for (let i = 0; i < menuItems.length; i++ ) {
        if (menuItems[i].nodeName == "html:li" &&
            menuItems[i].firstChild.getAttribute("value") == aFilter) {
          let target = menuItems[i].firstChild;
          let icon = document.createElementNS("http://www.w3.org/1999/xhtml", "i");
          icon.className = "icon-ok list-filter";
          target.insertBefore(icon, target.firstChild);
          break;
        }
      }
    }
  },

  /**
   * Returns the filter funtion to be used according to the specified filter
   * name
   * @param aFilterName the name of the filter to return its filtering function
   * @return the filtering function that corresponds to the specified filter
   * name
   */
  _getFilterFunction : function (aFilterName) {
    let filter;
    let dataSource = EbayCompanion.Datasource;
    let userId = dataSource.activeAccount().get("userId");
    // when new windows are opened, the item class might not be defined
    if (typeof(EbayCompanion.Item) == "undefined") {
      EbayCompanion._importModule("objects/item.js");
    }

    try {
      switch (aFilterName) {
        case "buying.all":
          filter = function(item) {
                     return item.get("sellerUserId").toLowerCase() !=
                              userId.toLowerCase();
                   }
          break;
        case "selling.all":
          filter = function(item) {
                     return item.get("sellerUserId").toLowerCase() ==
                              userId.toLowerCase();
                   }
          break;
        case "buying.outbid":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return !item.get("isEnded") && timeLeft > 0 &&
                            item.get("userMaxBid") > 0 &&
                            item.get("highBidderId").toLowerCase() !=
                              userId.toLowerCase() &&
                            (item.type == EbayCompanion.Item.ITEM_TYPE_BIDDING);
                   };
          break;
        case "buying.winning":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return !item.get("isEnded") && timeLeft > 0 &&
                            item.get("userMaxBid") > 0 &&
                            item.get("highBidderId").toLowerCase() ==
                              userId.toLowerCase() &&
                            (item.type == EbayCompanion.Item.ITEM_TYPE_BIDDING);
                   };
          break;
        case "active.all":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     // XXX: added won and sold conditions here to fix a bug with
                     // won and sold items randomly showing up in the active tab
                     return !item.get("isEnded") && timeLeft > 0 &&
                            item.get("type") != EbayCompanion.Item.ITEM_TYPE_WON &&
                            item.get("type") != EbayCompanion.Item.ITEM_TYPE_SOLD
                   };
          break;

        case "active.buying":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return !item.get("isEnded") && timeLeft > 0 &&
                            (item.type == EbayCompanion.Item.ITEM_TYPE_BIDDING ||
                             item.type == EbayCompanion.Item.ITEM_TYPE_BEST_OFFER);
                   }
          break;

        case "active.watching":
        case "buying.watching":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return !item.get("isEnded") && timeLeft > 0 &&
                            item.get("sellerUserId").toLowerCase() !=
                              userId.toLowerCase() &&
                            item.get("userMaxBid") == 0 &&
                            (item.get("type") ==
                              EbayCompanion.Item.ITEM_TYPE_WATCHING ||
                            item.get("type") ==
                              EbayCompanion.Item.ITEM_TYPE_BEST_OFFER);
                   }
          break;

        case "active.selling":
        case "selling.active":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return !item.get("isEnded") && timeLeft > 0 &&
                            item.get("sellerUserId").toLowerCase() ==
                              userId.toLowerCase();
                   }
          break;

        case "ended.all":
          // reset active tab filter to "all"
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return item.get("isEnded") || timeLeft == 0;
                   };
          break;

        case "ended.watching":
        case "buying.ended":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return (item.get("isEnded") || timeLeft == 0) &&
                            item.get("sellerUserId").toLowerCase() !=
                              userId.toLowerCase() &&
                            item.get("userMaxBid") == 0 &&
                            item.get("userQuantityWinning") == 0 &&
                            item.get("type") ==
                              EbayCompanion.Item.ITEM_TYPE_WATCHING;
                   }
          break;

        case "ended.won":
        case "buying.won":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return (item.get("isEnded") || timeLeft == 0) &&
                            item.get("userQuantityWinning") > 0 &&
                            item.get("type") == EbayCompanion.Item.ITEM_TYPE_WON;
                   }
          break;

        case "ended.lost":
        case "buying.lost":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return ((item.get("isEnded") || timeLeft == 0) &&
                            item.get("sellerUserId").toLowerCase() !=
                              userId.toLowerCase() &&
                            item.get("userMaxBid") > 0 &&
                            item.get("userQuantityWinning") == 0) ||
                            item.get("type") == EbayCompanion.Item.ITEM_TYPE_LOST;
                   }
          break;

        case "ended.sold":
        case "selling.sold":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return (item.get("isEnded") || timeLeft == 0) &&
                            item.get("sellerUserId").toLowerCase() ==
                              userId.toLowerCase() &&
                            item.get("quantitySold") > 0 &&
                            item.get("type") == EbayCompanion.Item.ITEM_TYPE_SOLD;
                   }
          break;

        case "ended.unsold":
        case "selling.unsold":
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return ((item.get("isEnded") || timeLeft == 0) &&
                            item.get("sellerUserId").toLowerCase() ==
                              userId.toLowerCase() &&
                            item.get("quantitySold") == 0) ||
                            item.get("type") ==
                              EbayCompanion.Item.ITEM_TYPE_UNSOLD;
                   }
          break;
        case "selling.offers":
          filter = function(item) {
                     return item.get("type") == EbayCompanion.Item.ITEM_TYPE_BEST_OFFER &&
                            item.getCurrentState(userId) == EbayCompanion.Item.ITEM_STATE_BEST_OFFER_PENDING;
                   }
          break;
        default:
          filter = function(item) {
                     let endTime = item.get("endTime");
                     let ebayTime =
                       dataSource.getEbayTime().getTime();
                     let timeLeft = Math.max(0, endTime - ebayTime);
                     return !item.get("isEnded") && timeLeft > 0;
                   };
          break;
      }
    } catch (ex) {
      // XXX: for Firefox 4 we get a "EbayCompanion is undefined" error.
      // There's no need to the anything else when catching that error.
    }

    return filter;
  },

  /**
   * Initialises the sidebar notification mechanism
   */
  _initNotifications : function() {
    let that = this;

    // making the import until we actually need the class
    if (typeof(EbayCompanion.WarningNotificationHelper) == "undefined") {
      EbayCompanion._importModule("helpers/warningNotificationHelper.js");
    }
    if (typeof(EbayCompanion.InformationNotificationHelper) == "undefined") {
      EbayCompanion._importModule("helpers/informationNotificationHelper.js");
    }

    EbayCompanion.Observers.add(
      this, EbayCompanion.Topics.NOTIFICATION_WARN_SHOW);
    EbayCompanion.Observers.add(
      this, EbayCompanion.Topics.NOTIFICATION_INFO_SHOW);
    EbayCompanion.Observers.add(
      this, EbayCompanion.Topics.NOTIFICATION_WARN_CLEAR);
    EbayCompanion.Observers.add(
      this, EbayCompanion.Topics.NOTIFICATION_INFO_CLEAR);
    EbayCompanion.Observers.add(
      this, EbayCompanion.Topics.NOTIFICATION_SIGNED_IN_WARN_CLEAR);

    // Set notification boxes' queue helpers, so that they can notify them when
    // the user closes the notification.
    let warnNotificationBox =
      document.getElementById("gs-ebay-warning-notificationbox");
    warnNotificationBox.queueHelper =
      EbayCompanion.WarningNotificationHelper;
    let signedOutWarnNotificationBox =
      document.getElementById("gs-ebay-signed-out-warning-notificationbox");
    signedOutWarnNotificationBox.queueHelper =
      EbayCompanion.WarningNotificationHelper;
    let infoNotificationBox =
      document.getElementById("gs-ebay-information-notificationbox");
    infoNotificationBox.queueHelper =
      EbayCompanion.InformationNotificationHelper;

    //XXX: hack used to style the mac skin mostly
    warnNotificationBox.setAttribute("FF4", true);
    signedOutWarnNotificationBox.setAttribute("FF4", true);
    infoNotificationBox.setAttribute("FF4", true);

    // Show any pre-existing notifications
    EbayCompanion.InformationNotificationHelper.showPendingNotifications();
    EbayCompanion.WarningNotificationHelper.showPendingNotifications();
  },

  /**
   * Displays the next pending notification
   * @param aNotificationBoxId the id of the notification box to be refreshed
   * @param aNotification the notification to be displayed
   */
  _showPendingNotification : function(aNotificationBoxId, aNotification) {
    try {
      let notificationBox = document.getElementById(aNotificationBoxId);
      let that = this;

      // prevent ended items moved to ended list notifications from being
      // displayed while we are in the ended tab
      let selectedTab = this._getTabSelection();
      if (aNotification.get("type") ==
            EbayCompanion.WarningNotificationHelper.
              ITEM_MOVED_TO_ENDED_TAB_WARNING) {
        let itemList = document.getElementById("gs-ebay-sidebar-item-list");
        if (itemList.detailsShown) {
          that.forceHideItemDetails({
            callback : null,
            animateListDisplay : true
          });
        }
        if (selectedTab == EbayCompanion.StateHelper.ENDED_TAB) {
          EbayCompanion.WarningNotificationHelper.dismissNotification(
            EbayCompanion.WarningNotificationHelper.
              ITEM_MOVED_TO_ENDED_TAB_WARNING);
          return;
        }
      }

      // make sure the notification box exists
      if (notificationBox) {
        // clear the notification box first to make sure we only have 1
        // notification queued in the binding any time.
        if (null != notificationBox.currentNotification) {
          // this code prevents notificationbox blinking when the same
          // notification has to be shown again (triggered from somewhere else
          // when it is currently being displayed)
          let currentText = notificationBox.currentNotification.notificationText;
          let notificationText =
            EbayCompanion.Constants.parseMarkupString(
              aNotification.get("content"));
          if (notificationBox.currentNotification.notificationType
                != aNotification.get("type") || currentText != notificationText) {
            notificationBox.removeAllNotifications(true);
            notificationBox.appendCustomNotification(aNotification);
          }
        } else {
          notificationBox.removeAllNotifications(true);
          notificationBox.appendCustomNotification(aNotification);
        }
      }

    } catch(e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Clears the notification box depending on the aNotificationBoxId value
   * passed as parameter
   * @param aNotificationBoxId the id of the notification box to be cleared
   */
  clearNotificationBox : function(aNotificationBoxId) {
    try {
      let notificationBox = document.getElementById(aNotificationBoxId);
      if (notificationBox &&
          notificationBox.allNotifications &&
          notificationBox.allNotifications.length > 0) {
        notificationBox.removeAllNotifications(true);
      }
    } catch(e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Adds a event listener to handle the focus in all elements.
   */
  _addFocusHandler : function() {
    let sidebar = document.getElementById("ebayCompanionSidebar");
    this._handleFocusClick = function(aEvent) {
      if ("ebaysearchbox" != aEvent.target.localName) {
        aEvent.target.focus();
      }
    }

    sidebar.addEventListener("click", this._handleFocusClick, false);
  },

  /**
   * Adds a event listener to handle the focus in all elements.
   */
  _removeFocusHandler : function() {
    let sidebar = document.getElementById("ebayCompanionSidebar");

    sidebar.removeEventListener("click", this._handleFocusClick, false);
  },

  /**
   * Enables/Disables the Deals tab based on the value of enableDealsTab
   * preference
   * @return boolnea the tab is displayed or not.
   */
  enableDealsTab : function() {
    let displayDeals = false;
    let prefEnabled = EbayCompanion.Constants.prefBranch.get("dealsTabEnabled");
    // root preferences
    let language = UtilityHelper.getSelectedLocale().toUpperCase();
    let languageArray = language.split("-");
    let siteId = EbayCompanion.Constants.siteIdForSite(
                   EbayCompanion.Datasource.homeSite());
    /*
    if (EbayCompanion.Datasource.homeSite() != undefined)
      siteId = EbayCompanion.Constants.siteIdForSite(
          EbayCompanion.Datasource.homeSite());
    else
      siteId = EbayCompanion.Constants.siteIdForSite(languageArray[1]);

    if (siteId == 0 || // us
        siteId == 3 || // uk
        siteId == 15 || // australia
        siteId == 2 || // canada US
        siteId == 77 || // germany
        siteId == 210 // canada FR
        ) {
      displayDeals = true;
    }*/

    // XXX: Enabling deals only for browsers in en-US
    if (prefEnabled && language.toLowerCase().indexOf("us") != -1 &&
        siteId == 0) {
      displayDeals = true;
    }

    let dealsTab = document.getElementById("tab-deals");
    dealsTab.parentNode.style.display = displayDeals ? "" : "none";

    return displayDeals;
  },

  /**
   * Enables/Disables the dialy deals UI based on the browser's locale and
   * default site selected
   * @param aPrefChange whether there was a change in the chosenSite or
   * useragent preferences or not
   */
  _enableDealsUI : function(aPrefChange) {
    let displayDeals = this.enableDealsTab();

    if (displayDeals) {
      EbayCompanion.Constants.prefBranch.set("enable.deals.api.requests", true);
      if (aPrefChange) {
        EbayCompanion.ApiCoordinator.pingDealsAPI();
      }
    } else {
      if (EbayCompanion.StateHelper.getSelectedTab() ==
          EbayCompanion.StateHelper.DEALS_TAB) {
        let tab = EbayCompanion.Constants.prefBranch.get("default.tab");
        this.selectTab(tab);
      }
      // set preference to prevent requests being sent to the deals API
      EbayCompanion.Constants.prefBranch.set("enable.deals.api.requests", false);
    }
  },

  enforceViewCustomization : function() {
    let active = document.getElementById("tab-buying");
    let ended = document.getElementById("tab-selling");
    let buying = document.getElementById("tab-active");
    let selling = document.getElementById("tab-ended");

    if (EbayCompanion.Constants.prefBranch.get("default.view") == 0) {
      active.parentNode.style.display = "none";
      ended.parentNode.style.display = "none";
      buying.parentNode.style.display = "";
      selling.parentNode.style.display = "";
    } else {
      active.parentNode.style.display = "";
      ended.parentNode.style.display = "";
      buying.parentNode.style.display = "none";
      selling.parentNode.style.display = "none";
    }
  },

  changeViewCustomization : function(aView) {
    let selectedTab = EbayCompanion.StateHelper.getSelectedTab();
    let defaultTab = EbayCompanion.Constants.prefBranch.get("default.tab");

    //view 0 => Active/Ended,
    if (aView == 0 &&
      (selectedTab == EbayCompanion.StateHelper.BUYING_TAB ||
       selectedTab ==  EbayCompanion.StateHelper.SELLING_TAB)) {
        selectedTab = EbayCompanion.StateHelper.ACTIVE_TAB;
        EbayCompanion.StateHelper.setCurrentTab(selectedTab);
        EbayCompanion.StateHelper.setCurrentFilter("active","all");
    }
    if (aView == 1 &&
      (selectedTab == EbayCompanion.StateHelper.ACTIVE_TAB ||
       selectedTab == EbayCompanion.StateHelper.ENDED_TAB)) {
        selectedTab = EbayCompanion.StateHelper.BUYING_TAB;
        EbayCompanion.StateHelper.setCurrentTab(selectedTab);
        EbayCompanion.StateHelper.setCurrentFilter("buying","watching");
    }
    if (defaultTab != EbayCompanion.StateHelper.DEALS_TAB &&
        defaultTab != EbayCompanion.StateHelper.HOME_TAB) {
      EbayCompanion.Constants.prefBranch.set("default.tab", selectedTab);
    } else {
      // default tab enforces tab, so we only notify SIDEBAR_ENFORCE_TAB
      // if we don't change the default tab value to avoid duplication.
      EbayCompanion.Observers.notify(
        null, EbayCompanion.Topics.SIDEBAR_ENFORCE_TAB, selectedTab);
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    let notificationBoxId;
    let connected;

    switch (aTopic) {
      case EbayCompanion.Topics.ACCOUNT_LOGGED_IN:
        this._signingInUpdating = true;
        this.restoreState();
        // clear the sign in form
        let userId =
          document.getElementById("gs-ebay-sidebar-userId-textbox");
        let pwd =
          document.getElementById("gs-ebay-sidebar-password-textbox");
        if (userId)
          userId.value = "";
        if (pwd)
          pwd.value = "";

        EbayCompanion.EventAnalytics.push(
          ['_trackEvent', "account", "signedIn"]);
        break;
      case EbayCompanion.Topics.ACCOUNT_LOGGED_OUT:
        // reset the state to prevent displaying the last filter used by the
        // previously signed in account
        EbayCompanion.StateHelper.resetState();
        this.restoreState();
        EbayCompanion.InformationNotificationHelper.clearQueue();
        this._showSignedOutNotification();
        EbayCompanion.WarningNotificationHelper.dismissSignedInNotifications();
        EbayCompanion.EventAnalytics.push(
          ['_trackEvent', "account", "signedOut"]);
        break;
      case EbayCompanion.Topics.LIST_RESET_CURRENT_FILTER:
        this._resetCurrentTabFilter();
        break;
      case EbayCompanion.Topics.SIDEBAR_APPLY_TAB:
        this._forceTab(aData);
        break;
      case EbayCompanion.Topics.SIDEBAR_ENFORCE_TAB:
        let selectedFilter = EbayCompanion.StateHelper.getSelectedFilter();
        this._setFilterSelection(selectedFilter[0], selectedFilter[1]);
        this.switchToTab(aData);
        break;
      case EbayCompanion.Topics.LIST_APPLY_FILTER:
        this._forceFilter(aData);
        break;
      case EbayCompanion.Topics.LIST_FORCE_APPLY_FILTER:
        this.applyFilter(true);
        break;
      case EbayCompanion.Topics.ITEM_SHOW_DETAILS:
        this.forceShowItemDetails(aSubject, aData);
        break;
      case EbayCompanion.Topics.ITEM_HIDE_DETAILS:
        this.forceHideItemDetails(aSubject);
        break;
      case EbayCompanion.Topics.DEAL_SHOW_DETAILS:
        this.forceShowDealDetails(aSubject, aData);
        break;
      case EbayCompanion.Topics.DEAL_HIDE_DETAILS:
        this.forceHideDealDetails(aSubject);
        break;
      case EbayCompanion.Topics.NOTIFICATION_WARN_SHOW:
        notificationBoxId = "gs-ebay-warning-notificationbox";
        connected = EbayCompanion.Datasource.activeAccount() != null;
        if (!connected &&
            (aSubject.get("type") ==
            EbayCompanion.WarningNotificationHelper.SIGNED_OUT_WARNING ||
            aSubject.get("type") ==
            EbayCompanion.WarningNotificationHelper.SECURITY_SIGN_OUT_WARNING ||
            aSubject.get("type") ==
            EbayCompanion.WarningNotificationHelper.LOGIN_FAILED_WARNING ||
            aSubject.get("type") ==
             EbayCompanion.WarningNotificationHelper.ACCOUNT_BLOCKED_WARNING)) {
          notificationBoxId = "gs-ebay-signed-out-warning-notificationbox";
        }
        this._showPendingNotification(notificationBoxId, aSubject);
        break;
      case EbayCompanion.Topics.NOTIFICATION_INFO_SHOW:
        notificationBoxId = "gs-ebay-information-notificationbox";
        this._showPendingNotification(notificationBoxId, aSubject);
        break;
      case EbayCompanion.Topics.NOTIFICATION_WARN_CLEAR:
        this.clearNotificationBox("gs-ebay-warning-notificationbox");
        this.clearNotificationBox("gs-ebay-signed-out-warning-notificationbox");
        break;
      case EbayCompanion.Topics.NOTIFICATION_SIGNED_IN_WARN_CLEAR:
        this.clearNotificationBox("gs-ebay-warning-notificationbox");
        break;
      case EbayCompanion.Topics.NOTIFICATION_INFO_CLEAR:
        this.clearNotificationBox("gs-ebay-information-notificationbox");
        break;
      case Topics.UPDATE_FINISHED:
        this._signingInUpdating = false;
        let myEbayTabDeck =
          document.getElementById("gs-ebay-sidebar-myebay-tab-deck");
        myEbayTabDeck.selectedIndex = 1;
        break;
    }
  }
};
