/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["ApiCoordinator"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

var ApiCoordinator = {
  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      Cu.import("resource://ebaycompanion/common.js");
      Cu.import("resource://ebaycompanion/helpers/logger.js");
      Cu.import("resource://ebaycompanion/helpers/observers.js");
      Cu.import("resource://ebaycompanion/helpers/timer.js");
      Cu.import("resource://ebaycompanion/helpers/eventTracker.js");
      Cu.import("resource://ebaycompanion/helpers/apiHelper.js");
      Cu.import("resource://ebaycompanion/helpers/itemList.js");
      Cu.import("resource://ebaycompanion/helpers/warningNotificationHelper.js");
      Cu.import("resource://ebaycompanion/objects/item.js");
      Cu.import("resource://ebaycompanion/objects/notification.js");
      Cu.import("resource://ebaycompanion/apis/tradingApi.js");
      Cu.import("resource://ebaycompanion/apis/shoppingApi.js");
      Cu.import("resource://ebaycompanion/apis/clientAlertsApi.js");
      Cu.import("resource://ebaycompanion/apis/myEbayApplicationApi.js");
      Cu.import("resource://ebaycompanion/apis/systemMessageApi.js");
      Cu.import("resource://ebaycompanion/datasource.js");
      Cu.import("resource://ebaycompanion/constants.js");

      Observers.add(this, Topics.QUIT_APPLICATION_GRANTED);

      let that = this;
      // listen to default site changes to force a hard update for the converted
      // prices to be shown correctly.
      Constants.prefBranch.addObserver(
        function() {
          if (Datasource.activeAccount()) {
            that.hardUpdate();
          }
        },
        "chosenSite");

      this._signInUpdate = false;

      // now we must retrieve the deals at startup, not like we used to do before,
      // only after the user signed in. We wait though to see if the extension
      // signs in automatically
      new Timer(function() {
        if (!Datasource.activeAccount()) {
          that.pingDealsAPI();
        }
      }, 1000);

      this._numActiveUpdates = 0;
      // define a limit for client alerts connect attempts
      this._caConnectAttempts = 5;
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Uninitializes the resource.
   */
  _uninit : function() {
    Observers.removeAll(this);
  },

  /**
   * The number of currently active updates
   */
  get numActiveUpdates() {
    return this._numActiveUpdates;
  },

  /**
   * The current client alerts polling rate
   */
  get clientAlertsPollRate() {
    // return the current alerts polling rate if not null or 5 mins if null
    return this._clientAlertsPollRate ? this._clientAlertsPollRate : 5 * 60 * 1000;
  },

  /**
   * resets the number of active updates to 0
   */
  resetNumActiveUpdates : function() {
    this._numActiveUpdates = 0;
  },

  /**
   * Enable updates
   * @param aAutomaticLogin whether this is an automatic login or not. Helps
   * knowing whether we should call SetNotificationPreferences during Client
   * Alerts login process or not.
   */
  enableUpdates : function(aAutomaticLogin) {
    try {
      const HARD_UPDATE_INTERVAL = 1000 * 60 * 60 * 8;
      // we change the soft update time to 3:58 to prevent both soft and hard
      // updates to be triggered at the same time.
      const SOFT_UPDATE_INTERVAL = (1000 * 60 * 60 * 4) - (1000 * 60 * 2);
      const CLIENT_ALERTS_RECONNECT_INTERVAL = 5 * 60 * 1000; // 5 mins
      let that = this;

      // Disable existing timers before resetting them
      if (this._updatesEnabled) {
        this.disableUpdates();
      }

      // Hard Update
      this._hardUpdateTimer =
        new Timer(function() that.hardUpdate(),
                  HARD_UPDATE_INTERVAL, Timer.TYPE_REPEATING_SLACK);

      // Soft Update
      this._softUpdateTimer =
        new Timer(function() that.softUpdate(), SOFT_UPDATE_INTERVAL,
                  Timer.TYPE_REPEATING_SLACK);

      // Client Alerts
      let callback =
        function(numErrors, systemErrors) {
          if (numErrors > 0) {
            Logger.error("Unable to log in to Client Alerts.");
            // schedule a timer to try to sign in again in 5 minutes
            if (that._caConnectAttempts > 0 && systemErrors) {
              new Timer(function() {
                  // retry sign in only if there's a connected user
                  if (Datasource.activeAccount()) {
                    that._connectClientAlerts(callback, aAutomaticLogin);
                  }
                },
                CLIENT_ALERTS_RECONNECT_INTERVAL, Timer.TYPE_ONE_SHOT);
              that._caConnectAttempts--;
            }
            return;
          }
          that._clientAlertsTimer =
            new Timer(function() that.clientAlertsUpdate(),
                      that._optimalTimeToNextClientAlertsUpdate());
        }
      this._connectClientAlerts(callback, aAutomaticLogin);

      // Maintain list of active items sorted by ending time, so that we can
      // adjust the Client Alerts poll rate as necessary.
      this._itemsByEndingTime = new ItemList("itemsByEndTime");
      this._itemsByEndingTime.filter(function(item) {
                                       return !item.get("isEnded");
                                     });
      this._itemsByEndingTime.sort(function(a, b) {
                                     return a.get("endTime") < b.get("endTime");
                                   });
      this._itemsByEndingTime.
        addListener({
          itemAddedEvent: function(index, itemId) {
            that._adjustClientAlertsPollPeriod();
          },
          itemRemovedEvent: function(index) {
            that._adjustClientAlertsPollPeriod();
          }
        });

      // Updates Enabled
      this._updatesEnabled = true;

      // Do hard update straight away
      this._signInUpdate = true;
      this.hardUpdate();

      // get messages for the day
      SystemMessageApi.getMessagesForTheDay();

      // pings the deals API if necessary
      this.pingDealsAPI();

    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Disable updating
   */
  disableUpdates : function() {
    try {
      // Make sure the timers are actually active
      if (!this._updatesEnabled) {
        return;
      }

      // Hard Update
      this._hardUpdateTimer.cancel();

      // Soft Update
      this._softUpdateTimer.cancel();

      // Client Alerts
      // (User may log out before CA has had a chance to log in.)
      if (this._clientAlertsTimer) {
        this._clientAlertsTimer.cancel();
        delete this._clientAlertsTimer;
      }

      // Delete any pending requests to avoid data being updated when they
      // return
      ApiHelper.abortPendingRequests();

      // Updates Disabled
      delete this._updatesEnabled;

      // reset the number of active updates to 0 in case it isn't
      this.resetNumActiveUpdates();
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Returns the period of time that should elapse before the next Client Alerts
   * update.
   * @returns time in ms
   */
  _optimalTimeToNextClientAlertsUpdate : function() {
    const maxPollingPeriod = 5 * 60 * 1000; // 5 mins
    const minPollingPeriod = 10 * 1000; // 10 secs
    const pollRatio = 0.2; // Poll at this ratio to remaining time (20%)
    const maxTimeLeftAfterEnded = -30000; // 30 secs.

    let itemEndingSoonest = this._itemsByEndingTime.filteredItems[0];
    let endTime, ebayTime, timeLeft;
    let i = 1;
    while (itemEndingSoonest) {
      endTime = itemEndingSoonest.get("endTime");
      ebayTime = Datasource.getEbayTime().getTime();
      timeLeft = endTime - ebayTime;
      if (timeLeft > maxTimeLeftAfterEnded) {
        // we found the item we needed, then we break
        break;
      } else {
        // this means we reached the end of the array and didn't find
        // an item we could use to adjust the client alerts polling rate
        if (i >= this._itemsByEndingTime.filteredItems.length) {
          itemEndingSoonest = null;
        } else {
          itemEndingSoonest = this._itemsByEndingTime.filteredItems[i];
          i++;
        }
      }
    }
    let pollPeriod;
    if (itemEndingSoonest) {
      timeLeft = Math.max(0, endTime - ebayTime);
      // Start with ratio, then apply limits
      pollPeriod = Math.round(timeLeft * pollRatio);
      pollPeriod = Math.max(pollPeriod, minPollingPeriod);
      pollPeriod = Math.min(pollPeriod, maxPollingPeriod);
    } else {
      pollPeriod = maxPollingPeriod;
    }

    this._clientAlertsPollRate = pollPeriod;

    return pollPeriod;
  },

  /**
   * Adjusts the Client Alerts polling period, which depends on how soon the
   * next item will end
   */
  _adjustClientAlertsPollPeriod : function() {
    try {
      // Client alerts may not be connected yet, in which case we don't need to
      // adjust anything.
      if (!this._clientAlertsTimer) {
        return;
      }

      let oldPollPeriod = this._clientAlertsTimer.interval;
      let newPollPeriod = this._optimalTimeToNextClientAlertsUpdate();

      if (newPollPeriod != oldPollPeriod) {
        // Add or remove from the time that's already elapsed since the last
        // update
        let timeSinceLastUpdate = this._clientAlertsTimer.elapsedTime;
        let adjustedPeriod = newPollPeriod - timeSinceLastUpdate;
        adjustedPeriod = Math.max(0, adjustedPeriod);
        this._clientAlertsTimer.cancel();
        this._clientAlertsTimer =
          new Timer(let (that = this) function() that.clientAlertsUpdate(),
                    adjustedPeriod);
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Connects and configures the Client Alerts service
   * @param callback
   * @param automaticLogin whether this is an automatic login or not. It helps
   * determining whether we should call SetNotificationPreferences or not
   */
  _connectClientAlerts : function(callback, automaticLogin) {
    let eventTracker = new EventTracker("Connect Client Alerts Sequence");
    let notificationsSet = eventTracker.addCallbackEvent();
    let gotAuthToken = eventTracker.addCallbackEvent();
    let loggedIn = eventTracker.addCallbackEvent();
    let systemErrors = false;
    eventTracker.doWhenAllFinished(function () {
        callback (eventTracker.eventErrors(), systemErrors);
      });
    let that = this;

    let errorWatcher = new EventTracker("Connect Client Alerts Calls");
    errorWatcher.doWhenAllFinished(function(numErrors) {
      eventTracker.failRemainingEvents();
    });

    let notificationsCallback =
      function(result) {
        if (result.errors) {
          Logger.error("SetNotificationPreferences returned an error.");
          if (result.errors.systemErrors) {
            systemErrors = true;
          }
          eventTracker.failRemainingEvents();
          return;
        }
        notificationsSet();
        errorWatcher.addRequest(
          TradingApi.getClientAlertsAuthToken(tokenCallback));
      };

    let tokenCallback =
      function(result) {
        if (!result.token || result.errors) {
          Logger.error("GetClientAlertsAuthToken returned an empty token or " +
                       "errors.");
          if (result.errors && result.errors.systemErrors) {
            systemErrors = true;
          }
          eventTracker.failRemainingEvents();
          return;
        }
        gotAuthToken();
        errorWatcher.addRequest(
          ClientAlertsApi.login(result.token, loginCallback));
      }
    let loginCallback =
      function(result) {
        if (!result.sessionData || !result.sessionId) {
          Logger.error("No SessionData or SessionID returned by Login call");
          eventTracker.failRemainingEvents();
          return;
        }
        loggedIn();
      }

    // if this is an automatic login, we can skip the call to
    // SetNotificationPreferences
    if (automaticLogin) {
      // we didn't have to call SetNotificationPreferences, so we call the
      // callback event to make the event tracker finish
      notificationsSet();
      errorWatcher.addRequest(
        TradingApi.getClientAlertsAuthToken(tokenCallback));
    } else {
      errorWatcher.addRequest(
        TradingApi.setNotificationPreferences(notificationsCallback));
    }
  },

  /**
   * Called when an update starts
   */
  _updateStarted : function(updateName) {
    this._numActiveUpdates++;
    Observers.notify(null, Topics.UPDATE_STARTED, updateName);
  },

  /**
   * Called when an update finished
   */
  _updateFinished : function(updateName) {
    if (this._numActiveUpdates > 0) {
      this._numActiveUpdates--;
    }
    Observers.notify(null, Topics.UPDATE_FINISHED, updateName);
  },

  /**
   * Hard update
   */
  hardUpdate : function(callback) {
    try {
      let that = this;
      let numPhaseOneCallsReceived = 0;
      let buyingSellingCallsProcessed = 0;
      this._updateStarted("Hard Update");

      // Using "let" for these seems to produce a race condition or similar.  I
      // get "reserved slot index out of range" now and then.
      var trackedItems = [];
      var trackedTransactions = [];
      var localCallback;

      // Track each event so that we can know when we're done
      let hardUpdateTracker = new EventTracker("Hard Update");
      let phaseOneComplete = hardUpdateTracker.addCallbackEvent();
      let phaseTwoComplete = hardUpdateTracker.addCallbackEvent();
      let accountUpdateComplete = hardUpdateTracker.addCallbackEvent();
      hardUpdateTracker.doWhenAllFinished(function(numErrors) {
        if (that._signInUpdate) {
          // reset the sidebar (full) list to make sure it is current
          that._signInUpdate = false;
          Observers.notify(null, Topics.SIDEBAR_LIST_RESET, "sidebarList");
        }
        if (Constants.prefBranch.get("update.collection")) {
          Constants.prefBranch.set("update.collection", false);
        }
        if (callback) {
          callback(numErrors);
        }
        that._updateFinished("Hard Update");
      });

      // flag used to know when there where eBay API response errors.
      let apiErrors = false;

      // Phase One involves calling GetMyeBayBuying and GetMyeBaySelling.  When
      // both have returned, we check the items they've returned against the
      // datasource to remove any old items.
      let phaseOneTracker = new EventTracker("Hard Update Phase One");
      let phaseOneCallsReceived = phaseOneTracker.addCallbackEvent();
      phaseOneTracker.doWhenAllFinished(function(numErrors) {
        if (numErrors > 0 || apiErrors) {
          Logger.error("Errors in Hard Update Phase One; " +
                       "skipping removal of old items as the tracked item " +
                       "list is probably incorrect.");
        } else {
          let hashList = {};
          for (let i = 0; i < trackedItems.length; i++) {
            hashList[trackedItems[i].get("itemId")] = true;
          }
          let dsItems = Datasource.items();
          for each (let [itemId, item] in Iterator(dsItems)) {
            if (!hashList[itemId]) {
              Datasource.removeItem(item);
            }
          }

          // delete those transactions that are not being tracked anymore
          let dsTransactions = Datasource.getAllTransactions();

          for each (let [itemId, transactionArray] in Iterator(dsTransactions)) {
            for each (let [transactionId, transaction] in Iterator(transactionArray)) {
              let removeTransaction = true;
              for (let i = 0; i < trackedTransactions.length; i++) {
                let trackedTransaction = trackedTransactions[i];
                if (transaction.get("itemId") == trackedTransaction.get("itemId") &&
                    transaction.get("transactionId") == trackedTransaction.get("transactionId")) {
                  removeTransaction = false;
                  break;
                }
              }
              if (removeTransaction) {
                Datasource.removeTransaction(transaction);
              }
            }
          }
          if (Constants.prefBranch.get("update.collection")) {
            Constants.prefBranch.set("update.collection", false);
          }

        }
        phaseOneComplete(numErrors);

        // If we're not tracking any items, the phase two tracker doesn't
        // contain any events, so finish phase two directly:
        /*if (phaseTwoTracker.numPendingEvents() == 0) {
          phaseTwoComplete();
        }*/
      });

      // Phase Two contains the GetItem calls that are spawned as the Phase One
      // calls return.  We don't wait until both Phase One calls have returned
      // to start on Phase Two -- we start it as soon as the first one returns.
      let phaseTwoTracker = new EventTracker("Hard Update Phase Two");
      let buyingSellingProcessed = phaseTwoTracker.addCallbackEvent();
      phaseTwoTracker.doWhenAllFinished(function() {
          phaseTwoComplete();
        });

      // GetMyeBayBuying & GetMyeBaySelling
      localCallback =
        function(result) {
          if (result.error) {
            Logger.warning("Hard update reported API errors");
            // we create the sidebar notification to let the user know
            // addToWatchList failed
            let notification =
              new Notification(
                WarningNotificationHelper.HARD_UPDATE_ERROR_WARNING);
            notification.set(
              "content", Constants.stringBundle.getString(
                "notification.error.download"));
            notification.set("priority", 1);
            WarningNotificationHelper.queueNotification(notification);
            // there were API errors
            apiErrors = true;
            return;
          }

          // Log buying/selling received
          numPhaseOneCallsReceived++;

          // dismiss any download error notification if we succeed in the
          // subsequent requests
          WarningNotificationHelper.dismissNotification(
            WarningNotificationHelper.HARD_UPDATE_ERROR_WARNING);

          let items = result.items;
          let transactions = result.transactions;
          let favoriteSellers = result.favoriteSellers;

          if (favoriteSellers) {
            // delete the favorite sellers for the current account and clear
            // the array kept in memory
            Datasource.clearAccountFavoriteSellers();
            // Update or add favoriteSellers to Datasource
            for (let i = 0; i < favoriteSellers.length; i++) {
              Datasource.addFavoriteSeller(favoriteSellers[i]);
            }
            // notify observer to refresh favorite sellers UI
            Observers.notify(null, Topics.FAVORITE_SELLERS_UPDATED, null);
          }

          let favoriteSearches = result.favoriteSearches;
          if (favoriteSearches) {
            // delete the favorite searches for the current account and clear
            // the array kept in memory
            Datasource.clearAccountFavoriteSearches();
            // Update or add favoriteSearches to Datasource
            for (let i = 0; i < favoriteSearches.length; i++) {
              Datasource.addFavoriteSearch(favoriteSearches[i]);
            }
            // notify observer to refresh favorite searches UI
            Observers.notify(null, Topics.FAVORITE_SEARCHES_UPDATED, null);
          }

          // Update eBay time
          if (result.timestamp) {
            Datasource.setEbayTime(result.timestamp);
          }

          // Update or add transactions to Datasource
          for (let i = 0; i < transactions.length; i++) {
            let resultTransaction = transactions[i];
            // if the related item is not stored yet, change the _signInUpdate
            // flag to reset the sidebar list when the hard udpate finishes.
            let transItemId = resultTransaction.get("itemId");
            let transItem = Datasource.items()[transItemId];
            if (!transItem) {
              that._signInUpdate = true;
            }
            Datasource.addOrUpdateTransaction(resultTransaction);
          }

          // Keep a list of tracked items and transactions for use at the end
          // of Phase One
          trackedItems = trackedItems.concat(items);
          trackedTransactions = trackedTransactions.concat(transactions);

          // If buying/selling items are now in trackedItems, trigger phaseOneCompleted.
          if (numPhaseOneCallsReceived == 2) {
            phaseOneCallsReceived();
          }

          // XXX: change to replace GetItem calls with GetMultipleItems calls
          // and improve performance and prevent Trading API calls (expensive)
          // GetMultipleItems
          var multipleItemsCallback = function(aItems, aRequestTracker) {
            for each (let [aIndex, aItem] in Iterator(aItems)) {
              Datasource.addOrUpdateItem(aItem);
            }
            // Call this to tell the tracker we are done with this set of items.
            if (aRequestTracker) {
              aRequestTracker();
            } else {
              // Log buying/Selling items completely filled.
              buyingSellingCallsProcessed++;
              // Trigger phaseTwoCompleted after storing all item & transaction info
              if (buyingSellingCallsProcessed == 2) {
                buyingSellingProcessed();
              }
            }
          };

          // prepare a list of items including only those that are new or
          // are still active
          let itemsToUpdate = [];
          for (var k = 0; k < items.length; k++) {
            let tempItemId = items[k].get("itemId");
            let dsItem = Datasource.items()[tempItemId];
            // Call GetMultilpleItems only if the item is not yet being
            // tracked or if it's still active or the type is undefined
            // or if the update.collection preference is set to true
            let updateCollection = Constants.prefBranch.get("update.collection");
            if (!dsItem || !dsItem.get("isEnded") || !dsItem.type ||
                !dsItem.get("isReserveMet") || updateCollection) {
              itemsToUpdate.push(items[k]);
            }
          }

          that._getMultipleItemsInfo(
            phaseTwoTracker, itemsToUpdate, multipleItemsCallback);

        }

      getMyEbaySellingCallback =
        function(result) {
          if (result.error) {
            Logger.warning("Hard update reported API errors");
            // we create the sidebar notification to let the user know
            // addToWatchList failed
            let notification =
              new Notification(
                WarningNotificationHelper.HARD_UPDATE_ERROR_WARNING);
            notification.set(
              "content", Constants.stringBundle.getString(
                "notification.error.download"));
            notification.set("priority", 1);
            WarningNotificationHelper.queueNotification(notification);
            // there were API errors
            apiErrors = true;
            return;
          }

          phaseTwoTracker.addRequest(
            TradingApi.getSellerList(result, localCallback));
        }

      phaseOneTracker.addRequest(
        TradingApi.getMyeBayBuying(localCallback));
      phaseOneTracker.addRequest(
        TradingApi.getMyeBaySelling(getMyEbaySellingCallback));

      // Update account data
      this.accountUpdate(accountUpdateComplete);
      // adjust client alerts polling rate, to keep it as accurate as possible
      this._adjustClientAlertsPollPeriod();
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Utilitary method to retrieve multiple items information and update it
   * locally
   * @param aEventTracker the event tracker that will handle the requests
   * @param aItemsToFetch a list of the items to be queried using
   * GetMultipleItems
   * @param aCallback the callback to be called when the request is processed.
   */
  _getMultipleItemsInfo : function(aEventTracker, aItemsToFetch, aCallback) {

    // XXX: change to replace GetItem calls with GetMultipleItems calls
    // and improve performance and prevent Trading API calls (expensive)
    // GetMultipleItems
    var multipleItemsCallback = function(aItems, aResult, aRequestTracker) {
      if (!aResult.items) {
        Logger.warning(
          "GetMultipleItems encountered an error, so the associated " +
          "items will not be updated.");
        aEventTracker.failRemainingEvents();
        return;
      }
      let resultItems = aResult.items;
      let tempItem;
      for (let j = 0; j < resultItems.length; j++) {
        if (!resultItems[j]) {
          Logger.warning(
            "GetMultipleItemsCallback encountered an error, so the " +
            "associated item will not be updated.");
          aEventTracker.failRemainingEvents();
          return;
        }

        tempItem = aItems[resultItems[j].get("itemId")];
        let ignoreProperties =
          ["userMaxBid", "userQuantityWinning", "userQuantityBidFor",
           "convertedMaxBid", "convertedMaxBidCurrency", "numWatching",
           "bestOffer", "bestOfferCurrency", "convertedBestOffer",
           "convertedBestOfferCurrency", "bestOfferCount"];

        // XXX massive hack to prevent changes in pageViews value if the new
        // value is less than the current one
        let newPageViews = resultItems[j].get("pageViews");
        let currentPageViews = tempItem.get("pageViews");
        if(newPageViews < currentPageViews) {
          ignoreProperties =
            ["userMaxBid", "userQuantityWinning", "userQuantityBidFor",
             "convertedMaxBid", "convertedMaxBidCurrency", "numWatching",
             "pageViews", "bestOffer", "bestOfferCurrency",
             "convertedBestOffer", "convertedBestOfferCurrency",
             "bestOfferCount"];
        }

        tempItem.updateTo(resultItems[j], null, ignoreProperties);

      }
      aCallback(aItems, aRequestTracker);
    };

    while (aItemsToFetch.length > 0) {
      let itemListSubset = aItemsToFetch.splice(0, 20);
      let requestTracker = aEventTracker.addCallbackEvent();

      aEventTracker.addRequest(
        ShoppingApi.getMultipleItems(
          itemListSubset, multipleItemsCallback, requestTracker));
    }
    aCallback(new Array());
  },

  /**
   * Utilitary method to retrieve multiple deals information and update it
   * locally
   * @param aDealsToFetch a list of the deal items to be queried using
   * GetMultipleItems
   * @param aCallback the callback to be called when the request is processed.
   */
  _getDealsInfo : function(aDealsToFetch, aCallback) {
    let that = this;
    var multipleItemsCallback = function(aDealsArray, aResult) {
      if (!aResult.items) {
        Logger.warning(
          "GetMultipleItems encountered an error, so the associated " +
          "deals will not be updated.");
        aCallback(aDealsToFetch);
        return;
      }
      let resultItems = aResult.items;
      let tempDeal;
      for (let j = 0; j < aDealsArray.length; j++) {
        let resultItemInfo =
          that._findItem(aDealsArray[j].get("itemId"), resultItems);
        if (!resultItemInfo) {
          Logger.warning(
            "GetMultipleItemsCallback encountered an error, so the " +
            "associated deal will not be updated.");
          aCallback(aDealsToFetch);
          return;
        }

        aDealsArray[j].set("sellerUserId", resultItemInfo.get("sellerUserId"));
        aDealsArray[j].set(
          "sellerFeedbackRating", resultItemInfo.get("sellerFeedbackRating"));
        aDealsArray[j].set(
          "sellerFeedbackPercent", resultItemInfo.get("sellerFeedbackPercent"));
        aDealsArray[j].set("pageViews", resultItemInfo.get("pageViews"));
        aDealsArray[j].set("viewItemUrl", resultItemInfo.get("viewItemUrl"));
      }
      aCallback(aDealsArray);
    };

    ShoppingApi.getMultipleItems(aDealsToFetch, multipleItemsCallback, null, true);
  },

  /**
   * Finds the item specified by aItemId in the given array passed as parameter
   * @param aItemId the id of the item to look for
   * @param aArray the array to search into
   * @return the specified item or null if not present
   */
  _findItem : function(aItemId, aArray) {
    let item = null;
    for (let i = 0; i < aArray.length; i++) {
      if (aArray[i].get("itemId") == aItemId) {
        item = aArray[i];
        break;
      }
    }
    return item;
  },

  /**
   * Soft Update
   */
  softUpdate : function(callback) {
    try {
      let that = this;
      this._updateStarted("Soft Update");

      var messagesCallback, multipleItemsCallback;

      let eventTracker = new EventTracker("Soft Update");
      eventTracker.doWhenAllFinished(function(numErrors) {
        if (callback) {
          callback(numErrors);
        }
        that._updateFinished("Soft Update");
      });

      // GetMyMessages
      messagesCallback = function(result) {
        Datasource.activeAccount().
          updateProperty("numUnreadMessages", result.numUnreadMessages);
      }
      eventTracker.addRequest(TradingApi.getMyMessages(messagesCallback));

      multipleItemsCallback = function(aItems, aRequestTracker) {
        for each (let [aIndex, aItem] in Iterator(aItems)) {
          Datasource.addOrUpdateItem(aItem);
        }
        // Call this to tell the tracker we are done with this set of items.
        if (aRequestTracker) {
          aRequestTracker();
        }

      };

      // GetMultipleItems
      let items = Datasource.items();

      // prepare a list of items including only those that are new or
      // are still active
      let itemsToUpdate = [];
      for each ( let [itemId, item] in Iterator(items)) {
        // Call GetMultipleItems only if the item is not yet being tracked or
        // if it's still active
        if (!item.get("isEnded")) {
          itemsToUpdate.push(item);
        }
      }

      this._getMultipleItemsInfo(
        eventTracker, itemsToUpdate, multipleItemsCallback);
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Account Update
   * @param callback
   */
  accountUpdate : function(callback) {
    let localCallback;
    let that = this;

    let eventTracker = new EventTracker("Account Update");
    eventTracker.doWhenAllFinished(callback);

    let activeAccount = Datasource.activeAccount();

    // GetUser
    localCallback =
      function(result) {
        if (result && result.feedbackRating != null) {
          // only notify if the feedback rating actually changes
          if (result.feedbackRating != activeAccount.get("feedbackRating")) {
            let info = {};
            info.originalObject = activeAccount.copy();
            activeAccount.updateProperty(
              "feedbackRating", result.feedbackRating);
            info.object = activeAccount;
            // notify to see if we should generate an alert for a new feedback
            // milestone achieved
            Observers.notify(info, Topics.ACCOUNT_FEEDBACK_CHANGED, null);
          }
          activeAccount.
            updateProperty("registrationSite", result.registrationSite);

          if (result.email && result.email.indexOf("@") != -1) {
            activeAccount.updateProperty("email", result.email);
          }

        } else {
          Logger.warning("getUser API call reported errors when " +
                         "trying to update user feedback score");
        }
      }
    eventTracker.addRequest(TradingApi.getUser(localCallback));

    // GetMyMessages
    localCallback =
      function(result) {
        // only notify if the number of unread messages increases
        let info = {};
        info.originalObject = activeAccount.copy();
        let originalNumUnreadMessages = activeAccount.get("numUnreadMessages");
        activeAccount.updateProperty(
          "numUnreadMessages", result.numUnreadMessages);
        info.object = activeAccount;
        if (result.numUnreadMessages >
            originalNumUnreadMessages || that._signInUpdate) {
          // notify number of unread messages changed.
          Observers.notify(
            info, Topics.ACCOUNT_UNREAD_MESSAGES_CHANGED, null);
        }
      }
    eventTracker.addRequest(TradingApi.getMyMessages(localCallback));
  },

  /**
   * Client Alerts Update
   */
  clientAlertsUpdate : function() {
    try {
      let that = this;
      this._updateStarted("Client Alerts Update");

      let mainTracker = new EventTracker("Client Alerts Update");
      mainTracker.doWhenAllFinished(function(numErrors) {
        let newPollPeriod = that._optimalTimeToNextClientAlertsUpdate();
        // Set timer for next update
        that._clientAlertsTimer.cancel(); // In case it was reset by updates
        that._clientAlertsTimer =
          new Timer(function() { that.clientAlertsUpdate(); }, newPollPeriod);
        that._updateFinished("Client Alerts Update");
      });

      let publicAlertsCallback =
        function(result) {
          try {

            let callback;
            if (result.feedbackReceived) {
              callback = mainTracker.addCallbackEvent();
              that._updateFeedbackScore(callback);
            }

            if (result.needUpdate.length > 0) {
              that._processNeedUpdateItems(result.needUpdate, mainTracker);
            }

            if (result.removedItems.length > 0) {
              callback = mainTracker.addCallbackEvent();
              that._processRemovedItems(result.removedItems, callback);
            }

          }
          catch (e) {
            Logger.exception(e, Logger.DUMP_STACK)
          }
        }

      let userAlertsCallback =
        function(result) {
          // We only want to call GetPublicAlerts if we have active items to track
          let activeItemsExist = false;
          for (let [itemId, item] in Iterator(Datasource.items())) {
            if (!item.get("isEnded")) {
              activeItemsExist = true;
              break;
            }
          }
          // this means we must call public alerts, so make the intermediate
          // connection
          if (activeItemsExist) {
            mainTracker.addRequest(
              ClientAlertsApi.getPublicAlerts(result, publicAlertsCallback));
          } else {
            // call publicAlertsCallback directly
            publicAlertsCallback(result);
          }
        }

      let resultObject = ClientAlertsApi.emptyResultObject();

      mainTracker.addRequest(
        ClientAlertsApi.getUserAlerts(resultObject, userAlertsCallback));

    }
    catch (e) {
      this._updateFinished("Client Alerts Update");
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Updates the feedback score of the active account by making a GetUserProfile
   * call
   */
  _updateFeedbackScore : function(callback) {
    let tracker = new EventTracker("Update Feedback Score");
    tracker.doWhenAllFinished(callback);

    let activeAccount = Datasource.activeAccount();

    let localCallback = function(result) {
      if (result && result.feedbackRating) {
        // only notify if the feedback rating actually changes
        if (result.feedbackRating != activeAccount.get("feedbackRating")) {
          let info = {};
          info.originalObject = activeAccount.copy();
          activeAccount.updateProperty("feedbackRating", result.feedbackRating);
          info.object = activeAccount;
          // notify to see if we should generate an alert for a new feedback
          // milestone achieved
          Observers.notify(info, Topics.ACCOUNT_FEEDBACK_CHANGED, null);
        }
      } else {
        Logger.warning("getUserProfile API call reported errors when trying " +
                       "to update user feedback score");
      }
    }

    tracker.addRequest(
      ShoppingApi.getUserProfile(localCallback));
  },

  /**
   * Given an array of items, makes the necessary work to update them using API
   * calls
   */
  _processNeedUpdateItems : function(aNeedUpdateItems, aEventTracker) {
    let itemsToFetch = {};
    let needBuying = false;
    let needSelling = false;
    let finishedEvent = null;
    let that = this;

    for each (let [aIndex, aItem] in Iterator(aNeedUpdateItems)) {
      itemsToFetch[aItem.get("itemId")] = aItem;
      if (!needBuying || !needSelling) {
        switch(aItem.type) {
          case Item.ITEM_TYPE_BIDDING:
          case Item.ITEM_TYPE_WON:
          case Item.ITEM_TYPE_LOST:
            needBuying = true;
          break;
          case Item.ITEM_TYPE_SELLING:
          case Item.ITEM_TYPE_SOLD:
          case Item.ITEM_TYPE_UNSOLD:
            needSelling = true;
          break;
        }
      }
    };

    // GetMyeBayBuying & GetMyeBaySelling
    var buyingSellingCallback = function(aResult) {
      if (aResult.errors) {
        Logger.warn("_processNeedUpdateItems reported API errors");
        return;
      }

      var items = new Array();
      var transactions = aResult.transactions;
      var favoriteSellers = aResult.favoriteSellers;

      if (favoriteSellers) {
        // delete the favorite sellers for the current account and clear
        // the array kept in memory
        Datasource.clearAccountFavoriteSellers();
        // Update or add favoriteSellers to Datasource
        for (let i = 0; i < favoriteSellers.length; i++) {
          Datasource.addFavoriteSeller(favoriteSellers[i]);
        }
      }

      let favoriteSearches = aResult.favoriteSearches;
      if (favoriteSearches) {
        // delete the favorite searches for the current account and clear
        // the array kept in memory
        Datasource.clearAccountFavoriteSearches();
        // Update or add favoriteSearches to Datasource
        for (let i = 0; i < favoriteSearches.length; i++) {
          Datasource.addFavoriteSearch(favoriteSearches[i]);
        }
        // notify observer to refresh favorite searches UI
        Observers.notify(null, Topics.FAVORITE_SEARCHES_UPDATED, null);
      }

      // Update eBay time
      if (aResult.timestamp) {
        Datasource.setEbayTime(aResult.timestamp);
      }

      // Update or add transactions to Datasource
      if (transactions) {
        for (let i = 0; i < transactions.length; i++) {
          Datasource.addOrUpdateTransaction(transactions[i]);
        }
      }

      // create a list of items that require to be fetch by getMultipleItems
      for each (let [aIndex, aItem] in Iterator(aResult.items)) {
        if (itemsToFetch[aItem.get('itemId')]) {
          items.push(aItem);
        }
      }

      // XXX: change to replace GetItem calls with GetMultipleItems calls
      // and improve performance and prevent Trading API calls (expensive)
      // GetMultipleItems
      var multipleItemsCallback = function(aItems, aRequestTracker) {
        for each (let [aIndex, aItem] in Iterator(aItems)) {
          Datasource.addOrUpdateItem(aItem);
        }
        if (aRequestTracker) {
          aRequestTracker();
        }
        if (finishedEvent) {
          finishedEvent();
        }
      };

      that._getMultipleItemsInfo(aEventTracker, items, multipleItemsCallback);

    };

    // Call getMyeBayBuying
    if (needBuying) {
      if (!finishedEvent) {
        finishedEvent = aEventTracker.addCallbackEvent();
      }
      aEventTracker.addRequest(
        TradingApi.getMyeBayBuying(buyingSellingCallback));
    }

    if (needSelling) {
      if (!finishedEvent) {
        finishedEvent = aEventTracker.addCallbackEvent();
      }
      aEventTracker.addRequest(
        TradingApi.getMyeBaySelling(buyingSellingCallback));
    }

    if (!needBuying && !needSelling) {
      buyingSellingCallback({items : aNeedUpdateItems});
    }
  },

  /**
   * Given an array of item IDs, removes each relevant item from the Datasource
   */
  _processRemovedItems : function(removedItems, callback) {
    let mainTracker = new EventTracker("Removed Items");
    mainTracker.doWhenAllFinished(callback);

    // Remove removed items
    for (let i = 0; i < removedItems.length; i++) {
      let dsItem = Datasource.items()[removedItems[i]];
      if (dsItem) {
        Datasource.removeItem(dsItem);
      } else {
        Logger.warning("Client Alerts tried to remove item " + removedItems[i] +
                       ", which isn't being tracked.");
      }
    }

    // Call the callback on return to the event loop as we have no actual
    // asynchronous requests
    new Timer(callback, 0);
  },

  /**
   * Adds the item with the given itemId to the user's watch list
   * @param itemId ItemID of the item to add
   * @param callback Function that is called when the process is complete
   */
  addToWatchList : function(itemId, callback) {
    try {
      if (Datasource.items()[itemId]) {
        Logger.warning("addToWatchList called for item " + itemId + ", which is " +
                       "already being tracked.");
        return;
      }

      this._updateStarted("Add To Watch List");
      let eventTracker = new EventTracker("Add Item to Watchlist");
      let finishedGettingItem = eventTracker.addCallbackEvent();
      eventTracker.doWhenAllFinished(callback);

      let that = this;
      let localCallback = function(result) {
        if (result.error) {
          Logger.warning(
            "Adding item failed because the API reported an error.");
          // we create the sidebar notification to let the user know
          // addToWatchList failed
          let notification =
            new Notification(
              WarningNotificationHelper.ADD_TO_WATCHLIST_ERROR_WARNING);
          notification.set(
            "content", Constants.stringBundle.getString(
              "notification.error.addWatch"));
          notification.set("priority", 1);
          WarningNotificationHelper.queueNotification(notification);
          return;
        }
        // Do GetSingleItem call to get item details
        let getSingleItemCallback = function(result) {
          let item = result.item;
          if (!item || result.error) {
            Logger.warning("AddItem encountered an error, so the associated " +
                           "item will not be updated.");
            // we create the sidebar notification to let the user know
            // addToWatchList failed
            let notification =
              new Notification(
                WarningNotificationHelper.ADD_TO_WATCHLIST_ERROR_WARNING);
            notification.set(
              "content", Constants.stringBundle.getString(
                "notification.error.addWatch"));
            notification.set("priority", 1);
            WarningNotificationHelper.queueNotification(notification);
            return;
          }

          item.type = Item.ITEM_TYPE_WATCHING;

          Datasource.addOrUpdateItem(item);
          // dismiss any add to watchlist error notification if we succeed in
          // the subsequent requests
          WarningNotificationHelper.dismissNotification(
            WarningNotificationHelper.ADD_TO_WATCHLIST_ERROR_WARNING);
          finishedGettingItem();
          that._updateFinished("Add To Watch List");
        }
        ShoppingApi.getSingleItem(itemId, getSingleItemCallback);
      }

      eventTracker.addRequest(
        TradingApi.addToWatchList(itemId, localCallback));
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Removes the item with the given itemId from the user's watch list
   * @param itemId ItemID of the item to remove
   * @param callback Function that is called when the process is complete
   */
  removeFromWatchList : function(itemId, callback) {
    try {
      if (!Datasource.items()[itemId]) {
        Logger.warning("removeFromWatchList called for item " + itemId +
                       ", which is not being tracked.");
        return;
      }
      if (Datasource.transactions(itemId)) {
        Logger.warning("removeFromWatchList called for item " + itemId +
                       ", which has tracked transactions.");
        return;
      }

      this._updateStarted("Remove From Watch List");
      let eventTracker = new EventTracker("Remove Item from Watchlist");
      let removedFromDatasource = eventTracker.addCallbackEvent();
      eventTracker.doWhenAllFinished(callback);

      let that = this;
      let localCallback = function(result) {
        if (result.error) {
          Logger.warning("Skipping removal of the item from " +
                         "datasource because the API reported an error.");
          // we create the sidebar notification to let the user know
          // removeFromWatchList failed
          let notification =
            new Notification(
              WarningNotificationHelper.REMOVE_FROM_WATCHLIST_ERROR_WARNING);
          notification.set(
            "content", Constants.stringBundle.getString(
              "notification.error.stopWatch"));
          notification.set("priority", 1);
          WarningNotificationHelper.queueNotification(notification);
          return;
        }
        // It's possible that the item was removed before this callback was
        // called
        let item = Datasource.items()[itemId];
        if (item) {
          Datasource.removeItem(item);
        }
        // dismiss any remove from watchlist error notification if we succeed in
        // the subsequent requests
        WarningNotificationHelper.dismissNotification(
          WarningNotificationHelper.REMOVE_FROM_WATCHLIST_ERROR_WARNING);
        removedFromDatasource();
        that._updateFinished("Remove From Watch List");
      }

      eventTracker.addRequest(
        TradingApi.removeFromWatchList(itemId, localCallback));
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Removes the item with the given itemId from the user's didn't win list
   * @param aItemId ItemID of the item to remove
   * @param callback Function that is called when the process is complete
   */
  removeFromDidntWinList : function(aItemId, callback) {
    try {
      if (!Datasource.items()[aItemId]) {
        Logger.warning("removeFromDidntWinList called for item " + itemId +
                       ", which is not being tracked.");
        return;
      }

      this._updateStarted("Remove From Didn't Win List");
      let eventTracker = new EventTracker("Remove Item from Didn't win list");
      let removedFromDatasource = eventTracker.addCallbackEvent();
      eventTracker.doWhenAllFinished(callback);

      let that = this;
      let localCallback = function(result) {
        if (result.error) {
          Logger.warning("Skipping removal of the item from " +
                         "datasource because the API reported an error.");
          // we create the sidebar notification to let the user know
          // removeFromWatchList failed
          let notification =
            new Notification(
              WarningNotificationHelper.REMOVE_FROM_DIDNTWINLIST_ERROR_WARNING);
          notification.set(
            "content", Constants.stringBundle.getString(
              "notification.error.removeFromDidntWin.list"));
          notification.set("priority", 1);
          WarningNotificationHelper.queueNotification(notification);
          return;
        }
        // It's possible that the item was removed before this callback was
        // called
        let item = Datasource.items()[aItemId];
        if (item) {
          Datasource.removeItem(item);
        }
        // dismiss any remove from watchlist error notification if we succeed in
        // the subsequent requests
        WarningNotificationHelper.dismissNotification(
          WarningNotificationHelper.REMOVE_FROM_DIDNTWINLIST_ERROR_WARNING);
        removedFromDatasource();
        that._updateFinished("Remove From Didn't Win List");
      }

      eventTracker.addRequest(
        MyEbayApplicationApi.removeFromDidntWinList(aItemId, localCallback));
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Removes the item with the given itemId from the user's won list
   * @param item the item to be removed, including the respective transaction
   * @param callback Function that is called when the process is complete
   */
  removeFromWonList : function(aItem, callback) {
    try {
      if (!Datasource.items()[aItem.get("itemId")]) {
        Logger.warning("removeFromWonList called for item " + itemId +
                       ", which is not being tracked.");
        return;
      }
      let itemId = aItem.get("itemId");
      let transactionId = aItem.transaction.get("transactionId");

      this._updateStarted("Remove From Won List");
      let eventTracker = new EventTracker("Remove Item from Won list");
      let removedFromDatasource = eventTracker.addCallbackEvent();
      eventTracker.doWhenAllFinished(callback);

      let that = this;
      let localCallback = function(result) {
        if (result.error) {
          Logger.warning("Skipping removal of the item from " +
                         "datasource because the API reported an error.");
          // we create the sidebar notification to let the user know
          // removeFromWatchList failed
          let notification =
            new Notification(
              WarningNotificationHelper.REMOVE_FROM_WONLIST_ERROR_WARNING);
          notification.set(
            "content", Constants.stringBundle.getString(
              "notification.error.removeFromWon.list"));
          notification.set("priority", 1);
          WarningNotificationHelper.queueNotification(notification);
          return;
        }
        // It's possible that the item was removed before this callback was
        // called
        let dbTransaction = Datasource.transactions(itemId)[transactionId];
        if (dbTransaction) {
          Datasource.removeTransaction(dbTransaction);
        }
        // dismiss any remove from watchlist error notification if we succeed in
        // the subsequent requests
        WarningNotificationHelper.dismissNotification(
          WarningNotificationHelper.REMOVE_FROM_WONLIST_ERROR_WARNING);
        removedFromDatasource();
        that._updateFinished("Remove From Won List");
      }

      eventTracker.addRequest(
        MyEbayApplicationApi.removeFromWonList(
          itemId, transactionId, localCallback));
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Pings the deals API if the enable.deals.api.requests preference is set to
   * true
   */
  pingDealsAPI : function() {
    if (Constants.prefBranch.get("enable.deals.api.requests")) {
      // get daily deals
      if ("undefined" == typeof(DealsApi)) {
        Cu.import("resource://ebaycompanion/apis/dealsApi.js");
      }
      let that = this;
      let getDealsCallback = function(aDealsArray) {
        Datasource.dailyDeals = aDealsArray;

        let getDealsInfoCallback = function(aUpdatedDealsArray) {
          Datasource.dailyDeals = aUpdatedDealsArray;
          Observers.notify(null, Topics.DAILY_DEALS_INFO_UPDATED, null);
        };
        that._getDealsInfo(Datasource.dailyDeals, getDealsInfoCallback);
        Observers.notify(null, Topics.DAILY_DEALS_UPDATED, null);
      };
      DealsApi.getDailyDeals(getDealsCallback);
    } else {
      Logger.warning("DEALS API ACCESS DISABLED\n");
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    if (Topics.QUIT_APPLICATION_GRANTED == aTopic) {
      this._uninit();
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(ApiCoordinator);
