/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["ClientAlertsApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/apiHelper.js");
Cu.import("resource://ebaycompanion/services/apiAccessService.js");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/transaction.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://ebaycompanion/apiCoordinator.js");
Cu.import("resource://ebaycompanion/constants.js");

/**
 * Client Alerts API actions
 */
ClientAlertsApi = {

  /**
   * Used to know if we are connected to client alerts
   */
  get connected() {
    return this._sessionId && this._sessionData;
  },

  /**
   * Client Alerts Login
   * @param token Client Alerts Auth token
   * @param callback
   * @returns request object
   */
  login : function(token, callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Client Alerts Login call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }
    let parameters = {};

    parameters.ClientAlertsAuthToken = token;

    // do the call
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseLogin(response);
        if (result.sessionId && result.sessionData) {
          // for GetUserAlerts calls
          that._sessionId = result.sessionId;
          that._sessionData = result.sessionData;
          if (result.timestamp) {
            that._lastPublicUpdateTime = result.timestamp;
          }
        }
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall("Login", parameters, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a Login response
   * @param response
   */
  _parseLogin : function(response, callback) {
    let jsonObject = JSON.parse(response);
    let errors = this._detectAndReportErrors(jsonObject);

    let result = {};
    result.sessionId = jsonObject.SessionID;
    result.sessionData = jsonObject.SessionData;
    result.timestamp = jsonObject.Timestamp;
    result.errors = errors;

    return result;
  },

  /**
   * Client Alerts GetPublicAlerts
   * @param resultObject the result object to store the retured data
   * @param callback
   * @returns request object
   */
  getPublicAlerts : function(resultObject, callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Client Alerts GetPublicAlerts call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }
    if (!this.connected) {
      Logger.warning("Making Client Alerts GetPublicAlerts call when " +
                     "not logged in to Client Alerts!", Logger.DUMP_STACK);
    }
    let parameters = {};

    if (this._lastPublicUpdateTime) {
      // TODO: We can't use LastRequestTime, because ClientAlerts requires that
      // we call GetPublicAlerts at least once without it for each item, and
      // that's a real headache.  By not specifying one, we get public alerts we
      // don't need, but at least we stay up to date.

      parameters.LastRequestTime = this._lastPublicUpdateTime;
    }

    let items = Datasource.items();
    parameters.ChannelDescriptor = [];
    for each (let [itemId, item] in Iterator(items)) {
      // Only retrieve alerts for active items
      if (!item.get("isEnded")) {
        let channelDescriptor = {};
        channelDescriptor.ChannelID = itemId;
        channelDescriptor.ChannelType = "Item";
        channelDescriptor.EventType = ["ItemEnded", "PriceChange"];
        parameters.ChannelDescriptor.push(channelDescriptor);
      }
    }

    // Abort the call if there's nothing to watch
    if (parameters.ChannelDescriptor.length == 0) {
      Logger.warning("Attempt to make Client Alerts GetPublicAlerts call " +
                     "when there are no items to track!", Logger.DUMP_STACK);
      return;
    }

    // do the call
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetPublicAlerts(response, resultObject);
        if (result.timestamp) {
          that._lastPublicUpdateTime = result.timestamp;
        }
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request =
      this._doCall("GetPublicAlerts", parameters, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetPublicAlerts response
   * @param response the API response
   * @param resultObject the result object to store the retured data
   */
  _parseGetPublicAlerts : function(response, resultObject) {
    let jsonObject = JSON.parse(response);
    this._detectAndReportErrors(jsonObject);

    for (let i = 0; i < jsonObject.Content.length; i++) {
      let channelEvents = jsonObject.Content[i].ChannelEvent;
      for (let j = 0; j < channelEvents.length; j++) {
        let event = channelEvents[j][channelEvents[j].EventType];
        this._processAlert(event, resultObject);
      }
    }

    resultObject.timestamp = jsonObject.Timestamp;

    return resultObject;
  },

  /**
   * Client Alerts GetUserAlerts
   * @param resultObject the result object to store the retured data
   * @param callback
   * @returns request object
   */
  getUserAlerts : function(resultObject, callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Client Alerts GetUserAlerts call when " +
                   "no account is active.", Logger.DUMP_STACK);
      throw "Attempt to make Client Alerts GetUserAlerts call when " +
                   "no account is active.";
      return;
    }
    if (!this.connected) {
      Logger.error("Attempt to make Client Alerts GetUserAlerts call when " +
                   "not logged in to Client Alerts!", Logger.DUMP_STACK);
      throw "Attempt to make Client Alerts GetUserAlerts call when " +
                   "no account is active.";
      return;
    }
    let parameters = {};

    parameters.SessionID = this._sessionId;
    parameters.SessionData = this._sessionData;

    // do the call
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetUserAlerts(response, resultObject);
        if (result.sessionData) {
          that._sessionData = result.sessionData;
        }
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request =
      this._doCall("GetUserAlerts", parameters, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetUserAlerts response
   * @param response the API response
   * @param resultObject the result object to store the retured data
   */
  _parseGetUserAlerts : function(response, resultObject) {
    let jsonObject = JSON.parse(response);

    this._detectAndReportErrors(jsonObject);

    let clientAlerts = jsonObject.ClientAlerts;
    if (clientAlerts) {
      let alertEvents = clientAlerts.ClientAlertEvent;
      for (let i = 0; i < alertEvents.length; i++) {
        let eventType = alertEvents[i].EventType;
        // XXX: necessary due to a camel case violation returned by the API
        if (eventType == "OutBid") {
          eventType = "Outbid";
        }
        let event = alertEvents[i][eventType];
        if (null != event) {
          this._processAlert(event, resultObject);
        }
      }
    }

    resultObject.sessionData = jsonObject.SessionData;

    return resultObject;
  },

  /**
   * Returns an empty result object ready to hold information from _processAlert
   */
  emptyResultObject : function() {
    let result = {};
    result.feedbackReceived = false;  // boolean
    result.removedItems = new Array();// Item IDs array
    result.needUpdate = new Array();  // Partial Item objects

    return result;
  },

  /**
   * Performs appropriate actions for the given Client Alert object
   * @param alertObject the alert object, containing relevant data
   * @param result accumulator object that collects parsed data
   */
  _processAlert : function(alertObject, result) {
    Logger.log("Event: " + alertObject.EventType +
               ", ID: " + alertObject.ItemID);

    // The switch statement doesn't consider each case a block, so 'let' doesn't
    // work as might be expected.
    let eventType = alertObject.EventType;
    let itemId = alertObject.ItemID;
    let item;
    let feedbackDetail;
    let itemType = null;

    switch (eventType) {
      case "ItemAddedToWatchList":
        item = Datasource.items()[itemId];
        if (!item) {
          item = new Item(itemId);
          item.type = Item.ITEM_TYPE_WATCHING;
          result.needUpdate.push(item);
        }
        // if this event comes in the same response as a previous
        // ItemRemovedFromWatchList event, we should remove the itemId from the
        // removedItems array
        for (var i = 0; i < result.removedItems.length; i++) {
          if (result.removedItems[i] == itemId) {
            result.removedItems.splice(i, 1);
            break;
          }
        }
        break;

      case "ItemRemovedFromWatchList":
        item = Datasource.items()[itemId];
        if (item) {
          result.removedItems.push(itemId);
        }
        // if this event comes in the same response as a previous
        // ItemAddedToWatchList event, we should remove the itemId from the
        // watchedItems array
        for (var i = 0; i < result.needUpdate.length; i++) {
          if (result.needUpdate[i].get("itemId") == itemId) {
            result.needUpdate.splice(i, 1);
            break;
          }
        }
        break;

      case "ItemMarkedPaid":
        var transactions =
            Datasource.transactions(itemId);
        var transaction =
          transactions[alertObject.TransactionID];
        if (transaction) {
          transaction = transaction.copy();
          transaction.set("isPaidFor", true);
          // save the transaction immediately
          Datasource.addOrUpdateTransaction(transaction);
        }
        break;

      case "ItemMarkedShipped":
        var transactions =
            Datasource.transactions(itemId);
        var transaction =
          transactions[alertObject.TransactionID];
        if (transaction) {
          transaction = transaction.copy();
          transaction.set("isShipped", true);
          // save the transaction immediately
          Datasource.addOrUpdateTransaction(transaction);
        }
        break;

      case "OutBid":
      case "BidReceived":
      case "BidPlaced":
        item = new Item(itemId);
        if (eventType == "BidReceived") {
          item.type = Item.ITEM_TYPE_SELLING;
        } else {
          item.type = Item.ITEM_TYPE_BIDDING;
        }
        // If the items exists, just update the fields
        var localItem = Datasource.items()[itemId];
        if (localItem && eventType == "OutBid") {
          item = localItem.copy();
          item.set("numBids", alertObject.BidCount);
          item.set("currentPrice", alertObject.CurrentPrice.Value);
          item.set("currency", alertObject.CurrentPrice.CurrencyID);
          item.set("highBidderId", alertObject.HighBidderUserID);
          if (alertObject.ReserveMet) {
            item.set("isReserveMet", alertObject.ReserveMet);
          }

          // we require immediate action here
          Datasource.addOrUpdateItem(item);
        } else { //If not, go get the item info from the API.
          result.needUpdate.push(item);
        }
        break;

      case "PriceChange":
        item = Datasource.items()[itemId];
        if (item) {
          item = item.copy();
          item.set("currentPrice", alertObject.CurrentPrice.Value);
          item.set("numBids", alertObject.BidCount);
          if (alertObject.ReserveMet) {
            item.set("isReserveMet", alertObject.ReserveMet);
          }
          // we require immediate action here
          Datasource.addOrUpdateItem(item);
        }
      break;

      case "ItemSold":
      case "ItemWon":
      case "ItemUnsold":
      case "ItemLost":
        item = Datasource.items()[itemId];
        if (item) {
          item = item.copy();
        }
        if (eventType == "ItemSold") {
          item.type = Item.ITEM_TYPE_SOLD;
        } else if (eventType == "ItemWon") {
          item.type = Item.ITEM_TYPE_WON;
        } else if (eventType == "ItemUnsold") {
          item.type = Item.ITEM_TYPE_UNSOLD;
          item.set("isEnded", true);
        } else {
          item.type = Item.ITEM_TYPE_LOST;
          item.set("isEnded", true);
        }

        // For Won items, we need to get the transaction with getBuying.
        // Sold items will get transaction infor from EndOfAuction and
        // fixedPriceTransaction events.
        if (eventType == "ItemWon") {
          result.needUpdate.push(item);
        } else {
          item.set("numBids", alertObject.BidCount);
          item.set("currentPrice", alertObject.CurrentPrice.Value);
          item.set("currency", alertObject.CurrentPrice.CurrencyID);
          // we require immediate action here
          Datasource.addOrUpdateItem(item);
        }
      break;

      case "ItemEnded":
        item = Datasource.items()[itemId];
        if (item) {
          item = item.copy();
          item.set("isEnded", true);
        }
        // we require immediate action here
        Datasource.addOrUpdateItem(item);
      break;

      case "EndOfAuction":
      case "FixedPriceTransaction":
      case "FixedPriceEndOfTransaction":
        item = Datasource.items()[itemId];
        //XXX: We should be extracting all the information from the API
        // response. However, FixedPriceTransaction returns a bad QuantitySold
        // and EndOfAuction does not return a TransactionId. There are bugs
        // filed at EDS for this reasons, so until those are addressed, the
        // safest solution is to add the item to the needUpdate array to
        // update it using GetMyEbayBuying/Selling + GetMultipleItems calls
        // combination
        if (!item) {
          item = new Item(itemId);
          var sellerId = alertObject.SellerUserID;
          // if the user is the seller, the item is sold. It is won
          // otherwise
          if (sellerId.toLowerCase() ==
                Datasource.activeAccount().get("userId").toLowerCase()) {
            item.type == Item.ITEM_TYPE_SOLD;
          } else {
            item.type == Item.ITEM_TYPE_WON;
          }
        }
        result.needUpdate.push(item);

      break;

      case "FeedbackLeft":
      case "FeedbackReceived":
        itemId = alertObject.FeedbackDetail.ItemID;
        var transactionId = alertObject.FeedbackDetail.TransactionID;
        var transactions =
          Datasource.transactions(itemId);
        var transaction;
        if (transactions) {
          transaction = transactions[transactionId];
        }

        if (!transaction) {
          item = Datasource.items()[itemId];
          if (item) {
            // XXX: massive hack because there are some cases where the API
            // returns transaction ids that don't match what
            // GetMyEbayBuying/Selling returns, so we have no choice but to
            // update the item using additional calls
            result.needUpdate.push(item);
          }
        } else {
          transaction = transaction.copy();
          if (eventType == "FeedbackReceived") {
            var feedbackType = alertObject.FeedbackDetail.CommentType;
            transaction.set("feedbackReceivedType", feedbackType);
            transaction.set("userHasReceivedFeedback", true);
            result.feedbackReceived = true;
          } else {
            transaction.set("userHasSentFeedback", true);
          }
          // save the transaction immediately
          Datasource.addOrUpdateTransaction(transaction);
        }
      break;
      case "BestOfferPlaced":
      case "BestOfferDeclined":
      case "CounterOfferReceived":
        item = Datasource.items()[itemId];
        if (item) {
          let sellerId = item.get("sellerUserId");
          let currentUser = Datasource.activeAccount().get("userId");
          // if the current user is the seller, set the item for update
          if (sellerId.toLowerCase() == currentUser.toLowerCase()) {
            // XXX: massive hack because there are some cases where the API
            // returns transaction ids that don't match what
            // GetMyEbayBuying/Selling returns, so we have no choice but to
            // update the item using additional calls
            result.needUpdate.push(item);
          } else {
            // update it right away in any other case
            item = item.copy();
            if (eventType == "BestOfferDeclined") {
              item.set("bestOfferStatus", "Declined");
            } else if (eventType == "CounterOfferReceived") {
              item.set("bestOfferStatus", "Countered");
            } else {
              item.set("bestOfferStatus", "Pending");
            }
            // we require immediate action here
            Datasource.addOrUpdateItem(item);
          }
        }
        break;
      case "BestOffer":
        item = Datasource.items()[itemId];
        if (item) {
          // update it right away in any other case
          item = item.copy();
          // increase the number of offers
          let currentCount =
            item.get("bestOfferCount") ? item.get("bestOfferCount") : 0;
          item.set("bestOfferCount", ++currentCount);

          // we require immediate action here
          Datasource.addOrUpdateItem(item);
        }
        break;
      default:
        Logger.warning("Unrecognised Client Alert: " +
                       "\"" + alertObject.EventType + "\"");
        Logger.warning(JSON.stringify(alertObject));
        break;
    }
  },

  /**
   * Reports any error messages in the API response
   * @param jsonObject the response
   */
  _detectAndReportErrors : function(jsonObject) {
    let errors = jsonObject.Errors;
    if (errors) {
      for (let i = 0; i < errors.length; i++) {
        let error = errors[i];
        let errorCode = Number(error.ErrorCode);
        switch (errorCode) {
          case 11.3:
            Logger.error("Client Alerts Session has expired; " +
                           "resetting update cycle.");
            // force the user sign out to renew the token
            //Datasource.logoutUser();
            //Datasource.showSecuritySignOutWarningNotification();
            // we used to sign the user out, but we got many complains about it
            // so we simulate an automatic sign in to try to keep the user
            // signed in
            ApiCoordinator.disableUpdates();
            ApiCoordinator.enableUpdates(true);
            break;

          default:
            Logger.error("Client Alerts API Error:\n" + "Error Code: " +
                         errorCode +
                         "\nShort Message: " +
                         String(errors[i]["ShortMessage"]) +
                         "\nLong Message: " +
                         String(errors[i]["LongMessage"]) +
                         "\nSeverity: " +
                         String(errors[i]["SeverityCode"]) +
                         "\nError Clasification: " +
                         String(errors[i]["ErrorClassification"]) + "\n",
                         Logger.DUMP_STACK);
            break;
        }
      }
    }
  },

  /**
   * Performs ClientAlerts API call
   * @param requestName
   * @param parameters object with parameters to add to the call
   * @param callback The callback function
   * @returns request object
   */
  _doCall : function(requestName, parameters, useSandbox, callback) {
    // Flatten the parameters object into a string for inclusion in the URI
    let unflattened = [];
    let parameterString = "";
    for each (let [name, value] in Iterator(parameters)) {
      unflattened.push({name: name, value: value});
    }
    while (unflattened.length > 0) {
      let entry = unflattened.shift();
      if (typeof(entry.value) != "object") {
        // Base type
        parameterString += "&" + entry.name +
                           "=" + encodeURIComponent(entry.value);
      } else {
        if (entry.value.constructor.name == "Array") {
          // Array
          for (let i = 0; i < entry.value.length; i++) {
            let newEntry = {};
            newEntry.name = entry.name + "(" + i + ")";
            newEntry.value = entry.value[i];
            unflattened.splice(i, 0, newEntry);
          }
        } else {
          // Object
          let i = 0;
          for each (let [name, value] in Iterator(entry.value)) {
            let newEntry = {};
            newEntry.name = entry.name + "." + name;
            newEntry.value = value;
            unflattened.splice(i++, 0, newEntry);
          }
        }
      }
    }

    let callDescription = "Client Alerts API " + requestName;

    let apiCallback =
      ApiHelper.generateApiCallback(callDescription, callback);

/*    if (!ApiHelper.accessService) {
      // The binary component is missing.
      return;
    }*/
    let siteId = Constants.siteIdForSite(Datasource.homeSite());
    //ApiHelper.accessService.setSiteId(siteId);
    ApiAccessService.siteId = siteId;
    //ApiHelper.accessService.setApiVersion(627);
    ApiAccessService.apiVersion = 691;
    let request =
      ApiAccessService.doClientAlertsAPICall(
        requestName, parameterString, useSandbox, apiCallback);

    ApiHelper.addPendingRequest(request, callDescription);

    let requestLog = {};
    requestLog.requestName = requestName;
    requestLog.properties = parameters;
    requestLog.api = "Client Alerts";

    // notify the observer to log the request
    Observers.notify(requestLog, Topics.API_CLIENT_ALERTS_CALL, null);

    return request;
  }
};
