/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["FindingApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/apiHelper.js");
Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
Cu.import("resource://ebaycompanion/services/apiAccessService.js");
Cu.import("resource://ebaycompanion/helpers/informationNotificationHelper.js");
Cu.import("resource://ebaycompanion/helpers/warningNotificationHelper.js");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/notification.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/storage/propertiesStorage.js");

/**
 * Finding API actions
 */
FindingApi = {

  _XML_HEADER : "<?xml version=\"1.0\" encoding=\"utf-8\"?>",
  _XML_NAMESPACE : "xmlns=\"http://www.ebay.com/marketplace/search/v1/services\"",

  /**
   * Gets the items being sold by the given user
   * @param aSellerId the id of the seller to retrieve their items
   * @param aCallback
   */
  getSellerItems : function(aSellerId, aCallback) {

    var wrappedBody =
      this._XML_HEADER +
      "<findItemsAdvancedRequest " + this._XML_NAMESPACE +">" +
      "<itemFilter>" +
      "<name>Seller</name>" +
      "<value>" + aSellerId + "</value>" +
      "</itemFilter>" +
      "<paginationInput>" +
      "<entriesPerPage>3</entriesPerPage>" +
      "</paginationInput>" +
      "<outputSelector>SellerInfo</outputSelector>" +
      "</findItemsAdvancedRequest>";

    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetSellerItemsResponse(response);
        try {
          if (aCallback) aCallback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let request = this._doCall(wrappedBody, localCallback);

    return request;
  },

  /**
   * Processes a getSellerItems response
   * @param response
   * @returns toolbar activity response details
   */
  _parseGetSellerItemsResponse : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");
    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");

    let result = {};

    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
    xmlObject = xmlObject["findItemsAdvancedResponse"];

    let foundErrors = this._detectAndReportErrors(xmlObject);
    if (!foundErrors) {
      // process response
    } else {
      result.error = foundErrors;
    }
    return result;

  },

  /**
   * Reports any error messages in the API response
   * @param xmlTree the API response
   */
  _detectAndReportErrors : function(xmlTree) {
    let foundErrors = null;

    // Check for errors
    if (xmlTree["Errors"]) {
      foundErrors = {};
      let errors = xmlTree["Errors"];

      if (!errors)
        return foundErrors;
      if (!Array.isArray(errors))
        errors = [errors];

      for (let i = 0; i < errors.length; i++) {
        let errorCode = Number(errors[i]["ErrorCode"]);
        let errorType = String(errors[i]["ErrorClassification"]);
        if (errorType) {
          // we use this flag to know if the failed API calls can be retried
          // (applies only to client alerts login logic)
          if (errorType.indexOf("SystemError") != -1) {
            foundErrors.systemErrors = true;
          }
        }
        switch (errorCode) {
          // Catch-all
          default:
            Logger.error("Finding API Error:\n" + "Error Code: " + errorCode +
                         "\nShort Message: " +
                         String(errors[i]["ShortMessage"]) +
                         "\nLong Message: " +
                         String(errors[i]["LongMessage"]) +
                         "\nSeverity: " +
                         String(errors[i]["SeverityCode"]) +
                         "\nError Clasification: " +
                         String(errors[i]["ErrorClassification"]) + "\n",
                         Logger.DUMP_STACK);
            break;
        }
      }
    }

    return foundErrors;
  },

  /**
   * Performs Finding API call
   * @param body The full body of the call, as will be POSTed
   * @param callback The callback function
   * @returns request object
   */
  _doCall : function(body, callback) {
    // Extract the call name from the body --- we need it in the headers
    let requestName;
    try {
      requestName = /><(.*?)Request/.exec(body)[1];
    }
    catch (e) {
      Logger.error("Finding API request will not be sent, as it is " +
                   "badly-formed.", Logger.DUMP_STACK);
      return;
    }
    let callDescription = "Finding API " + requestName;

    let apiCallback =
      ApiHelper.generateApiCallback(callDescription, callback);

    let siteId = Constants.siteIdForSite(Datasource.homeSite());
    ApiAccessService.siteId = siteId;
    let request =
      ApiAccessService.doFindingAPICall(
        requestName, body, false, apiCallback);

    ApiHelper.addPendingRequest(request, callDescription);

    let requestLog = {};
    requestLog.requestName = requestName;
    requestLog.properties = body;
    requestLog.api = "Finding";

    // notify the observer to log the request
    Observers.notify(requestLog, Topics.API_FINDING_CALL, null);

    return request;
  }
};
