/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["ShoppingApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
Cu.import("resource://ebaycompanion/helpers/apiHelper.js");
Cu.import("resource://ebaycompanion/objects/product.js");
Cu.import("resource://ebaycompanion/services/apiAccessService.js");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://ebaycompanion/constants.js");

/**
 * Shopping API actions
 */
ShoppingApi = {

  SHOPPING_REQUEST_TYPE_KEYWORDS : "KEYWORDS",
  SHOPPING_REQUEST_TYPE_ISBN : "ISBN",
  SHOPPING_REQUEST_TYPE_UPC : "UPC",
  SHOPPING_REQUEST_TYPE_ITEM_MODEL : "ITEM_MODEL",
  SHOPPING_REQUEST_TYPE_EAN : "EAN",

  SHOPPING_FORMAT_XML : "XML",
  SHOPPING_FORMAT_NV : "NV",
  SHOPPING_FORMAT_JSON : "JSON",
  SHOPPING_FORMAT_SOAP : "SOAP",

  SHOPPING_MAX_ENTRIES : 5,

  SHOPPING_WARNING_CODES : ["10.20", "10.85", "10.12", "10.15"],

  /**
   * Shopping API GetUserProfile
   * @param callback
   * @returns request object
   */
  getUserProfile : function(callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Shopping API GetUserProfile call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }
    let innerBody = "<IncludeSelector>Details</IncludeSelector>" +
                    "<UserID>" + Datasource.activeAccount().get("userId") + "</UserID>";

    // do the call
    let wrappedBody = this._wrapCall("GetUserProfile", innerBody);
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetUserProfile(response);
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetUserProfile response
   * @param response
   * @returns current user's feedback score
   */
  _parseGetUserProfile : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");
    let xmlTree = XML(response);

    let errorsObject = this._detectAndReportErrors(xmlTree);
    if (errorsObject.isError) {
      // The returned item will be null
      return;
    }

    let feedbackRating = xmlTree.User.FeedbackScore.toString();
    let registrationSite = xmlTree.User.RegistrationSite.toString();

    let ret = {};
    ret.feedbackRating = feedbackRating;
    ret.registrationSite = registrationSite;

    return ret;
  },

  /**
   * Shopping API GetMultipleItems
   * @param aItems a array with the items that will be fetched
   * @param callback
   * @param aRequestTracker the event tracker to be notified when the request
   * returns
   * @param aForceRequest whether to force the request even if there is no
   * signed in user
   * @returns request object
   */
  getMultipleItems : function(aItems, callback, aRequestTracker, aForceRequest) {
    if (!Datasource.activeAccount() && !aForceRequest) {
      Logger.error("Attempt to make Shopping API GetMultipleItems call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }

    let itemHashList = {};
    // create XML string.
    let innerBody = '<IncludeSelector>Details,ItemSpecifics,ShippingCosts</IncludeSelector>';

    for (let i = 0; i < aItems.length; i++) {
      let aItem = aItems[i];
      innerBody += '<ItemID>' + aItem.get("itemId") + '</ItemID>';
      itemHashList[aItem.get("itemId")] = aItem;
    }

    // do the call
    let wrappedBody = this._wrapCall("GetMultipleItems", innerBody);
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetMultipleItems(response);
        try {
          if (callback) {
            if (aRequestTracker) {
              callback(itemHashList, result, aRequestTracker);
            } else {
              callback(aItems, result);
            }
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      };

    var useSandbox = false;
    if (Datasource.activeAccount()) {
      useSandbox = Datasource.activeAccount().get("isSandbox");
    } else {
      // this means we are forcing the request even if there is no user signed
      // in
      useSandbox = false;
    }
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetMultipleItems response
   * @param response
   * @returns current user's feedback score
   */
  _parseGetMultipleItems : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
//    dump(JSON.stringify(xmlObject));
    xmlObject = xmlObject["GetMultipleItemsResponse"];

    this._detectAndReportErrors(xmlObject);

    let ret = {};
    ret.items = [];

    let items = xmlObject["Item"];
    if (!Array.isArray(items))
      items = [items];
    for (let i = 0; i < items.length; i++) {
      let retItem = new Item(Number(items[i]["ItemID"]));
      retItem.fromJXONObject(items[i]);
      ret.items.push(retItem);
    }

    return ret;
  },

  /**
   * Shopping API GetSingleItem
   * @param aItemId the id of the item to be retrieved
   * @param callback
   * @returns request object
   */
  getSingleItem : function(aItemId, callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Shopping API GetSingleItem call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }
    let innerBody = "<IncludeSelector>Details,ItemSpecifics,ShippingCosts</IncludeSelector>" +
                 "<ItemID>"+ aItemId + "</ItemID>";

    // do the call
    let wrappedBody = this._wrapCall("GetSingleItem", innerBody);
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetSingleItem(response);
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetSingleItem response
   * @param response
   * @returns the information of the requested item
   */
  _parseGetSingleItem : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
//    dump(JSON.stringify(xmlObject));
    xmlObject = xmlObject["GetSingleItemResponse"];

    let ret = {};
    ret.item = null;

    let errorsObject = this._detectAndReportErrors(xmlObject);
    let foundErrors = errorsObject.isError;

    if (!foundErrors) {
      let item = xmlObject["Item"];
      if (item != undefined) {
        if (Array.isArray(item))
          item = item[0];
        let retItem = new Item(Number(item["ItemID"]));
        retItem.fromJXONObject(item);
        ret.item = retItem;
      }
    } else {
      ret.error = foundErrors;
    }

    return ret;
  },

  /**
   * Reports any error messages in the API response
   * @param aResponse the API response
   * @param aResultsFormat the API response format (XML, JSON, NV, SOAP)
   * @returns an array of Product objects if products are found or an object
   * which contains the error description (error.isErrror, error.isWarning,
   * error.errorCode, error.errorMessage, error.severity)
   */
  _detectAndReportErrors : function(aResponse, aResultsFormat) {

    let foundErrors = false;
    let usingJSON = false;
    let errorsObject = {};

    if (aResultsFormat && aResultsFormat == ShoppingApi.SHOPPING_FORMAT_JSON) {
      usingJSON = true;
    }

    if (usingJSON) {
      foundErrors = aResponse["Errors"] !== undefined ? true : false;
    }
    else {
      foundErrors = (aResponse["Errors"] && aResponse["Errors"].length > 0);
    }

    if (foundErrors) {
      let errors = aResponse["Errors"];
      let errorsLength = usingJSON ? errors.length : errors.length;

      for (let i = 0; i < errorsLength; i++) {
        let errorCode = String(errors[i]["ErrorCode"]);
        let severity = usingJSON ? String(errors[i]["SeverityCode"]) :
          String(errors[i]["SeverityCode"]);

        let shortMessage = usingJSON ? String(errors[i]["ShortMessage"]) :
          String(errors[i]["ShortMessage"]);
        let errorClasification = usingJSON ?
          String(errors[i]["ErrorClassification"]) :
          String(errors[i]["ErrorClassification"]);

        errorsObject.errorCode = errorCode;
        errorsObject.errorMessage = shortMessage;
        errorsObject.severity = severity;

        if (severity.indexOf("Error") != -1 &&
            !Constants.arrayContains(ShoppingApi.SHOPPING_WARNING_CODES,
                                    errorCode)) {
          errorsObject.isError = true;
          Logger.error("Shopping API Error:\n" + "Error Code: " + errorCode
                       + "\nShort Message: " + shortMessage
                       + "\nSeverity: " + severity
                       + "\nError Clasification: " + errorClasification + "\n"
                       + Logger.DUMP_STACK);
        } else {
          //a warning generally implies no products were found.
          //eBay API treats the "no products found" response as an error, so,
          //we convert the error to a warning
          errorsObject.isWarning = true;
          Logger.warning("Shopping API Warning:\n" + "Error Code: " + errorCode
                       + "\nShort Message: " + shortMessage
                       + "\nSeverity: " + severity
                       + "\nError Clasification: " + errorClasification + "\n"
                       + Logger.DUMP_STACK);
        }
      }
    }

    return errorsObject;
  },

  /**
   * Wraps an XML fragment into a Shopping API call
   * @param callName The name of the call (without "Request")
   * @param body The body of the call
   * @returns the fully-formed text
   */
  _wrapCall : function(callName, innerBody) {
    let xmlHeader = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
    let xmlns = "xmlns=\"urn:ebay:apis:eBLBaseComponents\"";

    let wrappedBody = xmlHeader +
                      "<" + callName + "Request " + xmlns + ">" +
                      innerBody +
                      "</" + callName + "Request>";

    return wrappedBody;
  },


  /**
   * Performs Shopping API call
   * @param body The full body of the call, as will be POSTed
   * @param useSandbox boolean
   * @param callback The callback function
   * @param resultsFormat string indicates the format for getting the ebay API
   * @param requestName the name of the eBay Shoppping API function that will
   * be called
   * query results
   */
  _doCall : function(body, useSandbox, callback, resultsFormat, requestName) {

    if (!resultsFormat) {
      resultsFormat = ShoppingApi.SHOPPING_FORMAT_XML;
    }

    if (resultsFormat == ShoppingApi.SHOPPING_FORMAT_XML) {
      // Extract the call name from the body --- we need it in the headers
      try {
        requestName = /><(.*?)Request/.exec(body)[1];
      }
      catch (e) {
        Logger.error("Shopping API request will not be sent, as it is " +
                     "badly-formed.", Logger.DUMP_STACK);
        return;
      }
    }

    let callDescription = "Shopping API " + requestName;

    let apiCallback =
      ApiHelper.generateApiCallback(callDescription, callback);

/*    if (!ApiHelper.accessService) {
      // The binary component is missing.
      return;
    }*/
    let siteId = Constants.siteIdForSite(Datasource.homeSite());
    //ApiHelper.accessService.setSiteId(siteId);
    ApiAccessService.siteId = siteId;
    //ApiHelper.accessService.setApiVersion(623);
    ApiAccessService.apiVersion = 693;
    let request =
      ApiAccessService.doShoppingAPICall(
        requestName, body, useSandbox, apiCallback, resultsFormat);

    ApiHelper.addPendingRequest(request, callDescription);

    let requestLog = {};
    requestLog.requestName = requestName;
    requestLog.properties = body;
    requestLog.api = "Shopping";

    // notify the observer to log the request
    Observers.notify(requestLog, Topics.API_SHOPPING_CALL, null);

    return request;
  },


  /**
   * Shopping API FindProducts
   * Find products from the eBay shopping API. We can search for products by
   * their ISBN, UPC or keywords from the product title
   * @param callback
   * @param aRequestTracker the event tracker to be notified when the request
   * returns
   * @param aResultsFormat string indicates the format for getting the ebay API
   * query results. For this method the format will be JSON
   * @returns request object
   */
  findProducts : function(aSearchType, aSearchValue, callback, aRequestTracker,
                          aResultsFormat) {

    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Shopping API FindProducts call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return null;
    }

    if (null == aSearchType || null == aSearchValue) {
      return null;
    }

    let wrappedBody = "";
    let functionName = "FindProducts";

    if (!aResultsFormat) {
      aResultsFormat = ShoppingApi.SHOPPING_FORMAT_JSON;
    }

    switch (aSearchType) {
      case ShoppingApi.SHOPPING_REQUEST_TYPE_ISBN:
        wrappedBody += "ProductID.type=ISBN&ProductID.Value="
          + aSearchValue.replace(/-/g,"");
        break;
      case ShoppingApi.SHOPPING_REQUEST_TYPE_UPC:
        wrappedBody += "ProductID.type=UPC&ProductID.Value=" + aSearchValue;
        break;
      case ShoppingApi.SHOPPING_REQUEST_TYPE_EAN:
        wrappedBody += "ProductID.type=EAN&ProductID.Value=" + aSearchValue;
        break;
      case ShoppingApi.SHOPPING_REQUEST_TYPE_ITEM_MODEL:
      case ShoppingApi.SHOPPING_REQUEST_TYPE_KEYWORDS:
        wrappedBody += "QueryKeywords=" + encodeURIComponent(aSearchValue);
        break;
    }

    if (0 < wrappedBody.length) {
      wrappedBody += "&MaxEntries=" + ShoppingApi.SHOPPING_MAX_ENTRIES +
        "&ProductSort=Popularity&AvailableItemsOnly=true";

      let localCallback =
        let(that = this) function(response) {
          let result = that._parseFindProducts(response, aResultsFormat);

          try {
            if (callback) {
              callback(result);
            }
          }
          catch (e) {
            Logger.exception(e, Logger.DUMP_STACK)
          }
        }

      let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
      let request =
        this._doCall(wrappedBody, useSandbox, localCallback, aResultsFormat,
          functionName);

      return request;
    } else {
      return null;
    }
  },


  /**
   * Processes a FindProducts response
   * @param aResponse the API response
   * @param aResultsFormat the API response format (XML, JSON, NV, SOAP)
   * @returns an array of Product objects if products are found or an object
   * which contains the error description (error.isErrror, error.isWarning,
   * error.errorCode, error.errorMessage, error.severity)
   */
  _parseFindProducts : function (aResponse, aResultsFormat) {

    let result = {};
    let products = [];
    //we will need to parse the response in JSON format
    json = JSON.parse(aResponse);

    let errorObject = this._detectAndReportErrors(json, aResultsFormat);

    if (!errorObject.isError && !errorObject.isWarning) {
      for (key in json.Product) {
        product = new Product();

        if (json.Product[key].Title !== undefined) {
          product.set("title", json.Product[key].Title);
        }

        if (json.Product[key].StockPhotoURL !== undefined) {
          product.set("thumbnail", json.Product[key].StockPhotoURL);
        }

        for (let i = 0; i < json.Product[key].ProductID.length; i++) {
          if (json.Product[key].ProductID[i].Type == "Reference") {
            product.set("productId", json.Product[key].ProductID[i].Value);
            i = json.Product[key].ProductID.length;
          }
        }

        products.push(product);
      }

      result.products = products;
    } else {
      result.error = errorObject;
    }

    return result;
  }


};
