/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["SignInApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const SOAP_URL =
  "https://svcs.ebay.com/services/mobile/v1/UserAuthenticationService";
//const SOAP_URL =
//  "http://api4.qa.ebay.com/services/mobile/v1/MessagesForTheDayService";

/**
 * The Authentication API.
 */
SignInApi = {

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/constants.js");
    Cu.import("resource://ebaycompanion/datasource.js");
    Cu.import("resource://ebaycompanion/helpers/logger.js");
    Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
    Cu.import("resource://ebaycompanion/helpers/accessKeysHelper.js");
    Cu.import("resource://ebaycompanion/objects/account.js");
  },

  /**
   * authenticates a user using the sign in API
   * @param aUsername the username to be authenticated
   * @param aPassword the password to be authenticated
   * @param aCallback the callback to be called when the authentication request
   * returns
   */
  signIn : function(aUsername, aPassword, aCallback) {
    let that = this;
    let methodName = "authenticate";
    let callback = function(aResponse) {
      if (aResponse) {
        let body = that._extractSoapResponse(aResponse.responseText);
        let response = body["authenticateResponse"];
        let ack = response["ack"];
        let result = {};

        if ("Success" == ack) {
          result.success = true;
          result.token = response["token"];
        } else {
          result.success = false;
          let message = response["errorMessage"]["error"]["message"];
          Logger.error("SignIn: " + ack + ": " + message);
          result.errorMessage = message;
          result.errorCode = response["errorMessage"]["error"]["errorId"];
        }
        if (aCallback) {
          aCallback(result);
        }
      }
    };

    this._doCall(methodName, aUsername, aPassword, callback);
  },

  /**
   * Extracts the SOAP response.
   * @param aResponseText the xml.
   * @return the body XML of the response.
   */
  _extractSoapResponse : function(aResponseText) {
    let xmlResponse = null;

    // workaround for E4X bug in Firefox and remove namespaces.
    aResponseText = aResponseText.replace(/<\?xml .*?>/, "");
    aResponseText = aResponseText.replace(/soapenv:/gi, "");
    aResponseText = aResponseText.replace(/ xmlns:soapenv=["'].*?["']/, "");
    aResponseText = aResponseText.replace(/ xmlns:ms=["'].*?["']/, "");
    aResponseText = aResponseText.replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(aResponseText,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);

    xmlObject = xmlObject["Envelope"];

    return xmlObject["Body"];
  },

  /**
   * Creates a SOAP request.
   * @param aMethod the method.
   * @param aUsername the username to be sent in the request body
   * @param aPassword the password to be sent in the request body
   * @return the request text.
   */
  _createSoapRequest : function(aMethod, aUsername, aPassword) {
    let soapRequest = "<soapenv:Envelope " +
      "xmlns:soapenv=\"http://www.w3.org/2003/05/soap-envelope\" " +
      "xmlns:ser=\"http://www.ebay.com/marketplace/security/v1/services\">";

    soapRequest += "<soapenv:Body>";
    soapRequest += "<ser:" + aMethod + "Request>";
    soapRequest += "<ser:credentials>";
    soapRequest += "<ser:userId>" + aUsername + "</ser:userId>";
    soapRequest += "<ser:password>" + aPassword + "</ser:password>";
    soapRequest += "<ser:type>PASSWORD</ser:type>";
    soapRequest += "<ser:domain>EBAYUSER</ser:domain>";
    soapRequest += "</ser:credentials>";
    soapRequest += "<ser:options>";
    soapRequest += "<ser:tokenType>SHORT</ser:tokenType>";
    soapRequest += "</ser:options>";
    soapRequest += "</ser:" + aMethod + "Request>";
    soapRequest += "</soapenv:Body>";
    soapRequest += "</soapenv:Envelope>";
    //dump("REQUEST: " + soapRequest + "\n");

    return soapRequest;
  },

  /**
   * Gets the application name.
   * @return the application name.
   */
  _getKeyset : function() {
    let site = Datasource.homeSite();
    let siteId = Constants.siteIdForSite(site);
    let keyset = AccessKeysHelper.getKeySet(false, siteId);

    return keyset;
  },

  /**
   * Sends a request to the WS.
   * @param aMethod the request method.
   * @param aUsername the username to be authenticated
   * @param aPassword the password to be authenticated
   * @param aCallback the request callback.
   */
  _doCall : function(aMethod, aUsername, aPassword, aCallback) {
    let globalId = Constants.getGlobalId();
    let keyset = this._getKeyset();
    let request =
      Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
        createInstance(Ci.nsIXMLHttpRequest);
    let headers = {
      "Content-Type" : "application/soap+xml; charset=UTF-8",
      "X-EBAY-SOA-SERVICE-NAME" : "UserAuthenticationService",
      "X-EBAY-SOA-MESSAGE-PROTOCOL" : "SOAP12",
      "X-EBAY-SOA-SERVICE-VERSION" : "1.0.0",
      "X-EBAY-SOA-OPERATION-NAME" : aMethod,
      "X-EBAY-SOA-SECURITY-APPNAME" : keyset.appName,
      "X-EBAY-DEVNAME" : keyset.devName,
      "X-EBAY-CERTNAME" : keyset.certName,
      "X-EBAY-SOA-GLOBAL-ID" : globalId

    };
    let requestBody = this._createSoapRequest(aMethod, aUsername, aPassword);

    request.mozBackgroundRequest = true;
    request.open("POST", SOAP_URL, true);
    request.onreadystatechange = function() {
      if (4 != request.readyState) { return; }
      aCallback(request);
    };
    for (let header in headers) {
      request.setRequestHeader(header, headers[header]);
    }
    request.send(requestBody);
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(SignInApi);
