/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["SystemMessageApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const SOAP_URL =
  "https://svcs.ebay.com/services/mobile/v1/MessagesForTheDayService";
//const SOAP_URL =
//  "http://api4.qa.ebay.com/services/mobile/v1/MessagesForTheDayService";

/**
 * The Message of the day API.
 */
SystemMessageApi = {
  // Array to map global ids and site data
  _GLOBAL_ID_DATA : {
    "Australia"      : "EBAY-AU",
    "Austria"        : "EBAY-AT",
    "Belgium.Dutch"  : "EBAY-NLBE",
    "Belgium.French" : "EBAY-FRBE",
    "Canada"         : "EBAY-ENCA",
    "CanadaFrench"   : "EBAY-FRCA",
    "France"         : "EBAY-FR",
    "Germany"        : "EBAY-DE",
    "HongKong"       : "EBAY-HK",
    "India"          : "EBAY-IN",
    "Ireland"        : "EBAY-IE",
    "Italy"          : "EBAY-IT",
    "Netherlands"    : "EBAY-NL",
    "Philippines"    : "EBAY-PH",
    "Malaysia"       : "EBAY-MY",
    "Poland"         : "EBAY-PL",
    "Singapore"      : "EBAY-SG",
    "Spain"          : "EBAY-ES",
    "Switzerland"    : "EBAY-CH",
    "UK"             : "EBAY-GB",
    "US"             : "EBAY-US"
  },

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/constants.js");
    Cu.import("resource://ebaycompanion/datasource.js");
    Cu.import("resource://ebaycompanion/helpers/logger.js");
    Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
    Cu.import("resource://ebaycompanion/helpers/accessKeysHelper.js");
    Cu.import("resource://ebaycompanion/objects/systemMessage.js");
    Cu.import("resource://ebaycompanion/services/systemMessageService.js");
  },

  /**
   * Calls the getMessagesForTheDay method in the web service.
   */
  getMessagesForTheDay : function() {
    let that = this;
    let methodName = "getMessagesForTheDay";
    let callback = function(aResponse) {
      if (aResponse) {
        let body = that._extractSoapResponse(aResponse.responseText);
        let response = body["getMessagesForTheDayResponse"];
        let ack = response["ack"];

        if ("Success" == ack) {
          that._parseGetMessagesForTheDay(response);
        } else {
          let message = response["errorMessage"]["error"]["message"];
          Logger.error("getMessagesForTheDay: " + ack + ": " + message);
        }
      }
    };

    this._doCall(methodName, callback);
    this._removeOldSystemMessages();
  },

  /**
   * Removes the old system messages.
   */
  _removeOldSystemMessages : function() {
    if (typeof(ObjectsStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
    }
    ObjectsStorage.removeSystemMessages(Datasource.getEbayTime().getTime());
  },

  /**
   * Extracts the SOAP response.
   * @param aResponseText the xml.
   * @return the body XML of the response.
   */
  _extractSoapResponse : function(aResponseText) {
    let xmlResponse = null;

    // workaround for E4X bug in Firefox and remove namespaces.
    aResponseText = aResponseText.replace(/<\?xml .*?>/, "");
    aResponseText = aResponseText.replace(/soapenv:/gi, "");
    aResponseText = aResponseText.replace(/ xmlns:soapenv=["'].*?["']/, "");
    aResponseText = aResponseText.replace(/ xmlns:ms=["'].*?["']/, "");
    aResponseText = aResponseText.replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(aResponseText,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);

    xmlObject = xmlObject["Envelope"];

    return xmlObject["Body"];
  },

  /**
   * Processes a GetMessagesForTheDay response.
   * @param aResponse the API response.
   */
  _parseGetMessagesForTheDay : function(aResponse) {
    let message = null;
    let validToTime = null;
    let validFromTime = null;
    let systemMessage = null;
    let userId = Datasource.activeAccount().get("userId");
    let jsonBody = null;
    let messageList = aResponse["messageList"];

    if (!Array.isArray(messageList))
      messageList = [messageList];

    for (let messageIndex in messageList) {
      message = messageList[messageIndex];
      if (message) {
        // in this case, dates are in Date format already, so we just get the
        // timestamp
        validFromTime = message["validFromTime"].getTime();
        validToTime = message["validToTime"].getTime();
        jsonBody = String(message["message"]);

        systemMessage =
          new SystemMessage(String(message["messageId"]), String(userId));
        systemMessage.set("jsonBody", jsonBody);
        systemMessage.set("type", String(message["messageType"]));
        systemMessage.set("maxNumberOfViews", String(message.maxNumberOfViews));
        systemMessage.set("validFromTime", validFromTime);
        systemMessage.set("validToTime", validToTime);
        SystemMessageService.addOrUpdateSystemMessage(systemMessage);
      }
    }
  },

  /**
   * Gets the timestamp from a given time string.
   * @param aTimeString the time string.
   * @return the timestamp
   */
  _getTimestamp : function(aTimeString) {
    let dateRegExp = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})$/;
    let dateResults = dateRegExp.exec(aTimeString);
    let date = new Date();
    let timestamp = null;

    if (dateResults) {
      // the date object deals with months from 0-11, not from 1-12
      date.setUTCFullYear(dateResults[1], dateResults[2]-1, dateResults[3]);
      date.setUTCHours(dateResults[4], dateResults[5], dateResults[6]);
    } else {
      Logger.error(
        "Incompatible date format: " + aTimeString, Logger.DUMP_STACK);
    }
    timestamp = String(date.getTime());

    return timestamp;
  },

  /**
   * Creates a SOAP request.
   * @param aMethod the method.
   * @return the request text.
   */
  _createSoapRequest : function(aMethod) {
    let site = Datasource.homeSite();
    let siteId = Constants.siteIdForSite(site);
    let language = "en-US";
    let soapRequest = "<soapenv:Envelope " +
      "xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" " +
      "xmlns:ser=\"http://www.ebay.com/marketplace/mobile/v1/services\">";

    for (locale in Constants.localeSiteDataSelector) {
      if (Constants.localeSiteDataSelector[locale] == site) {
        language = locale;
      }
    }

    soapRequest += "<soapenv:Header/>";
    soapRequest += "<soapenv:Body>";
    soapRequest += "<ser:" + aMethod + "Request>";
    soapRequest += "<ser:clientId>FIREFOXEXTENSION</ser:clientId>";
    soapRequest += "<ser:language>" + language + "</ser:language>";
    soapRequest += "<ser:siteId>" + siteId + "</ser:siteId>";
    soapRequest += "</ser:" + aMethod + "Request>";
    soapRequest += "</soapenv:Body>";
    soapRequest += "</soapenv:Envelope>";

    return soapRequest;
  },

  /**
   * Gets the application name.
   * @return the application name.
   */
  _getAppName : function() {
    let site = Datasource.homeSite();
    let siteId = Constants.siteIdForSite(site);
    let keyset = AccessKeysHelper.getKeySet(false, siteId);

    return keyset.appName;
  },

  /**
   * Sends a request to the WS.
   * @param aMethod the request method.
   * @param aCallback the request callback.
   */
  _doCall : function(aMethod, aCallback) {
    let globalId = Constants.getGlobalId();
    let appName = this._getAppName();
    let request =
      Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
        createInstance(Ci.nsIXMLHttpRequest);
    let headers = {
      "X-EBAY-SOA-SERVICE-NAME": "MessagesForTheDayService",
      "X-EBAY-SOA-MESSAGE-PROTOCOL" : "SOAP12",
      "X-EBAY-SOA-OPERATION-NAME" : aMethod,
      "X-EBAY-SOA-GLOBAL-ID" : globalId,
      "X-EBAY-SOA-SECURITY-APPNAME" : appName
    };
    let requestBody = this._createSoapRequest(aMethod);

    request.mozBackgroundRequest = true;
    request.open("POST", SOAP_URL, true);
    request.onreadystatechange = function() {
      if (4 != request.readyState) { return; }
      aCallback(request);
    };
    for (let header in headers) {
      request.setRequestHeader(header, headers[header]);
    }
    request.send(requestBody);
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(SystemMessageApi);
