/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["TradingApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/apiHelper.js");
Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
Cu.import("resource://ebaycompanion/services/apiAccessService.js");
Cu.import("resource://ebaycompanion/helpers/informationNotificationHelper.js");
Cu.import("resource://ebaycompanion/helpers/warningNotificationHelper.js");
Cu.import("resource://ebaycompanion/objects/account.js");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/transaction.js");
Cu.import("resource://ebaycompanion/objects/favoriteSeller.js");
Cu.import("resource://ebaycompanion/objects/favoriteSearch.js");
Cu.import("resource://ebaycompanion/objects/notification.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/storage/propertiesStorage.js");

/**
 * Trading API actions
 */
TradingApi = {

  /**
   * Reports Toolbar Activity (install/uninstall/upgrade)
   * @param aActivity the activity to be reported
   * @param aCredentialsArray an array with the credentials to be sent with the
   * request
   * @param aUseSandbox whether to use the sandbox or not
   * @param callback Provides token
   */
  reportToolbarActivity : function(
    aActivity, aCredentialsArray, aUseSandbox, aCallback) {

    let toolbarId = PropertiesStorage.get("ToolbarId");
    Logger.log("Reporting toolbar activity: \"" + aActivity + "\" with " +
               "toolbarId: " + toolbarId);

    if (aActivity != "Install" && !toolbarId) {
      Logger.error("Attempt to make Trading API non-install " +
                   "ReportToolbarActivity call when there is no toolbarId " +
                   "stored locally.", Logger.DUMP_STACK);
      return;
    }

    if (aActivity == "Install" && toolbarId) {
      Logger.error("Attempt to make Trading API install " +
                   "ReportToolbarActivity call when there is already a " +
                   "toolbarId stored locally.", Logger.DUMP_STACK);
      let tempResult = {};
      tempResult.error = true;
      tempResult.existingToolbarId = true;
      if (aCallback) {
        aCallback(tempResult);
      }
      return;
    }

    let toolbarIdValue = "";
    if (toolbarId) {
      toolbarIdValue = "<ToolbarID>" + toolbarId + "</ToolbarID>";
    }

    let promptedForUpgrade = "";
    if (aActivity == "Upgrade") {
      promptedForUpgrade = "<PromptedForUpgrade>0</PromptedForUpgrade>";
    }

    let credentials = "";
    for each (let [name, value] in Iterator(aCredentialsArray)) {
      credentials += "<" + name + ">" + value + "</" + name + ">";
    }

    let requestIdentifier = Datasource.getRequestIdentifier();

    let wrappedBody =
      "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
      "<ReportToolbarActivityRequest xmlns=\"urn:ebay:apis:eBLBaseComponents\">" +
        "<MessageID>" + requestIdentifier + "</MessageID>" +
        "<RequesterCredentials>" +
          credentials +
        "</RequesterCredentials>" +
        "<ToolbarVersion>1</ToolbarVersion>" +
        "<Activity>" + aActivity + "</Activity>" +
          toolbarIdValue +
          promptedForUpgrade +
      "</ReportToolbarActivityRequest>";

    let localCallback =
      let(that = this) function(response) {
        let result = that._parseReportToolbarActivityResponse(response);
        try {
          if (aCallback) aCallback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let request = this._doCall(wrappedBody, aUseSandbox, localCallback);

    return request;
  },

  /**
   * Processes a ReportToolbarActivity response
   * @param response
   * @returns toolbar activity response details
   */
  _parseReportToolbarActivityResponse : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");
    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
    xmlObject = xmlObject["ReportToolbarActivityResponse"];

    let foundErrors = this._detectAndReportErrors(xmlObject);
    let result = {};

    if (!foundErrors) {
      let toolbarId = xmlObject["ToolbarID"];
      if (toolbarId) {
        result.toolbarId = toolbarId;
      }
    } else {
      result.error = foundErrors;
    }

    return result;

  },


  /**
   * Trading API GetSessionID
   * @param aUseSandbox whether or not to use the sandbox
   * @param aCallback Processes the returned sessionID
   */
  getSessionID : function(aUseSandbox, aCallback) {
    let requestIdentifier = Datasource.getRequestIdentifier();
    let runame = Constants.getRuname(aUseSandbox, Datasource.homeSite());
    let wrappedBody =
      "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
      "<GetSessionIDRequest xmlns=\"urn:ebay:apis:eBLBaseComponents\">" +
        "<MessageID>" + requestIdentifier + "</MessageID>" +
        "<RuName>" + runame + "</RuName>" +
      "</GetSessionIDRequest>";

    let localCallback =
      let(that = this) function(response) {
        // Workaround for E4X bug in Firefox
        // Also get rid of that annoying namespace
        response = response.replace(/<\?xml .*?>/, "").
                            replace(/ xmlns=["'].*?["']/, "");
        var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
          .createInstance(Ci.nsIDOMParser);
        let xmlTree = parser.parseFromString(response,"text/xml");
        var xmlObject = ObjectHelper.getJXONTree(xmlTree);
        xmlObject = xmlObject["GetSessionIDResponse"];

        let foundErrors = that._detectAndReportErrors(xmlObject);
        let result = {};

        if (!foundErrors) {
          let sessionId = String(xmlObject["SessionID"]);
          result.sessionId = sessionId;
        } else {
          result.errors = foundErrors;
        }

        try {
          if (aCallback) {
            aCallback(result);
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let request = this._doCall(wrappedBody, aUseSandbox, localCallback);
    return request;
  },

  /**
   * Trading API FetchToken
   * @param isSandboxAccount boolean
   * @param sessionId the session id requested previously when loading the
   * login page (stored in Datasource)
   * @param callback Provides token
   */
  fetchToken : function(isSandboxAccount, sessionId, callback) {
    let requestIdentifier = Datasource.getRequestIdentifier();
    let wrappedBody =
      "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
      "<FetchTokenRequest xmlns=\"urn:ebay:apis:eBLBaseComponents\">" +
        "<MessageID>" + requestIdentifier + "</MessageID>" +
        "<SessionID>" + sessionId + "</SessionID>" +
      "</FetchTokenRequest>";

    let localCallback =
      let(that = this) function(response) {
        // Workaround for E4X bug in Firefox
        // Also get rid of that annoying namespace
        response = response.replace(/<\?xml .*?>/, "").
                            replace(/ xmlns=["'].*?["']/, "");
        var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
          .createInstance(Ci.nsIDOMParser);
        let xmlTree = parser.parseFromString(response,"text/xml");
        var xmlObject = ObjectHelper.getJXONTree(xmlTree);
        xmlObject = xmlObject["FetchTokenResponse"];

        let errors = that._detectAndReportErrors(xmlObject);

        let authToken = String(xmlObject["eBayAuthToken"]);
        try {
          if (callback) {
            let result = {};
            result.token = authToken;
            result.errors = errors;
            callback(result);
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let request = this._doCall(wrappedBody, isSandboxAccount, localCallback);
    return request;
  },

  /**
   * Trading API GetUser
   * @param callback the callback method to be called when the response is
   * processed
   */
  getUser : function(aCallback) {
    let activeAccount = Datasource.activeAccount();
    if (!activeAccount) {
      Logger.error("Attempt to make Trading API getUser call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }
    var wrappedBody;
    var localCallback;
    var that = this;
    var request;
    var innerBody = "" +
      "<UserID>"+activeAccount.get("userId")+"</UserID>";

    // do the call
    wrappedBody =
      this._wrapCall("GetUser", innerBody);
    localCallback = function(aResponse) {
      var result = that._parseGetUserResponse(aResponse);

      try {
        if (aCallback) {
          aCallback(result);
        }
      }
      catch(e) {
        Logger.exception(e, Logger.DUMP_STACK)
      }
    };

    request =
      this._doCall(
        wrappedBody, activeAccount.get("isSandboxAccount"), localCallback);

    return request;
  },

  /**
   * Processes a GetUser response
   * @param aResponse
   * @returns an object with partial information about the current user
   */
  _parseGetUserResponse : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");
    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
    xmlObject = xmlObject["GetUserResponse"];
    let foundErrors = this._detectAndReportErrors(xmlObject);
    let result = {};

    if (!foundErrors) {
      result.feedbackRating = Number(xmlObject["User"]["FeedbackScore"]);
      result.registrationSite = String(xmlObject["User"]["Site"]);
      result.email = String(xmlObject["User"]["Email"]);
    } else {
      result.error = foundErrors;
    }

    return result;
  },


  /**
   * Trading API GetMyeBayBuying
   * @param callback
   * @returns request object
   */
  getMyeBayBuying : function(callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API GetMyeBayBuying call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }
    let innerBody = "" +
      // Won List
      "<WonList>" +
      "<DurationInDays>60</DurationInDays>" +
      "</WonList>" +
      // Lost List
      "<LostList>" +
      "<DurationInDays>60</DurationInDays>" +
      "</LostList>" +
      // Watch List
      "<WatchList>" +
      "<Pagination>" +
      "<EntriesPerPage>100</EntriesPerPage>" +
      "<PageNumber>1</PageNumber>" +
      "</Pagination>" +
      "<Sort>EndTime</Sort>" +
      "</WatchList>" +
      // Bid List
      "<BidList>" +
      "<Pagination>" +
      "<EntriesPerPage>100</EntriesPerPage>" +
      "<PageNumber>1</PageNumber>" +
      "</Pagination>" +
      "<Sort>EndTime</Sort>" +
      "</BidList>" +
      // Best offer List
      "<BestOfferList>" +
      "<Pagination>" +
      "<EntriesPerPage>100</EntriesPerPage>" +
      "<PageNumber>1</PageNumber>" +
      "</Pagination>" +
      "<Sort>EndTime</Sort>" +
      "</BestOfferList>" +
      // Favourite sellers
      "<FavoriteSellers>" +
      "<Include>true</Include>" +
      "</FavoriteSellers>" +
      // Saved searches
      "<FavoriteSearches>" +
      "<Include>true</Include>" +
      "</FavoriteSearches>";

    // do the call
    let wrappedBody = this._wrapCall("GetMyeBayBuying", innerBody);
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetMyeBayBuyingResponse(response);
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetMyeBayBuying response
   * @param response
   * @returns Transaction objects and partially-initialised Item objects
   */
  _parseGetMyeBayBuyingResponse : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
    xmlObject = xmlObject["GetMyeBayBuyingResponse"];
    let foundErrors = this._detectAndReportErrors(xmlObject);
    let result = {};


    if (!foundErrors) {
      // Extract item IDs from the call (avoiding duplicates)
      let itemIds = [];
      let xmlItemIds = xmlTree.getElementsByTagName("ItemID");

      for (let i=0; i<xmlItemIds.length;i++) {
        let xmlItemId = xmlItemIds[i];
        if (xmlItemId.childNodes) {
          // Using this array as a hash list to avoid duplicates
          itemIds[xmlItemId.childNodes[0].nodeValue] = true;
        }
      }

      let bestOfferItems = {};
      if (xmlObject["BestOfferList"]) {
        let xmlBestOfferItems = xmlObject["BestOfferList"]["ItemArray"]["Item"];
        if (!Array.isArray(xmlBestOfferItems))
          xmlBestOfferItems = [xmlBestOfferItems];
        for each (let xmlItem in xmlBestOfferItems) {
          bestOfferItems[xmlItem["ItemID"]] = xmlItem;
        }
      }

      // Keep an array with the watched items to get their thumbnail image
      let watchedItems = {};
      if (xmlObject["WatchList"]) {
        let xmlWatchedItems = xmlObject["WatchList"]["ItemArray"]["Item"];
        if (!Array.isArray(xmlWatchedItems))
          xmlWatchedItems = [xmlWatchedItems];
        for each (let xmlItem in xmlWatchedItems) {
          watchedItems[xmlItem["ItemID"]] = xmlItem;
        }
      }

      // Keep an array with the bid items to get their thumbnail image
      let bidItems = {};
      if (xmlObject["BidList"]) {
        let xmlBidItems = xmlObject["BidList"]["ItemArray"]["Item"];
        if (!Array.isArray(xmlBidItems))
          xmlBidItems = [xmlBidItems];
        for each (let xmlItem in xmlBidItems) {
          bidItems[xmlItem["ItemID"]] = xmlItem;
        }
      }

      // Keep an array with the lost items to get their thumbnail image
      let lostItems = {};
      if (xmlObject["LostList"]) {
        let xmlLostItems = xmlObject["LostList"]["ItemArray"]["Item"];
        if (!Array.isArray(xmlLostItems))
          xmlLostItems = [xmlLostItems];
        for each (let xmlItem in xmlLostItems) {
          lostItems[xmlItem["ItemID"]] = xmlItem;
        }
      }

      // Keep an array with the won items to get their thumbnail image
      let wonItems = {};
      if (xmlObject["WonList"]) {
        let xmlWonItems = xmlObject["WonList"]["OrderTransactionArray"]["OrderTransaction"];
        if (!Array.isArray(xmlWonItems))
          xmlWonItems = [xmlWonItems];
        for each (let xmlItem in xmlWonItems) {
          if (xmlItem["Order"]) {
            for each (let itemTrans in xmlItem["Order"]["TransactionArray"]["Transaction"]) {
              if (itemTrans["Item"]) {
                wonItems[itemTrans["Item"]["ItemID"]] = itemTrans["Item"];
              }
            }
          } else {
            wonItems[xmlItem["Transaction"]["Item"]["ItemID"]] = xmlItem["Transaction"]["Item"];
          }
        }
      }

      // For each tracked item, extract necessary information and add to array
      let items = [];
      for (let itemId in itemIds) {
        let item = new Item(Number(itemId));

        let xmlItem;

        // The QuantityWon element will not be returned for ended items, but we
        // can infer userQuantityWinning from which list the item was found in
        let quantityWinning = 0;
        // flag used to prevent bestOffer items not returned in watched items
        // list from being added to the items array
        let addItemToArray = true;
        if (bidItems[itemId]) {
          xmlItem = bidItems[itemId];
          item.fromJXONObject(xmlItem);
          item.type = Item.ITEM_TYPE_BIDDING;
        } else if (wonItems[itemId]) {
          xmlItem = wonItems[itemId];
          item.fromJXONObject(xmlItem);
          // Set the highbidder to the current user, so the sidebar knows we've
          // won the item
          let currentUser = Datasource.activeAccount();
          if (currentUser) {
            item.set("highBidderId", currentUser.get("userId"));
          }
          item.type = Item.ITEM_TYPE_WON;
        } else if (lostItems[itemId]) {
          xmlItem = lostItems[itemId];
          item.fromJXONObject(xmlItem);
          item.type = Item.ITEM_TYPE_LOST;
        } else if (bestOfferItems[itemId]) {
          xmlItem = bestOfferItems[itemId];
          item.fromJXONObject(xmlItem);
          item.type = Item.ITEM_TYPE_BEST_OFFER;
        } else {
          xmlItem = watchedItems[itemId];
          item.fromJXONObject(xmlItem);
          item.type = Item.ITEM_TYPE_WATCHING;
          let currentUser = Datasource.activeAccount();
          // don't add as watching items, those that the current user is selling
          if (currentUser &&
              currentUser.get("userId").toLowerCase() ==
                item.get("sellerUserId").toLowerCase()) {
            addItemToArray = false;
          }
        }

        if (addItemToArray && this.isValidXMLItem(xmlItem)) {
          items.push(item);
        }

      }

      // Extract transactions from the call
      let xmlTransactions = xmlTree.getElementsByTagName("Transaction");
      let transactions = [];
      for (let i=0; i<xmlTransactions.length; i++) {
        let xmlTransaction = xmlTransactions[i];
        let jxonTransaction = ObjectHelper.getJXONTree(xmlTransaction);
        let transaction = new Transaction(jxonTransaction["Item"]["ItemID"],jxonTransaction["TransactionID"]);
        transaction.fromJXONObject(jxonTransaction);

        transactions.push(transaction);
      }

      // Extract favorite sellers from the call (if any)
      let favoriteSellers = [];
      let xmlFavoriteSellers = xmlTree.getElementsByTagName("FavoriteSeller");
      let activeAccount = Datasource.activeAccount();
      for (let i=0; i<xmlFavoriteSellers.length;i++) {
        let xmlFavoriteSeller = xmlFavoriteSellers[i];
        let jxonFavoriteSeller = ObjectHelper.getJXONTree(xmlFavoriteSeller);
        let sellerId = jxonFavoriteSeller["UserID"];
        let storeName =
          jxonFavoriteSeller["StoreName"] ? jxonFavoriteSeller["StoreName"] : "";
        let favoriteSeller =
          new FavoriteSeller(
            activeAccount.get("userId"),
            activeAccount.get("isSandboxAccount"),
            sellerId);
        favoriteSeller.set("storeName", storeName);
        favoriteSellers.push(favoriteSeller);
      }


      // Extract favorite searches from the call (if any)
      let favoriteSearches = [];
      let xmlFavoriteSearches = xmlTree.getElementsByTagName("FavoriteSearch");
      for (let i=0; i<xmlFavoriteSearches.length;i++) {
        let xmlFavoriteSearch = xmlFavoriteSearches[i];
        let jxonFavoriteSearch = ObjectHelper.getJXONTree(xmlFavoriteSearch);
        let searchName = jxonFavoriteSearch["SearchName"];
        let searchQuery = jxonFavoriteSearch["SearchQuery"];
        let favoriteSearch =
          new FavoriteSearch(
            activeAccount.get("userId"),
            activeAccount.get("isSandboxAccount"),
            searchName);
        favoriteSearch.set("searchQuery", searchQuery);
        favoriteSearches.push(favoriteSearch);
      }

      // Extract the timestamp (used for tracking eBay time)
      let timestamp = xmlTree["Timestamp"]

      result.items = items;
      result.transactions = transactions;
      result.timestamp = timestamp;
      result.favoriteSellers = favoriteSellers;
      result.favoriteSearches = favoriteSearches;
    } else {
      result.error = foundErrors;
    }

    return result;
  },

  /**
   * Trading API GetMyeBaySelling
   * @param callback
   * @returns request object
   */
  getMyeBaySelling : function(callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API GetMyeBaySelling call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }

    let innerBody = "" +
      // Active List
      "<ActiveList>" +
      "<Pagination>" +
      "<EntriesPerPage>200</EntriesPerPage>" +
      "<PageNumber>1</PageNumber>" +
      "</Pagination>" +
      "<Sort>EndTime</Sort>" +
      "</ActiveList>" +
      // Active List
      "<SoldList>" +
      "<DurationInDays>60</DurationInDays>" +
      "<Pagination>" +
      "<EntriesPerPage>200</EntriesPerPage>" +
      "<PageNumber>1</PageNumber>" +
      "</Pagination>" +
      "<Sort>EndTime</Sort>" +
      "</SoldList>" +
      // Unsold List
      "<UnsoldList>" +
      "<DurationInDays>60</DurationInDays>" +
      "<Pagination>" +
      "<EntriesPerPage>200</EntriesPerPage>" +
      "<PageNumber>1</PageNumber>" +
      "</Pagination>" +
      "<Sort>EndTime</Sort>" +
      "</UnsoldList>";

    // do the call
    let wrappedBody = this._wrapCall("GetMyeBaySelling", innerBody);
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetMyeBaySellingResponse(response);
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetMyeBaySelling response
   * @param response
   * @returns Transaction objects and partially-initialised Item objects
   */
  _parseGetMyeBaySellingResponse : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");
    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
    xmlObject = xmlObject["GetMyeBaySellingResponse"];

    let foundErrors = this._detectAndReportErrors(xmlTree);
    let result = {};

    if (!foundErrors) {
      let itemIds = [];
      let xmlItemIds = xmlTree.getElementsByTagName("ItemID");

      for (let i=0; i<xmlItemIds.length;i++) {
        let xmlItemId = xmlItemIds[i];
        if (xmlItemId.childNodes) {
          // Using this array as a hash list to avoid duplicates
          itemIds[xmlItemId.childNodes[0].nodeValue] = true;
        }
      }

      let activeItems = {};
      if (xmlObject["ActiveList"]) {
        let xmlActiveItems = xmlObject["ActiveList"]["ItemArray"]["Item"];
        if (!Array.isArray(xmlActiveItems))
          xmlActiveItems = [xmlActiveItems];
        for each (let xmlItem in xmlActiveItems) {
          activeItems[xmlItem["ItemID"]] = xmlItem;
        }
      }

      // Keep an array with the won items to get their thumbnail image
      let soldItems = {};
      if (xmlObject["SoldList"]) {
        let xmlSoldItems = xmlObject["SoldList"]["OrderTransactionArray"]["OrderTransaction"];
        if (!Array.isArray(xmlSoldItems))
          xmlSoldItems = [xmlSoldItems];
        for each (let xmlItem in xmlSoldItems) {
          if (xmlItem["Order"]) {
            for each (let itemTrans in xmlItem["Order"]["TransactionArray"]["Transaction"]) {
              if (itemTrans["Item"]) {
                soldItems[itemTrans["Item"]["ItemID"]] = itemTrans["Item"];
              }
            }
          } else {
            soldItems[xmlItem["Transaction"]["Item"]["ItemID"]] = xmlItem["Transaction"]["Item"];
          }
        }
      }

      let unsoldItems = {};
      if (xmlObject["UnsoldList"]) {
        let xmlUnsoldItems = xmlObject["UnsoldList"]["ItemArray"]["Item"];
        if (!Array.isArray(xmlUnsoldItems))
          xmlUnsoldItems = [xmlUnsoldItems];
        for each (let xmlItem in xmlUnsoldItems) {
          unsoldItems[xmlItem["ItemID"]] = xmlItem;
        }
      }

      // For each tracked item, extract necessary information and add to array
      let items = {};
      for (let itemId in itemIds) {
        let item = new Item(Number(itemId));
        let xmlItem;

        item.set("userMaxBid", 0);
        item.set("userQuantityBidFor", 0);

        let thumbnailUrl = "";
        let convertedCurrentPrice = 0;
        let convertedCurrentPriceCurrency = "";

        if (activeItems[itemId]) {
          xmlItem = activeItems[itemId];
          item.type = Item.ITEM_TYPE_SELLING;
        } else if (soldItems[itemId]) {
          xmlItem = soldItems[itemId];
          item.type = Item.ITEM_TYPE_SOLD;
        } else if (unsoldItems[itemId]) {
          xmlItem = unsoldItems[itemId];
          item.type = Item.ITEM_TYPE_UNSOLD;
        }

        item.fromJXONObject(xmlItem);

        if (this.isValidXMLItem(xmlItem)) {
          // use the items array as a hash too to use it during the
          // GetSellerList call
          items[itemId] = item;
          //items.push(item);
        }
      }

      // Extract transactions from the call
      let xmlTransactions = xmlTree.getElementsByTagName("Transaction");
      let transactions = [];
      for (let i=0; i<xmlTransactions.length; i++) {
        let xmlTransaction = xmlTransactions[i];
        let jxonTransaction = ObjectHelper.getJXONTree(xmlTransaction);
        let transaction = new Transaction(jxonTransaction["Item"]["ItemID"],jxonTransaction["TransactionID"]);
        transaction.fromJXONObject(jxonTransaction);

        transactions.push(transaction);
      }

      result.items = items;
      result.transactions = transactions;

    } else {
      result.error = foundErrors;
    }

    return result;
  },

  /**
   * Trading API GetSellerList
   * @param getMyeBaySellingResults an object with the information (items and
   * transactions) returned by the GetMyEbaySelling call that we will need
   * during GetSellerList call and later on GetMultipleItems
   * @param callback
   * @returns request object
   */
  getSellerList : function(getMyeBaySellingResults, callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API GetSellerList call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }

    let today = new Date();
    let fromDate = new Date().setDate(today.getDate()-119);
    let dateFormat = "$YYYY-$MM-$DD $hh:$mm:$ss";

    let innerBody = "" +
      // Sold List
      "<OutputSelector>" +
      "ItemArray.Item.ItemID," +
      "ItemArray.Item.HitCount," +
      "ItemArray.Item.HitCounter" +
      "</OutputSelector>" +
      "<GranularityLevel>Coarse</GranularityLevel>" +
      "<StartTimeFrom>" + Constants.formatDate(fromDate, dateFormat) + "</StartTimeFrom>" +
      "<StartTimeTo>" + Constants.formatDate(today, dateFormat) + "</StartTimeTo>" +
      "<Pagination>" +
      "<EntriesPerPage>200</EntriesPerPage>" +
      "</Pagination>";

    // do the call
    let wrappedBody = this._wrapCall("GetSellerList", innerBody);
    let localCallback =
      let(that = this) function(response) {
        let result =
          that._parseGetSellerListResponse(
            response, getMyeBaySellingResults.items);

        result.transactions = getMyeBaySellingResults.transactions;
        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetSellerList response
   * @param itemsList the list of the items returned by GetMyEbaySelling, so we
   * can update the fields we are interested in when performing the
   * GetSellerList call
   * @param response
   * @returns partially-initialised Item objects
   */
  _parseGetSellerListResponse : function(response, itemsList) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
    xmlObject = xmlObject["GetSellerListResponse"];

    let foundErrors = this._detectAndReportErrors(xmlObject);
    let result = {};

    if (!foundErrors) {
      // Extract item IDs from the call (avoiding duplicates)
      let itemIds = [];
      let xmlItemIds = xmlTree.getElementsByTagName("ItemID");

      for (let i=0; i<xmlItemIds.length;i++) {
        let xmlItemId = xmlItemIds[i];
        if (xmlItemId.childNodes) {
          // Using this array as a hash list to avoid duplicates
          itemIds[xmlItemId.childNodes[0].nodeValue] = true;
        }
      }

      let sellingItems = {};
      if (xmlObject["ItemArray"] && xmlObject["ItemArray"] != true) {
        let xmlActiveItems = xmlObject["ItemArray"]["Item"];
        if (!Array.isArray(xmlActiveItems))
          xmlActiveItems = [xmlActiveItems];
        for each (let xmlItem in xmlActiveItems) {
          sellingItems[xmlItem["ItemID"]] = xmlItem;
        }
      }

      // For each tracked item, extract necessary information and add to array
      let items = [];
      for (let itemId in itemIds) {
        let xmlItem;
        let item = itemsList[itemId];
        if (item) {
          xmlItem = sellingItems[itemId];

          //XXX: For some reason GetSellerList is not returning hitcount. Commenting for now.
//          item.set("pageViews", Number(xmlItem["HitCount"]));
          item.set("hitCounterType", String(xmlItem["HitCounter"]));

          if (this.isValidXMLItem(xmlItem)) {
            itemsList[itemId] = item;
          }
        }

      }

      // now we can convert the item list from a hash to a regular array
      for each (let [itemId, item] in Iterator(itemsList)) {
        items.push(item);
      }

      result.items = items;

    } else {
      result.error = foundErrors;
    }

    return result;
  },

  /**
   * Trading API GetMyMessages
   * @param callback
   * @returns request object
   */
  getMyMessages : function(callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API GetMyMessages call when " +
                   "no account is active.", Logger.DUMP_STACK);
      return;
    }
    let innerBody =
      "<OutputSelector>" +
        "Messages.Message.Read" +
        "</OutputSelector>" +
        "<DetailLevel>" +
        "ReturnHeaders" +
        "</DetailLevel>";

    // do the call
    let wrappedBody = this._wrapCall("GetMyMessages", innerBody);
    let localCallback =
      let(that = this) function(response) {
        let result = that._parseGetMyMessagesResponse(response);
        try {
          if (callback) {
            callback(result);
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Processes a GetMyMessages response
   * @param response
   * @returns number of unread messages
   */
  _parseGetMyMessagesResponse : function(response) {
    // Workaround for E4X bug in Firefox
    // Also get rid of that annoying namespace
    response = response.replace(/<\?xml .*?>/, "").
                        replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(response,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);
    xmlObject = xmlObject["GetMyMessagesResponse"];

    let foundErrors = this._detectAndReportErrors(xmlObject);
    let result = {};

    if (!foundErrors) {
      let messages = xmlObject["Messages"] && xmlObject["Messages"]["Message"] ? xmlObject["Messages"]["Message"] : [];
      let alerts = xmlObject["Alerts"] ? xmlObject["Alerts"]["Alert"] : [];
      let numUnread = 0;

      // XXX: this kind of sintax doesn't work on FF3.5
      // let numUnread = xmlTree..Message.(Read == "false").length();
      // so we have to iterate the array manually to guarantee the right values
      for (var i = 0; i < messages.length; i++) {
        if(messages[i]["Read"] == false) {
          numUnread++;
        }
      }
      // count alerts as messages too
      for (var i = 0; alerts && i < alerts.length; i++) {
        if(alerts[i]["Read"] == false) {
          numUnread++;
        }
      }
      result.numUnreadMessages = numUnread;
    } else {
      result.error = foundErrors;
    }


    return result;
  },

  /**
   * Trading API GetClientAlertsAuthToken
   * @param callback Provides token
   * @returns request object
   */
  getClientAlertsAuthToken : function(callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API GetClientAlertsAuthToken " +
                   "call when no account is active.", Logger.DUMP_STACK);
      return;
    }

    // do the call
    let wrappedBody =
      this._wrapCall("GetClientAlertsAuthToken", "");
    let localCallback =
      let(that = this) function(response) {
        // Workaround for E4X bug in Firefox
        // Also get rid of that annoying namespace
        response = response.replace(/<\?xml .*?>/, "").
                            replace(/ xmlns=["'].*?["']/, "");

        var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
          .createInstance(Ci.nsIDOMParser);
        let xmlTree = parser.parseFromString(response,"text/xml");
        var xmlObject = ObjectHelper.getJXONTree(xmlTree);
        xmlObject = xmlObject["GetClientAlertsAuthTokenResponse"];

        let errors = that._detectAndReportErrors(xmlObject);

        let authToken = String(xmlObject["ClientAlertsAuthToken"]);
        try {
          if (callback) {
            let result = {};
            result.token = authToken;
            result.errors = errors;
            callback(result);
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Trading Api SetNotificationPreferences
   * @param callback
   * @returns request object
   */
  setNotificationPreferences : function(callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API SetNotificationPreferences " +
                   "call when no account is active.", Logger.DUMP_STACK);
      return;
    }

    let notifications = [
      "ItemAddedToWatchList",
      "ItemRemovedFromWatchList",
      "ItemMarkedPaid",
      "ItemMarkedShipped",
      "ItemListed",
      "ItemWon",
      "ItemLost",
      "ItemSold",
      "ItemUnsold",
      "OutBid",
      "BidPlaced",
      "BidReceived",
      "EndOfAuction",
      "FixedPriceTransaction",
      "FeedbackLeft",
      "FeedbackReceived",
      "BestOffer",
      "BestOfferPlaced",
      "BestOfferDeclined",
      "CounterOfferReceived"
    ];

    let innerBody =
      "<ApplicationDeliveryPreferences>" +
        "<DeviceType>ClientAlerts</DeviceType>" +
        "<PayloadVersion>657</PayloadVersion>" +
        "</ApplicationDeliveryPreferences>" +
        "<UserDeliveryPreferenceArray>";
    for (let i = 0; i < notifications.length; i++) {
      innerBody +=
        "<NotificationEnable>" +
          "<EventType>" + notifications[i] + "</EventType>" +
          "<EventEnable>Enable</EventEnable>" +
          "</NotificationEnable>";
    }
    innerBody += "</UserDeliveryPreferenceArray>";

    // do the call
    let wrappedBody =
      this._wrapCall("SetNotificationPreferences", innerBody);
    let localCallback =
      let(that = this) function(response) {
        // Workaround for E4X bug in Firefox
        // Also get rid of that annoying namespace
        response = response.replace(/<\?xml .*?>/, "").
                            replace(/ xmlns=["'].*?["']/, "");
        var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
          .createInstance(Ci.nsIDOMParser);
        let xmlTree = parser.parseFromString(response,"text/xml");
        var xmlObject = ObjectHelper.getJXONTree(xmlTree);
        xmlObject = xmlObject["SetNotificationPreferencesResponse"];

        let errors = that._detectAndReportErrors(xmlObject);

        let result = {};
        result.errors = errors;

        try {
          if (callback) callback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Trading API AddToWatchList
   * @param callback
   * @returns request object
   */
  addToWatchList : function(itemId, callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API AddToWatchList " +
                   "call when no account is active.", Logger.DUMP_STACK);
      return;
    }

    let innerBody = "<ItemID>" + itemId + "</ItemID>";

    // do the call
    let wrappedBody =
      this._wrapCall("AddToWatchList", innerBody);
    let localCallback =
      let(that = this) function(response) {
        // Workaround for E4X bug in Firefox
        // Also get rid of that annoying namespace
        response = response.replace(/<\?xml .*?>/, "").
                            replace(/ xmlns=["'].*?["']/, "");
        var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
          .createInstance(Ci.nsIDOMParser);
        let xmlTree = parser.parseFromString(response,"text/xml");
        var xmlObject = ObjectHelper.getJXONTree(xmlTree);
        xmlObject = xmlObject["AddToWatchListResponse"];

        let foundErrors =
          that._detectAndReportErrors(xmlObject);

        try {
          if (callback) {
            let result = {};
            result.error = foundErrors
            callback(result);
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Trading API RemoveFromWatchList
   * @param callback
   * @returns request object
   */
  removeFromWatchList : function(itemId, callback) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to make Trading API RemoveFromWatchList " +
                   "call when no account is active.", Logger.DUMP_STACK);
      return;
    }

    let innerBody = "<ItemID>" + itemId + "</ItemID>";


    // do the call
    let wrappedBody =
      this._wrapCall("RemoveFromWatchList", innerBody);
    let localCallback =
      let(that = this) function(response) {
        // Workaround for E4X bug in Firefox
        // Also get rid of that annoying namespace
        response = response.replace(/<\?xml .*?>/, "").
                            replace(/ xmlns=["'].*?["']/, "");
        var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
          .createInstance(Ci.nsIDOMParser);
        let xmlTree = parser.parseFromString(response,"text/xml");
        var xmlObject = ObjectHelper.getJXONTree(xmlTree);
        xmlObject = xmlObject["RemoveFromWatchListResponse"];

        let foundErrors = that._detectAndReportErrors(xmlObject);

        try {
          if (callback) {
            let result = {};
            result.error = foundErrors
            callback(result);
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let useSandbox = Datasource.activeAccount().get("isSandboxAccount");
    let request = this._doCall(wrappedBody, useSandbox, localCallback);

    return request;
  },

  /**
   * Checks if an item violates any policy and has been removed for this reason
   * @param aXMLItem the xml node with the item information
   */
  isValidXMLItem : function(aXMLItem) {
    //TODO: Check if ItemPolicyViolation is in the root of the item or not
    let itemViolatesPolicy = (aXMLItem["ItemPolicyViolation"] && String(aXMLItem["ItemPolicyViolation"]).length > 0);
    return !itemViolatesPolicy;
  },

  /**
   * Reports any error messages in the API response
   * @param xmlTree the API response
   */
  _detectAndReportErrors : function(xmlTree) {
    let foundErrors = null;

    // Check for errors
    if (xmlTree["Errors"]) {
      foundErrors = {};
      let errors = xmlTree["Errors"];

      if (!errors)
        return foundErrors;
      if (!Array.isArray(errors))
        errors = [errors];

      for (let i = 0; i < errors.length; i++) {
        let errorCode = Number(errors[i]["ErrorCode"]);
        let errorType = String(errors[i]["ErrorClassification"]);
        if (errorType) {
          // we use this flag to know if the failed API calls can be retried
          // (applies only to client alerts login logic)
          if (errorType.indexOf("SystemError") != -1) {
            foundErrors.systemErrors = true;
          }
        }
        switch (errorCode) {

          // Invalid IAF token (upgrade from secretId authentication to new one)
          case 21916984:
            Logger.error("The new authentication mechanism requires you to " +
                       "sign in again.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;

          // suspended account
          case 841:
            Logger.error("The user account has been suspended. Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;
          // Invalid token
          case 931:
            Logger.error("Auth token is invalid. Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;
          // Token hard expired
          case 932:
            Logger.error("Auth token is hard expired. Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;
          // Security token expired
          case 17470:
            Logger.error("Please login again now. Your security token has " +
                         "expired. Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;
          // Token expired
          case 16110:
            Logger.error("The Trading API token has expired. Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;

          // invalid authentication method
          case 16112:
            Logger.error("The authentication method used is invalid. " +
                         "Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;

          // token retrieval window expired
          case 16118:
            Logger.error("The token retrieval window has expired. " +
                         "Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;

          // token does not exist
          case 16119:
            Logger.error("The token does not exist. " +
                         "Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;

          // token does not match headers credentials
          case 17476:
            Logger.error("The token does not match headers credentials. " +
                         "Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;

          // Token has been revoked by App
          case 21916013:
            Logger.error("The token Token has been revoked by App. " +
                         "Forcing logout.");
            Datasource.logoutUser();
            Datasource.showSecuritySignOutWarningNotification();
            break;

          // Invalid item Id (only active items can be added to the watchlist)
          case 21000:
            Logger.error("Tried to add an ended item to the watchlist.",
                         Logger.DUMP_STACK);
            break;

          // Item not in watch list
          case 21002:
            let itemId = errors[i]["ErrorParameters"][0]["Value"];
            Logger.error("The Item is not in the watch list.",
                         Logger.DUMP_STACK);
            break;

          // Items were not removed from watch list
          case 20820:
            // We can safely ignore this
            break;

          // Item not found
          case 1505:
            itemId = errors[i]["ErrorParameters"][0]["Value"];
            Logger.error("Trading API reports that the item was not found.",
                         Logger.DUMP_STACK);
            break;

          // Item already in watch list
          case 21003:
            itemId = errors[i]["ErrorParameters"][0]["Value"];
            Logger.error("Item is already in the watch list.",
                         Logger.DUMP_STACK);
            break;

          // Items were not added to watch list
          case 20819:
            // We can safely ignore this
            break;

          // Catch-all
          default:
            Logger.error("Trading API Error:\n" + "Error Code: " + errorCode +
                         "\nShort Message: " +
                         String(errors[i]["ShortMessage"]) +
                         "\nLong Message: " +
                         String(errors[i]["LongMessage"]) +
                         "\nSeverity: " +
                         String(errors[i]["SeverityCode"]) +
                         "\nError Clasification: " +
                         String(errors[i]["ErrorClassification"]) + "\n",
                         Logger.DUMP_STACK);
            break;
        }
      }
    }

    // Check for imminent token expiration
    if (xmlTree["HardExpirationWarning"]) {
      // Create Zulu-format date string
//      let expirationDateString = String(xmlTree["HardExpirationWarning"]).
//                                   replace(" ", "T") + ".000Z";
      // Convert to date object
      let expirationDate = xmlTree["HardExpirationWarning"];
      Observers.notify(expirationDate, Topics.API_TRADING_TOKEN, null);
    }

    return foundErrors;
  },

  /**
   * Wraps an XML fragment into a Trading API call
   * @param callName The name of the call (without "Request")
   * @param body The body of the call
   * @returns the fully-formed text
   */
  _wrapCall : function(callName, innerBody) {
    if (!Datasource.activeAccount()) {
      Logger.error("Attempt to wrap Trading API call without active account.",
                   Logger.DUMP_STACK);
      return "";
    }

    let xmlHeader = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
    let token = Datasource.activeAccount().get("token");
    let credentials = "<RequesterCredentials><eBayAuthToken>" +
                      token +
                      "</eBayAuthToken></RequesterCredentials>";
    let xmlns = "xmlns=\"urn:ebay:apis:eBLBaseComponents\"";

    let requestIdentifier = Datasource.getRequestIdentifier();
    let messageID = "<MessageID>" + requestIdentifier + "</MessageID>";

    let wrappedBody = xmlHeader +
                      "<" + callName + "Request " + xmlns + ">" +
                      messageID + credentials + innerBody +
                      "</" + callName + "Request>";
    /*let wrappedBody = xmlHeader +
                      "<" + callName + "Request " + xmlns + ">" +
                      messageID + innerBody +
                      "</" + callName + "Request>";*/


    return wrappedBody;
  },

  /**
   * Performs Trading API call
   * @param body The full body of the call, as will be POSTed
   * @param useSandbox boolean
   * @param callback The callback function
   * @returns request object
   */
  _doCall : function(body, useSandbox, callback) {
    // Extract the call name from the body --- we need it in the headers
    let requestName;
    try {
      requestName = /><(.*?)Request/.exec(body)[1];
    }
    catch (e) {
      Logger.error("Trading API request will not be sent, as it is " +
                   "badly-formed.", Logger.DUMP_STACK);
      return;
    }
    let callDescription = "Trading API " + requestName;

    let apiCallback =
      ApiHelper.generateApiCallback(callDescription, callback);

/*    if (!ApiHelper.accessService) {
      // The binary component is missing.
      return;
    }*/
    let siteId = Constants.siteIdForSite(Datasource.homeSite());
    //ApiHelper.accessService.setSiteId(siteId);
    ApiAccessService.siteId = siteId;
    //ApiHelper.accessService.setApiVersion(627);
    ApiAccessService.apiVersion = 693;
    let request =
      ApiAccessService.doTradingAPICall(
        requestName, body, useSandbox, apiCallback);

    ApiHelper.addPendingRequest(request, callDescription);

    let requestLog = {};
    requestLog.requestName = requestName;
    requestLog.properties = body;
    requestLog.api = "Trading";

    // notify the observer to log the request
    Observers.notify(requestLog, Topics.API_TRADING_CALL, null);

    return request;
  }
};
