/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["Datasource"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/logger.js");

var Datasource = {

  /* Analytics consecutive number */
  _analyticsCount : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      Cu.import("resource://ebaycompanion/common.js");
      Cu.import("resource://ebaycompanion/eventAnalytics.js");
      Cu.import("resource://ebaycompanion/helpers/observers.js");
      Cu.import("resource://ebaycompanion/helpers/timer.js");
      Cu.import("resource://ebaycompanion/helpers/httpRequestHeaderVisitor.js");
      Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
      Cu.import("resource://ebaycompanion/constants.js");

      _analyticsCount = 0;

      // import debugger if debugging is enabled
      if (Constants.prefBranch.get("debugging.enabled")) {
        Cu.import("resource://ebaycompanion/debug/debugger.js");
      }

      Observers.add(this, Topics.QUIT_APPLICATION_GRANTED);
      Observers.add(this, Topics.ACCOUNT_PROPERTY_UPDATED);
      Observers.add(this, Topics.ITEM_PROPERTY_UPDATED);
      Observers.add(this, Topics.TRANSACTION_PROPERTY_UPDATED);
      Observers.add(this, Topics.SYSTEM_MESSAGE_PROPERTY_UPDATED);
      // XXX: this is the necessary observer to catch every http request made
      // from the browser. We will use this to extract the username used
      // when the user signs into the extension.
      Observers.add(this, Topics.HTTP_ON_MODIFY_REQUEST);

      let that = this;
      // Periodically flush dirty objects to storage
      new Timer(function() { that._flushDirtyObjects(); },
                60 * 1000, Timer.TYPE_REPEATING_PRECISE);

      new Timer(function() {
          // make the import until we actually need it
          if (typeof(ObjectsStorage) == "undefined") {
            Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
          }
          Cu.import("resource://ebaycompanion/services/recentSearchService.js");
          Cu.import("resource://ebaycompanion/services/recentBidService.js");
          Cu.import("resource://ebaycompanion/services/recentViewService.js");
          Cu.import("resource://ebaycompanion/services/recentWatchService.js");
          Cu.import("resource://ebaycompanion/services/recentAlertService.js");
          // Get active account, if one is stored
          that._activeAccount = ObjectsStorage.activeAccount();

          // Members at least need to be sane
          that._items = {};
          that._transactions = {};
          that._favoriteSellers = [];
          that._favoriteSearches = [];
          that._dailyDeals = [];
          that._clientRoverIds = ObjectsStorage.getRoverIds();
          that._clientInstallTrackingCodes =
            ObjectsStorage.getInstallTrackingCodes();

          let currentVersion = Constants.prefBranch.get("version");
          if (currentVersion && currentVersion.match(/^1\.(.)*$/g) &&
              that._activeAccount) {
            // log out any active account to display the T & C change notification
            that.logoutUser();
          }

          // tries to connect the user automatically if the conditions are met
          // (there is an active account and the connectAutomatically flag is true)
          that.tryAutomaticConnect();

          // get the number of tracked url records currently stored locally
          that._trackedUrlsCount = ObjectsStorage.getTrackedUrlsCount();
        }, 10, Timer.TYPE_ONE_SHOT);
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Uninitializes the resource.
   */
  _uninit : function() {
    this._flushDirtyObjects();
    Observers.removeAll(this);
  },

  /**
   * returns the analytics count
   */
  getAnalyticsCount : function() {
    this._analyticsCount++;
    return this._analyticsCount;
  },

  /**
   * Extracts the username used when the user submits the sidebar sign in form
   * @param aHTTPRequest
   */
  _extractSignInUsername : function(aHTTPRequest) {
    // most of this code is adapted from TamperData add-on
    aHTTPRequest.QueryInterface(Ci.nsIHttpChannel);
    let headerArray = new Array();
    let url = aHTTPRequest.URI.asciiSpec;
    /* Regular expression used to match the login URL. */
    const AUTH_AUTH_LOGIN_URL_REGEXP =
      /^https:\/\/signin(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll\?/i;

    let result = url.match(AUTH_AUTH_LOGIN_URL_REGEXP);
    if (result != null && !this.activeAccount()) {

      let visitor = new HttpRequestHeaderVisitor(aHTTPRequest);
      // this is necessary to extract all the information from the
      // request
      let requestHeaders = visitor.inspectRequest();
      // here we iterate the request headers if necessary
      /*for each (let [name, value] in Iterator(requestHeaders)) {
      }*/
      let postData = visitor.getPostData();
      if (postData) {
        // separate all the pairs name=value
        let parameters = postData.split("&");
        let postParamsArray = new Array();
        for(var i = 0; i < parameters.length; i++) {
          let param = parameters[i].split("=");
          postParamsArray[param[0]] = param[1];
        }
        // set the lastestUsername
        // this code is to be used when testing with the sandbox
        let username = postParamsArray["userid"];
        //this._latestUsername = username?username:this._latestUsername;
        this._latestUsername = username;
        //Logger.log(
              //"Latest username used to sign in: " + this._latestUsername);
      }
    }
  },

  /**
   * Returns an Auth & Auth URL for login
   * @param sourceAreaName used for rover tracking
   * @param useSandbox boolean
   * @param generateUrlCallback callback method to be called once the url is
   * generated
   */
  generateLoginUrl : function(sourceAreaName, useSandbox, generateUrlCallback) {
    let that = this;

    let runame = Constants.getRuname(useSandbox, this.homeSite());

    // import until we actually need the component
    if (typeof(TradingApi) == "undefined") {
      Cu.import("resource://ebaycompanion/apis/tradingApi.js");
    }

    let getSessionIdCallback = function(result) {
      try {
        if (!result.sessionId || result.errors) {
          Logger.error("Unable to retrieve a SessionID. Login aborted.");
          // create sidebar notification
          // we import these classes here because there is no real need to
          // load them at init
          Cu.import(
            "resource://ebaycompanion/helpers/warningNotificationHelper.js");
          Cu.import("resource://ebaycompanion/objects/notification.js");
          let notification =
            new Notification(
              WarningNotificationHelper.LOGIN_FAILED_WARNING);
          notification.set(
            "content", Constants.stringBundle.getString(
              "notification.error.connection"));
          notification.set("priority", 1);
          WarningNotificationHelper.queueNotification(notification);
          // notify the observer to change the sidebar status and display the
          // button
          Observers.notify(null, Topics.SIGN_IN_STOP, null);
          return;
        } else {
          that._loginSessionId = result.sessionId;

          let params = "isSandboxAccount=" + useSandbox;
          let encodedParams = encodeURIComponent(params);

          // Parameters required by the URL template
          let templateArgs = {
            runame :   runame,
            sessid :   that._loginSessionId,
            ruparams : encodedParams
          };

          let url;
          if (!useSandbox) {
            // Production
            url = Constants.getUrl(sourceAreaName, "authAuth", templateArgs);
          } else {
            // Sandbox
            url =
              Constants.getUrl(sourceAreaName, "authAuthSandbox", templateArgs);
          }

          // now that the url has been generated, call the callback method
          generateUrlCallback(url);
          // notify the observer to change the sidebar status and display the
          // button
          Observers.notify(null, Topics.SIGN_IN_STOP, null);
        }
      } catch(e) {
        Logger.exception(e, Logger.DUMP_STACK)
        // notify the observer to change the sidebar status and display the
        // button
        Observers.notify(null, Topics.SIGN_IN_STOP, null);
      }

    };

    TradingApi.getSessionID(useSandbox, getSessionIdCallback);
    // notify the observer to change the sidebar status and display the
    // throbber
    Observers.notify(null, Topics.SIGN_IN_START, null);

  },

  /**
   * Attempts to connect the user automatically if there is an active account
   * and the connectAutomatically flag is set
   */
  tryAutomaticConnect : function() {
    // If there is an active account, get saved items from storage and start
    // the update process
    let connectAutomatically =
      Constants.prefBranch.get("connectAutomatically");
    let activeAccount = this._activeAccount;
    if (activeAccount && connectAutomatically) {
      EventAnalytics.push(
        ['_trackEvent', '101-startup-status-signed-in', 'startup']);
      // import until we actually need the component
      if (typeof(TradingApi) == "undefined") {
        Cu.import("resource://ebaycompanion/apis/tradingApi.js");
      }

      // notify the observer to change the sidebar status and display the
      // throbber
      Observers.notify(null, Topics.SIGN_IN_START, null);

      this._loadDataFromStorage();

      // make the import until we actually need it
      if (typeof(EventTracker) == "undefined") {
        Cu.import("resource://ebaycompanion/helpers/eventTracker.js");
      }

      let eventTracker = new EventTracker("Installation and upgrade report");
      let extensionActivated = eventTracker.addCallbackEvent();
      eventTracker.doWhenAllFinished();

      let isSandboxAccount = activeAccount.get("isSandboxAccount");

      let reportInstallCallback =
        let (that = this) function(result) {
          try {
            if (result.error) {
              Logger.warning(
                "Extension installation/activation report reported errors");
              if (result.existingToolbarId) {
                Logger.warning(
                  "There is already a toolbarId stored locally");
                Constants.prefBranch.set("firstSignIn", false);
              } else {
                // Inform event tracker
                extensionActivated();
              }
            }
            if (result.toolbarId) {
              Logger.log(
                "Installation/Activation reported through API. Returned " +
                "toolbarId: " + result.toolbarId);
              if (typeof(PropertiesStorage) == "undefined") {
                Cu.import(
                  "resource://ebaycompanion/storage/propertiesStorage.js");
              }
              PropertiesStorage.set("ToolbarId", result.toolbarId);
              Constants.prefBranch.set("firstSignIn", false);

              // the report upgrade can be made independently, we don't have to
              // wait for it to complete
              let reportUpgrade = Constants.prefBranch.get("reportUpgrade");
              if (reportUpgrade) {
                let ebayAuthToken = that._activeAccount.get("token");
                let credentialsArray = { eBayAuthToken: ebayAuthToken };

                TradingApi.reportToolbarActivity(
                  "Upgrade", credentialsArray,
                  isSandboxAccount, reportUpgradeCallback);
              }
              // Inform event tracker
              extensionActivated();
            }
          }
          catch (e) {
            Logger.exception(e, Logger.DUMP_STACK)
          }
        }

      let reportUpgradeCallback =
        function(result) {
          try {
            if (result.error) {
              Logger.warning("Extension upgrade report reported errors");
            } else {
              Logger.log("Upgrade reported successfully");
            }
            // set the preference to false so we don't send the upgrade request
            // everytime someone signs in
            Constants.prefBranch.set("reportUpgrade", false);
          }
          catch (e) {
            Logger.exception(e, Logger.DUMP_STACK)
          }
        }

      // Use timer to start updates after all other initialisation
      new Timer(let (that = this) function() {
        // send the activation request if this is the first time ever signing
        // in
        let firstSignIn = Constants.prefBranch.get("firstSignIn");

        // change the sidebar to the signed in state and enable the updates
        that._loggedIn(firstSignIn, true);

        if (firstSignIn) {
          let ebayAuthToken = activeAccount.get("token");
          let credentialsArray = { eBayAuthToken: ebayAuthToken };

          let request = TradingApi.reportToolbarActivity(
            "Install", credentialsArray,
            isSandboxAccount, reportInstallCallback);
          // send first sign in tracking request
          that.sendTrackingRequest("activate");
          // the call failed for some reason
          if (!request) {
            // Inform event tracker
            extensionActivated();
          }
        } else {
          // if this is not the first sign in, check if this is an upgrade
          // and send the upgrade request
          // the report upgrade can be made independently, we don't have to
          // wait for it to complete
          let reportUpgrade = Constants.prefBranch.get("reportUpgrade");
          if (reportUpgrade) {
            let ebayAuthToken = that._activeAccount.get("token");
            let credentialsArray = { eBayAuthToken: ebayAuthToken };

            TradingApi.reportToolbarActivity(
              "Upgrade", credentialsArray,
              isSandboxAccount, reportUpgradeCallback);
          }

          // Inform event tracker
          extensionActivated();
        }

        }, 0);
    } else if (!connectAutomatically) {
      EventAnalytics.push(
        ['_trackEvent', '102-startup-status-signed-out', 'startup']);
      // do as if the user has signed out
      this._activeAccount = null;
      this._latestUsername = null;
      ObjectsStorage.setActiveAccount(null);
    }
  },

  /**
   * Called when user has passed Auth & Auth -- fetches token and starts to
   * fetch data
   * @param username eBay UserId
   * @param isSandboxAccount boolean
   * @param callback function to call when done
   * @return false if the user is using an old sign in page (from a previous
   * browsing session), true otherwise
   */
  loginUser : function(username, isSandboxAccount, callback) {
    if (!this._loginSessionId) {
      Logger.error("Attempt to login when no Session ID was generated!",
                   Logger.DUMP_STACK);
      return false;
    }

    // import until we actually need the component
    if (typeof(TradingApi) == "undefined") {
      Cu.import("resource://ebaycompanion/apis/tradingApi.js");
    }

    // make the import until we actually need it
    if (typeof(EventTracker) == "undefined") {
      Cu.import("resource://ebaycompanion/helpers/eventTracker.js");
    }

    let eventTracker = new EventTracker("User Login");
    let gotToken = eventTracker.addCallbackEvent();
    let updatedAccount = eventTracker.addCallbackEvent();
    let extensionActivated = eventTracker.addCallbackEvent();
    eventTracker.doWhenAllFinished(callback);

    let tokenCallback =
      let (that = this) function(result) {
        try {
          let token = result.token;
          if (token == "" || result.errors) {
            Logger.error("FetchToken returned an empty token; login aborted.");
            // create sidebar notification
            // we import these classes here because there is no real need to
            // load them at init
            Cu.import(
              "resource://ebaycompanion/helpers/warningNotificationHelper.js");
            Cu.import("resource://ebaycompanion/objects/notification.js");
            let notification =
              new Notification(
                WarningNotificationHelper.LOGIN_FAILED_WARNING);
            notification.set(
              "content", Constants.stringBundle.getString(
                "notification.error.connection"));
            notification.set("priority", 1);
            WarningNotificationHelper.queueNotification(notification);
            // notify the observer to change the sidebar status and display the
            // button
            Observers.notify(null, Topics.SIGN_IN_STOP, null);
            return;
          }
          // Log out if there is an active account
          if (that._activeAccount) {
            that.logoutUser();
          }

          // Try to load account from storage
          let storedAccount =
            ObjectsStorage.getAccount(username, isSandboxAccount);

          // If we have a local copy of the account data, we don't need to do an
          // account update now and can rely on the hard update that immediately
          // follows login
          if (storedAccount) {
            that._activeAccount = storedAccount;
            that._activeAccount.set("token", token);
            // allow feedback score notifications be displayed
            Constants.prefBranch.set(
              "prevent.first.feedback.score.notification", false);
            // Shortcut to account update callback
            accountUpdateCallback(0);
          } else {
            // The account is not stored locally, so we must retrive it from
            // the server and store it locally before we continue the login
            // process.

            // this is the first time we actually need the account class, so
            // we import it
            Cu.import("resource://ebaycompanion/objects/account.js");
            that._activeAccount = new Account(username, isSandboxAccount);
            that._activeAccount.set("token", token);
            that._activeAccount.set("registrationSite",   // temporary value
                                    Constants.prefBranch.get("chosenSite"));
            that._activeAccount.set("feedbackRating", 0);
            that._activeAccount.set("numUnreadMessages", 0);
            // set the preference to prevent first display of feedback score
            // change notification when the user is signing in for the first
            // time
            Constants.prefBranch.set(
              "prevent.first.feedback.score.notification", true);
            // import until we actually need the component
            if (typeof(ApiCoordinator) == "undefined") {
              Cu.import("resource://ebaycompanion/apiCoordinator.js");
            }
            ApiCoordinator.accountUpdate(accountUpdateCallback);
          }

          // Inform event tracker
          gotToken();
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
    let accountUpdateCallback =
      let (that = this) function(numErrors) {
        try {
          // We write the account data as it's now up-to-date.
          ObjectsStorage.store(that._activeAccount);
          ObjectsStorage.setActiveAccount(that._activeAccount);
          that._loadDataFromStorage();
          that._loggedIn(false, false);
          // send the activation request if this is the first time ever signing
          // in
          let firstSignIn = Constants.prefBranch.get("firstSignIn");
          if (firstSignIn) {
            let ebayAuthToken = that._activeAccount.get("token");
            let credentialsArray = { eBayAuthToken: ebayAuthToken };

            let request = TradingApi.reportToolbarActivity(
              "Install", credentialsArray,
              isSandboxAccount, reportInstallCallback);
            // send first sign in tracking request
            that.sendTrackingRequest("activate");
            // the call failed for some reason
            if (!request) {
              // Inform event tracker
              extensionActivated();
            }
          } else {
            // if this is not the first sign in, check if this is an upgrade
            // and send the upgrade request
            // the report upgrade can be made independently, we don't have to
            // wait for it to complete
            let reportUpgrade = Constants.prefBranch.get("reportUpgrade");
            if (reportUpgrade) {
              let ebayAuthToken = that._activeAccount.get("token");
              let credentialsArray = { eBayAuthToken: ebayAuthToken };

              TradingApi.reportToolbarActivity(
                "Upgrade", credentialsArray,
                isSandboxAccount, reportUpgradeCallback);
            }

            // Inform event tracker
            extensionActivated();
          }

          // Inform event tracker
          updatedAccount();
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let reportInstallCallback =
      let (that = this) function(result) {
        try {
          if (result.error) {
            Logger.warning(
              "Extension installation/activation report reported errors");
            if (result.existingToolbarId) {
              Logger.warning(
                "There is already a toolbarId stored locally");
              Constants.prefBranch.set("firstSignIn", false);
            } else {
              // Inform event tracker
              extensionActivated();
            }
          }
          if (result.toolbarId) {
            Logger.log(
              "Installation/Activation reported through API. Returned " +
              "toolbarId: " + result.toolbarId);
            if (typeof(PropertiesStorage) == "undefined") {
              Cu.import(
                "resource://ebaycompanion/storage/propertiesStorage.js");
            }
            PropertiesStorage.set("ToolbarId", result.toolbarId);
            Constants.prefBranch.set("firstSignIn", false);

            // the report upgrade can be made independently, we don't have to
            // wait for it to complete
            let reportUpgrade = Constants.prefBranch.get("reportUpgrade");
            if (reportUpgrade) {
              let ebayAuthToken = that._activeAccount.get("token");
              let credentialsArray = { eBayAuthToken: ebayAuthToken };

              TradingApi.reportToolbarActivity(
                "Upgrade", credentialsArray,
                isSandboxAccount, reportUpgradeCallback);
            }
            // Inform event tracker
            extensionActivated();
          }
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    let reportUpgradeCallback =
      let (that = this) function(result) {
        try {
          if (result.error) {
            Logger.warning("Extension upgrade report reported errors");
          } else {
            Logger.log("Upgrade reported successfully");
          }
          // set the preference to false so we don't send the upgrade request
          // everytime someone signs in
          Constants.prefBranch.set("reportUpgrade", false);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }

    TradingApi.fetchToken(isSandboxAccount, this._loginSessionId,
                          tokenCallback);
    return true;
  },

  /**
   * Called once an account is logged in
   * @param aDisableNotifications whether or not to disable the feedback score
   * notifications. This is optional and used when the user upgrades from 1.6
   * to prevent the automatic login from displaying the feedback score change
   * notifications.
   * @param aAutomaticLogin whether this is an automatic login or not. Helps
   * knowing whether we should call SetNotificationPreferences during Client
   * Alerts login process or not.
   */
  _loggedIn : function(aDisableNotifications, aAutomaticLogin) {
    try {
      let info = {};
      info.object = this._activeAccount;
      Observers.notify(info, Topics.ACCOUNT_LOGGED_IN, null);

      // import until we actually need the component
      if (typeof(AlertsGenerator) == "undefined") {
        Cu.import("resource://ebaycompanion/services/alertsGenerator.js");
      }
      // let the sign in update notification take care of enabling the alerts
      // generator
      //AlertsGenerator.enable();

      if (typeof(ApiCoordinator) == "undefined") {
        Cu.import("resource://ebaycompanion/apiCoordinator.js");
      }
      ApiCoordinator.enableUpdates(aAutomaticLogin);      // Start updating
      if (!aDisableNotifications) {
        // now we allow feedback score notifications be displayed
        Constants.prefBranch.set(
          "prevent.first.feedback.score.notification", false);
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Invalidates the active account.
   */
  logoutUser : function() {
    try {
      let info = {};
      if (!this._activeAccount) {
        Logger.warning("Attempt to log out when there is no active account.");
      } else {
        ApiCoordinator.disableUpdates(); // Stop any further requests
        AlertsGenerator.disable();
        info.object = this._activeAccount.copy();
      }

      this._flushDirtyObjects();
      delete this._activeAccount;
      this._activeAccount = null;
      this._latestUsername = null;
      this._favoriteSearches = [];
      ObjectsStorage.setActiveAccount(null);
      Observers.notify(info, Topics.ACCOUNT_LOGGED_OUT, null);
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Helps displaying a warning notification when there are invalid token
   * API errors
   */
  showSecuritySignOutWarningNotification : function() {
    if (typeof(WarningNotificationHelper) == "undefined") {
      Cu.import(
        "resource://ebaycompanion/helpers/warningNotificationHelper.js");
    }
    if (typeof(Notification) == "undefined") {
      Cu.import("resource://ebaycompanion/objects/notification.js");
    }
    let notification =
      new Notification(
        WarningNotificationHelper.SECURITY_SIGN_OUT_WARNING);
    notification.set(
      "content", Constants.stringBundle.getString(
        "notification.warning.signedOut"));
    notification.set("priority", 1);
    WarningNotificationHelper.queueNotification(notification);
  },

  /**
   * Sends a (silent) tracking request using client-specific rover ids when
   * available, or default ones otherwise.
   * @param aType install or activate values to send installation or
   * first sign in tracking request.
   */
  sendTrackingRequest : function(aType) {
    // Generate unique ID to prevent caching
    let time = Date.now();
    let random = String(Math.random()).split(".")[1];
    let uniqueId = time + random;
    let homeSite = this.homeSite();

    // a client specific installation tracking pixel is requested based on
    // the home site specified by the user
    let trackTag = this.getClientTrackingCode(homeSite, false);
    let usingClientSpecificTag = false;

    // there is no client-specific track tag, so we use the default one for
    // the current homesite
    let language = "EN-US";
    if (!trackTag) {
      for (locale in Constants.localeSiteDataSelector) {
        if (Constants.localeSiteDataSelector[locale] == homeSite) {
          language = locale;
          break;
        }
      }
      trackTag = Constants.installTrackTags[language];
    } else {
      usingClientSpecificTag = true;
    }

    let args = {
      tracktag : trackTag,
      mfe   : aType,
      uniqueid : uniqueId
    };

    let request =
      Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
        createInstance(Ci.nsIXMLHttpRequest);
    let url = Constants.getUrl("installTracker", "installTrack", args);

    request.open("GET", url, true);
    request.addEventListener(
      "load",
      function() {
        let sanDiskBuild = Constants.prefBranch.get("SanDiskBuild");
        let message = "";
        if (sanDiskBuild && usingClientSpecificTag) {
          message += "San Disk-Build ";
        } else if (usingClientSpecificTag) {
          message += "Client-Specific ";
        }
        message += aType + " Tracking Pixel Loaded: " + url;
        Logger.log(message);
      },
      false);
    request.addEventListener(
      "error",
      function() {
        Logger.error("error");
        let message = "Error loading ";
        message += usingClientSpecificTag ? "client-specific " : "";
        message +=  aType + " tracking pixel: " + url;
        Logger.error(message);
      },
      false);
    request.send(null);
  },

  /**
   * Returns the ID of the eBay site that is used for user interaction
   */
  homeSite : function() {
    let siteId;
    if (Constants.prefBranch.get("useRegistrationSite") &&
        this._activeAccount)
    {
      siteId = this._activeAccount.get("registrationSite");
      let supportedSites = Constants.supportedSites();
      // XXX: on 11/30/2011 we found out some users might have eBayMotors
      // as their registration site, so we should default to US in that case
      // to prevent a chain of major issues
      let supported = false;
      for (let i = 0; i < supportedSites.length; i++) {
        if (supportedSites[i] == siteId) {
          supported = true;
          break;
        }
      }
      if (!siteId || !supported) {
        siteId = "US";
      }
    } else {
      siteId = Constants.prefBranch.get("chosenSite");
    }
    return siteId;
  },

  /**
   * Returns the currently active account
   */
  activeAccount : function() {
    return this._activeAccount;
  },

  /**
   * Returns the toolbarID
   */
  get toolbarID() {
    if (typeof(PropertiesStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/propertiesStorage.js");
    }

    return PropertiesStorage.get("ToolbarId");
  },

  /**
   * Returns the number of tracked urls
   */
  get trackedUrlsCount() {
    return this._trackedUrlsCount;
  },

  /**
   * Sets the number of tracked urls
   * @param aValue the value to be set
   */
  set trackedUrlsCount(aValue) {
    this._trackedUrlsCount = aValue;
  },

  /**
   * Returns the client specific tracking code for the given country
   * @param aCountry the country to get its tracking code or null if not
   * found
   * @param aIsRover whether to search in the client rover ids or installation
   * tracking codes
   */
  getClientTrackingCode : function(aCountry, aIsRover) {
    // in some cases the rover ids array hasn't been initialized just yet, so
    // we force its initialization
    if (aIsRover && !this._clientRoverIds) {
      // make the import until we actually need it
      if (typeof(ObjectsStorage) == "undefined") {
        Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
      }
      this._clientRoverIds = ObjectsStorage.getRoverIds();
    }
    let trackingCodesArray =
      aIsRover ? this._clientRoverIds : this._clientInstallTrackingCodes;
    let trackingCodeValue = null;
    if (trackingCodesArray) {
      for (var i = 0; i < trackingCodesArray.length; i++) {
        let trackingCode = trackingCodesArray[i];
        if (trackingCode.get("country") == aCountry) {
          trackingCodeValue = trackingCode.get("value");
          break;
        }
      }
    }
    return trackingCodeValue;
  },

  /**
   * Sets the active account. ONLY Used when the user upgrades from 1.6
   * @param aAccount the account to be set as active
   */
  setActiveAccount : function(aAccount) {
    this._activeAccount = aAccount;
    ObjectsStorage.setActiveAccount(aAccount);
    // store the account locally since we are sure it is not there
    ObjectsStorage.store(aAccount);
  },

  /**
   * Returns all tracked item objects in a hash list object, hashed by itemId
   */
  items : function() {
    return this._items;
  },

  getItem : function(aItemId) {
    let item = null;

    if (this._items[aItemId]) {
      item = this._items[aItemId];
    }

    return item;
  },

  /**
   * Returns all tracked favorite search objects
   */
  favoriteSearches : function() {
    return this._favoriteSearches;
  },

  /**
   * Returns latest username used to sign in to the extension
   */
  latestUsername : function() {
    return this._latestUsername;
  },

  /**
   * Adds an item to the local store, or updates it to match the given item.
   * Notifications are generated to inform listeners that the item has been
   * added or changed.
   * @param item Item object
   */
  addOrUpdateItem : function(item) {
    let itemId = item.get("itemId");
    let dsItem = this._items[itemId];

    if (dsItem) {
      let originalObject = dsItem.copy();
      // XXX: this code prevents the page views of an item to be reset to 0 due
      // to trading API alternating between the real value and -1 for unknown
      // reasons
      if (originalObject.get("pageViews") > 0 && item.get("pageViews") == 0) {
        item.set("pageViews", originalObject.get("pageViews"));
      }
      let objectChanged = dsItem.updateTo(item);
      if (objectChanged) {
        let info = {};
        info.object = dsItem;
        info.originalObject = originalObject;
        Observers.notify(info, Topics.ITEM_CHANGED, null);
      }
    } else {
      this._items[itemId] = item;
      // instead of inserting it immediately, we just mark it as dirty and let
      // the flusher to take care of it later
      //ObjectsStorage.addItemToAccount(item, this._activeAccount);
      this._markObjectDirty(item);

      let info = {}
      info.object = item;
      Observers.notify(info, Topics.ITEM_NEW, null);
    }
  },

  /**
   * Removes an item from the local store
   * @param item Item object
   */
  removeItem : function(item) {
    let itemId = item.get("itemId");
    if (!itemId) {
      Logger.warning("Empty object passed to removeItem.", Logger.DUMP_STACK);
    }

    // Remove any transactions for the item
    let transactions = this._transactions[itemId];
    if (transactions) {
      for each (let [transactionId, transaction] in Iterator(transactions)) {
        let info = {};
        info.object = transaction;
        Observers.notify(info, Topics.TRANSACTION_REMOVED, null);
      }
      delete this._transactions[itemId];
    }

    let info = {};
    info.object = this._items[itemId];
    Observers.notify(info, Topics.ITEM_REMOVED, null);

    ObjectsStorage.
      removeItemFromAccount(this._items[itemId], this._activeAccount);
    delete this._items[itemId];
  },

  /**
   * Returns all the transactions currently in memory
   */
  getAllTransactions : function() {
    return this._transactions;
  },

  /**
   * Returns all tracked transaction objects for the given itemId in a hash list
   * object, hashed by transaction Id
   */
  transactions : function(itemId) {
    return this._transactions[itemId];
  },

  /**
   * Adds a transactions to the local store
   * @param transaction Transaction object
   */
  addOrUpdateTransaction : function(transaction) {
    let itemId = transaction.get("itemId");
    let transactionId = transaction.get("transactionId");
    let transactions = this._transactions[itemId];

    if (transactions && transactions[transactionId]) {
      let dsTransaction = transactions[transactionId];
      let originalObject = dsTransaction.copy();
      let objectChanged = dsTransaction.updateTo(transaction);
      if (objectChanged) {
        let info = {};
        info.object = dsTransaction;
        info.originalObject = originalObject;
        Observers.notify(info, Topics.TRANSACTION_CHANGED, null);
      }
    } else {
      if (!transactions) {
        this._transactions[itemId] = [];
        transactions = this._transactions[itemId];
      }
      transactions[transactionId] = transaction;
      // instead of inserting it immediately, we just mark it as dirty and let
      // the flusher to take care of it later
      //ObjectsStorage.store(transaction);
      this._markObjectDirty(transaction);

      let info = {};
      info.object = transaction;
      Observers.notify(info, Topics.TRANSACTION_NEW, null);
    }
  },

  /**
   * Removes an transaction from the local store
   * @param aTransaction Transaction object
   */
  removeTransaction : function(aTransaction) {

    let itemId = aTransaction.get("itemId");
    let transactions = this._transactions[itemId];
    if (transactions) {
      let newArray = [];
      let dsItem = this._items[itemId];
      let transRemoved = false;
      for each (let [transactionId, transaction] in Iterator(transactions)) {
        if (transactionId == aTransaction.get("transactionId")) {
          ObjectsStorage.removeTransaction(aTransaction);
          dsItem.transaction = aTransaction;
          transRemoved = true;
        } else {
          newArray[transactionId] = transaction;
        }
      }
      // set the transactions array to the purged one. It will remain as
      // undefined unless the new array actually contains transactions
      delete this._transactions[itemId];
      for each (let [transactionId, transaction] in Iterator(newArray)) {
        if (!this._transactions[itemId]) {
          this._transactions[itemId] = [];
        }
        this._transactions[itemId][transactionId] = transaction;
      }

      // remove it from the dirty list so the record is not inserted back to
      // the database
      this._removeFromDirtyObjects(aTransaction);

      // ok, now notify the lists so they also remove the transaction item
      if (transRemoved) {
        let info = {};
        info.object = dsItem;
        Observers.notify(info, Topics.TRANSACTION_ITEM_REMOVED, null);
      }
    }
  },

  /**
   * Returns all the user's favorite sellers
   */
  favoriteSellers : function() {
    return this._favoriteSellers;
  },

  /**
   * Adds an favorite seller to the local store.
   * @param aFavoriteSeller FavoriteSeller object
   */
  addFavoriteSeller : function(aFavoriteSeller) {
    this._favoriteSellers[this._favoriteSellers.length] = aFavoriteSeller;
    this._markObjectDirty(aFavoriteSeller);
    //ObjectsStorage.store(aFavoriteSeller);
  },

  /**
   * Deletes all the records in the favorite sellers table and clears the array
   * kept in memory, for the current account
   */
  clearAccountFavoriteSellers : function() {
    this._favoriteSellers = [];
    ObjectsStorage.removeFavoriteSellersFromAccount(this._activeAccount);
  },

  /**
   * Adds an favorite search to the local store.
   * @param aFavoriteSearch FavoriteSearch object
   */
  addFavoriteSearch : function(aFavoriteSearch) {
    this._favoriteSearches[this._favoriteSearches.length] = aFavoriteSearch;
    this._markObjectDirty(aFavoriteSearch);
    //ObjectsStorage.store(aFavoriteSearch);
  },

  /**
   * Deletes all the records in the favorite searches table and clears the array
   * kept in memory, for the current account
   */
  clearAccountFavoriteSearches : function() {
    this._favoriteSearches = [];
    ObjectsStorage.removeFavoriteSearchesFromAccount(this._activeAccount);
  },

  /**
   * Returns the daily deals array
   */
  get dailyDeals() {
    return this._dailyDeals;
  },

  /**
   * Sets the daily deals array
   */
  set dailyDeals(aValue) {
    this._dailyDeals = aValue;
  },

  /**
   * Adds an white list element to the local store, or updates it to match the
   * given element.
   * @param aWhiteListUrl WhiteListUrl object
   */
  addOrUpdateWhiteListUrl : function(aWhiteListUrl) {
    this._markObjectDirty(aWhiteListUrl);
  },

  /**
   * Returns all the white list urls from the local database
   */
  getAllWhiteListUrls : function() {
    return ObjectsStorage.getAllWhiteListUrls();
  },

  /**
   * Removes an WhiteListUrl from the local store
   * @param aWhiteListUrl WhiteListUrl object
   */
  removeWhiteListUrl : function(aWhiteListUrl) {
    ObjectsStorage.removeWhiteListUrl(aWhiteListUrl);
  },

  /**
   * Adds a DOM list element to the local store, or updates it to match the
   * given element.
   * @param aDOMListUrl DOMListUrl object
   */
  addOrUpdateDOMListUrl : function(aDOMListUrl) {
    this._markObjectDirty(aDOMListUrl);
  },

  /**
   * Returns all the DOM list urls from the local database
   */
  getAllDOMListUrls : function() {
    return ObjectsStorage.getAllDOMListUrls();
  },

  /**
   * Removes an DOMListUrl from the local store
   * @param aDOMListUrl DOMListUrl object
   */
  removeDOMListUrl : function(aDOMListUrl) {
    ObjectsStorage.removeDOMListUrl(aDOMListUrl);
  },

  /**
   * Adds an tracked url element to the local store, or updates it to match the
   * given element.
   * @param aTrackedUrl TrackedUrl object
   */
  addOrUpdateTrackedUrl : function(aTrackedUrl) {
    this._markObjectDirty(aTrackedUrl);
  },

  /**
   * Returns all the tracked urls from the local database
   */
  getAllTrackedUrls : function() {
    return ObjectsStorage.getAllTrackedUrls();
  },

  /**
   * Returns top tracked urls from the local database
   */
  getTopTrackedUrls : function(aCount) {
    return ObjectsStorage.getTopTrackedUrls(aCount);
  },

  /**
   * Deletes all the records in the trackedUrls table
   */
  clearTrackedUrls : function() {
    ObjectsStorage.removeTrackedUrls();
  },

  /**
   * Deletes top records in the trackedUrls table
   */
  clearTopTrackedUrls : function(aCount) {
    ObjectsStorage.removeTopTrackedUrls(aCount);
  },

  /**
   * Sets the current eBay time, making sure that nothing too funky is happening
   * that could cause visible time-jumps
   */
  setEbayTime : function(claimedTime) {
    // Avoid letting time go backwards
    if (this._lastEbayTime && claimedTime < this._lastEbayTime) {
      return;
    }
    this._lastEbayTime = claimedTime;

    let localTime = Date.now();
    let ebayTime = claimedTime.getTime();
    let offset = ebayTime - localTime;

    this._ebayTimeOffset = offset;
    if (typeof(PropertiesStorage) == "undefined") {
      Cu.import(
        "resource://ebaycompanion/storage/propertiesStorage.js");
    }
    PropertiesStorage.set("ebayTimeOffset", this._ebayTimeOffset);
  },

  /**
   * Returns the current eBay time as a Date object
   */
  getEbayTime : function() {
    if (!this._ebayTimeOffset) {
      if (typeof(PropertiesStorage) == "undefined") {
        Cu.import(
          "resource://ebaycompanion/storage/propertiesStorage.js");
      }
      // Retrieve the offset from storage, if it is there
      let storedOffset = PropertiesStorage.get("ebayTimeOffset");
      this._ebayTimeOffset = (storedOffset != null) ? Number(storedOffset) : 0;
    }
    return new Date(Date.now() + this._ebayTimeOffset);
  },

  /**
   * Returns the "unique" instance identifier (if stored) or generates and
   * stores one
   */
  getInstanceIdentifier : function() {
    if (!this._instanceIdentifier) {
      if (typeof(PropertiesStorage) == "undefined") {
        Cu.import(
          "resource://ebaycompanion/storage/propertiesStorage.js");
      }
      // Retrieve the identifier from storage, if it is there
      let storedIdentifier = PropertiesStorage.get("instanceIdentifier");
      if (null == storedIdentifier) {
        // no instance identifier? generate and store one
        this._instanceIdentifier = UtilityHelper.generateUUID();
        PropertiesStorage.set("instanceIdentifier", this._instanceIdentifier);
      } else {
        this._instanceIdentifier = storedIdentifier;
      }
    }
    return this._instanceIdentifier;
  },

  /**
   * Gets the identifier to be sent with each Trading API request. It is a
   * combination of the instance identifier and a timestamp
   */
  getRequestIdentifier : function() {
    let instanceIdentifier = this.getInstanceIdentifier();
    let timestamp = new Date().getTime();
    return instanceIdentifier + "*" + timestamp;
  },

  /**
   * Marks an object (Data Type) for later storage on disk
   * @param object Data Type from objects modules
   */
  _markObjectDirty : function(object) {
    if (!this._dirtyObjects) {
      this._dirtyObjects = [];
    }

    let objectType = object.constructor.name;
    let entry = {};
    entry.type = objectType;

    // The idea here is to produce a key that will be unique for each object,
    // but clash when the same object is marked dirty twice, to avoid it being
    // written to disk several times, and also store enough information to
    // identify the object that should be stored.
    switch (objectType) {
      case "Account":
        let userId = object.get("userId");
        let isSandboxAccount = object.get("isSandboxAccount");
        let key = objectType + "." + userId + "." + isSandboxAccount;
        entry.userId = userId;
        entry.isSandboxAccount = isSandboxAccount;
        this._dirtyObjects[key] = entry;
        break;

      case "Item":
        let itemId = object.get("itemId");
        key = objectType + "." + itemId;
        entry.itemId = itemId;
        this._dirtyObjects[key] = entry;
        break;

      case "Transaction":
        itemId = object.get("itemId");
        let transactionId = object.get("transactionId");
        key = objectType + "." + itemId + "." + transactionId;
        entry.itemId = itemId;
        entry.transactionId = transactionId;
        this._dirtyObjects[key] = entry;
        break;

      case "FavoriteSearch":
        userId = object.get("userId");
        let searchName = object.get("searchName");
        key = objectType + "." + userId + "." + searchName;
        entry.object = object;
        this._dirtyObjects[key] = entry;
        break;

      case "FavoriteSeller":
        userId = object.get("userId");
        let sellerId = object.get("sellerId");
        key = objectType + "." + userId + "." + sellerId;
        entry.object = object;
        this._dirtyObjects[key] = entry;
        break;

      case "SystemMessage":
        let messageId = object.get("messageId");
        userId = object.get("userId");
        key = objectType + "." + messageId + "." + userId;
        entry.object = object;
        this._dirtyObjects[key] = entry;
        break;

      case "WhiteListUrl":
        let urlId = object.get("urlId");
        key = objectType + "." + urlId;
        entry.object = object;
        this._dirtyObjects[key] = entry;
        break;

      case "DOMListUrl":
        let id = object.get("id");
        key = objectType + "." + id;
        entry.object = object;
        this._dirtyObjects[key] = entry;
        break;

      case "TrackedUrl":
        let timestamp = object.get("timestamp");
        key = objectType + "." + timestamp;
        entry.object = object;
        this._dirtyObjects[key] = entry;
        break;

      case "RecentSearch":
      case "RecentBid":
      case "RecentView":
      case "RecentWatch":
      case "RecentAlert":
        timestamp = object.get("timestamp");
        key = objectType + "." + timestamp;
        entry.object = object;
        this._dirtyObjects[key] = entry;
        break;

      default:
        Logger.error("Unable to mark object of type \"" + objectType + "\" " +
                     "as dirty, because it is not a recognised type.",
                     Logger.DUMP_STACK);
    }
  },

  /**
   * Removes an object from the dirty objects list
   * @param aObject the object to be removed
   */
  removeDirtyObject : function(aObject) {
    let objectType = aObject.constructor.name;

    switch (objectType) {
      case "RecentSearch":
        for (let [key, dirtyObject] in Iterator(this._dirtyObjects)) {
          if (key.indexOf(objectType) != -1 &&
              dirtyObject.object.get("userId") == aObject.get("userId") &&
              dirtyObject.object.get("searchString") ==
                aObject.get("searchString")) {
            this._dirtyObjects[key] = null;
            delete this._dirtyObjects[key];
          }
        }
        break;
      case "RecentBid":
      case "RecentView":
      case "RecentWatch":
      case "RecentAlert":
        for (let [key, dirtyObject] in Iterator(this._dirtyObjects)) {
          if (key.indexOf(objectType) != -1 &&
              dirtyObject.object.get("userId") == aObject.get("userId") &&
              dirtyObject.object.get("itemId") == aObject.get("itemId")) {
            this._dirtyObjects[key] = null;
            delete this._dirtyObjects[key];
          }
        }
        break;
      default:
        Logger.error("Unable to remove object of type \"" + objectType + "\" " +
                     "from dirty objects, because it is not a recognised type.",
                     Logger.DUMP_STACK);
    }
  },

  /**
   * Flushes all objects to disk that have been marked as dirty.
   */
  _flushDirtyObjects : function() {
    if (typeof(StorageHelper) == "undefined") {
      Cu.import("resource://ebaycompanion/helpers/storageHelper.js");
    }
    try {
      if (!this._dirtyObjects) {
        return;
      }

      StorageHelper.connection.beginTransaction();
      let notifyTrackedUrlsLimitReached = false;

      for each (let [key, entry] in Iterator(this._dirtyObjects)) {
        switch (entry.type) {
          case "Account":
            let account = this._activeAccount;
            if (account) {
              ObjectsStorage.store(account);
            }
            break;

          case "Item":
            let item = this._items[entry.itemId];
            if (item) {
              ObjectsStorage.addItemToAccount(this._items[entry.itemId],
                                            this._activeAccount);
            }
            break;

          case "Transaction":
            let transactions = this._transactions[entry.itemId];
            if (transactions && transactions[entry.transactionId]) {
              ObjectsStorage.store(transactions[entry.transactionId]);
            }
            break;

          case "FavoriteSearch":
            ObjectsStorage.store(entry.object);
            break;
          case "FavoriteSeller":
            ObjectsStorage.store(entry.object);
            break;

          case "SystemMessage":
            ObjectsStorage.store(entry.object);
            break;

          case "TrackedUrl":
            ObjectsStorage.store(entry.object);
            // increase the tracked urls count
            this._trackedUrlsCount++;
            if (this._trackedUrlsCount > Constants.STORAGE_TRACKED_URLS_LIMIT) {
              //Removes the oldest tracked url stored in the database
              ObjectsStorage.removeTopTrackedUrls(1);
              this._trackedUrlsCount--;
            }
            // check if the limit of tracked url records has been reached, to
            // force submit to ebay when the dirty data flush finishes
            if (this._trackedUrlsCount >= Constants.TRACKED_URLS_COUNT_LIMIT) {
              notifyTrackedUrlsLimitReached = true;
            }
            break;

          case "WhiteListUrl":
            ObjectsStorage.store(entry.object);
            break;

          case "DOMListUrl":
            ObjectsStorage.store(entry.object);
          break;

          case "RecentSearch":
          case "RecentBid":
          case "RecentView":
          case "RecentWatch":
          case "RecentAlert":
            ObjectsStorage.store(entry.object);
          break;

          default:
            Logger.error("Unable to flush dirty object of type " +
                         "\"" + entry.type + "\" because it is not a " +
                         "recognised type.");
        }
        delete this._dirtyObjects[key];
      }
      StorageHelper.connection.commitTransaction();
      // after the flush process has finished, we check if we have reached the
      // records limit and notify the observer
      if (notifyTrackedUrlsLimitReached) {
        Constants.prefBranch.set("tracking.submitData", true);
        Observers.notify(null, Topics.TRACKED_URLS_LIMIT_REACHED, null);
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
      StorageHelper.connection.rollbackTransaction();
    }
  },

  /**
   * Removes the item sent as paramter from the dirty objects list
   * @param object the object to be removed from the dirty objects list
   */
  _removeFromDirtyObjects : function(object) {
    let objectType = object.constructor.name;
    let entry = {};
    entry.type = objectType;
    let key = null;

    // The idea here is to produce a key that will be unique for each object,
    // but clash when the same object is marked dirty twice, to avoid it being
    // written to disk several times, and also store enough information to
    // identify the object that should be stored.
    switch (objectType) {
      case "Account":
        let userId = object.get("userId");
        let isSandboxAccount = object.get("isSandboxAccount");
        key = objectType + "." + userId + "." + isSandboxAccount;
        break;

      case "Item":
        let itemId = object.get("itemId");
        key = objectType + "." + itemId;
        break;

      case "Transaction":
        itemId = object.get("itemId");
        let transactionId = object.get("transactionId");
        key = objectType + "." + itemId + "." + transactionId;
        break;

      case "FavoriteSearch":
        userId = object.get("userId");
        let searchName = object.get("searchName");
        key = objectType + "." + userId + "." + searchName;
        break;

      case "FavoriteSeller":
        userId = object.get("userId");
        let sellerId = object.get("sellerId");
        key = objectType + "." + userId + "." + sellerId;
        break;

      case "SystemMessage":
        let messageId = object.get("messageId");
        userId = object.get("userId");
        key = objectType + "." + messageId + "." + userId;
        break;

      default:
        Logger.error("Unable to remvoe object of type \"" + objectType + "\" " +
                     "from dirty list, because it is not a recognised type.",
                     Logger.DUMP_STACK);
    }

    if (key && this._dirtyObjects) {
      delete this._dirtyObjects[key];
    }

  },

  /**
   * Loads Item and Transaction objects from storage
   */
  _loadDataFromStorage : function() {
    this._items = [];
    this._transactions = [];
    this._favoriteSellers = [];
    this._favoriteSearches = [];

    // Load stored items
    let storedItems = ObjectsStorage.itemsForAccount(this._activeAccount);
    for (let i = 0; i < storedItems.length; i++) {
      let itemId = storedItems[i].get("itemId");
      this._items[itemId] = storedItems[i];
    }

    // Load stored transactions
    let storedTransactions =
      ObjectsStorage.transactionsForAccount(this._activeAccount);
    for (let i = 0; i < storedTransactions.length; i++) {
      let transaction = storedTransactions[i];
      let itemId = transaction.get("itemId");
      let transactionId = transaction.get("transactionId");
      if (!this._transactions[itemId]) {
        this._transactions[itemId] = [];
      }
      this._transactions[itemId][transactionId] = transaction;
    }

    this._favoriteSellers =
      ObjectsStorage.favoriteSellersForAccount(this._activeAccount);
    Observers.notify(null, Topics.FAVORITE_SELLERS_UPDATED, null);
    this._favoriteSearches =
      ObjectsStorage.favoriteSearchesForAccount(this._activeAccount);
    Observers.notify(null, Topics.FAVORITE_SEARCHES_UPDATED, null);
    Observers.notify(null, Topics.SIDEBAR_LIST_RESET, "sidebarList");
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case Topics.QUIT_APPLICATION_GRANTED:
        this._uninit();
        break;
      case Topics.ACCOUNT_PROPERTY_UPDATED:
      case Topics.ITEM_PROPERTY_UPDATED:
      case Topics.TRANSACTION_PROPERTY_UPDATED:
      case Topics.SYSTEM_MESSAGE_PROPERTY_UPDATED:
        this._markObjectDirty(aSubject.object);
        break;
      case Topics.HTTP_ON_MODIFY_REQUEST:
        this._extractSignInUsername(aSubject);
        break;
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(Datasource);
