/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["HttpRequestHeaderVisitor"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/logger.js");

/**
 * Utilitary class to visit http request headers and extract POST parameters
 */
function HttpRequestHeaderVisitor (aHttpChannel) {
  this._httpChannel = aHttpChannel;
  this._requestHeadersArray = new Array();
  this._postBody = null;
}

HttpRequestHeaderVisitor.prototype =  {
  /**
   * Returns an instance of the ecPostDataExtractor class that will help us
   * extracting request body
   */
  getPostDataExtractor : function(aHttpChannel) {
    try {
      // "type cast" to nsIUploadChannel so we can process post data
      aHttpChannel.QueryInterface(Ci.nsIUploadChannel);
      let postStream = aHttpChannel.uploadStream;
      // if there is no POST body, there is nothing to do
      if (postStream) {
        // "type cast" nsISeekableStream so we can rewind after extracting the
        // post and prevent the request from stalling
        postStream.QueryInterface(Ci.nsISeekableStream);
        // And return a postData object
        return new ecPostDataExtractor(postStream);
      }
    } catch (ex) {
      Logger.exception("There was an error extracting the POST DATA: " + ex);
    }
    return null;
  },

  /**
   * This method makes this class implement the nsIHttpHeaderVisitor interface,
   * so we can send the class as parameter when we call visitRequestHeaders on
   * any instance of nsIHttpChannel, and extract the header values to our own
   * array
   */
  visitHeader : function(name, value) {
    this._requestHeadersArray[name] = value;
  },

  /**
   * Inspects the requests headers and also extracts its post parameters (if
   * any)
   */
  inspectRequest : function () {
    this._requestHeadersArray = {};
    // first get the request headers
    this._httpChannel.visitRequestHeaders(this);

    // There may be post data in the request
    var postDataExtractor = this.getPostDataExtractor(this._httpChannel);
    if (postDataExtractor) {
      let postBody = postDataExtractor.getPostBody();
      if (postBody != null) {
        this._postBody = postBody;
      }
    }
    return this._requestHeadersArray;
  },

  getPostData : function() {
    return this._postBody;
  }

};

/**
 * Helper class to extract post data from a request
 */
function ecPostDataExtractor(aStream) {
  // the stream to be processed
  this._stream = aStream;
  // necessary to extract the stream contents
  this._scriptableStream =
    Cc["@mozilla.org/scriptableinputstream;1"].
      createInstance(Ci.nsIScriptableInputStream);
  this._scriptableStream.init(this._stream);
}

ecPostDataExtractor.prototype = {
  // method to go back to the beginning of the stream
  rewind: function() {
    this._stream.seek(0,0);
  },

  /**
   * Extracts the post body from the stream
   */
  getPostBody: function() {
    let streamLength = this._scriptableStream.available();
    let postString = "";
    try {
      // prevent any "NS_BASE_STREAM_CLOSED" thrown occasionally
      for (var i = 0; i < streamLength; i++) {
        let character = this._scriptableStream.read(1);
        character ? postString += character : postString+='\0';
      }
    } catch (ex) {
      return "" + ex;
    } finally {
      this.rewind();
    }
    // get rid of those "\r\n" occurrences in the string
    while (postString.indexOf("\r\n") == (postString.length - 2)) {
      postString = postString.substring(0, postString.length - 2);
    }
    return postString;
  }
};
