/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["ItemList"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/datasource.js");


/**
 * This object provides an interface for sorting and filtering item objects
 * @param listName the name of the list so we can identify it when needed
 */
function ItemList(listName) {
  this._listName = listName;
  this._filteredItems = [];
  // Reset the list
  this.reset(true);

  Observers.add(this, Topics.SIDEBAR_LIST_RESET);
  Observers.add(this, Topics.ITEM_NEW);
  Observers.add(this, Topics.ITEM_REMOVED);
  Observers.add(this, Topics.ITEM_CHANGED);
  Observers.add(this, Topics.TRANSACTION_NEW);
  Observers.add(this, Topics.TRANSACTION_ITEM_REMOVED);

  // List of listeners to notify on changes
  this._listeners = [];
}

ItemList.prototype = {
  /**
   * Returns the name of the list
   */
  get listName() {
    return this._listName;
  },

  /**
   * Returns the items held by this list
   */
  get items() {
    return this._itemArray;
  },

  /**
   * Returns the items held by this list after the current filter has been
   * applied
   */
  get filteredItems() {
    return this._filteredItems;
  },

  /**
   * Resets this item list to contain all items in the datasource, unfiltered
   * and unsorted.
   * @param aStartupReset whether this is a startup reset or not
   */
  reset : function(aStartupReset) {
    this._itemArray = [];


    // Add existing items from Datasource to array
    let hashList = Datasource.items();
    if (hashList) {
      for (let [itemId, item] in Iterator(hashList)) {

        // we want to take item transactions into account (if it has any) and add
        // a representative item for each one of them.
        // If the item has transactions, but hasn't ended (dutch items), we also
        // add items with transactions as items to be considered for the
        // filter method, but only for items where the user is the seller.
        let itemTransactions = Datasource.transactions(item.get("itemId"));
        if (itemTransactions) {
          let userId = Datasource.activeAccount().get("userId");
          for (let [transactionId, transaction] in Iterator(itemTransactions)) {
            let copy = item.copy();
            copy.set("isEnded", true);
            // XXX: for BIN items with many units, the highBidderId column
            // is not set, so we have to set it in the representative item for
            // each transaction where the user was involved. The same happens
            // for ended auction items.
            let highBidder = copy.get("highBidderId");
            if (highBidder == null || highBidder.length == 0) {
              copy.set("highBidderId", transaction.get("buyerUserId"));
            }
            // user is seller
            if (copy.get("sellerUserId").toLowerCase() == userId.toLowerCase()) {
              copy.set(
                "quantitySold", transaction.get("quantityPurchased"));
              copy.set(
                "userQuantityWinning", 0);
              copy.type = Item.ITEM_TYPE_SOLD;
            } else {
            // user is buyer
              copy.set(
                "userQuantityWinning", transaction.get("quantityPurchased"));
              copy.set(
                "quantitySold", 0);
              copy.type = Item.ITEM_TYPE_WON;
            }
            copy.transaction = transaction;
            this._itemArray.push(copy);
          }
          if (!item.get("isEnded")) {
            // if we get here it is because the item hasn't ended, but we should
            // also check if it has time left as in some cases the item might
            // have ended (because its time is over), but the isEnded flag
            // hasn't been set yet. In that case, we already have the
            // transaction items representing it.
            let endTime = item.get("endTime");
            let ebayTime =
              Datasource.getEbayTime().getTime();
            let timeLeft = Math.max(0, endTime - ebayTime);
            if (item.get("sellerUserId").toLowerCase() == userId.toLowerCase() &&
                timeLeft > 0) {
              this._itemArray.push(item);
            }
          }
        } else {
          this._itemArray.push(item);
        }
      }
    }

    // set the default filter and the default sort mechanism only for startup
    // resets
    if (aStartupReset) {
      // Default blank filter
      this._filter =
        function(item) {
          return true;
        };

      // Default sort by itemId
      this._lessThan =
        function(a, b) {
          return a.get("itemId") < b.get("itemId");
        };
    }

  },

  /**
   * Filters the items in this list using the function provided, and sets the
   * filter used for automatically-added and removed items.
   * @param filter Function taking an item object as a parameter and returning
   *               true if it should be in the list, and false if it should not.
   */
  filter : function(filter) {
    if (filter) {
      this._filter = filter;
    }
    let filteredArray = [];
    let item;
    for (let i = 0; i < this._itemArray.length; i++) {
      item = this._itemArray[i];
      if (this._filter(item)) {
        filteredArray.push(item);
      }
    }

    this._filteredItems.splice(0, this._filteredItems.length);
    this._filteredItems = null;
    this._filteredItems = filteredArray;
  },

  /**
   * Sorts the items in this list using the quicksort algorithm, and sets the
   * sort order used for automatically-added and removed items.
   * @param lessThan Function taking two item objects as parameters and
   *                 returning true if the first should be sorted lower than
   *                 the second. If the parameter is null, the previous value
   *                 passed to this method is used.
   */
  sort : function(lessThan) {
    if (lessThan) {
      this._lessThan = lessThan;
    }
    this._filteredItems = quicksort(this._filteredItems, this._lessThan);
  },

  /**
   * Gets the summary of watching/selling/buying items
   * @returns the summary of watching/selling/buying items
   */
  getCollectionSummary : function() {

    let collectionSummary = {};
    let watching = 0;
    let bidding = 0;
    let selling = 0;
    let won = 0;
    let sold = 0;
    let item;

    for (let i = 0; i < this.items.length; i++) {
      item = this.items[i];

      switch(item.type) {
        // iOS app counts only active items
        case Item.ITEM_TYPE_WATCHING:
          if (!item.get("isEnded")) {
            watching++;
          }
          break;
        case Item.ITEM_TYPE_WON:
          won++;
          break;
        case Item.ITEM_TYPE_BIDDING:
        case Item.ITEM_TYPE_OFFER:
          bidding++;
          break;
        // iOS app counts only actively selling items
        case Item.ITEM_TYPE_SOLD:
          sold++;
          break;
        // iOS app counts only actively selling items
        case Item.ITEM_TYPE_SELLING:
          selling++;
          break;
      }
    }

    collectionSummary.watching = watching;
    collectionSummary.bidding = bidding;
    collectionSummary.sold = sold;
    collectionSummary.won = won;
    collectionSummary.selling = selling;
    return collectionSummary;
  },

  /**
   * Adds an object that will listen for add / remove events
   * @param listener The object that will listen for these events
   */
  addListener : function(listener) {
    this._listeners.push(listener);
  },

  /**
   * Removes an object that listens for add / remove events
   * @param listener The object that will listen for these events
   */
  removeListener : function(aListener) {
    for (let i = 0; i < this._listeners.length; i++) {
      let listener = this._listeners[i];
      try {
        if (aListener == listener) {
          this._listeners.splice(i, 1);
        }
      }
      catch (e) {
        // this will happen if the event handler isn't implemented
      }
    }
  },

  /**
   * Notifies all listeners of the given event
   * @param eventName Name of the event (e.g. "itemAdded")
   * @param params Parameter list for the event. These will be passed into the
   *               event handler as arguments.
   */
  _notifyListeners : function(eventName, params) {
    try {
      let methodName = eventName + "Event";
      for (let i = 0; i < this._listeners.length; i++) {
        let listener = this._listeners[i];
        try {
          listener[methodName].apply(listener, params);
        }
        catch (e) {
          // this will happen if the event handler isn't implemented
        }
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Adds an item to the item list
   * @param item Item object to add
   * @param toFilteredList whether to add the item to the filtered list or to
   * the global item list
   */
  _addItem : function(item, toFilteredList) {
    try {
      // Reject the item if the filter rejects it
      if (toFilteredList && !this._filter(item)) {
        return;
      }
      let itemList = this._itemArray;
      if (toFilteredList) {
        itemList = this._filteredItems;
      }

      let newIndex = binarySearch(item, itemList, this._lessThan);
      // Avoid duplicates
      let existingItem = itemList[newIndex];
      let sameItem = (existingItem &&
                      existingItem.get("itemId") == item.get("itemId"));
      if (!sameItem) {
        // XXX: in case we didn't find the item with the binary search, we try
        // to iterate the whole list to find it
        let exhaustiveIndex = exhaustiveSearch(item, itemList);
        if (null != exhaustiveIndex) {
          existingItem = itemList[exhaustiveIndex];
          sameItem = (existingItem.get("itemId") == item.get("itemId"));
          newIndex = exhaustiveIndex;
        }
      }

      if (existingItem) {
        if (sameItem) {
          if (existingItem.transaction != null &&
              item.transaction != null &&
              existingItem.transaction.get("transactionId") !=
                item.transaction.get("transactionId")) {
            sameItem = false;
          }
          // this removes items from the active tab when the user buys or wins
          // the auction. The item has to be active, the user be the buyer and
          // the collection item shouldn't a related transaction despite there
          // are transactions in the datasource. When these conditions are met,
          // we have to remove the item from the active list.
          let userId = Datasource.activeAccount().get("userId");
          let itemTransactions = Datasource.transactions(item.get("itemId"));
          if (!existingItem.get("isEnded") &&
              existingItem.get("sellerUserId").toLowerCase() !=
                userId.toLowerCase() &&
              existingItem.transaction == null &&
              itemTransactions) {
            this._removeItem(item, true, toFilteredList);
          }
          // seller user id should never be empty, this is case when
          // something happenedon the item. E.g. buyer account was suspended.
          if (item.get("sellerUserId") == "") {
            this._removeItem(item, false, toFilteredList);
          }
        }
      }

      if (!existingItem || !sameItem) {
        itemList.splice(newIndex, 0, item);
        // we only notify the observers if the items is added to the filtered
        // list
        if (toFilteredList) {
          this._notifyListeners("itemAdded", [newIndex, item]);
        }
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Removes an item from the list
   * @param item object to remove
   * @force determines either we should or shouldn't display a "Recently ended
   * items moved to ended list" notification or not
   * @param toFilteredList whether to add the item to the filtered list or to
   * the global item list
   */
  _removeItem : function(item, force, toFilteredList) {
    try {
      let itemList = this._itemArray;
      if (toFilteredList) {
        itemList = this._filteredItems;
      }
      let itemIndex = binarySearch(item, itemList, this._lessThan);
      let existingItem = itemList[itemIndex];
      if (!existingItem) {
        return;
      }

      let sameItem = (existingItem.get("itemId") == item.get("itemId"));
      if (!sameItem) {
        // XXX: in case we didn't find the item with the binary search, we try
        // to iterate the whole list to find it
        let exhaustiveIndex = exhaustiveSearch(item, itemList);
        if (null != exhaustiveIndex) {
          existingItem = itemList[exhaustiveIndex];
          sameItem = (existingItem.get("itemId") == item.get("itemId"));
          itemIndex = exhaustiveIndex;
        }
      }

      if (sameItem) {
        // if the existing item has a transaction we shouldn't remove it because
        // that means we are in the ended tab, and we don't want to remove
        // transaction items
        // Also, seller user id should never be empty, this is case when
        // something happened on the item. E.g. buyer account was suspended.
        if (existingItem.transaction != null &&
            item.get("sellerUserId") != "") {
          sameItem = false;
        }
      }
      if (sameItem) {
        itemList.splice(itemIndex, 1);
        // we only notify the observers if the items is removed to the filtered
        // list
        if (toFilteredList) {
          this._notifyListeners(
            "itemRemoved", [itemIndex, force, item]);
        }
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Re-applies the filter to an item. Ensures the item is in the list if it
   * passes, and removed if not.
   * @param item The item object to filter.
   */
  _refilterItem : function(item) {
    if (this._filter(item)) {
      this._addItem(item);
      // these are only used if we were implementing the filteredList
      // approach for the sidebar items
      this._addItem(item, true);
    } else {
      // if we are refiltering the item and it doesn't match the current filter
      // we only have to remove it from the filtered list
      //this._removeItem(item);
      // these are only used if we were implementing the filteredList
      // approach for the sidebar items
      this._removeItem(item, false, true);
    }
  },

  /**
   * Adds a transaction item to the item list
   * @param transaction transaction object to add
   * @param toFilteredList whether to add the item to the filtered list or to
   * the global item list
   */
  _addTransactionItem : function(transaction, toFilteredList) {
    try {
      // get the related item first
      let item = Datasource.items()[transaction.get("itemId")];
      let userId = Datasource.activeAccount().get("userId");
      if (item) {
        // work with a copy so we can set it as ended without affecting the
        // original one that might not have ended.
        let itemCopy = item.copy();
        itemCopy.set("isEnded", true);
        itemCopy.transaction = transaction;
        // XXX: for BIN items with many units, the highBidderId column
        // is not set, so we have to set it in the representative item for
        // each transaction where the user was involved. The same happens
        // for ended auction items.
        let highBidder = itemCopy.get("highBidderId");
        if (highBidder == null || highBidder.length == 0) {
          itemCopy.set("highBidderId", transaction.get("buyerUserId"));
        }
        if (itemCopy.get("sellerUserId").toLowerCase() ==
              userId.toLowerCase()) {
          itemCopy.set(
            "quantitySold", transaction.get("quantityPurchased"));
          itemCopy.set(
            "userQuantityWinning", 0);
          itemCopy.type = Item.ITEM_TYPE_SOLD;
        } else {
          itemCopy.set(
            "userQuantityWinning", transaction.get("quantityPurchased"));
          itemCopy.set(
            "quantitySold", 0);
          itemCopy.type = Item.ITEM_TYPE_WON;
        }
        // Reject the item if the filter rejects it
        if (toFilteredList && !this._filter(itemCopy)) {
          return;
        }

        let itemList = this._itemArray;
        if (toFilteredList) {
          itemList = this._filteredItems;
        }

        let newIndex = binarySearch(itemCopy, itemList, this._lessThan);
        // Avoid duplicates
        let existingItem = itemList[newIndex];
        let sameItem = false;
        let replaceExisting = 0;

        if (existingItem) {
          sameItem = (existingItem.get("itemId") == itemCopy.get("itemId"));
        }
        if (!sameItem) {
          // XXX: in case we didn't find the item with the binary search, we try
          // to iterate the whole list to find it
          let exhaustiveIndex = exhaustiveSearch(itemCopy, itemList);
          if (null != exhaustiveIndex) {
            existingItem = itemList[exhaustiveIndex];
            sameItem = (existingItem.get("itemId") == itemCopy.get("itemId"));
            newIndex = exhaustiveIndex;
          }
        }

        if (sameItem) {
          let EC_LISTING_TYPE_CHINESE = "Chinese";
          let EC_LISTING_TYPE_DUTCH = "Dutch";
          let EC_LISTING_TYPE_LIVE = "Live";
          let listingFormat = existingItem.get("listingFormat");

          // in case of an auction item, if a new transaction comes up,
          // this means the auction ended and the item was sold, so we have
          // to replace the current item from the list with the new one with
          // the transaction
          if ((EC_LISTING_TYPE_CHINESE == listingFormat ||
                EC_LISTING_TYPE_DUTCH == listingFormat ||
                EC_LISTING_TYPE_LIVE == listingFormat) &&
              (existingItem.transaction == null &&
               itemCopy.transaction != null)) {
            sameItem = false;
            replaceExisting = 1;
          } else if((existingItem.transaction == null &&
              itemCopy.transaction != null)) {
            // this is a new transaction for a BIN item that doesn't have any
            // transactions yet, so we DO want to replace it in the list
            sameItem = false;
            replaceExisting = 1;
          } else if (existingItem.transaction != null &&
              itemCopy.transaction != null &&
              existingItem.transaction.get("transactionId") !=
                itemCopy.transaction.get("transactionId")) {
            // this is a new BIN item transaction, so we don't want
            // to replace the existing element in the list
            sameItem = false;
          }
        }

        if (!existingItem || !sameItem) {
          itemList.splice(newIndex, replaceExisting, itemCopy);
          // we only notify the observers if the items is added to the filtered
          // list, and it is not replacing an existing item.
          if (toFilteredList && replaceExisting == 0) {
            // at this point, if the item has to be added to the filtered list
            // and as it is a new transaction, should be added at the top of
            // the list, because we are in the ended tab
            newIndex = 0;
            this._notifyListeners("itemAdded", [newIndex, itemCopy]);
          }
        }
      }
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Removes an transaction item from the list
   * @param transactionItem object to remove
   * @force determines either we should or shouldn't display a "Recently ended
   * items moved to ended list" notification or not
   * @param toFilteredList whether to remove the item from the filtered list or
   * from the global item list
   */
  _removeTransactionItem : function(transactionItem, force, toFilteredList) {

    let itemList = this._itemArray;
    if (toFilteredList) {
      itemList = this._filteredItems;
    }

    for (var i = 0; i < itemList.length; i++) {
      let listItem = itemList[i];
      if (listItem.transaction != null &&
          listItem.get("itemId") == transactionItem.get("itemId") &&
          listItem.get("transactionId") == transactionItem.get("transactionId")) {
        itemList.splice(i, 1);
        if (toFilteredList) {
          this._notifyListeners(
            "itemRemoved", [i, force, transactionItem]);
        }
        break;
      }
    }

    // if the item was a buying item and there are no more transactions
    // related to it, we have to check if we should show it again in the sidebar
    // list, as watching item
    let dsItem = Datasource.items()[transactionItem.get("itemId")];
    let itemTransactions = Datasource.transactions(dsItem.get("itemId"));
    let userId = Datasource.activeAccount().get("userId");
    if (!itemTransactions &&
        dsItem.get("sellerUserId").toLowerCase() != userId.toLowerCase()) {
      // set the item as watching and update it in the datasource.
      let itemCopy = dsItem.copy();
      // set the type to watching and remove any transaction in it
      itemCopy.type = Item.ITEM_TYPE_WATCHING;
      itemCopy.transaction = null;
      Datasource.addOrUpdateItem(itemCopy);
      this._addItem(itemCopy);
      this._addItem(itemCopy, true);
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case Topics.SIDEBAR_LIST_RESET:
        if (this._listName && this._listName.indexOf(aData) != -1) {
          this.reset();
          Observers.notify(null, Topics.LIST_FORCE_APPLY_FILTER, null);
        }
        break;
      case Topics.ITEM_NEW:
        this._addItem(aSubject.object);
        // these are only used if we were implementing the filteredList
        // approach for the sidebar items
        this._addItem(aSubject.object, true);
        break;
      case Topics.ITEM_REMOVED:
        this._removeItem(aSubject.object, true);
        // these are only used if we were implementing the filteredList
        // approach for the sidebar items
        this._removeItem(aSubject.object, true, true);
        break;
      case Topics.ITEM_CHANGED:
        this._refilterItem(aSubject.object);
        break;
      case Topics.TRANSACTION_NEW:
        this._addTransactionItem(aSubject.object);
        // these are only used if we were implementing the filteredList
        // approach for the sidebar items
        this._addTransactionItem(aSubject.object, true);
        break;
      case Topics.TRANSACTION_ITEM_REMOVED:
        this._removeTransactionItem(aSubject.object);
        // these are only used if we were implementing the filteredList
        // approach for the sidebar items
        this._removeTransactionItem(aSubject.object, true, true);
        break;
    }
  }
}

/**
 * Quicksort algorithm
 * @param array The array to sort
 * @param lessThan Function taking two item objects and returning true
 *                 if the first should be sorted lower than the second
 * @return The sorted array
 */
function quicksort(array, lessThan) {
  if (array.length <= 1) {
    return array;
  }

  let less = [];
  let greater = [];
  let pivot = array.shift();
  for (let i = 0; i < array.length; i++) {
    if (lessThan(array[i], pivot)) {
      less.push(array[i]);
    } else {
      greater.push(array[i]);
    }
  }

  let left = quicksort(less, lessThan);
  let right = quicksort(greater, lessThan);

  return left.concat(pivot).concat(right);
}

/**
 * Searches for the given item in the given array, using the lessThan method to
 * determine when one item comes before another in the array.
 * @param item The Item object to search for
 * @param array The array of Item objects to search
 * @param lessThan Function taking two item objects and returning true if the
 *                 first appears before the other.
 * @return Array index that provides the closest match to the provided item.  If
 *         the provided item was not in the array, it will not be equal.
 */
function binarySearch(item, array, lessThan) {
  let low = 0;
  let high = array.length;
  while (low < high) {
    let mid = Math.floor((low + high) / 2);
    if (lessThan(array[mid], item)) {
      low = mid + 1;
    } else {
      high = mid;
    }
  }
  // Since there may be several items that are equal by sort order, so we need
  // to iterate through the list until we find an item with the same itemId, or
  // until we're sure it's not there.
  while (array[high + 1] && !lessThan(array[high], array[high + 1]) &&
         array[high].get("itemId") != item.get("itemId")) {
    high++;
  }
  return high;
}

/**
 * Iterates the entire list of items looking for a particular item. Used only
 * to remove an item when the binary search doesn't return it
 * @param aItem the item to be searched
 * @param aArray the array to be searched
 * @return the index of the item being searched, or null if not in the list
 */
function exhaustiveSearch(aItem, aArray) {
  let item;
  let index = null;
  for (var i = 0; i < aArray.length; i++) {
    if (aArray[i].get("itemId") == aItem.get("itemId")) {
      index = i;
      break;
    }
  }
  return index;
}
