/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["Logger"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/log4moz.js");

/**
 * Logger service.
 */
var Logger = {
  /* Logger level preferece. */
  LOGGER_LEVEL_PREF : eBayComp.PrefBranch + "common.logger.level",
  LOGGER_APPENDER_PREF : eBayComp.PrefBranch + "common.logger.appender",
  // Used as a constant to make code easier to read
  DUMP_STACK : true,
  // Glaxstar's common logger class
  _logger : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    this._initAppenders();
    this._logger = this._createLogger("Logger");
    this._logger.debug("init");
  },

  /**
   * Initializes the appenders.
   */
  _initAppenders : function() {
    let root = Log4Moz.repository.rootLogger;
    let formatter = new Log4Moz.AdvancedFormatter();
    let logFile = eBayComp.getExtensionDirectory();
    let loggerLevelPref =
      eBayComp.Application.prefs.get(this.LOGGER_LEVEL_PREF);
    let loggerLevel = (loggerLevelPref ? loggerLevelPref.value : "All");
    let app;

    logFile.append("ebayLog.txt");

    // Loggers are hierarchical, lowering this log level will affect all
    // output.
    root.level = Log4Moz.Level["All"];

    switch (eBayComp.Application.prefs.get(this.LOGGER_APPENDER_PREF).value) {
      case "rotating":
        // this appender will log to the file system.
        app = new Log4Moz.RotatingFileAppender(logFile, formatter);
        app.level = Log4Moz.Level[loggerLevel];
        root.addAppender(app);
        break;
      case "console":
        // A console appender outputs to the JS Error Console.
        app = new Log4Moz.ConsoleAppender(formatter);
        app.level = Log4Moz.Level[loggerLevel];
        root.addAppender(app);
        break;
      case "dump":
      default:
        // A dump appender outputs to standard out.
        app = new Log4Moz.DumpAppender(formatter);
        app.level = Log4Moz.Level[loggerLevel];
        root.addAppender(app);
        break;
    }

  },

  /**
   * Creates a logger repository from Log4Moz.
   * @param aName the name of the logger to create.
   * @param aLevel (optional) the logger level.
   * @return the generated logger.
   */
  _createLogger : function(aName, aLevel) {
    let logger = Log4Moz.repository.getLogger(aName);

    logger.level = Log4Moz.Level[(aLevel ? aLevel : "All")];

    return logger;
  },

  /**
   * Enter an exception in the log
   * @param message the message the user wishes to log
   * @param dumpStack A stack dump is produced if true
   * @param nonStandardStack If set, this stack is dumped instead
   */
  exception : function(message, dumpStack, nonStandardStack) {
    let logString = message;

    if (dumpStack) {
      logString += "\n" +
        this._dumpXpcomStack(
          nonStandardStack ? nonStandardStack : Components.stack.caller);
    }

    this._logger.fatal(logString);
  },

  /**
   * Enter generic event in the log
   * @param message the message the user wishes to log
   * @param dumpStack A stack dump is produced if true
   * @param nonStandardStack If set, this stack is dumped instead
   */
  log : function(message, dumpStack, nonStandardStack) {
    let logString = message;

    if (dumpStack) {
      logString += "\n" +
        this._dumpXpcomStack(
          nonStandardStack ? nonStandardStack : Components.stack.caller);
    }

    this._logger.info(logString);
  },

  /**
   * Enter a warning in the log
   * @param message the message the user wishes to log
   * @param dumpStack A stack dump is produced if true
   * @param nonStandardStack If set, this stack is dumped instead
   */
  warning : function(message, dumpStack, nonStandardStack) {
    let logString = message;

    if (dumpStack) {
      logString += "\n" +
        this._dumpXpcomStack(
          nonStandardStack ? nonStandardStack : Components.stack.caller);
    }

    this._logger.warn(logString);
  },

  /**
   * Enter an error in the log
   * @param message the message the user wishes to log
   * @param dumpStack A stack dump is produced if true
   * @param nonStandardStack If set, this stack is dumped instead
   */
  error : function(message, dumpStack, nonStandardStack) {
    let logString = message;

    if (dumpStack) {
      logString += "\n" +
        this._dumpXpcomStack(
          nonStandardStack ? nonStandardStack : Components.stack.caller);
    }

    this._logger.error(logString);
  },

  /**
   * Strips the useless components of a file:// path so that we end up with a
   * more readable path
   * @param path A full file:// path
   * @returns path relative to the extension directory
   */
  _getRelativePath : function(path) {
    // might as well cache this value
    if (!this._baseStart) {
      let index = path.indexOf(eBayComp.ExtensionId);
      if (index != -1) {
        // Add 8 to get rid of the "file:///" prefix
        this._baseStart = index + eBayComp.ExtensionId.length;
      } else {
        this.warning(
          "Extension ID in Logger module is set incorrectly. " +
          "Can't calculate relative paths for stack trace.");
        // Add 7 to get rid of the "file://" prefix, leaving one leading /
        this._baseStart = 7;
      }
    }

    return path.slice(this._baseStart);
  },

  /**
   * Dumps the provided JavaScript stack
   * @param stack
   */
  _dumpJsStack : function(stack) {
    let message = "--- JS Stack Trace:\n";

    let stackArray = stack.split("\n");
    stackArray.pop();  // the last element is empty
    for (let [, frame] in Iterator(stackArray)) {
      frame = frame.split("@");
      let context = frame[0];
      let filePath = unescape(frame[1]);
      // If the path uses the file:// protocol, strip that down to make it
      // more readable
      if (filePath.indexOf("file://") != -1) {
        filePath = this._getRelativePath(filePath);
      }
      // removed the context as it might contain the API response in some cases
      //message += "---   " + filePath + " " + context + "\n";
      message += "---   " + filePath + "\n";
    }
    return message;
  },

  /**
   * Dumps the provided XPCOM stack
   * @param stack
   */
  _dumpXpcomStack : function(stack) {
    let message = "--- XPCOM Stack Trace:\n";

    let frame = stack;
    while (frame) {
      let filePath = unescape(frame.filename);
      // If the path uses the file:// protocol, strip that down to make it
      // more readable
      if (filePath.indexOf("file://") != -1) {
        filePath = this._getRelativePath(filePath);
      }
      message += "---   " + filePath + ":" + frame.lineNumber + "\n";
      frame = frame.caller;
    }
    return message;
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(Logger);
