/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["ObjectHelper"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");

var ObjectHelper = {
  flags : {
    NO_NOTIFICATIONS : 1
  },

  /**
   * Returns a value that the given property expects, and/or generates an error
   * if there is a type mismatch.  For example, if a value "true" (string) is
   * given to a property of type boolean, this will return the expected boolean
   * value ("true" => true, "false" => false)
   * @param property Name of property (used in exception string)
   * @param type Type that the value should match
   * @param value Value that should match type
   * @returns filtered value
   */
  filterValue : function(property, type, value) {
    const TYPE_MISMATCH =
      "Type Mismatch: %1 (\"%2\") given for %3, which is of type %4";

    let reportError = function() {
      let str = TYPE_MISMATCH.replace(/%1/, typeof(value)).
                              replace(/%2/, value).
                              replace(/%3/, property).
                              replace(/%4/, type);
      Logger.error(str, Logger.DUMP_STACK, Components.stack.caller.caller.caller);
    }

    let newValue = value;
    switch (type) {
      case "number":
        newValue = Number(value);
        if (isNaN(newValue)) {
          reportError();
        }
        break;

      case "listingFormat":
        switch (value) {
          case "Chinese":
          case "Dutch":
          case "FixedPriceItem":
          case "StoresFixedPrice":
          case "LeadGeneration":
          case "AdType":
          case "Live":
            newValue = value;
            break;
          default:
            newValue = "FixedPriceItem";
            reportError();
        }
        break;

      case "boolean":
        switch (typeof(value)) {
          case "boolean":
            newValue = value;
            break;
          case "string":
            if (value == "true") {
              newValue = true;
            } else if (value == "false") {
              newValue = false;
            } else {
              newValue = false;
              reportError();
            }
            break;
          default:
            newValue = false;
            reportError();
        }
        break;

      case "object":
        switch (typeof(value)) {
          case "string":
            newValue = value;
        }
        break;

      default:
        if (typeof(value) != type) {
          // XXX: change to prevent number values causing errors when
          // set in string fields
          if (typeof(value) == "number" && type == "string") {
            newValue = "" + value;
          } else{
            newValue = undefined;
            reportError();
          }

        }
    }

    return newValue;
  },

  /**
   * Updates one object's properties to match those of another object, and
   * generates a notification with the given name for each property that
   * changed.
   * @param curObject the object to update
   * @param newObject the object to copy from
   * @param notificationName string indicating name of notification
   * @param flags may be NO_NOTIFICATONS to suppress observer notifications
   * @param ignorePropertiesArray an array with the names of the properties to
   * be ignored during the update
   * @return true if the object received modifications, false if not
   */
  updateObject : function(curObject, newObject, notificationName,
                          flags, ignorePropertiesArray) {
    let objectModified = false;
    for (let [property, value] in Iterator(newObject)) {
      if (property.charAt(0) == "_") {
        let ignoreProperty = false;
        if (ignorePropertiesArray) {
          for (let i = 0; i < ignorePropertiesArray.length; i++) {
            let tempProperty = "_" + ignorePropertiesArray[i];
            if (property == tempProperty) {
              ignoreProperty = true;
              break;
            }
          }
        }
        if (!ignoreProperty) {
          let curValue = curObject[property];
          curObject[property] = value;
          if (curValue != value) {
            objectModified = true;
            if (flags != this.flags.NO_NOTIFICATIONS) {
              let member = property.slice(1);
              let info = {};
              info.object = curObject;
              info.oldValue = curValue;
              info.newValue = value;
              Observers.notify(info, notificationName, member);
            }
          }
        }
      }
    }
    return objectModified;
  },

  /**
   * Updates the given property of the given object, and generates a
   * notification with the given name if it was different to the previous value.
   * @param curObject the object to update
   * @param property the name of the property
   * @param value the new value for the property
   * @param notificationName string indicating name of notification
   */
  updateProperty : function(curObject, property, value, notificationName) {
    let curValue = curObject["_" + property];
    curObject.set(property, value);
    if (curValue != value) {
      let info = {};
      info.object = curObject;
      info.oldValue = curValue;
      info.newValue = value;
      Observers.notify(info, notificationName, property);
    }
  },

  /**
   * Returns the value of the given property from the given object
   */
  getProperty : function(object, property) {
    let value = object["_" + property];
    if (value == undefined) {
      Logger.warning("Attempt to get property \"" + property + "\" of " +
                   object.constructor.name + " object, which is undefined.",
                   Logger.DUMP_STACK);
    }
    return value;
  },

  /**
   * Returns the value of the given property from the given object
   */
  getPropertyNoUnderscore : function(object, property) {
    let value = object[property];
    if (value == undefined) {
      Logger.warning("Attempt to get property \"" + property + "\" of " +
                   object.constructor.name + " object, which is undefined.",
                   Logger.DUMP_STACK);
    }
    return value;
  },

  fromJXON : function(aJXON, aObject) {
    var object = this._getObjectFromName(aObject.constructor.name);
    var jxonName = null;
    var jxonValue = null;
    var that = this;

    let objectPropertyTypes = aObject.constructor.prototype.propertyTypes;
    let objectPropertyXML = aObject.constructor.prototype.propertyXML;
    for each (let [property, type] in Iterator(objectPropertyTypes)) {
      jxonValue = that._getJXONValue(aJXON, type, objectPropertyXML[property]);
      if (jxonValue != null) {
        object.set(property, jxonValue);
      }
    }

    return object;
  },

  _getJXONValue : function(aJXONObject, aType, aJXONName) {
    let jxonNames = aJXONName.split("|");
    let jxonValue = null;

    for each (let [aIndex, aName] in Iterator(jxonNames)) {
      let jxonFields = aName.split(".");
      let jxonNode = aJXONObject;

      for each(let [aFieldIndex, aField] in Iterator(jxonFields)) {
        if (aField[0] != "@" && jxonNode) {
          // Check if field name contains an array notation
          let arrayStart = aField.indexOf('[');
          if (arrayStart != -1) {
            let arrayIndex = aField.substring(arrayStart+1, aField.indexOf(']'));
            aField = aField.substring(0,arrayStart);
            jxonNode = jxonNode[aField][arrayIndex];
          } else {
            jxonNode = jxonNode[aField];
          }
        }
      }

      if (jxonNode != undefined && jxonNode != null) {
        if (jxonFields[(jxonFields.length -1)][0] == "@") {
          let jxonAttr = jxonFields[(jxonFields.length -1)];
          if (aType == "number") {
            jxonValue = Number(jxonNode[jxonAttr]);
          } else {
            jxonValue = String(jxonNode[jxonAttr]);
          }
        } else {
          if (aType == "number") {
            jxonValue = (jxonNode["keyValue"] != undefined ? Number(jxonNode["keyValue"]) : Number(jxonNode));
          } else {
            jxonValue = (jxonNode["keyValue"] != undefined ? String(jxonNode["keyValue"]) : String(jxonNode));
          }
        }
        break;
      }
    }

    return jxonValue;
  },

  /**
   * Populates an object from a XML.
   * @param aXMLNode the xml node.
   * @param aObject the object schema.
   * @return the populated object from the xml.
   */
  fromXML : function(aXMLNode, aObject) {

    var myObject = this.getJXONTree(aXMLNode);
    var object = this._getObjectFromName(aObject.constructor.name);
    var xmlName = null;
    var xmlValue = null;
    var that = this;

    let objectPropertyTypes = aObject.constructor.prototype.propertyTypes;
    let objectPropertyXML = aObject.constructor.prototype.propertyXML;
    for each (let [property, type] in Iterator(objectPropertyTypes)) {
      xmlValue = that._getXMLValue(aXMLNode, type, objectPropertyXML[property]);
      if (xmlValue != null) {
        object.set(property, xmlValue);
      }
    }

    return object;
  },


  /**
   * Gets a value from an XML node, given its name(s) separated by '|'.
   * It allows nested tags, using '.' as a separator and @ for attributes.
   * @param aXMLNode the XML node to traverse.
   * @param aType the type of the field to get its value
   * @param aXMLName the tag name to find
   */
  _getXMLValue : function(aXMLNode, aType, aXMLName) {
    let xmlNames = aXMLName.split("|");
    let xmlValue = null;

    for each (let [aIndex, aName] in Iterator(xmlNames)) {
      let xmlFields = aName.split(".");
      let xmlNode = aXMLNode;

      var field = "xmlNode."+"aXMLName";

      xmlValue = eval(field);

//      for each(let [aFieldIndex, aField] in Iterator(xmlFields)) {
//        if (aField[0] != "@") {
//          // Check if field name contains an array notation
//          let arrayStart = aField.indexOf('[');
//          if (arrayStart != -1) {
//            let arrayIndex = aField.substring(arrayStart+1, aField.indexOf(']'));
//            aField = aField.substring(0,arrayStart);
//            xmlNode = xmlNode.children(aField)[arrayIndex];
//          } else {
//            xmlNode = xmlNode.descendants(aField);
//          }
//        }
//      }
//
//      if (xmlNode.length() > 0) {
//        if (xmlFields[(xmlFields.length -1)][0] == "@") {
//          let xmlAttr = xmlFields[(xmlFields.length -1)].substring(1);
//          if (aType == "number") {
//            xmlValue = Number(xmlNode.attribute(xmlAttr));
//          } else {
//            xmlValue = String(xmlNode.attribute(xmlAttr));
//          }
//        } else {
//          if (aType == "number") {
//            xmlValue = Number(xmlNode.text());
//          } else {
//            xmlValue = String(xmlNode.text());
//          }
//        }
//        break;
//      }
    }

    return xmlValue;
  },

  /**
   * Gets an object from the name.
   * @param aName the name of the object.
   * @return the object.
   */
  _getObjectFromName : function(aName) {

    Cu.import("resource://ebaycompanion/objects/item.js");
    Cu.import("resource://ebaycompanion/objects/account.js");
    Cu.import("resource://ebaycompanion/objects/transaction.js");
    Cu.import("resource://ebaycompanion/objects/favoriteSearch.js");
    Cu.import("resource://ebaycompanion/objects/favoriteSeller.js");
    Cu.import("resource://ebaycompanion/objects/systemMessage.js");
    Cu.import("resource://ebaycompanion/objects/deal.js");

    var object;

    switch (aName.toLowerCase()) {
      case "account":
        object = new Account();
        break;
      case "item":
        object = new Item();
        break;
      case "transaction":
        object = new Transaction();
        break;
      case "favoriteseller":
        object = new FavoriteSeller();
        break;
      case "favoritesearch":
        object = new FavoriteSearch();
        break;
      case "systemmessage":
        object = new SystemMessage();
        break;
      case "deal":
        object = new Deal();
        break;
      default:
        break;
    }

    return object;
  },

  parseText : function (sValue) {
    if (/^\s*$/.test(sValue)) { return null; }
    if (/^(?:true|false)$/i.test(sValue)) { return sValue.toLowerCase() === "true"; }
    if (isFinite(sValue)) { return parseFloat(sValue); }
    if (isFinite(Date.parse(sValue))) { return new Date(sValue); }
    return sValue;
  },

  getJXONTree : function (oXMLParent) {
    var vResult = /* put here the default value for empty nodes! */ true, nLength = 0, sCollectedTxt = "";
    if (oXMLParent.hasAttributes()) {
      vResult = {};
      for (nLength; nLength < oXMLParent.attributes.length; nLength++) {
        oAttrib = oXMLParent.attributes.item(nLength);
        vResult["@" + oAttrib.name] = this.parseText(oAttrib.value.trim());
      }
    }
    if (oXMLParent.hasChildNodes()) {
      for (var oNode, sProp, vContent, nItem = 0; nItem < oXMLParent.childNodes.length; nItem++) {
        oNode = oXMLParent.childNodes.item(nItem);
        if (oNode.nodeType === 4) { sCollectedTxt += oNode.nodeValue; } /* nodeType is "CDATASection" (4) */
        else if (oNode.nodeType === 3) { sCollectedTxt += oNode.nodeValue.trim(); } /* nodeType is "Text" (3) */
        else if (oNode.nodeType === 1 && !oNode.prefix) { /* nodeType is "Element" (1) */
          if (nLength === 0) { vResult = {}; }
          sProp = oNode.nodeName;
          vContent = this.getJXONTree(oNode);
          if (vResult.hasOwnProperty(sProp)) {
            if (vResult[sProp].constructor !== Array) { vResult[sProp] = [vResult[sProp]]; }
            vResult[sProp].push(vContent);
          } else { vResult[sProp] = vContent; nLength++; }
        }
      }
    }
    if (sCollectedTxt) { nLength > 0 ? vResult.keyValue = this.parseText(sCollectedTxt) : vResult = this.parseText(sCollectedTxt); }
    /* if (nLength > 0) { Object.freeze(vResult); } */
    return vResult;
  }

}
