/**
 * Copyright (c) 2011-2013 eBay Inc. All rights reserved.
 */

var EXPORTED_SYMBOLS = ["ProgressListener"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://ebaycompanion/constants.js");

/**
 * Progress Listener.
 */
ProgressListener = {
  /* Windows manager. */
  _windowManager : null,
  /* Document load service */
  _documentLoadService : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    this._windowManager =
      Cc["@mozilla.org/appshell/window-mediator;1"].
        getService(Ci.nsIWindowMediator);
    this._documentLoadService =
      Cc["@mozilla.org/docloaderservice;1"].getService(Ci.nsIWebProgress);

    // add this object as a global web progress listener.
    this._documentLoadService.addProgressListener(this,
      (Ci.nsIWebProgress.NOTIFY_LOCATION |
       Ci.nsIWebProgress.NOTIFY_STATE_REQUEST));
  },

  /**
   * Called when the location of the window being watched changes. This is not
   * when a load is requested, but rather once it is verified that the load is
   * going to occur in the given window. For instance, a load that starts in a
   * window might send progress and status messages for the new site, but it
   * will not send the onLocationChange until we are sure that we are loading
   * this new page here.
   * @param aWebProgress the nsIWebProgress instance that fired the
   * notification.
   * @param aRequest the associated nsIRequest. This may be null in some cases.
   * @param aLocation the URI of the location that is being loaded.
   */
  onLocationChange : function(aWebProgress, aRequest, aLocation) {
  },

  /**
   * (nsIWebProgressListener) Notification indicating the state has changed for
   * one of the requests associated with aWebProgress.
   * @param aWebProgress the nsIWebProgress instance that fired the
   * notification.
   * @param aRequest the nsIRequest that has changed state.
   * @param aStateFlags flags indicating the new state. This value is a
   * combination of one of the State Transition Flags and one or more of the
   * State Type Flags. Any undefined bits are reserved for future use.
   * @param aStatus error status code associated with the state change. This
   * parameter should be ignored unless stateFlags includes the STATE_STOP bit.
   * The status code indicates success or failure of the request associated with
   * the state change. NOTE: status may be a success code even for server
   * generated errors, such as the HTTP 404 error. In such cases, the request
   * itself should be queried for extended error information (e.g., for HTTP
   * requests see nsIHttpChannel).
   */
  onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus) {
    Constants.handleStateChange(aWebProgress, aRequest, aStateFlags, aStatus);
  },

  /**
   * (nsIWebProgressListener) Notification that the progress has changed for one
   * of the requests associated with aWebProgress. This should never be called.
   */
  onProgressChange : function(aWebProgress, aRequest, aCurSelfProgress,
    aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {
  },

  /**
   * (nsIWebProgressListener) Notification called for security progress. This
   * should never be called.
   */
  onSecurityChange : function(aWebProgress, aRequest, aState) {
  },

  /**
   * (nsIWebProgressListener) Notification that the status of a request has
   * changed. This should never be called.
   */
  onStatusChange : function(aWebProgress, aRequest, aStatus, aMessage) {
  },

  /**
   * Indicates if the given window is a sub-frame or the main DOM window in a
   * page.
   * @param aDOMWindow the window to check whether it's the main DOM window or
   * not.
   * @return true if the given DOM window is the main DOM window. false if it's
   * a subframe of a page.
   */
  _isMainDOMWindow : function(aDOMWindow) {
    return (aDOMWindow == aDOMWindow.top);
  },

  /**
   * Gets the chrome window associated with the given DOM HTML window.
   * @param aDOMWindow the nsIDOMWindow that corresponds to the page or frame
   * being loaded.
   * @return the chrome window object that contains the given DOM window object.
   */
  _getChromeWindowForDOMWindow : function(aDOMWindow) {
    let windows = this._windowManager.getEnumerator("navigator:browser");
    let window = null;
    let chromeWindow = null;
    let tab = null;

    while (windows.hasMoreElements()) {
      window = windows.getNext();
      tab = this._getTabForDOMWindow(window, aDOMWindow);

      if (null != tab) {
        chromeWindow = window;
        break;
      }
    }

    return chromeWindow;
  },

  /**
   * Gets the tab object associated with the given DOM HTML window.
   * @param aWindow the window where the page or frame is being loaded.
   * @param aDOMWindow the nsIDOMWindow that corresponds to the page or frame
   * being loaded.
   * @return the tab object that contains the given DOM window object.
   */
  _getTabForDOMWindow : function(aWindow, aDOMWindow) {
    let tab = null;

    if (aWindow.gBrowser) {
      let tabCount = aWindow.gBrowser.mTabs.length;
      let browserTab = null;

      try {
        for (let i = 0; i < tabCount; i++) {
          browserTab = aWindow.gBrowser.mTabs[i];

          if (browserTab.linkedBrowser &&
              browserTab.linkedBrowser.contentWindow == aDOMWindow.top) {
            tab = browserTab;
            break;
          }
        }
      } catch (e) {
        Logger.error("_getTabForDOMWindow. \n" + e);
      }
    }

    return tab;
  },

  /**
   * Filters the chrome and about schemes.
   * @param aScheme the location scheme.
   * @return true if the location is valid.
   */
  _filterScheme : function(aScheme) {
    return ("chrome" != aScheme && "about" != aScheme);
  },

  /**
   * The QueryInterface method provides runtime type discovery.
   * More: http://developer.mozilla.org/en/docs/nsISupports
   * @param aIID the IID of the requested interface.
   * @return the resulting interface pointer.
   */
  QueryInterface : function(aIID) {
    if (aIID.equals(Ci.nsIWebProgressListener) ||
        aIID.equals(Ci.nsISupportsWeakReference) ||
        aIID.equals(Ci.nsISupports)) {
      return this;
    }
    throw Cr.NS_NOINTERFACE;
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(ProgressListener);
