/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["StateHelper"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/constants.js");

/**
 * State helper.
 */
var StateHelper = {

  // TAB CONSTANTS
  HOME_TAB : 0,
  BUYING_TAB : 1,
  SELLING_TAB : 2,
  ACTIVE_TAB : 3,
  ENDED_TAB : 4,
  DEALS_TAB : 5,

  // DEFAULT FILTER
  DEFAULT_FILTER : "all",
  BUYING_DEFAULT_FILTER : "watching",
  SELLING_DEFAULT_FILTER : "active",
  ACTIVE_DEFAULT_FILTER : "all",
  ENDED_DEFAULT_FILTER : "all",

  /* State keep timer. */
  _STATE_KEEP_TIME : 60 * 1000, //60 seconds
  /* Timer. */
  _timer : null,
  /* Selected tab. */
  _selectedTab : null,
  /* Selected filter. */
  _selectedFilter : null,
  /* Item details */
  _itemDetails : null,
  /* Deal details */
  _dealDetails : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    this._timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    this.resetState();
  },

  /**
   * Resets the keep timer.
   */
  stopKeepTimer : function() {
    if (null != this._timer) {
      this._timer.cancel();
    }
  },

  /**
   * Starts the keep timer.
   */
  startKeepTimer : function() {
    let that = this;

    this._timer.initWithCallback({
      notify : function() { that.resetState(); }
    }, this._STATE_KEEP_TIME, Ci.nsITimer.TYPE_ONE_SHOT);
  },

  /**
   * Resets the state to the default values.
   */
  resetState : function() {
    this._selectedTab = Constants.prefBranch.get("default.tab");
    let tabNames = ["home", "buying", "selling", "active", "ended", "deals"];
    this._selectedPrefix = tabNames[this._selectedTab];
    this.resetSelectedFilter(this._selectedTab);
    this._itemDetails = null;
    this._dealDetails = null;
  },

  /**
   * Resets the default filter based on the tab passed as parameter
   */
  resetSelectedFilter : function(aTab) {
    let tabNames = ["home", "buying", "selling", "active", "ended", "deals"];
    let tab = tabNames[aTab];

    switch (tab) {
      case "buying":
        this.setCurrentFilter(tab, this.BUYING_DEFAULT_FILTER);
        break;
      case "selling":
        this.setCurrentFilter(tab, this.SELLING_DEFAULT_FILTER);
        break;
      case "active":
        this.setCurrentFilter(tab, this.ACTIVE_DEFAULT_FILTER);
        break;
      case "ended":
        this.setCurrentFilter(tab, this.ENDED_DEFAULT_FILTER);
        break;
      default:
        this.setCurrentFilter(tab, this.DEFAULT_FILTER);
        break;
    }
  },

  /**
   * Sets the current tab
   * @param aTab the current tab
   */
  setCurrentTab : function(aTab) {
    this._selectedTab = aTab;
  },

  /**
   * Sets the current filter.
   * @param aPrefix the current prefix (active/ended)
   * @param aFilter the current filter
   */
  setCurrentFilter : function(aPrefix, aFilter) {
    this._selectedPrefix = aPrefix;
    this._selectedFilter = aFilter;
  },

  /**
   * Get selected tab.
   * @return the selected tab.
   */
  getSelectedTab : function() {
    return this._selectedTab;
  },

  /**
   * Get selected filter.
   * @return the selected filter.
   */
  getSelectedFilter : function() {
    return [this._selectedPrefix, this._selectedFilter];
  },

  /**
   * Sets the accordion state. We use a preference so we don't lose the
   * accordion state even if the user closes the browser.
   * @param aSectionIndex the index of the array for the stored state of the
   * corresponding recent XXX section.
   * @param aState the section state: 1 opened, 0 closed.
   */
  setAccordionState : function(aSectionIndex, aState) {
    var accordionState = Constants.prefBranch.get("accordion.state");
    if (!accordionState) {
      Constants.prefBranch.set("accordion.state", "");
    }

    if (accordionState) {
      accordionState = accordionState.split(",");
    } else {
      accordionState = [];
    }

    accordionState[aSectionIndex] = aState;
    Constants.prefBranch.set("accordion.state", accordionState.join(","));
  },

  /**
   * Returns the accordion state stored in the preferences, corresponding
   * to a recent XXX section.
   * @param aSectionIndex the index of the array for the stored state of the
   * corresponding recent XXX section.
   */
  getAccordionStateForItem : function (aSectionIndex) {
    var accordionState = Constants.prefBranch.get("accordion.state");
    var state = -1;

    if (accordionState) {
      accordionState = accordionState.split(",");
      if (accordionState[aSectionIndex]) {
        state = accordionState[aSectionIndex];
      }
    }

    return state;
  },

  /**
   * Gets the item details.
   * @return the item details
   */
  getItemDetails : function() {
    return this._itemDetails;
  },

  /**
   * Sets the item details.
   * @param aItem the item
   */
  setItemDetails : function(aItem) {
    this._itemDetails = aItem;
  },

  /**
   * Gets the deal details.
   * @return the deal details
   */
  getDealDetails : function() {
    return this._dealDetails;
  },

  /**
   * Sets the deal details.
   * @param aDeal the deal
   */
  setDealDetails : function(aDeal) {
    this._dealDetails = aDeal;
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(StateHelper);
