/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["StorageHelper"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

// Database file name
const DB_FILE_NAME = "ebay-companion.sqlite";
const DB_TEMPLATE_FILE_NAME = "ebay-companion.template.sqlite";

var StorageHelper = {
  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      Cu.import("resource://ebaycompanion/common.js");
      Cu.import("resource://ebaycompanion/helpers/logger.js");
      Cu.import("resource://ebaycompanion/helpers/observers.js");
      Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
      Cu.import("resource://ebaycompanion/constants.js");

      Observers.add(this, Topics.QUIT_APPLICATION_GRANTED);

      // Define members
      let databaseFile =
        Cc["@mozilla.org/file/directory_service;1"].
          getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
      databaseFile.append("eBay Inc");
      databaseFile.append(DB_FILE_NAME);
      let databasePreexisted = databaseFile.exists();
      // the file exists, then check this is the first run and try to remove
      // the database file from the file system as it might belong to an
      // older version purged
      if (databasePreexisted && Constants.prefBranch.get("firstRun")) {
        databaseFile.remove(true);
        this._copyTemplate(databaseFile);
      } else if (!databasePreexisted) {
        // the database file doesn't exist, then we should copy the template
        this._copyTemplate(databaseFile);
      }
      // use a clone of the file so in windows the file is not locked
      // and the uninstall process can delete it. The file is locked anyway
      // on Windows so it is purged and not deleted.
      this._databaseFile = databaseFile.clone();

      this._storageService = Cc["@mozilla.org/storage/service;1"].
                               getService(Ci.mozIStorageService);
      this._databaseConnection =
        this._storageService.openDatabase(this._databaseFile);

      // We need "connection" to be defined before this can be imported
      if (typeof(Timer) == "undefined") {
        Cu.import("resource://ebaycompanion/helpers/timer.js");
      }
      new Timer(function() {
          Cu.import("resource://ebaycompanion/storage/migrations.js");
          Migrations.performMigrations();
        }, 500, Timer.TYPE_ONE_SHOT);
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Uninitializes the resource.
   */
  _uninit : function() {
    Observers.removeAll(this);
  },

  /**
   * Getter
   */
  get connection() {
    return this._databaseConnection;
  },

  /**
   * Copies the database template
   * @param aDBFile the target database file to copy the template to
   */
  _copyTemplate : function(aDBFile) {
    let rootDatabaseFile =
      Cc["@mozilla.org/file/directory_service;1"].
        getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
    rootDatabaseFile.append(DB_FILE_NAME);

    if (rootDatabaseFile.exists()) {
      rootDatabaseFile.moveTo(aDBFile.parent, aDBFile.leafName);
    } else {
      let copied =
        UtilityHelper.extractFileFromZip(DB_TEMPLATE_FILE_NAME, DB_FILE_NAME);

      if (!copied) {
        let defaultFile = UtilityHelper.getDefaultsFolder(eBayComp.ExtensionId);

        defaultFile.append(DB_TEMPLATE_FILE_NAME);
        defaultFile.copyTo(aDBFile.parent, aDBFile.leafName);
      }
    }
  },

  /**
   * Executes a statement on the database, optionally binding the given
   * parameters
   * @param statementText SQL statement; may include ?N-style placeholders
   * @param parameters Array containing the value to bind to the placeholders
   * @param async whether to execute the query asynchronously or not
   * @returns Array of hash objects, containing the data for each row
   */
  doStatement : function(statementText, parameters, async) {
    let rows = [];
    let statement = null;
    if (typeof(async) == "undefined") {
      async = false;
    }
    // flag to know whether the statemente was executed in sync mode or not
    let executedSync = true;
    try {
      statement = this.connection.createStatement(statementText);
      if (parameters) {
        for (let i = 0; i < parameters.length; i++) {
          statement.bindUTF8StringParameter(i, parameters[i]);
        }
      }

      if (async) {
        executedSync = false;
        // we can proceed asynchronously. At the moment, only insert/update
        // operations are done this way
        statement.executeAsync({
          handleResult: function(aResultSet) {
            // since we only perform write operations, we don't have to do
            // anything here
          },

          handleError: function(aError) {
            Logger.exception(aError.message);
          },

          handleCompletion: function(aReason) {
            if (aReason != Ci.mozIStorageStatementCallback.REASON_FINISHED) {
              Logger.exception("Database query canceled or aborted!");
            } else {
              Logger.log("Async database operation successful");
            }
          }
        });
      } else {
        while (statement.executeStep()) {
          let row = {};
          for (let i = 0; i < statement.numEntries; i++) {
            let key = statement.getColumnName(i);
            let value = statement.getUTF8String(i);
            row[key] = value;
          }
          rows.push(row);
        }
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
    finally {
      // reset and finalize the statement if it is performed synchronously
      if (statement != null && executedSync) {
        statement.reset();
        statement.finalize();
      }
    }

    return rows;
  },

  /**
   * Removes the database (used when uninstalling)
   */
  removeDatabase : function() {
    this._databaseFile.remove(true);
  },

  /**
   * Closes the database connection.
   */
  closeConnection : function() {
    try {
      this._databaseConnection.close();
      delete this._databaseConnection;
      this._databaseConnection = null;
    } catch (e) {
      this._databaseConnection = null;
      Logger.error("_closeDBConnection: " + e);
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    if (Topics.QUIT_APPLICATION_GRANTED == aTopic) {
      this._uninit();
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(StorageHelper);
