/**
 * Copyright (c) 2008-2013 Appcoast Ltd. All rights reserved.
 */

var EXPORTED_SYMBOLS = ["UtilityHelper"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");

// Operating system regular expressions.
const RE_OS_WINDOWS = /^Win/i;
const RE_OS_MAC = /^Mac/i;
const RE_OS_LINUX = /^Linux/i;
const RE_OS_WINDOWS_VISTA = /Windows NT 6.0/i;
const RE_OS_WINDOWS_WIN7 = /Windows NT 6.1/i;
const RE_OS_WINDOWS_WIN8 = /Windows NT 6.2/i;

// Regular expression to check a valid email address.
// See http://www.regular-expressions.info/email.html
const RE_EMAIL_ADDRESS = /^[a-z0-9._%+-]+@(?:[a-z0-9-]+\.)+[a-z]{2,4}$/i;
// Array constructor regular expression.
const RE_ARRAY_CONSTRUCTOR = /^function Array\(\)/;

/**
 * Utility service. Contains some general utility methods that are commonly
 * used in our extensions.
 */
var UtilityHelper = {
  /* Identifier for the operating system this extension is running on. */
  _os : null,

  /**
   * Obtains an identifier for the operating system this extension is running
   * on.
   * @return any of the OS constants defined in this interface.
   */
  getOperatingSystem : function() {
    let operatingSystem = "OTHER";

    if (null == this._os) {
      let appShellService =
        Cc["@mozilla.org/appshell/appShellService;1"].
          getService(Ci.nsIAppShellService);
      let platform = appShellService.hiddenDOMWindow.navigator.platform;

      if (platform.match(RE_OS_MAC)) {
        operatingSystem = "MAC";
      } else if (platform.match(RE_OS_WINDOWS)) {
        let userAgent = appShellService.hiddenDOMWindow.navigator.userAgent;

        if (userAgent.match(RE_OS_WINDOWS_VISTA)) {
          operatingSystem = "VISTA";
        } else if (userAgent.match(RE_OS_WINDOWS_WIN7)) {
          operatingSystem = "WIN7";
        } else if (userAgent.match(RE_OS_WINDOWS_WIN8)) {
          operatingSystem = "WIN8";
        } else {
          operatingSystem = "WINDOWS";
        }
      } else if (platform.match(RE_OS_LINUX)) {
        operatingSystem = "LINUX";
      } else {
        operatingSystem = "OTHER";
      }
      this._os = operatingSystem;
    } else {
      operatingSystem = this._os;
    }

    return operatingSystem;
  },

  /**
   * Returns the version of the extension and calls the specified callback with
   * the returned information. Firefox 4 introduced an asyncronous service in
   * spite of the extensions manager component, so this method adds the
   * necessary code to handle the new component and to keep backwards
   * compatibility
   * @param aCallback the callback method to be called with the version value
   * returned by the respective component
   */
  getVersion : function(aCallback) {
    if ("@mozilla.org/extensions/manager;1" in Cc) {
      // < Firefox 4
      let version =
        Cc["@mozilla.org/extensions/manager;1"].
          getService(Ci.nsIExtensionManager).
            getItemForID(eBayComp.ExtensionId).version;

      aCallback(version);
    } else {
      // > Firefox 4.
      Cu.import("resource://gre/modules/AddonManager.jsm");

      AddonManager.getAddonByID(eBayComp.ExtensionId, function(aAddon) {
        aCallback(aAddon.version);
      });
    }
  },

  /**
   * Returns the locale string.
   * @return the locale string.
   */
  getSelectedLocale : function() {
    let chrome =
      Cc["@mozilla.org/chrome/chrome-registry;1"].
        getService(Ci.nsIXULChromeRegistry).
          QueryInterface(Ci.nsIToolkitChromeRegistry);
    return chrome.getSelectedLocale("global");
  },

  /**
   * Obtains a reference to the defaults folder for a specific extension.
   * @param aExtensionID the ID of the extension.
   * @return nsIFile reference to the defaults folder for the extension matching
   * the given ID.
   */
  getDefaultsFolder : function(aExtensionID) {
    let directoryService =
      Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
    let defaultsFolder = directoryService.get("ProfD", Ci.nsIFile);
    defaultsFolder.append("extensions");
    defaultsFolder.append(aExtensionID);
    defaultsFolder.append("defaults");

    return defaultsFolder;
  },

  /**
   * Trims a string (both start and end).
   * @param aString the string to be trimmed.
   * @return the trimmed string.
   */
  trim : function(aString) {
    return aString.replace(/^\s+|\s+$/g, '');
  },

  /**
   * Returns true if the given string is null or empty.
   * @param aString the string to check.
   * @return true if the string is null or empty. Returns false otherwise. Note
   * that this method returns false for strings with blank characters, like " ".
   */
  isNullOrEmpty : function(aString) {
    return ((null == aString) || (0 == aString.length));
  },

  /**
   * Returns true if the given string is a valid email address.
   * @param aString the string to check.
   * @return true if the string is a valir email address. Returns false
   * otherwise. Note that this method returns false for empty an null strings.
   */
  isValidEmailAddress : function(aString) {
    return (!this.isNullOrEmpty(aString) && RE_EMAIL_ADDRESS.test(aString));
  },

  /**
   * Generates a UUID.
   * @return an UUID.
   */
  generateUUID : function() {
    var uuidGenerator =
      Cc["@mozilla.org/uuid-generator;1"].getService(Ci.nsIUUIDGenerator);
    var uuid = uuidGenerator.generateUUID().toString();

    // Remove the brackets from the generated UUID
    if (uuid.indexOf("{") == 0 ) {
      uuid = uuid.substring(1, (uuid.length - 1));
    }

    return uuid;
  },

  /**
   * Returns an array of elements that match the given aXPathExpression,
   * from the aXMLNode specified, using xpath.
   * @param aXMLNode the XML node or document to search for matches.
   * @param aXPathExpresion the xpath expression to be used in the search.
   * @return array with the elements that matched the xpath expression.
   */
  evaluateXPath : function (aXMLNode, aXPathExpression) {
    let matches = [];
    let xpe =
      Cc["@mozilla.org/dom/xpath-evaluator;1"].
        getService(Ci.nsIDOMXPathEvaluator);

    let nsResolver =
      xpe.createNSResolver(
        ((null == aXMLNode.ownerDocument) ? aXMLNode.documentElement :
         aXMLNode.ownerDocument.documentElement));
    let result = xpe.evaluate(aXPathExpression, aXMLNode, nsResolver, 0, null);
    let res = result.iterateNext();

    while (res) {
      matches.push(res);
      res = result.iterateNext();
    }

    return matches;
  },

  /**
   * Indicates if the given object is an array or not.
   * This code is based on code from:
   * http://snipplr.com/view/1996/typeof--a-more-specific-typeof/
   * @return true if the object is an array, false otherwise.
   */
  isArray : function(aObject) {
    let isArray =
      ((null != aObject) && ("object" == typeof(aObject)) &&
       (RE_ARRAY_CONSTRUCTOR.test(aObject.constructor.toString())));

    return isArray;
  },

  /**
   * Extracts file from the extension zip.
   * @param aFileName the file name to be extracted.
   * @param aFileNameTo the file name to be used when extracted.
   * @return result true if success, false otherwise.
   */
  extractFileFromZip : function(aFileName, aFileNameTo) {
    let result = false;
    let zipFile = eBayComp.getExtensionZip();

    if (zipFile.exists()) {
      let zipReader =
        Cc["@mozilla.org/libjar/zip-reader;1"].getService(Ci.nsIZipReader);
      let fileTo = eBayComp.getExtensionDirectory();
      let filePath = "defaults/" + aFileName;

      zipReader.open(zipFile);
      zipReader.test(null);

      fileTo.append(aFileNameTo);
      if (!fileTo.exists()) {
        fileTo.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0777);
        if (zipReader.hasEntry(filePath)) {
          zipReader.extract(filePath, fileTo);
        }
      }

      zipReader.close();
      result = true;
    }

    return result;
  },

  /**
   * Gets the URL from the active tab
   * @returns a string containing the URL from the active tab
   */
  getCurrentURL: function() {
    let currentURL = "";
    let windowMediator =
      Cc["@mozilla.org/appshell/window-mediator;1"].
        getService(Ci.nsIWindowMediator);
    let win = windowMediator.getMostRecentWindow("navigator:browser");

    if (null != win) {
      currentURL = win.gBrowser.selectedTab.linkedBrowser.currentURI.spec;
    }

    return currentURL;
  },

  /**
   * Determines if a string ends with another string
   * @param aStr the string where we will execute the search
   * @param aSuffix the string to look for
   */
  endsWith : function(aStr, aSuffix) {
    return aStr.indexOf(aSuffix, aStr.length - aSuffix.length) !== -1;
  },

  /**
   * Replaces the provided label with a series of label elements, of which those
   * that are intended to be clickable are assigned a callback.  The source
   * label should have a value containing text in the form "plain text [1 link]
   * plain text...".  The number at the start of the link construct is used as
   * an index into the provided "callbacks" array (1 -> 0th element etc...).
   * The retrieved element is assigned as on onclick handler.  This makes it
   * possible to reorder the text during localisation without changing the order
   * of elements in the "callbacks" array.  The link labels are assigned the
   * class "link" for styling.
   */
  explodeLabelWithLinks : function(label, callbacks) {
    let text = label.value;

    while (text.length > 0) {
      let element = label.cloneNode(false);

      if (text.charAt(0) == "[") {
        // link
        let endIndex = text.indexOf("]");
        let left = text.slice(0, endIndex + 1);
        let linkRe = /\[(\d+) (.*?)\]/g;
        linkRe.lastIndex = 0;
        let results = linkRe.exec(left);
        let index = results[1];

        element.value = results[2];
        element.setAttribute("class", "link");

        let callback = callbacks[index - 1];
        if (callback) {
          element.onclick = callback;
        }
        text = text.slice(endIndex + 1);
      } else {
        // text
        let endIndex = text.indexOf("[");
        if (endIndex == -1) {
          endIndex = text.length;
        }
        element.value = text.slice(0, endIndex);
        text = text.slice(endIndex);
      }

      label.parentNode.appendChild(element);
    }
    label.parentNode.removeChild(label);
  },

  /**
   * Gets the most recent window.
   * @returns the most recent window object.
   */
  getMostRecentWindow: function() {
    let windowMediator =
      Cc["@mozilla.org/appshell/window-mediator;1"].
        getService(Ci.nsIWindowMediator);
    let mostRecentWindow =
      windowMediator.getMostRecentWindow("navigator:browser");

    return mostRecentWindow;
  }

};
