/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["WarningNotificationHelper"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

var WarningNotificationHelper = {

  // Notification types

  // Warning notifications
  BEST_OFFER_NOT_SUPPORTED_WARNING : 10,
  LIST_LIMITED_WARNING : 11,
  API_CONNECTION_WARNING : 12,
  ADD_TO_WATCHLIST_ERROR_WARNING : 13,
  REMOVE_FROM_WATCHLIST_ERROR_WARNING : 14,
  HARD_UPDATE_ERROR_WARNING : 15,
  ITEM_MOVED_TO_ENDED_TAB_WARNING : 16,
  BIDDING_FILTER_WARNING : 17,
  WATCHING_FILTER_WARNING : 18,
  SELLING_FILTER_WARNING : 19,
  WATCHED_FILTER_WARNING : 20,
  WON_FILTER_WARNING : 21,
  LOST_FILTER_WARNING : 22,
  SOLD_FILTER_WARNING : 23,
  OUTBID_FILTER_WARNING : 24,
  WINNING_FILTER_WARNING : 25,
  OFFERS_FILTER_WARNING : 26,
  UNSOLD_FILTER_WARNING : 27,
  SIGNED_OUT_WARNING : 28,
  LOGIN_FAILED_WARNING : 29,
  SECURITY_SIGN_OUT_WARNING : 30,
  REMOVE_FROM_DIDNTWINLIST_ERROR_WARNING : 31,
  REMOVE_FROM_WONLIST_ERROR_WARNING : 32,
  //GIFTS_API_ERROR_WARNING : 33,
  ACCOUNT_BLOCKED_WARNING : 34,

  // we don't queue warning notifications, we just overwrite the existing one
  // if any. This are displayed on sidebar's top
  _warnNotification : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/common.js");
    Cu.import("resource://ebaycompanion/helpers/logger.js");
    Cu.import("resource://ebaycompanion/helpers/observers.js");

    Observers.add(this, Topics.QUIT_APPLICATION_GRANTED);
  },

  /**
   * Uninitializes the resource.
   */
  _uninit : function() {
    this._clearQueue();
    Observers.removeAll(this);
  },

  /**
   * Shows the pending notifications by getting the next one (if any) and
   * notifying the appropriate observer
   */
  showPendingNotifications : function() {
    try {
      let nextNotification = this.getNextNotification();

      if (null != nextNotification) {
        // XXX: hack to prevent the filter notifications from not being
        // displayed when they should when the user switches tabs very quickly
        if (typeof(Timer) == "undefined") {
          Cu.import("resource://ebaycompanion/helpers/timer.js");
        }
        new Timer(function() {
          Observers.notify(nextNotification, Topics.NOTIFICATION_WARN_SHOW, null);
        }, 100, Timer.TYPE_ONE_SHOT);
      } else {
        // no next notification, then clear the notification boxes (if there
        // is any displayed)
        Observers.notify(null, Topics.NOTIFICATION_WARN_CLEAR, null);
      }
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Returns the next warning notification to be displayed.
   * @return the next notification to be displayed
   */
  getNextNotification : function() {
    let next = null;
    try {
      next = this._warnNotification;
    } catch(e) {
      Logger.exception(e, Logger.DUMP_STACK)
      return null;
    }
    return next;
  },

  /**
   * Queues the notification passed as parameter.
   * Notifies the appropriate observer so the notification is
   * displayed in all windows sidebar.
   */
  queueNotification : function(aNotification) {
    try {
      // set notification image
      let type = aNotification.get("type");
      // for warning notifications we just replace the current
      // warnNotification (if any) and notify the observer
      this._warnNotification = aNotification;
      // finally, this refreshes all the sidebars with the latest
      // notification
      this.showPendingNotifications();
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

   /**
   * Dimisses an alert by removing it from the queue
   * @param aNotificationType the type of notification to be dismissed
   */
  dismissNotification : function(aNotificationType) {
    this._removeNotificationsByType(aNotificationType);
    this.showPendingNotifications();
    // all filter notifications dismissal notifies the observer to reset
    // active or ended tab filtering
    if (aNotificationType >= this.BIDDING_FILTER_WARNING &&
        aNotificationType <= this.UNSOLD_FILTER_WARNING) {
      Observers.notify(null, Topics.LIST_RESET_CURRENT_FILTER, null);
    }
  },

  /**
   * Dismisses all filter notifications
   */
  dismissFilterNotifications : function() {
    for (var i = WarningNotificationHelper.BIDDING_FILTER_WARNING;
          i <= WarningNotificationHelper.UNSOLD_FILTER_WARNING;
          i++) {
      this._removeNotificationsByType(i);
    }
    this.showPendingNotifications();
  },

  /**
   * Dismisses signed in notifications
   */
  dismissSignedInNotifications : function() {
    this.dismissFilterNotifications();
    // purge the notification box
    Observers.notify(null, Topics.NOTIFICATION_SIGNED_IN_WARN_CLEAR, null);
  },

  /**
   * Removes all the notifications of the given type so we don't have duplicates
   * @param aType the type of notifications to be removed from the array
   */
  _removeNotificationsByType : function(aType) {
    try {
      if (this._warnNotification &&
          aType == this._warnNotification.get("type")) {
        this._warnNotification = null;
      }
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Removes the warning notification from the system. Then requests the
   * display of any pending notifications
   */
  removeCurrentNotification : function(aIsPersistent) {
    try {
      // remove the warning notification for warn notifications
      this._warnNotification = null;
      // show any pending notifications
      this.showPendingNotifications();
    } catch(e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Clears the warning notification
   */
  _clearQueue : function() {
    try {
      this._warnNotification = null;
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Clears warnNotification variables when the sidebar is closed, so the next
   * time it is opened, all windows are refreshed properly.
   */
  clearCurrentNotification : function() {
    try {
      // non-persistent warning notifications should be removed here too
      if (null != this._warnNotification &&
          !this._warnNotification.get("isPersistent")) {
        this._warnNotification = null;
      }
    } catch(e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    if (Topics.QUIT_APPLICATION_GRANTED == aTopic) {
      this._uninit();
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(WarningNotificationHelper);
