/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["Deal"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
Cu.import("resource://ebaycompanion/constants.js");

/**
 * Ebay deal object.
 * @param itemId
 */
function Deal(itemId) {
  if (itemId) {
    this.set("itemId", itemId);
  }

}

Deal.prototype = {
  /**
   * Used to determine the type of the object
   */
  constructor : Deal,

  /**
   * Types for each property of this object
   */
  propertyTypes : {
    itemId :               "number",
    title :                "string",
    isEnded :              "boolean",
    currentPrice :         "number",
    currency :             "string",
    rrp :                  "number",
    rrpCurrency :          "string",
    shippingCost :         "number",
    freeShipping :         "boolean",
    quantitySold :         "number",
    quantity :             "number",
    startTime :            "string",
    endTime :              "string",
    imageUrl :             "string",
    thumbnailUrl :         "string",
    savingsRate :          "number",
    savingsRateText :      "string",
    sellerUserId :         "string",
    sellerFeedbackRating : "number",
    sellerFeedbackPercent : "number",
    pageViews :            "number",
    viewItemUrl :          "string"
  },

  /**
   * XML nodes related to each property
   */
  propertyXML : {
    itemId :               "@ItemID",
    title :                "ItemDetails.Title",
    isEnded :              "ItemMetaData.IsExpired",
    currentPrice :         "ItemDetails.ConvertedCurrentPrice",
    currency :             "ItemDetails.ConvertedCurrentPrice.@CurrencyID",
    rrp :                  "ItemDetails.RRP",
    rrpCurrency :          "ItemDetails.RRP.@CurrencyID",
    shippingCost :         "ItemDetails.ShippingPrice",
    freeShipping :         "ItemMetaData.FreeShipping",
    quantitySold :         "ItemMetaData.QuantitySold",
    quantity :             "ItemMetaData.Quantity",
    startTime :            "@DateAdded",
    endTime :              "ItemMetaData.EndTime",
    imageUrl :             "ItemDetails.PictureURL",
    thumbnailUrl :         "ItemDetails.GalleryURL",
    savingsRate :          "ItemMetaData.SavingsRate",
    savingsRateText :      "ItemMetaData.SavingsRateText",
    sellerUserId :         "",
    sellerFeedbackRating : "",
    sellerFeedbackPercent : "",
    pageViews :            "",
    viewItemUrl :          ""
  },

  /**
   * Returns a copy of this object.
   * @return the copy of this object.
   */
  copy : function() {
    let copy = new Deal(this.get("itemId"));
    copy.updateQuietlyTo(this);
    return copy;
  },

  /**
   * Updates this object with the new object.
   * @param aNewObject the new object.
   * @param aFlags the set of flags.
   * @param aIgnoreFields an array with the names of the fields to be ignored
   * during the update
   * @return the result.
   */
  updateTo : function(aNewObject, aFlags, aIgnorePropertiesArray) {
    return ObjectHelper.updateObject(
      this, aNewObject, "ebay-deal-property-updated",
      aFlags, aIgnorePropertiesArray);
  },

  /**
   * Updates quietly this object with the new object.
   * @param aNewObject the new object.
   * @return the result.
   */
  updateQuietlyTo : function(aNewObject) {
    return this.updateTo(aNewObject, ObjectHelper.flags.NO_NOTIFICATIONS);
  },

  /**
   * Updates a property of the object.
   * @param aProperty the property name.
   * @param aValue the property value.
   * @return the result.
   */
  updateProperty : function(aProperty, aValue) {
    return ObjectHelper.updateProperty(
      this, aProperty, aValue, "ebay-deal-property-updated");
  },

  /**
   * Returns the value of the given property.
   * @param aProperty the property name.
   * @return the property value.
   */
  get : function(aProperty) {
    return ObjectHelper.getProperty(this, aProperty);
  },

  /**
   * Sets the given property to the given value, checking that the value is of
   * the correct type for the property
   * @param aProperty the property name.
   * @param aValue the property value.
   */
  set : function(aProperty, aValue) {
    aValue = ObjectHelper.filterValue(
      aProperty, this.propertyTypes[aProperty], aValue);
    this["_" + aProperty] = aValue;
  },

  /**
   * Populates the item information with the data contained in the xmlNode
   * passed as parameter
   * @param aXMLItem the xml item to extract the data from
   */
  fromJXONObject : function(aJXONObject) {
    var deal = ObjectHelper.fromJXON(aJXONObject, this);
    this.updateTo(deal);

    // startTime and endTime
    var startDate = aJXONObject["@DateAdded"];

    if (startDate)
      this.set("startTime", String(startDate.getTime()));

    var endDate;
    if (aJXONObject["ItemMetaData"] && aJXONObject["ItemMetaData"]["EndTime"]) {
      endDate = aJXONObject["ItemMetaData"]["EndTime"];
    } else {
      endDate = aJXONObject["EndTime"];
    }
    if (endDate)
      this.set("endTime", String(endDate.getTime()));
  },

  /**
   * Gets the timestamp from a given time string.
   * @param aTimeString the time string.
     2010-12-07T22:25:38
     2010-12-07T19:07:49.494Z
     2010-11-28T22:25:23Z
   * @return the timestamp
   */
  _getTimestamp : function(aTimeString) {
    let dateRegExp = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})Z$/;
    let dateResults = dateRegExp.exec(aTimeString);
    let date = new Date();
    let timestamp = null;

    if (dateResults) {
      // the date object deals with months from 0-11, not from 1-12
      date.setUTCFullYear(dateResults[1], dateResults[2]-1, dateResults[3]);
      date.setUTCHours(dateResults[4], dateResults[5], dateResults[6]);
    } else {
      Logger.error(
        "Incompatible date format: " + aTimeString, Logger.DUMP_STACK);
    }
    timestamp = String(date.getTime());

    return timestamp;
  }
};
