/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["Transaction"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://ebaycompanion/constants.js");

/**
 * eBay Transaction
 * @param itemId
 * @param transactionId
 */
function Transaction(itemId, transactionId) {
  if (itemId) {
    this.set("itemId", itemId);
  }

  if (transactionId) {
    this.set("transactionId", transactionId);
  }
}

Transaction.prototype = {
  /**
   * Used to determine the type of the object
   */
  constructor : Transaction,

  /**
   * Types for each property of this object
   */
  propertyTypes : {
    itemId :                  "number",
    transactionId :           "number",
    creationTime :            "number",
    quantityPurchased :       "number",
    buyerUserId :             "string",
    isPaidFor :               "boolean",
    isShipped :               "boolean",
    userHasSentFeedback :     "boolean",
    userHasReceivedFeedback : "boolean",
    feedbackReceivedType :    "string",
    transactionPrice :        "number",
    transactionPriceCurrency :  "string",
    convertedTransactionPrice :  "number",
    convertedTransactionPriceCurrency :  "string"
  },

  /**
   * XML nodes related to each property
   */
  propertyXML : {
    itemId :                  "Item.ItemID",
    transactionId :           "TransactionID",
    creationTime :            "",
    quantityPurchased :       "QuantityPurchased",
    buyerUserId :             "Buyer.UserID",
    isPaidFor :               "",
    isShipped :               "",
    userHasSentFeedback :     "",
    userHasReceivedFeedback : "",
    feedbackReceivedType :    "FeedbackReceived.CommentType",
    transactionPrice :        "TransactionPrice|TotalTransactionPrice|Item.SellingStatus.CurrentPrice",
    transactionPriceCurrency :  "TransactionPrice.@currencyID|TotalTransactionPrice.@currencyID|Item.SellingStatus.CurrentPrice.@currencyID",
    convertedTransactionPrice :  "ConvertedTransactionPrice",
    convertedTransactionPriceCurrency :  "ConvertedTransactionPrice.@currencyID"
  },

  /**
   * Returns a copy of this object
   */
  copy : function() {
    let copy = new Transaction(this.get("itemId"), this.get("transactionId"));
    copy.updateQuietlyTo(this);
    return copy;
  },

  /**
   * updateTo
   */
  updateTo : function(newObject, flags) {
    return ObjectHelper.updateObject(
      this, newObject, Topics.TRANSACTION_PROPERTY_UPDATED, flags);
  },

  /**
   * updateQuietlyTo
   */
  updateQuietlyTo : function(newObject) {
    return this.updateTo(newObject, ObjectHelper.flags.NO_NOTIFICATIONS);
  },

  /**
   * updateProperty
   */
  updateProperty : function(property, value) {
    return ObjectHelper.updateProperty(
      this, property, value, Topics.TRANSACTION_PROPERTY_UPDATED);
  },

  /**
   * Returns the value of the given property
   */
  get : function(property) {
    return ObjectHelper.getProperty(this, property);
  },

  /**
   * Sets the given property to the given value, checking that the value is of
   * the correct type for the property
   */
  set : function(property, value) {
    value = ObjectHelper.
              filterValue(property, this.propertyTypes[property], value);
    this["_" + property] = value;
  },

  fromJXONObject : function(aJXONObject) {
    var transaction = ObjectHelper.fromJXON(aJXONObject, this);
    this.updateTo(transaction);

    // creationTime
    this.set("creationTime", aJXONObject["CreatedDate"].getTime());

    // If there's no buyer in the response, it means the user is the buyer(won item). Set it to reflect that.
    if (!this.get("buyerUserId")) {
      this.set("buyerUserId",  Datasource.activeAccount().get("userId"));
    }
    var paidStatus;
    switch (aJXONObject["SellerPaidStatus"]) {
      case "MarkedAsPaid":
      case "PaidWithEscrow":
      case "PaidWithPaisapay":
      case "PaidWithPayPal":
        paidStatus = true;
        break;
      default:
        paidStatus = false;
    }

    // if paid status is false at this point, check BuyerPaidStatus
    if (!paidStatus && null != aJXONObject["BuyerPaidStatus"]) {
      switch (aJXONObject["BuyerPaidStatus"]) {
        case "MarkedAsPaid":
        case "PaidWithEscrow":
        case "PaidWithPaisapay":
        case "PaidWithPayPal":
          paidStatus = true;
          break;
        default:
          paidStatus = false;
      }
    }

    this.set("isPaidFor", paidStatus);

    this.set("isShipped", (aJXONObject["ShippedTime"] ? true : false));
    this.set("userHasSentFeedback", (aJXONObject["FeedbackLeft"] ? true : false));
    this.set("userHasReceivedFeedback", (aJXONObject["FeedbackReceived"] ? true : false));
  }


};
