/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["AlertsGenerator"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

var AlertsGenerator = {
  /* Boolean indicates whether the alerts manager is enabled or not. */
  _enabled : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/common.js");
    Cu.import("resource://ebaycompanion/helpers/logger.js");
    Cu.import("resource://ebaycompanion/helpers/observers.js");
    Cu.import("resource://ebaycompanion/helpers/itemList.js");
    Cu.import("resource://ebaycompanion/helpers/timer.js");
    Cu.import("resource://ebaycompanion/helpers/warningNotificationHelper.js");
    Cu.import("resource://ebaycompanion/datasource.js");
    Cu.import("resource://ebaycompanion/constants.js");
    Cu.import("resource://ebaycompanion/objects/alert.js");
    Cu.import("resource://ebaycompanion/objects/item.js");
    Cu.import("resource://ebaycompanion/objects/feedback.js");
    Cu.import("resource://ebaycompanion/objects/notification.js");

    this._enabled = false;

    Observers.add(this, Topics.QUIT_APPLICATION_GRANTED);
    Observers.add(this, Topics.UPDATE_FINISHED);
    Observers.add(this, Topics.ITEM_NEW);
    Observers.add(this, Topics.ITEM_CHANGED);
    Observers.add(this, Topics.TRANSACTION_NEW);
    Observers.add(this, Topics.ACCOUNT_FEEDBACK_CHANGED);
  },

  /**
   * Uninitializes the resource.
   */
  _uninit : function() {
    Observers.removeAll(this);
  },

  /**
   * Enables alerts
   */
  enable : function() {
    let that = this;

    // Item list used to track the order in which items will end
    this._endingSoonList = new ItemList();
    this._endingSoonAlerted = {};  // Keeps track of which alerts we've generated
    this._enabled = true;
    this._endingSoonList.
      filter(function(item) {
        let alreadyAlerted = that._endingSoonAlerted[item.get("itemId")];
        return !item.get("isEnded") && !alreadyAlerted;
      });

    this._endingSoonTimer =
      new Timer(function() that._checkForEndingSoon(), 1000 * 60,
                Timer.TYPE_REPEATING_SLACK);
    this._checkForEndingSoon();
  },

  /**
   * Disables alerts
   */
  disable : function() {
    this._enabled = false;
    if (this._endingSoonAlerted) {
      this._endingSoonTimer.cancel();
    }
  },

  /**
   * Checks the item that will end soonest to see if an Ending Soon alert should
   * be generated for it.
   */
  _checkForEndingSoon : function() {
    try {
      // We need to apply the filter again to filter out any items that have
      // already had alerts generated.
      // Because we only need to filter items out of (not into) our Item List,
      // we can simply apply the filter again without needing to reset the list.
      this._endingSoonList.filter();
      this._endingSoonList.
        sort(function(a, b) {
          return a.get("endTime") < b.get("endTime");
        });
      let items = this._endingSoonList.filteredItems;

      let mayBeMore = true;
      for (let i = 0; mayBeMore && i < items.length; i++) {
        let item = items[i];
        mayBeMore = this._checkItemForEndingSoon(item);
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Generates an appropriate "Ending Soon" alert if necessary for the provided
   * item
   * @param item An Item object
   * @returns true if an alert was generated; false otherwise
   */
  _checkItemForEndingSoon : function(item) {
    let alertGenerated = false;
    let that = this;

    if (item.get("isEnded")) {
      // if the new item comes and it is ended, show the "ended items moved to
      // ended tab" notification
      // flag to prevent item ended notification be shown multiple times during
      // a hard update
      if (!this._preventItemEndedNotification) {
        // show ended items moved to ended list notification
        let notification =
          new Notification(
            WarningNotificationHelper.
              ITEM_MOVED_TO_ENDED_TAB_WARNING);
        notification.addLinkCallback(0, function(aEvent){
            let windowMediator =
              Cc["@mozilla.org/appshell/window-mediator;1"].
                getService(Ci.nsIWindowMediator);
            let mostRecentWindow =
              windowMediator.getMostRecentWindow("navigator:browser");
            let sidebar = mostRecentWindow.document.getElementById("sidebar");
            sidebar.contentWindow.Sidebar.switchToTab(4);
          });
        notification.set(
          "content", Constants.stringBundle.getString(
            "notification.info.ended"));
        notification.set("priority", 1);
        WarningNotificationHelper.queueNotification(notification);
        this._preventItemEndedNotification = true;
        new Timer(
          function() {
            that._preventItemEndedNotification = false; },
          1000 * 30,
          Timer.TYPE_ONE_SHOT);
        return alertGenerated;
      }
    }

    let userIsSeller =
      item.get("sellerUserId").toLowerCase() ==
        Datasource.activeAccount().get("userId").toLowerCase();
    let hasTransactions =
      Datasource.transactions([item.get("itemId")]) != null;

    // If the user is not the seller, and the item has transactions, that means
    // the user bought the item.  We treat such items as ended and are not
    // interested in when the listing ends.
    if (hasTransactions && !userIsSeller) {
      return alertGenerated;
    }

    let timeLeft = item.get("endTime") - Datasource.getEbayTime().getTime();
    let minutesLeft = timeLeft / 1000 / 60;
    if (timeLeft > 0 && minutesLeft < 20) {
      if (userIsSeller) {
        this._generateAlert(Alert.ALERT_TYPE_SELLING_ENDING_SOON, item);
      } else {
        let userHasBid = item.get("userMaxBid") > 0;
        if (userHasBid) {
          this._generateAlert(Alert.ALERT_TYPE_BIDDING_ENDING_SOON, item);
        } else {
          this._generateAlert(Alert.ALERT_TYPE_WATCHING_ENDING_SOON, item);
        }
      }
      this._endingSoonAlerted[item.get("itemId")] = true;
      alertGenerated = true;
    } else if (timeLeft <= 0) {
      // flag to prevent item ended notification be shown multiple times during
      // a hard update
      if (!this._preventItemEndedNotification) {
        // show ended items moved to ended list notification
        let notification =
          new Notification(
            WarningNotificationHelper.
              ITEM_MOVED_TO_ENDED_TAB_WARNING);
        notification.addLinkCallback(0, function(aEvent){
            let windowMediator =
              Cc["@mozilla.org/appshell/window-mediator;1"].
                getService(Ci.nsIWindowMediator);
            let mostRecentWindow =
              windowMediator.getMostRecentWindow("navigator:browser");
            let sidebar = mostRecentWindow.document.getElementById("sidebar");
            sidebar.contentWindow.Sidebar.switchToTab(4);
          });
        notification.set(
          "content", Constants.stringBundle.getString(
            "notification.info.ended"));
        notification.set("priority", 1);
        WarningNotificationHelper.queueNotification(notification);
        alertGenerated = true;
        this._preventItemEndedNotification = true;
        new Timer(
          function() {
            that._preventItemEndedNotification = false; },
          1000 * 30,
          Timer.TYPE_ONE_SHOT);
      }
    }

    return alertGenerated;
  },

  /**
   * Called when a new item is being tracked
   * @param item The new item object
   */
  _newItem : function(item) {
    try {
      this._checkItemForEndingSoon(item);
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Called when an item object has been changed
   * @param item The Item object that has changed
   * @param originalItem The Item object before the change occured
   */
  _itemChanged : function(item, originalItem) {
    let userId = Datasource.activeAccount().get("userId");
    let userIsSeller =
      item.get("sellerUserId").toLowerCase() == userId.toLowerCase();
    let itemJustEnded =
      item.get("isEnded") != originalItem.get("isEnded");
    let userHasBid = item.get("userMaxBid") > 0;
    let highBidder = item.get("highBidderId");
    let userIsHighBidder =
      highBidder && (highBidder.toLowerCase() == userId.toLowerCase());
    let oldHighBidder = originalItem.get("highBidderId");
    let userWasHighBidder =
      oldHighBidder && (oldHighBidder.toLowerCase() == userId.toLowerCase());
    let originalOfferStatus = originalItem.get("bestOfferStatus");
    let currentOfferStatus = item.get("bestOfferStatus");
    let offerWasPending =
      (originalOfferStatus && originalOfferStatus.toLowerCase() == "pending");
    let offerWasCountered =
      (originalOfferStatus &&
         originalOfferStatus.toLowerCase() == "countered");
    let offerIsDeclined =
      (currentOfferStatus && currentOfferStatus.toLowerCase() == "declined");
    let offerIsExpired =
      (currentOfferStatus && currentOfferStatus.toLowerCase() == "expired");
    let offerIsCountered =
      (currentOfferStatus &&
         currentOfferStatus.toLowerCase() == "countered");
    let bestOfferDeclined =
      (offerWasPending || offerWasCountered) && offerIsDeclined;
    let bestOfferExpired =
      (offerWasPending || offerWasCountered) && offerIsExpired;
    let bestOfferCountered = offerWasPending && offerIsCountered;

    let EC_LISTING_TYPE_FIXED = "FixedPriceItem";
    let EC_LISTING_TYPE_STORE_FIXED = "StoresFixedPrice";
    let itemHasBuyItNow = item.get("hasBuyItNow");
    let listingFormat = item.get("listingFormat");

    let isBINItem = (EC_LISTING_TYPE_STORE_FIXED == listingFormat ||
        EC_LISTING_TYPE_FIXED == listingFormat) && itemHasBuyItNow;
    // the first condition here applies for BIN items with multiple units. The
    // second condition applies to single unit BIN items. They behave
    // differently because for multiple units BIN items, the user is NEVER set
    // as high bidder and for single unit BIN items, the userQuantityWinning
    // column is never set.
    let buyerWon =
      (!userIsSeller &&
      item.get("userQuantityWinning") > originalItem.get("userQuantityWinning") &&
      item.get("quantityRemaining") < originalItem.get("quantityRemaining")) ||
      (!userIsSeller &&
      !userWasHighBidder && userIsHighBidder &&
      item.get("quantityRemaining") < originalItem.get("quantityRemaining"));

    if (userIsSeller) {
      // Selling
      let soldNone = item.get("quantitySold") == 0;
      let newBidPlaced = item.get("numBids") > originalItem.get("numBids");
      let newOfferPlaced =
        item.get("bestOfferCount") > originalItem.get("bestOfferCount");

      if (itemJustEnded && soldNone) {
        this._generateAlert(Alert.ALERT_TYPE_SELLING_ITEM_UNSOLD, item);
      } else if (newBidPlaced) {
        this._generateAlert(Alert.ALERT_TYPE_SELLING_BID_PLACED, item);
      } else if (itemJustEnded) {
        this._generateAlert(Alert.ALERT_TYPE_ITEM_ENDED, item);
      } else if (newOfferPlaced) {
        this._generateAlert(Alert.ALERT_TYPE_SELLING_BESTOFFER_NEWOFFER, item);
      }
    } else if (userHasBid || (isBINItem && buyerWon)) {
      // Bidding
      let priceIncreased =
        item.get("currentPrice") > originalItem.get("currentPrice");

      if (itemJustEnded) {
        if (!userIsHighBidder) {
          this._generateAlert(Alert.ALERT_TYPE_BIDDING_ITEM_LOST, item);
        }
      } else if (!isBINItem) {
        if (priceIncreased) {
          if (userIsHighBidder && userWasHighBidder) {
            this._generateAlert(Alert.ALERT_TYPE_BIDDING_RAISED_BID, item);
          } else if(userIsHighBidder && !userWasHighBidder) {
            this._generateAlert(Alert.ALERT_TYPE_BIDDING_HIGH_BIDDER, item);
          } else if (!userIsHighBidder && userWasHighBidder) {
            this._generateAlert(Alert.ALERT_TYPE_BIDDING_OUTBID, item);
          } else if (!userIsHighBidder && !userWasHighBidder && userHasBid &&
            originalItem.getCurrentState(userId) == Item.ITEM_STATE_WATCHING) {
            // this is the case where the user adds the item to the watchlist
            // then bids his bid is not enough to become the highest bidder
            // so the user is outbid
            this._generateAlert(Alert.ALERT_TYPE_BIDDING_OUTBID, item);
          }
        } else {
          if (!userWasHighBidder && userIsHighBidder) {
            // this means the user became the highest bidder by placing the
            // first bid on an item.
            this._generateAlert(Alert.ALERT_TYPE_BIDDING_HIGH_BIDDER, item);
          }
        }
      }
    } else if (isBINItem && bestOfferDeclined) {
      // the best offer was declined by the seller
      this._generateAlert(Alert.ALERT_TYPE_BIDDING_BESTOFFER_DECLINED, item);
    } else if (isBINItem && bestOfferExpired) {
      // the best offer expired
      this._generateAlert(Alert.ALERT_TYPE_BIDDING_BESTOFFER_EXPIRED, item);
    } else if (isBINItem && bestOfferCountered) {
      // the best offer countered
      this._generateAlert(Alert.ALERT_TYPE_BIDDING_BESTOFFER_COUNTERED, item);
    }
  },

  /**
   * Called when a new transaction is being tracked
   * @param transaction The new transaction object
   */
  _newTransaction : function(transaction) {
    let item = Datasource.items()[transaction.get("itemId")];
    // if the item is already in the local datasource, then we can show the
    // alert
    if (item) {
      let userIsSeller =
        item.get("sellerUserId").toLowerCase() ==
          Datasource.activeAccount().get("userId").toLowerCase();
      let endedCopy = item.copy();
      endedCopy.set("isEnded", true);

      // include a copy of the transaction with the item, so we can update
      // the alert status when the transaction gets updated
      endedCopy.transaction = transaction.copy();

      let alertType;

      let generateAlert = true;

      if (userIsSeller) {
        // the user sold an item
        endedCopy.type = Item.ITEM_TYPE_SOLD;
        // depending on the final transaction prices, this could be a regular
        // transaction or a best offer one if the transaction price is different
        // to the item's current price.
        if (endedCopy.get("currentPrice") !=
            endedCopy.transaction.get("transactionPrice")) {
          alertType = Alert.ALERT_TYPE_SELLING_BESTOFFER_ITEM_SOLD;
        } else {
          alertType = Alert.ALERT_TYPE_SELLING_ITEM_SOLD;
        }
      } else {
        let EC_LISTING_TYPE_FIXED = "FixedPriceItem";
        let EC_LISTING_TYPE_STORE_FIXED = "StoresFixedPrice";

        let listingFormat = endedCopy.get("listingFormat");
        // this is a BIN item, so we don't generate a desktop alert for them
        // (as per Jonathan's request)
        if ((EC_LISTING_TYPE_STORE_FIXED == listingFormat ||
            EC_LISTING_TYPE_FIXED == listingFormat)) {
          // depending on the final transaction prices, this could be a regular
          // transaction or a best offer one if the transaction price is
          // different to the item's current price. For best offer sales, we
          // DO display an alert to the user
          if (endedCopy.get("currentPrice") !=
              endedCopy.transaction.get("transactionPrice")) {
            // the user won an auction item
            endedCopy.type = Item.ITEM_TYPE_WON;
            alertType = Alert.ALERT_TYPE_BIDDING_BESTOFFER_ITEM_WON;
          } else {
            generateAlert = false;
          }
        } else {
          // the user won an auction item
          endedCopy.type = Item.ITEM_TYPE_WON;
          alertType = Alert.ALERT_TYPE_BIDDING_ITEM_WON;
        }
      }

      if (generateAlert) {
        this._generateAlert(alertType, endedCopy);
      }
    }
  },

  /**
   * Called when the active account feedback changes.
   * @param aAccount The Account object that has changed
   * @param aOriginalAccount The Account object before the change occured
   */
  _accountFeedbackScoreChanged : function(aAccount, aOriginalAccount) {
    // just to be sure the feedback actually changed... and possitively
    let oldFeedbackValue = aOriginalAccount.get("feedbackRating");
    let newFeedbackValue = aAccount.get("feedbackRating");
    let preventAlertDisplay =
      Constants.prefBranch.get("prevent.first.feedback.score.notification");
    if (newFeedbackValue > oldFeedbackValue && !preventAlertDisplay) {
      let listLength = Feedback.MILESTONES.length;
      let showedScoreAlert = false;
      let feedback;

      for (var i=listLength-1; !showedScoreAlert && i>=0; i--) {
        if ((Feedback.MILESTONES[i] > oldFeedbackValue) &&
            (Feedback.MILESTONES[i] <= newFeedbackValue)) {
          feedback = new Feedback();
          feedback.score = Feedback.MILESTONES[i];
          this._generateAlert(Alert.ALERT_TYPE_FEEDBACK_SCORE, feedback);
          showedScoreAlert = true;
        }
      }
    }
  },

  /**
   * Creates and dispatches an alert with the given name for the given item
   * @param aType the type of the alert
   * @param aObject used to hold the item that caused the alert or the feedback
   * information in case of feedback alerts
   */
  _generateAlert : function(aType, aObject) {
    let alert = new Alert(aType, aObject);
    alert.dispatch();
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case Topics.QUIT_APPLICATION_GRANTED:
        this._uninit();
        break;
      case Topics.UPDATE_FINISHED:
        if (aData == "Hard Update") {
          if (!this._enabled) {
            this.enable();
          }
        }
        break;
      case Topics.ITEM_NEW:
        if (this._enabled) {
          this._newItem(aSubject.object);
        }
        break;
      case Topics.ITEM_CHANGED:
        if (this._enabled) {
          this._itemChanged(aSubject.object, aSubject.originalObject);
        }
        break;
      case Topics.TRANSACTION_NEW:
        if (this._enabled) {
          this._newTransaction(aSubject.object);
        }
        break;
      case Topics.ACCOUNT_FEEDBACK_CHANGED:
        this._accountFeedbackScoreChanged(
          aSubject.object, aSubject.originalObject);
        break;
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(AlertsGenerator);
