/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 * implemented based on https://developer.mozilla.org/en/Creating_Sandboxed_HTTP_Connections
 */

var EXPORTED_SYMBOLS = ["ApiAccessService"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

var ApiAccessService = {
  // the version of the API to be used
  _apiVersion : null,
  // the site id to be used in the requests
  _siteId : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/helpers/logger.js");
    Cu.import("resource://ebaycompanion/constants.js");
    //Cu.import("resource://ebaycompanion/datasource.js");
    Cu.import("resource://ebaycompanion/helpers/httpRequestHeaderVisitor.js");
    Cu.import("resource://ebaycompanion/helpers/accessKeysHelper.js");

    this._apiVersion = 627;
    this._siteId = 0;
  },

  /**
   * Returns the site id currently in use
   */
  get siteId() {
    return this._siteId;
  },

  /**
   * Sets the site id to the given value
   * @param aValue the value to be set
   */
  set siteId(aValue) {
    this._siteId = aValue;
  },

  /**
   * Returns the apiVersion currently in use
   */
  get apiVersion() {
    return this._apiVersion;
  },

  /**
   * Sets the api version to the given value
   * @param aValue the value to be set
   */
  set apiVersion(aValue) {
    this._apiVersion = aValue;
  },

  /**
   * Visits the request headers and dumps their values
   */
  visitRequestHeaders : function(aHttpChannel) {

    let visitor = new HttpRequestHeaderVisitor(aHttpChannel);
    // this is necessary to extract all the information from the
    // request
    let requestHeaders = visitor.inspectRequest();
    // here we iterate the request headers if necessary
    let message = "REQUEST HEADERS\n";
    for each (let [name, value] in Iterator(requestHeaders)) {
      message += name + ": " + value + "\n";
    }
    let postData = visitor.getPostData();
    if (postData) {
      message += "POST DATA:\n";
      message += postData + "\n";
    }
    Logger.error(message);
  },

  /**
   * (ecIEbayAccessComponent)
   * Performs a Trading API call of the given name using the data provided, and
   * ensures that the given callback will be invoked when the call returns.
   */
  doTradingAPICall : function(callName, body, useSandbox, callback) {

    // Determine the correct URL to use
    let url;
    if (!useSandbox) {
      // Production
      url = "https://api.ebay.com/ws/api.dll";
    } else {
      // Sandbox
      url = "https://api.sandbox.ebay.com/ws/api.dll";
    }

    // Get hold of the necessary access keys
    let keyset = AccessKeysHelper.getKeySet(useSandbox, this._siteId);

    // Create a HTTP channel
    let httpChannel = this.makeChannelForUrl(url);

    // set the request headers.
    httpChannel.setRequestHeader(
      "X-EBAY-API-COMPATIBILITY-LEVEL", this._apiVersion, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-DEV-NAME", keyset.devName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-APP-NAME", keyset.appName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-CERT-NAME", keyset.certName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-CALL-NAME", callName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-SITEID", this._siteId, false);
    /*httpChannel.setRequestHeader(
      "X-EBAY-API-IAF-TOKEN", Datasource.activeAccount().get("token"), false);*/

    // set the extension version as request header (as suggested by Dan Power)
    httpChannel.setRequestHeader(
      "X-EBAY-SIDEBAR-VERSION", Constants.prefBranch.get("version"), false);


    // Set POST data and send the request
    this.setChannelPostData(httpChannel, body);

    // uncomment to visit request headers
    // we visit the headers before actually sending the request because once
    // we send it, we might lose the POST body content
    //this.visitRequestHeaders(httpChannel);

    let request = this.sendRequest(httpChannel, callback);

    return request;
  },

  /**
   * (ecIEbayAccessComponent)
   * Performs a Shopping API call of the given name using the data provided, and
   * ensures that the given callback will be invoked when the call returns.
   */
  doShoppingAPICall : function(callName, body, useSandbox, callback,
                               resultsFormat) {

    // Determine the correct URL to use
    let url;
    if (!useSandbox) {
      // Production
      url = "http://open.api.ebay.com/shopping?";
    } else {
      // Sandbox
      url = "http://open.api.sandbox.ebay.com/shopping?";
    }

    // Get hold of the necessary access keys
    let keyset = AccessKeysHelper.getKeySet(useSandbox, this._siteId);

    //when we are getting JSON formatted responses, we need to send the callname
    //in the URL. This seems like an ebay API bug
    url += "callname=" + callName;

    // Create a HTTP channel
    let httpChannel = this.makeChannelForUrl(url);

    // set the request headers.
    httpChannel.setRequestHeader(
      "X-EBAY-API-VERSION", this._apiVersion, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-APP-ID", keyset.appName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-CALL-NAME", callName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-SITE-ID", this._siteId, false);
    httpChannel.setRequestHeader(
      "X-EBAY-API-RESPONSE-ENCODING", resultsFormat, false);

    if (resultsFormat == "XML") {
      httpChannel.setRequestHeader(
        "X-EBAY-API-REQUEST-ENCODING", "XML", false);
    }
    else {
      //we use Name-Value as the default format for the request, even if we
      //are getting the result in NV,JSON o SOAP format
      httpChannel.setRequestHeader(
        "X-EBAY-API-REQUEST-ENCODING", "NV", false);
    }

    // set the extension version as request header (as suggested by Dan Power)
    httpChannel.setRequestHeader(
      "X-EBAY-SIDEBAR-VERSION", Constants.prefBranch.get("version"), false);

    // Set POST data and send the request
    this.setChannelPostData(httpChannel, body, resultsFormat);

    // uncomment to visit request headers
    // we visit the headers before actually sending the request because once
    // we send it, we might lose the POST body content
    //this.visitRequestHeaders(httpChannel);

    let request = this.sendRequest(httpChannel, callback);

    return request;
  },

  /**
   * (ecIEbayAccessComponent)
   * Performs a Client Alerts API call of the given name using the data provided,
   * and ensures that the given callback will be invoked when the call returns.
   */
  doClientAlertsAPICall : function(
    callName, parameterString, useSandbox, callback) {

    // Determine the correct URL to use
    let url;
    if (!useSandbox) {
      // Production
      url = "http://clientalerts.ebay.com/ws/ecasvc/ClientAlerts?";
    } else {
      // Sandbox
      url = "http://clientalerts.sandbox.ebay.com/ws/ecasvc/ClientAlerts?";
    }

    // Get hold of the necessary access keys
    let keyset = AccessKeysHelper.getKeySet(useSandbox, this._siteId);

    // Add parameters
    url += "callname=" + callName;
    url += parameterString;
    url += "&ApplicationID=" + keyset.appName;
    url += "&siteid=" + this._siteId;
    url += "&version=" + this._apiVersion;

    // Create a HTTP channel
    let httpChannel = this.makeChannelForUrl(url);

    // uncomment to visit request headers
    // we visit the headers before actually sending the request because once
    // we send it, we might lose the POST body content
    //this.visitRequestHeaders(httpChannel);

    // Send the request
    let request = this.sendRequest(httpChannel, callback);

    return request;
  },

  /**
   * (ecIEbayAccessComponent)
   * Performs a MyEbayApplicationAPI call of the given name using the data provided, and
   * ensures that the given callback will be invoked when the call returns.
   */
  doMyEbayApplicationAPICall : function(callName, token, body, useSandbox, callback) {

    // Determine the correct URL to use
    let url;
    if (!useSandbox) {
      // Production
      url = "https://svcs.ebay.com/ws/spf";
    } else {
      // Sandbox
      url = "https://svcs.ebay.com/ws/spf";
    }

    // Get hold of the necessary access keys
    let keyset = AccessKeysHelper.getKeySet(useSandbox, this._siteId);

    // Create a HTTP channel
    let httpChannel = this.makeChannelForUrl(url);

    // set the request headers.
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-SERVICE-NAME", "MyEbayApplicationService", false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-OPERATION-NAME", callName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-SECURITY-TOKEN", token, false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-SECURITY-APPNAME", keyset.appName, false);

    // set the extension version as request header (as suggested by Dan Power)
    httpChannel.setRequestHeader(
      "X-EBAY-SIDEBAR-VERSION", Constants.prefBranch.get("version"), false);

    // Set POST data and send the request
    this.setChannelPostData(httpChannel, body);

    // uncomment to visit request headers
    // we visit the headers before actually sending the request because once
    // we send it, we might lose the POST body content
    this.visitRequestHeaders(httpChannel);

    let request = this.sendRequest(httpChannel, callback);

    return request;
  },

  /**
   * (ecIEbayAccessComponent)
   * Performs a ProductStatisticsServiceAPI call of the given name using the data provided, and
   * ensures that the given callback will be invoked when the call returns.
   */
  doProductStatisticsAPICall : function(
    callName, parameterString, useSandbox, callback) {

    // Determine the correct URL to use
    let url;
    if (!useSandbox) {
      // Production
      url = "http://svcs.ebay.com/services/search/ProductStatisticsService/v1?";
    } else {
      // Sandbox
      url = "http://svcs.ebay.com/services/search/ProductStatisticsService/v1?";
    }

    // Get hold of the necessary access keys
    let keyset = AccessKeysHelper.getKeySet(useSandbox, this._siteId);

    // Add parameters
    url += "X-EBAY-SOA-SECURITY-APPNAME=" + keyset.appName;
    url += "&X-EBAY-SOA-SERVICE-NAME=ProductStatisticsService";
    url += "&X-EBAY-SOA-OPERATION-NAME=" + callName;
    url += "&X-EBAY-SOA-RESPONSE-DATA-FORMAT=JSON";
    url += parameterString;
    url += "&buyBoxPriceRequest.includeAllConditions=true";
    url += "&inventoryCountRequest.includeAllConditions=true";
    //Logger.error("Product Statistics API url: " + url);

    // Create a HTTP channel
    let httpChannel = this.makeChannelForUrl(url);

    // uncomment to visit request headers
    // we visit the headers before actually sending the request because once
    // we send it, we might lose the POST body content
    //this.visitRequestHeaders(httpChannel);

    // Send the request
    let request = this.sendRequest(httpChannel, callback);

    return request;
  },

  /**
   * (ecIEbayAccessComponent)
   * Performs a Finding API call of the given name using the data provided, and
   * ensures that the given callback will be invoked when the call returns.
   */
  doFindingAPICall : function(callName, body, useSandbox, callback) {

    // Determine the correct URL to use
    let url;
    if (!useSandbox) {
      // Production
      url = "http://svcs.ebay.com/services/search/FindingService/v1?";
    } else {
      // Sandbox
      url = "http://svcs.ebay.com/services/search/FindingService/v1?";
    }

    // Get hold of the necessary access keys
    let keyset = AccessKeysHelper.getKeySet(useSandbox, this._siteId);

    // Create a HTTP channel
    let httpChannel = this.makeChannelForUrl(url);

    // set the request headers.
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-SERVICE-NAME", "FindingService", false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-OPERATION-NAME", callName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-SECURITY-APPNAME", keyset.appName, false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-SERVICE-VERSION", "1.12.0", false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-GLOBAL-ID", Constants.getGlobalId(), false);
    httpChannel.setRequestHeader(
      "X-EBAY-SOA-REQUEST-DATA-FORMAT", "XML", false);

    // set the extension version as request header (as suggested by Dan Power)
    httpChannel.setRequestHeader(
      "X-EBAY-SIDEBAR-VERSION", Constants.prefBranch.get("version"), false);


    // Set POST data and send the request
    this.setChannelPostData(httpChannel, body);

    // uncomment to visit request headers
    // we visit the headers before actually sending the request because once
    // we send it, we might lose the POST body content
    //this.visitRequestHeaders(httpChannel);

    let request = this.sendRequest(httpChannel, callback);

    return request;
  },

  /**
   * Given a URL, returns a new HTTP channel object
   */
  makeChannelForUrl : function(urlSpec) {
    // the IO service
    let ioService =
      Cc["@mozilla.org/network/io-service;1"]
        .getService(Ci.nsIIOService);

    // create an nsIURI
    let uri = ioService.newURI(urlSpec, null, null);

    // get a channel for that nsIURI
    let channel = ioService.newChannelFromURI(uri);

    channel.QueryInterface(Ci.nsIHttpChannel);
    return channel;
  },

  /**
   * Sends the given data using the given HTTP channel and calls the given
   * callback when the call returns
   */
  setChannelPostData : function(channel, postData, resultsFormat) {

    let inputStream =
      Cc["@mozilla.org/io/string-input-stream;1"]
        .createInstance(Ci.nsIStringInputStream);
    inputStream.setData(postData, postData.length);

    let uploadChannel = channel.QueryInterface(Ci.nsIUploadChannel);

    if (resultsFormat == "XML") {
      uploadChannel.setUploadStream(inputStream, "text/xml", -1);
    } else {
      uploadChannel.setUploadStream(inputStream, "text/html", -1);
    }

    // order important - setUploadStream resets to PUT
    channel.requestMethod = "POST";
  },

  /**
   * Sends the given data using the given HTTP channel and calls the given
   * callback when the call returns
   */
  sendRequest : function(channel, callback) {
    // get an listener
    let requestListener = new RequestListener();
    requestListener.addCallback(callback);
    channel.asyncOpen(requestListener, null);
    return requestListener;
  }
};

/**
 * Class used to control the transmission of information
 */
function RequestListener() {
  this._callbacks = new Array();
  this._abortWhenReady = false;
  this._buffer = "";
}

RequestListener.prototype = {

  /**
   * Adds a callback to invoke when the request returns.
   */
  addCallback : function(callback) {
    this._callbacks.push(callback);
  },

  /**
   * (nsIRequestObserver)
   * Called to signify the beginning of an asynchronous request.
   * @param request request being observed
   * @param context user defined context
   */
  onStartRequest: function (aRequest, aContext) {
    this._buffer = "";
    this._request = aRequest;
    if (this._abortWhenReady) {
      this._request.cancel(Cr.NS_BINDING_ABORTED);
    }
  },

  /**
   * (nsIRequestObserver)
   * Called to signify the end of an asynchronous request.
   * @param request request being observed
   * @param context user defined context
   * @param status reason for stopping (NS_OK if completed successfully)
   */
  onStopRequest: function (aRequest, aContext, aStatus) {
    let err = 0;
    let response;

    switch (aStatus) {
      case Cr.NS_OK:
        err = 0;
        response = this._buffer;
        break;
      default:
        err = aStatus;
        response = "";
        break;
    }

    this.callCallbacks(err, response);
  },

  /**
   * (nsIStreamListener)
   * Manages data as it's returned from the server.
   * @param request request corresponding to the source of the data
   * @param context user define context (not used)
   * @param inputStream input stream containing the data chunk
   * @param offset Number of bytes that were sent in previous onDataAvailable
   *   calls for this request
   * @param count number of bytes available in the stream
   */
  onDataAvailable: function (aRequest, aContext, aStream, aSourceOffset, aLength) {
    var scriptableInputStream =
      Cc["@mozilla.org/scriptableinputstream;1"].
        createInstance(Ci.nsIScriptableInputStream);
    scriptableInputStream.init(aStream);

    this._buffer += scriptableInputStream.read(aLength);
  },

  /**
   * (ecIEbayApiRequest)
   * Cancels the active Api request
   * @param aTimeout whether this is a timeout cancel or not
   */
  cancel : function(aTimeout) {
    if (this._request) {
      this._request.cancel(Cr.NS_BINDING_ABORTED);
    } else {
      this._abortWhenReady = true;
      if (aTimeout) {
        this.callCallbacks(Cr.NS_ERROR_NET_TIMEOUT, this._buffer);
      }
    }
    return Cr.NS_OK;
  },

  /**
   * Calls the callback objects using the given parameters
   */
  callCallbacks : function(err, response) {
    // Iterate through our callback objects and trigger all of them.
    for (let [index, callback] in Iterator(this._callbacks)) {
      callback.trigger(err, response);
    }
    this._callbacks.splice(0, this._callbacks.length);
  },

  // nsIInterfaceRequestor
  getInterface: function (aIID) {
    try {
      return this.QueryInterface(aIID);
    } catch (e) {
      throw Cr.NS_NOINTERFACE;
    }
  },

  // nsIProgressEventSink (not implementing will cause annoying exceptions)
  onProgress : function (aRequest, aContext, aProgress, aProgressMax) { },
  onStatus : function (aRequest, aContext, aStatus, aStatusArg) { },

  // nsIHttpEventSink (not implementing will cause annoying exceptions)
  onRedirect : function (aOldChannel, aNewChannel) { },

  /**
   * The QueryInterface method provides runtime type discovery.
   * More: http://developer.mozilla.org/en/docs/nsISupports
   * @param aIID the IID of the requested interface.
   * @return the resulting interface pointer.
   */
  QueryInterface : function(aIID) {
    if (aIID.equals(Ci.nsISupports) ||
        aIID.equals(Ci.nsIInterfaceRequestor) ||
        aIID.equals(Ci.nsIChannelEventSink) ||
        aIID.equals(Ci.nsIProgressEventSink) ||
        aIID.equals(Ci.nsIHttpEventSink) ||
        aIID.equals(Ci.nsIStreamListener) ||
        aIID.equals(Ci.ecIEbayApiRequest)) {
      return this;
    }
    throw Cr.NS_NOINTERFACE;
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(ApiAccessService);
