/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["AutocompleteService"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

/**
 * The autocomplete service.
 */
var AutocompleteService = {
  /* Autocomplete search param. */
  _SEARCH_PARAM : "ebay-searchbox-history",
  /* Id of the history search. */
  ID_HISTORY_SEARCH : "0",
  /* Id of the ebay search. */
  ID_EBAY_SEARCH : "1",

  /* Auto complete search service. */
  _autoCompleteSearch : null,
  /* Form history service. */
  _formHistoryService : null,
  /* History results array. */
  _historyResults : null,
  /* Suggestions results array. */
  _suggestionResults : null,
  /* Current query. */
  _currentQuery : null,
  /* Request object. */
  _request : null,
  /* Use async form history or not. */
  _useAsyncFormHistory : false,

  // Engine Ids.
  get ID_SEARCH_HISTORY() { return "0"; },
  get ID_BOOKMARKS() { return "1"; },

  /**
   * Initializes the resource.
   */
  init : function() {
    this._autoCompleteSearch =
      Cc["@mozilla.org/autocomplete/search;1?name=form-history"].
        createInstance(Ci.nsIAutoCompleteSearch);
    // the new form history is available from version 23.
    this._useAsyncFormHistory =
      (Services.vc.compare(Services.appinfo.version, "23") >= 0);
    if (this._useAsyncFormHistory) {
      XPCOMUtils.defineLazyModuleGetter(this, "FormHistory",
                                        "resource://gre/modules/FormHistory.jsm");
    } else {
      this._formHistoryService =
        Cc["@mozilla.org/satchel/form-history;1"].getService(Ci.nsIFormHistory2);
    }
  },

  /**
   * Stores the search in the form history.
   * @param aQuery The query to be stored.
   */
  storeSearch : function(aValue) {
    if (this._useAsyncFormHistory) {
      this.FormHistory.update({ op : "add", fieldname : this._SEARCH_PARAM,
                           value : unescape(aValue) });
    } else {
      this._formHistoryService.addEntry(this._SEARCH_PARAM, aValue);
    }
  },

  /**
   * Clears the search history.
   */
  clearSearchHistory : function() {
    if (this._useAsyncFormHistory) {
      this.FormHistory.update({ op : "remove", fieldname : this._SEARCH_PARAM });
    } else {
      this._formHistoryService.removeEntriesForName(this._SEARCH_PARAM);
    }
  },

  /**
   * Searches results for the given query.
   * @param aQuery The query to be searched for.
   */
  search : function(aQuery) {
    this._currentQuery = aQuery;
    this._historyResults = null;
    this._suggestionResults = null;

    this._getSearchHistory(aQuery);
    this._getSearchSuggestions(aQuery);
  },

  /**
   * Gets search history.
   * @param aQuery The query.
   */
  _getSearchHistory : function(aQuery) {
    this._autoCompleteSearch.startSearch(
      aQuery, this._SEARCH_PARAM, null, this);
  },

  /**
   * This is the callback for the form history service to send us results.
   * @param aSearch The search.
   * @param aResult The result.
   */
  onSearchResult: function(aSearch, aResult) {
    let historyResult = aResult;

    if (historyResult) {
      let query = historyResult.searchString;
      let results = new Array();

      if (historyResult.searchResult ==
          Ci.nsIAutoCompleteResult.RESULT_SUCCESS) {
        let count = historyResult.matchCount;
        let result = null;
        let toBeRemoved = [];

        for (let i = 0; i < count; i++) {
          let title = historyResult.getValueAt(i);
          let isDuplicate = results.some(function(aEntry) {
            return (aEntry.title == unescape(title));
          });
          if (isDuplicate) {
            toBeRemoved.push(i);
            continue;
          }
          result = {};
          result.title = unescape(title);
          result.type = "History";
          results.push(result);
        }

        // remove duplicate entries.
        for (let i = (toBeRemoved.length - 1); i >= 0; i--) {
          historyResult.removeValueAt(toBeRemoved[i], true);
        }
      }

      this._historyResults = results;
      this._resultsReturned();
    } else {
      this._historyResults = new Array();
      this._resultsReturned();
    }
  },

  /**
   * Gets search suggestions from eBay.
   * @param aQuery The query.
   */
  _getSearchSuggestions : function(aQuery) {
    let suggestionsEnabled =
      Constants.prefBranch.get("searchbox.showSuggestions");

    if (suggestionsEnabled) {
      let that = this;
      let siteId = Constants.siteIdForSite(Datasource.homeSite());
      let queryParam = { query: aQuery, site: siteId };
      let url = Constants.getUrl("autosuggest", "autosuggest", queryParam);
      let callback = function(aRequest) {
        if (aRequest.status == 200) {
          that._handleQueryResponse(aRequest.responseText);
          that._request = null;
        } else {
          that._suggestionResults = new Array();
          that._resultsReturned();
        }
      }

      this._sendRequest(url, callback);
    } else {
      this._suggestionResults = new Array();
      this._resultsReturned();
    }
  },

  /**
   * Handles the response returned from a query.
   */
  _handleQueryResponse : function(aResponse) {
    // XXX: workaround for E4X bug in Firefox, also get rid of that namespace.
    let responseText =
      aResponse.replace(/<\?xml .*?>/, "").replace(/ xmlns=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(responseText,"text/xml");

    this._suggestionResults = this._parseXMLResponse(xmlTree);

    this._resultsReturned();
  },

  /**
   * Parse the XML response.
   * @param aXMLTree the xml tree.
   * @return the results array.
   */
  _parseXMLResponse : function(aXMLTree) {
    const ID_DELIMITER = "_pid%3D";
    let results = new Array();
    let jxonObject = ObjectHelper.getJXONTree(aXMLTree);
    let currentMatch = jxonObject["SearchSuggestion"]["Query"];
    let trimmedSearchString = /^(.*?)\s*$/.exec(this._currentQuery)[1];

    if (currentMatch && currentMatch == trimmedSearchString) {
      let xmlItems = aXMLTree.getElementsByTagName("Item");
      let result = null;

      for (let i=0; i< xmlItems.length; i++) {
        let xmlItem = xmlItems[i];
        let jxonItem = ObjectHelper.getJXONTree(xmlItem);
        result = {};
        result.title = jxonItem["Text"];
        result.url = jxonItem["Url"];
        result.id = (result.url ? result.url.substring(result.url.indexOf(ID_DELIMITER) + ID_DELIMITER.length) : null);
        result.type = "Suggestion";

        if (jxonItem["Image"]) {
          result.style = "popular-product";
          result.image = String(jxonItem["Image"]["@source"]);
        }

        if ("" != result.title) {
          results.push(result);
        }
      }
    }

    return results;
  },

  /**
   * Called when results are returned from history or query searches.
   */
  _resultsReturned : function() {
    if (this._historyResults && this._suggestionResults) {
      let results = null;

      this._removeDuplicates();
      results = this._buildResultArray();

      Observers.notify(
        results, Topics.AUTOCOMPLETE_RESULTS, this._currentQuery);
    }
  },

  /**
   * Removes duplicate entries from the suggestion results.
   */
  _removeDuplicates : function() {
    let historyCount = this._historyResults.length;
    let suggestionCount = this._suggestionResults.length;
    let historyItem = null;
    let suggestionItem = null;

    for (let i = 0; i < historyCount; i++) {
      historyItem = this._historyResults[i];
      suggestionCount = this._suggestionResults.length;
      for (let j = suggestionCount - 1; j >= 0; j--) {
        suggestionItem = this._suggestionResults[j];
        if (historyItem.title == suggestionItem.title) {
          this._suggestionResults.splice(j, 1);
        }
      }
    }
  },

  /**
   * Builds the result array by mixing both results.
   * @return the result array.
   */
  _buildResultArray : function() {
    const HISTORY_MAX = 3;    // the prefered number of history entries
    const SUGGESTION_MAX = 7; // the prefered number of suggestion entries

    let results = new Array();
    let historyCount = this._historyResults.length;
    let suggestionCount = this._suggestionResults.length;
    let historyDelta = historyCount - HISTORY_MAX;
    let suggestionDelta = suggestionCount - SUGGESTION_MAX;
    let historyTop = (historyCount < HISTORY_MAX ? historyCount : HISTORY_MAX);
    let suggestionTop =
      (suggestionCount < SUGGESTION_MAX ? suggestionCount : SUGGESTION_MAX);

    if (0 > historyDelta && 0 < suggestionDelta) {
      let posHistoryDelta = Math.abs(historyDelta);

      suggestionTop +=
        (suggestionDelta > posHistoryDelta ? posHistoryDelta : suggestionDelta);
    }
    if (0 > suggestionDelta && 0 < historyDelta) {
      let posSuggestionDelta = Math.abs(suggestionDelta);

      historyTop +=
        (historyDelta > posSuggestionDelta ? posSuggestionDelta : historyDelta);
    }

    for (let i = 0; i < historyTop; i++) {
      results.push(this._historyResults[i]);
    }
    for (let i = 0; i < suggestionTop; i++) {
      results.push(this._suggestionResults[i]);
    }

    return results;
  },

  /**
   * Creates a http request to the server.
   * @param aURL the request url.
   * @param aCallback a callback to be called when the server returns.
   */
  _sendRequest : function(aURL, aCallback) {
    let request =
      Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
        createInstance(Ci.nsIXMLHttpRequest);

    if (this._request) {
      this._request.abort();
    }
    this._request = request;

    request.open('GET', aURL, true);
    request.onreadystatechange = function() {
      if (request.readyState == 4 && aCallback) {
        aCallback(request);
      }
    };

    request.send(null);
  }
};

/**
 * Constructor.
 */
(function() {  this.init(); }).apply(AutocompleteService);
