/**
 * Copyright (C) 2013 eBay Inc. All Rights Reserved.
 */
var EXPORTED_SYMBOLS = ["RecentAlertService"];

const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://ebaycompanion/objects/alert.js");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/recentAlert.js");


/**
 * Recent Alert Service.
 */
var RecentAlertService = {
  /* Recent alerts list */
  _recentAlertsCache : [],
  /* Limit of alerts stored locally */
  ALERTS_LIMIT : 8,

  /**
   * Initializes the service.
   */
  init: function() {
    Observers.add(this, Topics.ACCOUNT_LOGGED_IN);
    Observers.add(this, Topics.ACCOUNT_LOGGED_OUT);
    Observers.add(this, Topics.ALERT_SHOW_PRIMARY);
    Observers.add(this, Topics.ALERT_SHOW_SECONDARY);
    Observers.add(this, Topics.ITEM_REMOVED);
  },

  /**
   * Gets the recent alerts.
   * @return the recent alerts.
   */
  get recentAlerts() {
    return this._recentAlertsCache;
  },

  /**
   * Loads the recent alerts from the local database.
   * @param aAccount the account to load their recent alerts.
   */
  _loadRecentAlerts : function(aAccount) {
    if (typeof(ObjectsStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
    }

    this._recentAlertsCache = [];
    this._recentAlertsCache =
      ObjectsStorage.getRecentAlerts(aAccount.get("userId"));
    Observers.notify(null, Topics.RECENT_ALERTS_UPDATED);
  },

  /**
   * Clears the recent alerts kept in memory.
   */
  clearRecentAlerts : function(aDeleteFromDatabase) {
    delete this._recentAlertsCache;
    this._recentAlertsCache = [];

    if (aDeleteFromDatabase) {
      ObjectsStorage.deleteRecentAlertsByUser(
        Datasource.activeAccount().get("userId"));
    }

    Observers.notify(null, Topics.RECENT_ALERTS_UPDATED);
  },

  /**
   * Searches the recent alerts in memory and returns the index in the array
   * if the recent search is found.
   * @param aRecentAlert the recent search object.
   * @return -1 if the recentAlert object is not found in the recent alerts
   * cache, the array index of the recent search object if found in the cache.
   */
  _checkRecentAlertAdded : function(aRecentAlert) {
    let recentAlertIndex = -1;
    let itemId = aRecentAlert.get("itemId");
    for (let i = 0; i < this._recentAlertsCache.length; i++) {
      if (itemId == this._recentAlertsCache[i].get("itemId")) {
        return i;
      }
    }
    return recentAlertIndex;
  },

  /**
   * Adds a recent alert item to the recent alerts cache and the database.
   * @param aRecentAlert the recent alert item.
   */
  addRecentAlert : function(aRecentAlert) {

    let recentAlertIndex = this._checkRecentAlertAdded(aRecentAlert);
    // remove the element from the array if it has been added before.
    if (recentAlertIndex > -1) {
      this._recentAlertsCache.splice(recentAlertIndex, 1);
    }

    this._recentAlertsCache[this._recentAlertsCache.length] = aRecentAlert;

    let newRecentAlert = aRecentAlert.copy();
    newRecentAlert.set("alert", JSON.stringify(aRecentAlert.get("alert")));
    Datasource._markObjectDirty(newRecentAlert);

    // purge the recent alert list, so we don't have more elements than we
    // actually need.
    let alertToDelete;
    while (this._recentAlertsCache.length > this.ALERTS_LIMIT) {
      alertToDelete = this._recentAlertsCache.shift();
      ObjectsStorage.removeRecentAlert(alertToDelete);
    }
    Observers.notify(null, Topics.RECENT_ALERTS_UPDATED);

  },

  /**
   * Removes a recent alert object.
   * @param aItemId the id of the item to remove.
   */
  removeRecentAlert: function(aItemId) {
    // iterate the recent alerts array
    let recentAlert;
    for (let i = 0; i < this._recentAlertsCache.length; i++) {
      recentAlert = this._recentAlertsCache[i];
      //right now we are only removing alerts of type "watching".
      if (recentAlert.get("itemId") == aItemId &&
          parseInt(recentAlert.get("alertType")) ==
            parseInt(Alert.ALERT_TYPE_WATCHING_ENDING_SOON)) {

        this._recentAlertsCache.splice(i, 1);
        // remove from recent alerts dirty objects (if any)
        Datasource.removeDirtyObject(recentAlert);
        ObjectsStorage.removeRecentAlert(recentAlert);
        // notify the observer to refresh the list in the sidebar
        Observers.notify(null, Topics.RECENT_ALERTS_UPDATED);
      }
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case Topics.ACCOUNT_LOGGED_IN:
        this._loadRecentAlerts(aSubject.object);
        break;
      case Topics.ACCOUNT_LOGGED_OUT:
        this.clearRecentAlerts();
        break;
      case Topics.ALERT_SHOW_PRIMARY:
      case Topics.ALERT_SHOW_SECONDARY:
        var alertType = aSubject.type;
        if (alertType != Alert.ALERT_TYPE_FEEDBACK_SCORE &&
            alertType != Alert.ALERT_TYPE_ITEM_ENDED &&
            aSubject != undefined && aSubject instanceof Alert) {
          var userId = Datasource.activeAccount().get("userId");
          var itemId = aSubject.object.get("itemId");
          var timestamp = new Date().getTime();
          var recentAlert = new RecentAlert(userId, itemId);
          recentAlert.set("alertType", alertType);
          recentAlert.set("timestamp", timestamp);
          recentAlert.set("alert", aSubject);
          this.addRecentAlert(recentAlert);
        }
        break;
      case Topics.ITEM_REMOVED:
        item = aSubject.object;
        this.removeRecentAlert(item.get("itemId"))
        break;
    }
  }

};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(RecentAlertService);
