/**
 * Copyright (C) 2013 eBay Inc. All Rights Reserved.
 */
var EXPORTED_SYMBOLS = ["RecentBidService"];

const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/recentBid.js");


/**
 * Recent Bid Service.
 */
var RecentBidService = {
  /* Recent bids list */
  _recentBidsCache : [],
  /* Limit of bids stored locally */
  BIDS_LIMIT : 8,

  /**
   * Initializes the service.
   */
  init: function() {
    Observers.add(this, Topics.ACCOUNT_LOGGED_IN);
    Observers.add(this, Topics.ACCOUNT_LOGGED_OUT);
    Observers.add(this, Topics.ITEM_NEW);
    Observers.add(this, Topics.ITEM_CHANGED);
  },

  /**
   * Gets the recent bids.
   * @return the recent bids.
   */
  get recentBids() {
    return this._recentBidsCache;
  },

  /**
   * Loads the recent bids from the local database.
   * @param aAccount the account to load their recent bids.
   */
  _loadRecentBids : function(aAccount) {
    if (typeof(ObjectsStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
    }

    this._recentBidsCache = [];
    this._recentBidsCache =
      ObjectsStorage.getRecentBids(aAccount.get("userId"));
    Observers.notify(null, Topics.RECENT_BIDS_UPDATED);
  },

  /**
   * Clears the recent bids kept in memory.
   */
  _clearRecentBids : function() {
    delete this._recentBidsCache;
    this._recentBidsCache = [];
    Observers.notify(null, Topics.RECENT_BIDS_REMOVED);
  },

  /**
   * Searches the recent bids in memory and returns the index in the array
   * if the recent search is found.
   * @param aRecentBid the recent search object.
   * @return -1 if the recentBid object is not found in the recent bids
   * cache, the array index of the recent search object if found in the cache.
   */
  _checkRecentBidAdded : function(aRecentBid) {
    let recentBidIndex = -1;
    let itemId = aRecentBid.get("itemId");
    for (let i = 0; i < this._recentBidsCache.length; i++) {
      if (itemId == this._recentBidsCache[i].get("itemId")) {
        return i;
      }
    }
    return recentBidIndex;
  },

  /**
   * Adds a recent bid item to the recent bids cache and the database.
   * @param aRecentBid the recent bid item.
   */
  addRecentBid : function(aRecentBid) {

    let recentBidIndex = this._checkRecentBidAdded(aRecentBid);
    // remove the element from the array if it has been added before.
    if (recentBidIndex > -1) {
      this._recentBidsCache.splice(recentBidIndex, 1);
    }

    this._recentBidsCache[this._recentBidsCache.length] = aRecentBid;
    Datasource._markObjectDirty(aRecentBid);
    // purge the recent bid list, so we don't have more elements than we
    // actually need.
    let bidToDelete;
    while (this._recentBidsCache.length > this.BIDS_LIMIT) {
      bidToDelete = this._recentBidsCache.shift();
      ObjectsStorage.removeRecentBid(bidToDelete);
    }
    Observers.notify(null, Topics.RECENT_BIDS_UPDATED);

  },

  /**
   * Removes a recent bid object.
   * @param aItemId the id of the item to remove.
   */
  removeRecentBid : function(aItemId) {
    // iterate the recent bids array
    let recentBid;
    for (var i = 0; i < this._recentBidsCache.length; i++) {
      recentBid = this._recentBidsCache[i];
      if (recentBid.get("itemId") == aItemId) {
        this._recentBidsCache.splice(i, 1);
        // remove from recent bids dirty objects (if any)
        Datasource.removeDirtyObject(recentBid);
        ObjectsStorage.removeRecentBid(recentBid);
        // notify the observer to refresh the list in the sidebar
        Observers.notify(null, Topics.RECENT_BIDS_UPDATED);
      }
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case Topics.ACCOUNT_LOGGED_IN:
        this._loadRecentBids(aSubject.object);
        break;
      case Topics.ACCOUNT_LOGGED_OUT:
        this._clearRecentBids();
        break;
      case Topics.ITEM_NEW:
        // case where the user bids to the item directly (even if he gets outbid)
        let item = aSubject.object;
        if (item.get("type") == Item.ITEM_TYPE_BIDDING &&
            !item.get("isEnded") &&
            item.get("userMaxBid") &&
            parseFloat(item.get("userMaxBid")) > 0) {
          var userId = Datasource.activeAccount().get("userId");
          var itemId = item.get("itemId");
          var timestamp = new Date().getTime();
          var recentBid = new RecentBid(userId, itemId);
          recentBid.set("timestamp", timestamp);
          this.addRecentBid(recentBid);
        }
        break;
      case Topics.ITEM_CHANGED:
        // case where the item is in the watchlist and the user bids on it
        // or increases its bid (even if he is outbid)
        item = aSubject.object;
        let originalItem = aSubject.originalObject;
        if (item.get("type") == Item.ITEM_TYPE_BIDDING &&
            !item.get("isEnded") &&
            item.get("userMaxBid") > originalItem.get("userMaxBid")) {
          var userId = Datasource.activeAccount().get("userId");
          var itemId = item.get("itemId");
          var timestamp = new Date().getTime();
          var recentBid = new RecentBid(userId, itemId);
          recentBid.set("timestamp", timestamp);
          this.addRecentBid(recentBid);
        }
        break;
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(RecentBidService);
