/**
 * Copyright (C) 2013 eBay Inc. All Rights Reserved.
 */
var EXPORTED_SYMBOLS = ["RecentSearchService"];

const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://ebaycompanion/objects/recentSearch.js");
Cu.import("resource://gre/modules/Services.jsm");

/**
 * Recent Search Service.
 */
var RecentSearchService = {
  /* Recent searches list */
  _recentSearchesCache : [],
  /* Limit of searches stored locally */
  SEARCHES_LIMIT : 8,

  /**
   * Initializes the service.
   */
  init: function() {
    Observers.add(this, Topics.ACCOUNT_LOGGED_IN);
    Observers.add(this, Topics.ACCOUNT_LOGGED_OUT);
    Observers.add(this, Topics.SEARCH_EXECUTED);
  },

  /**
   * Gets the recent searches.
   * @return the recent searches.
   */
  get recentSearches() {
    return this._recentSearchesCache;
  },

  /**
   * Loads the recent searches from the local database.
   * @param aAccount the account to load their recent searches.
   */
  _loadRecentSearches : function(aAccount) {
    if (typeof(ObjectsStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
    }

    this._recentSearchesCache = [];
    this._recentSearchesCache =
      ObjectsStorage.getRecentSearches(aAccount.get("userId"));

    Observers.notify(null, Topics.RECENT_SEARCHES_UPDATED);
  },

  /**
   * Clears the recent searches kept in memory.
   * @param aDeleteFromDatabase indicates if recent searches stored in the
   * database should be deleted.
   */
  clearRecentSearches: function(aDeleteFromDatabase) {
    delete this._recentSearchesCache;
    this._recentSearchesCache = [];

    if (aDeleteFromDatabase) {
      ObjectsStorage.deleteRecentSearchesByUser(
        Datasource.activeAccount().get("userId"));
    }

    Observers.notify(null, Topics.RECENT_SEARCHES_UPDATED);
  },

  /**
   * Searches the recent searches in memory and returns the index in the array
   * if the recent search is found.
   * @param aRecentSearch the recent search object.
   * @return -1 if the recentSearch object is not found in the recent searches
   * cache, the array index of the recent search object if found in the cache.
   */
  _checkRecentSearchAdded : function(aRecentSearch) {
    let recentSearchIndex = -1;
    let searchString = aRecentSearch.get("searchString");
    for (let i = 0; i < this._recentSearchesCache.length; i++) {
      if (searchString == this._recentSearchesCache[i].get("searchString")) {
        return i;
      }
    }
    return recentSearchIndex;
  },

  /**
   * Adds a recent search to the recent searches cache and the database.
   * @param aRecentSearch the recent search object.
   */
  addRecentSearch : function(aRecentSearch) {

    let recentSearchIndex = this._checkRecentSearchAdded(aRecentSearch);
    // remove the element from the array if it has been added before.
    if (recentSearchIndex > -1) {
      this._recentSearchesCache.splice(recentSearchIndex, 1);
    }

    this._recentSearchesCache[this._recentSearchesCache.length] = aRecentSearch;
    Datasource._markObjectDirty(aRecentSearch);
    // purge the recent search list, so we don't have more elements than we
    // actually need
    let searchToDelete;
    while (this._recentSearchesCache.length > this.SEARCHES_LIMIT) {
      searchToDelete = this._recentSearchesCache.shift();
      ObjectsStorage.removeRecentSearch(searchToDelete);
    }
    Observers.notify(null, Topics.RECENT_SEARCHES_UPDATED);
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case Topics.ACCOUNT_LOGGED_IN:
        this._loadRecentSearches(aSubject.object);
        break;
      case Topics.ACCOUNT_LOGGED_OUT:
        this._clearRecentSearches();
        break;
      case Topics.SEARCH_EXECUTED:
        // add the search term to the recent searches if the user is signed in
        let activeAccount = Datasource.activeAccount();
        if (activeAccount) {
          let userId = activeAccount.get("userId");
          let searchString = UtilityHelper.trim(aData);
          let timestamp = new Date().getTime();
          let recentSearch = new RecentSearch(userId, searchString);
          recentSearch.set("timestamp", timestamp);
          this.addRecentSearch(recentSearch);
        }
        break;
    }
  }

};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(RecentSearchService);
