/**
 * Copyright (C) 2013 eBay Inc. All Rights Reserved.
 */
var EXPORTED_SYMBOLS = ["RecentViewService"];

const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/recentView.js");

/**
 * Recent View Service.
 */
var RecentViewService = {
  /* Recent views list */
  _recentViewsCache : null,
  /* Limit of views stored locally */
  VIEWS_LIMIT : 8,

  /**
   * Initializes the service.
   */
  init: function() {
    Observers.add(this, Topics.ACCOUNT_LOGGED_IN);
    //Observers.add(this, Topics.ACCOUNT_LOGGED_OUT);
    //Observers.add(this, Topics.ITEM_NEW);
    //Observers.add(this, Topics.ITEM_REMOVED);
  },

  /**
   * Gets the recent views.
   * @return the recent views.
   */
  get recentViews() {
    return this._recentViewsCache;
  },

  /**
   * Loads the recent views from the local database.
   * @param aAccount the account to load their recent views.
   */
  _loadRecentViews : function(aAccount) {
    if (typeof(ObjectsStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
    }
    this._recentViewsCache =
      ObjectsStorage.getRecentViews(aAccount.get("userId"));
    Observers.notify(null, Topics.RECENT_VIEWS_UPDATED);
  },

  /**
   * Clears the recent views kept in memory.
   */
  clearRecentViews : function() {
    delete this._recentViewsCache;
    this._recentViewsCache = null;
    Observers.notify(null, Topics.RECENT_VIEWS_REMOVED);
  },

  /**
   * Adds a recent viewed item to the recent views cache and the database.
   * @param aRecentView the recent viewed item.
   */
  addRecentView : function(aRecentView) {

    if (aRecentView) {
      this._recentViewsCache[this._recentViewsCache.length] = aRecentView;
      Datasource._markObjectDirty(aRecentView);

      // purge the recent search list, so we don't have more elements than we
      // actually need
      let recentViewToDelete;
      while (this._recentViewsCache.length > this.VIEWS_LIMIT) {
        recentViewToDelete = this._recentViewsCache.shift();
        // remove from recent views dirty objects (if any)
        Datasource.removeDirtyObject(recentViewToDelete);
        ObjectsStorage.removeRecentView(recentViewToDelete);
      }
      Observers.notify(null, Topics.RECENT_VIEWS_UPDATED);
    }
  },

  /**
   * Removes a recent view object.
   * @param aItemId the id of the item to remove.
   */
  removeRecentView : function(aItemId) {
    // iterate the recent views array
    let recentView;
    for (var i = 0; i < this._recentViewsCache.length; i++) {
      recentView = this._recentViewsCache[i];
      if (recentView.get("itemId") == aItemId) {
        this._recentViewsCache.splice(i, 1);
        // remove from recent views dirty objects (if any)
        Datasource.removeDirtyObject(recentView);
        ObjectsStorage.removeRecentView(recentView);
        // notify the observer to refresh the list in the sidebar
        Observers.notify(null, Topics.RECENT_VIEWS_UPDATED);
      }
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    let item;
    switch (aTopic) {
      case Topics.ACCOUNT_LOGGED_IN:
        this._loadRecentViews(aSubject.object);
        break;
      case Topics.ACCOUNT_LOGGED_OUT:
        this._clearRecentViews();
        break;
    }
  }

};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(RecentViewService);
