/**
 * Copyright (C) 2013 eBay Inc. All Rights Reserved.
 */
var EXPORTED_SYMBOLS = ["RecentWatchService"];

const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/recentWatch.js");

/**
 * Recent Watch Service.
 */
var RecentWatchService = {
  /* Recent watches list */
  _recentWatchesCache : [],
  /* Limit of watches stored locally */
  WATCHES_LIMIT : 8,

  /**
   * Initializes the service.
   */
  init: function() {
    Observers.add(this, Topics.ACCOUNT_LOGGED_IN);
    Observers.add(this, Topics.ACCOUNT_LOGGED_OUT);
    Observers.add(this, Topics.ITEM_NEW);
    Observers.add(this, Topics.ITEM_REMOVED);
  },

  /**
   * Gets the recent watches.
   * @return the recent watches.
   */
  get recentWatches() {
    return this._recentWatchesCache;
  },

  /**
   * Loads the recent watches from the local database.
   * @param aAccount the account to load their recent watches.
   */
  _loadRecentWatches : function(aAccount) {
    if (typeof(ObjectsStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
    }

    this._recentWatchesCache = [];
    this._recentWatchesCache =
      ObjectsStorage.getRecentWatches(aAccount.get("userId"));
    Observers.notify(null, Topics.RECENT_WATCHES_UPDATED);
  },

  /**
   * Clears the recent watches kept in memory.
   */
  clearRecentWatches : function() {
    delete this._recentWatchesCache;
    this._recentWatchesCache = [];
    Observers.notify(null, Topics.RECENT_WATCHES_REMOVED);
  },

  /**
   * Adds a recent watched item to the recent watches cache and the database.
   * @param aRecentWatch the recent watched item.
   */
  addRecentWatch : function(aRecentWatch) {

    if (aRecentWatch) {
      this._recentWatchesCache[this._recentWatchesCache.length] = aRecentWatch;
      Datasource._markObjectDirty(aRecentWatch);

      // purge the recent search list, so we don't have more elements than we
      // actually need
      let recentWatchToDelete;
      while (this._recentWatchesCache.length > this.WATCHES_LIMIT) {
        recentWatchToDelete = this._recentWatchesCache.shift();
        // remove from recent watches dirty objects (if any)
        Datasource.removeDirtyObject(recentWatchToDelete);
        ObjectsStorage.removeRecentWatch(recentWatchToDelete);
      }
      Observers.notify(null, Topics.RECENT_WATCHES_UPDATED);
    }
  },

  /**
   * Removes a recent watch object.
   * @param aItemId the id of the item to remove.
   */
  removeRecentWatch : function(aItemId) {
    // iterate the recent watches array
    let recentWatch;
    for (var i = 0; i < this._recentWatchesCache.length; i++) {
      recentWatch = this._recentWatchesCache[i];
      if (recentWatch.get("itemId") == aItemId) {
        this._recentWatchesCache.splice(i, 1);
        // remove from recent watches dirty objects (if any)
        Datasource.removeDirtyObject(recentWatch);
        ObjectsStorage.removeRecentWatch(recentWatch);
        // notify the observer to refresh the list in the sidebar
        Observers.notify(null, Topics.RECENT_WATCHES_UPDATED);
      }
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    let item;
    switch (aTopic) {
      case Topics.ACCOUNT_LOGGED_IN:
        this._loadRecentWatches(aSubject.object);
        break;
      case Topics.ACCOUNT_LOGGED_OUT:
        this._clearRecentWatches();
        break;
      case Topics.ITEM_NEW:
        item = aSubject.object;
        if (item.get("type") == Item.ITEM_TYPE_WATCHING &&
            !item.get("isEnded")) {
          var userId = Datasource.activeAccount().get("userId");
          var itemId = item.get("itemId");
          var timestamp = new Date().getTime();
          var recentWatch = new RecentWatch(userId, itemId);
          recentWatch.set("timestamp", timestamp);
          this.addRecentWatch(recentWatch);
        }
        break;
      case Topics.ITEM_REMOVED:
        item = aSubject.object;
        if (item.get("type") == Item.ITEM_TYPE_WATCHING) {
          this.removeRecentWatch(item.get("itemId"));
        }
        break;
    }
  }

};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(RecentWatchService);
