/**
 * Copyright (C) 2011-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["SystemMessageService"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

/**
 * System Message Service.
 */
SystemMessageService = {
  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/datasource.js");
    Cu.import("resource://ebaycompanion/constants.js");
    Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
    Cu.import("resource://ebaycompanion/objects/notification.js");
    Cu.import("resource://ebaycompanion/helpers/informationNotificationHelper.js");
  },

  /**
   * Adds an item to the local store, or updates it to match the given item.
   * Notifications are generated to inform listeners that the item has been
   * added or changed.
   * @param aSystemMessage the system message.
   */
  addOrUpdateSystemMessage : function(aSystemMessage) {
    let validToTime = aSystemMessage.get("validToTime");
    let validFromTime = aSystemMessage.get("validFromTime");
    let ebayTime = Datasource.getEbayTime().getTime();

    if (validFromTime < ebayTime && ebayTime < validToTime) {
      let userId = aSystemMessage.get("userId");
      let messageId = aSystemMessage.get("messageId");
      let storedMessage = ObjectsStorage.getSystemMessage(messageId, userId);
      let generateNotification = false;

      if (storedMessage) {
        let numberOfViews = storedMessage.get("numberOfViews");
        let maxNumberOfViews = storedMessage.get("maxNumberOfViews");

        if (maxNumberOfViews > numberOfViews) {
          numberOfViews++;
          // this causes the item to be marked as dirty
          storedMessage.updateProperty("numberOfViews", numberOfViews);
          generateNotification = true;
        }
      } else {
        // instead of inserting it immediately, we just mark it as dirty and let
        // the flusher to take care of it later
        Datasource._markObjectDirty(aSystemMessage);
        generateNotification = true;
      }

      // generate and send the notification
      if (generateNotification) {
        this._generateSystemMessageNotification(aSystemMessage);
      }
    }
  },

  /**
   * Generates a system message notification.
   * @param aSystemMessage the system message.
   */
  _generateSystemMessageNotification : function(aSystemMessage) {
    let jsonBody = aSystemMessage.get("jsonBody").replace(/\n/ig, "");
    let messageObject = JSON.parse(jsonBody);
    let notification = new Notification(
      InformationNotificationHelper.SYSTEM_MESSAGE_NOTIFICATION);
    let windowMediator =
      Cc["@mozilla.org/appshell/window-mediator;1"].
        getService(Ci.nsIWindowMediator);
    let callback = function(aEvent) {
      let target = aEvent.originalTarget;
      let targetClass =
        target.getAttribute("class") ? target.getAttribute("class") : "";

      if ("messageCloseButton" != targetClass) {
        let mostRecentWindow =
          windowMediator.getMostRecentWindow("navigator:browser");
        let roveredUrl =
          Constants.roverUrl(
            messageObject.url, Datasource.homeSite(), "platformNotification");
        mostRecentWindow.EbayCompanion.openRawURL(roveredUrl, aEvent);
        InformationNotificationHelper.dismissNotification(
          InformationNotificationHelper.SYSTEM_MESSAGE_NOTIFICATION);
      }
    };

    notification.addLinkCallback(0, callback);
    notification.set("content", messageObject.message);
    notification.set("title", messageObject.title);
    notification.set("priority", 1);

    InformationNotificationHelper.queueNotification(notification);
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(SystemMessageService);
