/**
 * Copyright (c) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = [];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
Cu.import("resource://ebaycompanion/constants.js");

/**
 * Uninstall Service.
 */
eBayComp.UninstallService = {
  /* Flag indicates whether the uninstall should be executed on exit. */
  _shouldUninstall : false,

  /**
   * Initializes the resource.
   */
  init : function() {
    Observers.add(this, Topics.QUIT_APPLICATION);
    Observers.add(this, Topics.EM_ACTION_REQUESTED);

    // Firefox 4 requires an special listener to catch add-on related activity
    // (uninstall event mostly). If we try importing the module and there's an
    // exception, then we are not on Firefox 4
    try {
      Cu.import("resource://gre/modules/AddonManager.jsm");
      AddonManager.addAddonListener(this);
    } catch (ex) { }
  },

  /**
   * Cleans up code to remove the extension directory and related preferences.
   */
  _cleanUp : function() {
    let prefBranch =
      Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);

    this._reportUninstall();
    this._deleteShortcut();
    this._removeDatabase();
    // remove the preferences.
    prefBranch.deleteBranch(eBayComp.PrefBranch);
    // remove the extension directory if possible.
    this._removeExtensionFolder();
  },

  /**
   * Removes the extension folder.
   */
  _removeExtensionFolder : function() {
    let installFolder = eBayComp.getExtensionDirectory();

    try {
      if (installFolder.exists() && installFolder.isDirectory()) {
        installFolder.remove(true);
      }
    } catch (e) {
      // using debug instead of error because we know that the
      // sqlite file is not going to be removed if the connection is open.
      Logger.error("_removeExtensionFolder. Error:\n" + e);

      // XXX: failed to remove directory, so lets remove contents recursively.
      if (installFolder.exists()) {
        let directoryEntries = installFolder.directoryEntries;
        let entry = null;

        while (directoryEntries.hasMoreElements()) {
          entry = directoryEntries.getNext();
          entry.QueryInterface(Ci.nsIFile);

          try {
            entry.remove(true);
          } catch (e) {
            // if a remove failed, proceed with the next file.
            // XXX: using debug instead of error because we know that the sqlite
            // file is not going to be removed if the connection is open.
            Logger.error("_removeExtensionFolder. Error:\n" + e);
          }
        }
        // try one last time to remove the folder after we deleted its
        // contents.
        try {
          installFolder.remove(true);
        } catch (e) {
          // XXX: using debug instead of error because we know that the sqlite
          // file is not going to be removed if the connection is open.
          Logger.error("_removeExtensionFolder. Error:\n" + e);
        }
      }
    }
  },

  /**
   * Removes the database if possible, or clean it.
   */
  _removeDatabase : function() {
    if (typeof(StorageHelper) == "undefined") {
      Cu.import("resource://ebaycompanion/helpers/storageHelper.js");
    }
    try {
      StorageHelper.removeDatabase();
    } catch (e) {
      // make the import when we actually need it
      if (typeof(ObjectsStorage) == "undefined") {
        Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
      }
      // This will happen if the database file couldn't be removed.
      ObjectsStorage.removeTables();
      // make the import when we actually need it
      if (typeof(PropertiesStorage) == "undefined") {
        Cu.import("resource://ebaycompanion/storage/propertiesStorage.js");
      }
      PropertiesStorage.removeTables();
      StorageHelper.doStatement("VACUUM");
    }
  },

  /**
   * Sends the API requets to report extension uninstall.
   */
  _reportUninstall : function() {
    // make the import when we actually need it
    if (typeof(Datasource) == "undefined") {
      Cu.import("resource://ebaycompanion/datasource.js");
    }
    let activeAccount = Datasource.activeAccount();
    if (!activeAccount) {
      Logger.log("No user signed in, using the first local account to send " +
                 "the uninstall request");
      // make the import when we actually need it
      if (typeof(ObjectsStorage) == "undefined") {
        Cu.import("resource://ebaycompanion/storage/objectsStorage.js");
      }
      let localAccounts = ObjectsStorage.getAllAccounts();
      // the first local account (if any) will help us sending the uninstall
      // request
      if (localAccounts.length > 0) {
        activeAccount = localAccounts[0];
      }
    }
    // if we don't have an account at this point, we can't send the uninstall
    // API call because we don't have any eBayAuthToken to send with it.
    if (activeAccount) {
      let ebayAuthToken = activeAccount.get("token");
      let credentialsArray = { eBayAuthToken: ebayAuthToken };

      // make the import when we actually need it
      if (typeof(TradingApi) == "undefined") {
        Cu.import("resource://ebaycompanion/apis/tradingApi.js");
      }

      TradingApi.reportToolbarActivity(
        "Uninstall", credentialsArray,
        activeAccount.get("isSandboxAccount"), reportUninstallCallback);
    }

    let reportUninstallCallback =
      let (that = this) function(result) {
        try {
          if (result.error) {
            Logger.warning("Extension uninstall report reported errors");
          } else {
            Logger.log("Uninstall reported successfully");
          }
        } catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
  },

  /**
   * Deletes the shortcut in the desktop.
   */
  _deleteShortcut : function() {
    let directoryService =
      Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
    let shortcutFile = directoryService.get("Desk", Ci.nsIFile);
    let shortcutName = Constants.prefBranch.get("common.iconName");

    if (!Constants.prefBranch.get("common.icon")) {
      shortcutName =
        Constants.stringBundle.getString(
          "extensions." + eBayComp.ExtensionId + ".name");
    }

    switch (UtilityHelper.getOperatingSystem()) {
      case "MAC":
        shortcutFile.append(shortcutName + ".app");
        break;
      case "WINDOWS":
      case "VISTA":
      case "WIN7":
      case "WIN8":
        shortcutFile.append(shortcutName + ".lnk");
        break;
      case "LINUX":
        shortcutFile.append(shortcutName);
        break;
    }

    if (shortcutFile.exists()) {
      shortcutFile.remove(true);
    }
  },

  /**
   * Handles the on uninstalling observer.
   * @param aItem the item to be uninstalled.
   */
  onUninstalling : function(aItem) {
    if (eBayComp.ExtensionId == aItem.id) {
      this._shouldUninstall = true;
    }
  },

  /**
   * Handles the on operation cancelled observer.
   * @param aItem the item cancelled.
   */
  onOperationCancelled : function(aItem) {
    if (eBayComp.ExtensionId == aItem.id) {
      this._shouldUninstall = false;
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    switch(aTopic) {
      case Topics.QUIT_APPLICATION:
        if (this._shouldUninstall) {
          this._cleanUp();
        }
        break;
      case Topics.EM_ACTION_REQUESTED:
        aSubject.QueryInterface(Ci.nsIUpdateItem);

        if (eBayComp.ExtensionId == aSubject.id) {
          switch(aData) {
            case "item-cancel-action":
              if (this._shouldUninstall) {
                this._shouldUninstall = false;
              }
              break;
            case "item-uninstalled":
              this._shouldUninstall = true;
              break;
          }
        }
        break;
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(eBayComp.UninstallService);
