/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["ObjectsStorage"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/storageHelper.js");
Cu.import("resource://ebaycompanion/objects/account.js");
Cu.import("resource://ebaycompanion/objects/alert.js");
Cu.import("resource://ebaycompanion/objects/item.js");
Cu.import("resource://ebaycompanion/objects/transaction.js");
Cu.import("resource://ebaycompanion/objects/favoriteSeller.js");
Cu.import("resource://ebaycompanion/objects/favoriteSearch.js");
Cu.import("resource://ebaycompanion/objects/trackingCode.js");
Cu.import("resource://ebaycompanion/objects/systemMessage.js");
Cu.import("resource://ebaycompanion/objects/whiteListUrl.js");
Cu.import("resource://ebaycompanion/objects/DOMListUrl.js");
Cu.import("resource://ebaycompanion/objects/trackedUrl.js");
Cu.import("resource://ebaycompanion/objects/recentBid.js");
Cu.import("resource://ebaycompanion/objects/recentSearch.js");
Cu.import("resource://ebaycompanion/objects/recentView.js");
Cu.import("resource://ebaycompanion/objects/recentWatch.js");
Cu.import("resource://ebaycompanion/objects/recentAlert.js");

var ObjectsStorage = {
  // These tables will be created to house the properties of their corresponding
  // objects if those tables don't already exist.
  _tables : [
    {
      objectType : Account,
      tableName : "accounts",
      primaryKeys : ["userId", "isSandboxAccount"]
    },
    {
      objectType : Item,
      tableName : "items",
      primaryKeys : ["itemId"]
    },
    {
      objectType : Transaction,
      tableName : "transactions",
      primaryKeys : ["itemId", "transactionId"]
    },
    {
      objectType : FavoriteSeller,
      tableName : "favoriteSellers",
      primaryKeys : ["userId", "isSandboxAccount", "sellerId"]
    },
    {
      objectType : FavoriteSearch,
      tableName : "favoriteSearches",
      primaryKeys : ["userId", "isSandboxAccount", "searchName"]
    },
    {
      // custom table to specify many-to-many relationship between accounts and
      // items
      tableName : "account_items",
      columns : ["userId", "isSandboxAccount", "itemId"],
      primaryKeys : ["userId", "isSandboxAccount", "itemId"]
    },
    {
      objectType : TrackingCode,
      tableName : "trackingCodes",
      primaryKeys : ["country", "isRover"]
    },
    {
      objectType : SystemMessage,
      tableName : "systemMessages",
      primaryKeys : ["messageId", "userId"]
    },
    {
      objectType : WhiteListUrl,
      tableName : "whiteListUrls",
      primaryKeys : ["urlId"]
    },
    {
      objectType : DOMListUrl,
      tableName : "DOMListUrls",
      primaryKeys : ["id"]
    },
    {
      objectType : TrackedUrl,
      tableName : "trackedUrls",
      primaryKeys : ["timestamp"]
    },
    {
      objectType : RecentAlert,
      tableName : "recentAlerts",
      primaryKeys : ["userId", "itemId"]
    },
    {
      objectType : RecentBid,
      tableName : "recentBids",
      primaryKeys : ["userId", "itemId"]
    },
    {
      objectType : RecentSearch,
      tableName : "recentSearches",
      primaryKeys : ["userId", "searchString"]
    },
    {
      objectType : RecentView,
      tableName : "recentViews",
      primaryKeys : ["userId", "itemId"]
    },
    {
      objectType : RecentWatch,
      tableName : "recentWatches",
      primaryKeys : ["userId", "itemId"]
    },
  ],

  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      // check each table exists and create it if necessary
      for (let i = 0; i < this._tables.length; i++) {
        let table = this._tables[i];
        if (!StorageHelper.connection.tableExists(table.tableName)) {

          let schema;
          if (table.objectType) {
            // object table; build schema to suit object's properties
            schema =
              this._buildSchemaForObject(table.objectType, table.primaryKeys);
          } else {
            // custom table; build schema from specified columns
            schema =
              table.columns.join(",") + "," +
              "PRIMARY KEY(" + table.primaryKeys.join(",") + ")";
          }
          StorageHelper.connection.createTable(table.tableName, schema);
        }
      }
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Removes all tables used by this object; used when uninstalling
   */
  removeTables : function() {
    for (let i = 0; i < this._tables.length; i++) {
      let statement = "DROP TABLE IF EXISTS #{table}; ";
      statement = statement.replace(/#{table}/g, this._tables[i].tableName);
      StorageHelper.doStatement(statement);
    }
  },

  /**
   * Stores an object of the given type in the corresponding database table
   */
  store : function(object) {
    // Object types map to table names (Item => items etc...)
    let className = object.constructor.name.toLowerCase();
    let tableName = className + "s";
    if (className.indexOf("search") != -1 ||
        className.indexOf("watch") != -1) {
      tableName = className + "es";
    }
    let objectProperties = object.constructor.prototype.propertyTypes;
    let columnNames = "";
    let numValues = 0;
    let placeholders = "";
    let values = [];
    for each (let [property,] in Iterator(objectProperties)) {
      columnNames += property + ",";
      placeholders += "?" + ++numValues + ",";
      values.push(object.get(property));
    }
    // remove trailing commas
    columnNames = columnNames.slice(0, -1);
    placeholders = placeholders.slice(0, -1);

    let statement =
      "INSERT OR REPLACE INTO #{table} (#{columns}) VALUES (#{placeholders})";
    statement = statement.replace(/#{table}/g, tableName).
                          replace(/#{columns}/g, columnNames).
                          replace(/#{placeholders}/g, placeholders);
    let params = values;
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the account that is marked as active in the database
   */
  activeAccount : function() {
    // make the import until we actually need it
    if (typeof(PropertiesStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/propertiesStorage.js");
    }
    let accountProperty = PropertiesStorage.get("activeAccount");
    let account;
    if (accountProperty) {
      let statement =
        "SELECT * FROM accounts WHERE userId == ?1 AND isSandboxAccount == ?2";
      let params = accountProperty.split(",");
      let rows = StorageHelper.doStatement(statement, params);
      account = this._accountFromRow(rows[0]);
    }
    return account;
  },

  /**
   * Returns the account specified by username and isSandboxAccount
   * @param username The username for the account to be retrieved
   * @param isSandboxAccount Whether or not the account is a sandbox account
   * @return the relevant account
   */
  getAccount : function(username, isSandboxAccount) {
    let statement =
      "SELECT * FROM accounts WHERE userId == ?1 AND isSandboxAccount == ?2";
    let rows =
      StorageHelper.doStatement(statement, [username, isSandboxAccount]);

    let account;
    if (rows.length > 0) {
      account = this._accountFromRow(rows[0]);
    }

    return account;
  },

  /**
   * Marks the given account as active in the database
   */
  setActiveAccount : function(account) {
    if (account) {
      let userId = account.get("userId");
      let isSandboxAccount = account.get("isSandboxAccount");
      let accountProperty = [userId, isSandboxAccount].join(",");
      PropertiesStorage.set("activeAccount", accountProperty);
    } else {
      PropertiesStorage.remove("activeAccount");
    }
  },

  /**
   * Returns all the accounts stored locally
   * @return the accounts stored locally
   */
  getAllAccounts : function () {
    let statement =
      "SELECT accounts.* FROM accounts";
    let rows = StorageHelper.doStatement(statement);

    let accounts = [];
    for (let i = 0; i < rows.length; i++) {
      accounts.push(this._accountFromRow(rows[i]));
    }

    return accounts;
  },

  /**
   * Connects the given item to the given account.  The account should already
   * be stored in the database, but the item needn't be.
   */
  addItemToAccount : function(item, account) {
    // first, store the item in its own table
    this.store(item);

    // now, add the connection in the relationship table
    let statement =
      "INSERT OR REPLACE INTO account_items " +
        "(userId, isSandboxAccount, itemId) " +
        "VALUES (?1, ?2, ?3)";
    let params = [account.get("userId"),
                  account.get("isSandboxAccount"),
                  item.get("itemId")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Disconnects the given item from the given account, and also removes any
   * orphaned items in the items table.
   */
  removeItemFromAccount : function(item, account) {
    let statement =
      "DELETE FROM account_items WHERE " +
        "userId == ?1 AND isSandboxAccount == ?2 AND itemId == ?3";
    let params = [account.get("userId"),
                  account.get("isSandboxAccount"),
                  item.get("itemId")];
    StorageHelper.doStatement(statement, params, true);

    // The above operation could leave some orphaned items that aren't tied to
    // any account, so we have to clean them up
    statement =
      "DELETE FROM items WHERE itemId NOT IN " +
        "(SELECT itemId FROM account_items)";
    StorageHelper.doStatement(statement, null, true);

    // The same is true for transactions
    statement =
      "DELETE FROM transactions WHERE itemId NOT IN " +
        "(SELECT itemId FROM account_items)";
    StorageHelper.doStatement(statement, null, true);
  },

  /**
   * Returns the items being tracked by the given account
   */
  itemsForAccount : function(account) {
    let statement =
      "SELECT items.* FROM account_items,items " +
      "WHERE account_items.itemId == items.itemId AND " +
        "userId == ?1 AND isSandboxAccount == ?2";
    let params = [account.get("userId"),
                  account.get("isSandboxAccount")];
    let rows = StorageHelper.doStatement(statement, params);

    let items = [];
    for (let i = 0; i < rows.length; i++) {
      items.push(this._itemFromRow(rows[i]));
    }

    return items;
  },

  /**
   * Returns the transactions for every item being tracked by the given account
   */
  transactionsForAccount : function(account) {
    let statement =
      "SELECT transactions.* FROM account_items,transactions " +
      "WHERE account_items.itemId == transactions.itemId AND " +
        "userId == ?1 AND isSandboxAccount == ?2";
    let params = [account.get("userId"),
                  account.get("isSandboxAccount")];
    let rows = StorageHelper.doStatement(statement, params);

    let transactions = [];
    for (let i = 0; i < rows.length; i++) {
      transactions.push(this._transactionFromRow(rows[i]));
    }

    return transactions;
  },

  /**
   * Removes the transaction sent as parameter
   * @param aTransaction the transaction to be removed
   */
  removeTransaction : function(aTransaction) {
    let statement =
      "DELETE FROM transactions WHERE " +
        "itemId == ?1 AND transactionId == ?2";
    let params = [aTransaction.get("itemId"),
                  aTransaction.get("transactionId")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the favorite sellers of the given account
   */
  favoriteSellersForAccount : function(account) {
    let statement =
      "SELECT favoriteSellers.* FROM favoriteSellers " +
      "WHERE userId == ?1 AND isSandboxAccount == ?2";
    let params = [account.get("userId"),
                  account.get("isSandboxAccount")];
    let rows = StorageHelper.doStatement(statement, params);

    let favoriteSellers = [];
    for (let i = 0; i < rows.length; i++) {
      favoriteSellers.push(this._favoriteSellerFromRow(rows[i]));
    }

    return favoriteSellers;
  },

  /**
   * Removes the favorite sellers related to the given account
   * @param aAccount the account to delete its related favorite sellers
   */
  removeFavoriteSellersFromAccount : function(aAccount) {
    let statement =
      "DELETE FROM favoriteSellers WHERE " +
        "userId == ?1 AND isSandboxAccount == ?2";
    let params = [aAccount.get("userId"),
                  aAccount.get("isSandboxAccount")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the favorite searches of the given account
   */
  favoriteSearchesForAccount : function(account) {
    let statement =
      "SELECT favoriteSearches.* FROM favoriteSearches " +
      "WHERE userId == ?1 AND isSandboxAccount == ?2";
    let params = [account.get("userId"),
                  account.get("isSandboxAccount")];
    let rows = StorageHelper.doStatement(statement, params);

    let favoriteSearches = [];
    for (let i = 0; i < rows.length; i++) {
      favoriteSearches.push(this._favoriteSearchFromRow(rows[i]));
    }

    return favoriteSearches;
  },

  /**
   * Removes the favorite searches related to the given account
   * @param aAccount the account to delete its related favorite searches
   */
  removeFavoriteSearchesFromAccount : function(aAccount) {
    let statement =
      "DELETE FROM favoriteSearches WHERE " +
        "userId == ?1 AND isSandboxAccount == ?2";
    let params = [aAccount.get("userId"),
                  aAccount.get("isSandboxAccount")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns all the rover ids stored locally
   * @return the rover ids stored locally
   */
  getRoverIds : function () {
    let statement =
      "SELECT trackingCodes.* FROM trackingCodes WHERE isRover == ?1";
    let params = ["true"];
    let rows = StorageHelper.doStatement(statement, params);

    let roverIds = new Array();
    let trackingCode;
    for (let i = 0; i < rows.length; i++) {
      trackingCode = this._trackingCodeFromRow(rows[i]);
      roverIds[i] = trackingCode;
    }

    return roverIds;
  },

  /**
   * Returns all the rover ids stored locally
   * @return the rover ids stored locally
   */
  getInstallTrackingCodes : function () {
    let statement =
      "SELECT trackingCodes.* FROM trackingCodes WHERE isRover == ?1";
    let params = ["false"];
    let rows = StorageHelper.doStatement(statement, params);

    let trackingCodes = new Array();
    let trackingCode;
    for (let i = 0; i < rows.length; i++) {
      trackingCode = this._trackingCodeFromRow(rows[i]);
      trackingCodes[i] = trackingCode;
    }
    return trackingCodes;
  },

  /**
   * Returns the system message specified by the messageId passed as parameter
   * @param aMessageId the id of the message to be searched
   * @param aUserId the id of the user to narrow the search
   * @return the system message that matchs the criteria
   */
  getSystemMessage : function (aMessageId, aUserId) {
    let statement =
      "SELECT systemMessages.* FROM systemMessages WHERE " +
      "messageId = ?1 AND userId = ?2";
    let rows = StorageHelper.doStatement(statement, [aMessageId, aUserId]);

    let systemMessage;
    if (rows.length > 0) {
      systemMessage = this._systemMessageFromRow(rows[0]);
    }

    return systemMessage;
  },

  /**
   * Removes the system messages not valid anymore.
   * @param aTime the valid time.
   */
  removeSystemMessages : function(aTime) {
    let statement = "DELETE FROM systemMessages WHERE validToTime < ?1";
    StorageHelper.doStatement(statement, [aTime], true);
  },

  /**
   * Returns all the white list urls stored locally
   * @return the white list urls stored locally
   */
  getAllWhiteListUrls : function () {
    let statement =
      "SELECT whiteListUrls.* FROM whiteListUrls";
    let rows = StorageHelper.doStatement(statement);

    let whiteListUrls = [];
    for (let i = 0; i < rows.length; i++) {
      whiteListUrls.push(this._whiteListUrlFromRow(rows[i]));
    }

    return whiteListUrls;
  },

  /**
   * Removes the white list url specified
   * @param aWhiteListUrl the white list url to be removed
   */
  removeWhiteListUrl : function(aWhiteListUrl) {
    let statement =
      "DELETE FROM whiteListUrls WHERE urlId == ?1";
    let params = [aWhiteListUrl.get("urlId")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns all the DOM list urls stored locally
   * @return the DOM list urls stored locally
   */
  getAllDOMListUrls : function () {
    let statement =
      "SELECT DOMListUrls.* FROM DOMListUrls";
    let rows = StorageHelper.doStatement(statement);

    let DOMListUrls = [];
    for (let i = 0; i < rows.length; i++) {
      DOMListUrls.push(this._DOMListUrlFromRow(rows[i]));
    }

    return DOMListUrls;
  },

  /**
   * Removes the white list url specified
   * @param aWhiteListUrl the white list url to be removed
   */
  removeDOMListUrl : function(aDOMListUrl) {
    let statement =
      "DELETE FROM DOMListUrls WHERE id == ?1";
    let params = [aDOMListUrl.get("id")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the number of tracked urls stored locally
   * @return the number of tracked urls stored locally
   */
  getTrackedUrlsCount : function () {
    let statement =
      "SELECT count(*) as count FROM trackedUrls";
    let result = StorageHelper.doStatement(statement);

    return result[0].count;
  },

  /**
   * Returns all the tracked urls stored locally
   * @return the tracked urls stored locally
   */
  getAllTrackedUrls : function () {
    let statement =
      "SELECT trackedUrls.* FROM trackedUrls";
    let rows = StorageHelper.doStatement(statement);

    let trackedUrls = [];
    for (let i = 0; i < rows.length; i++) {
      trackedUrls.push(this._trackedUrlFromRow(rows[i]));
    }

    return trackedUrls;
  },

  /**
   * Returns the top aCount tracked urls stored locally
   * @param aCount: the number of urls to be returned from the top of the table
   * @return the tracked urls stored locally
   */
  getTopTrackedUrls : function (aCount) {
    let statement =
      "SELECT trackedUrls.* FROM trackedUrls ORDER BY timestamp LIMIT " + aCount;

    let rows = StorageHelper.doStatement(statement);

    let trackedUrls = [];
    for (let i = 0; i < rows.length; i++) {
      trackedUrls.push(this._trackedUrlFromRow(rows[i]));
    }

    return trackedUrls;
  },

  /**
   * Removes all tracked urls
   */
  removeTrackedUrls : function() {
    let statement = "DELETE FROM trackedUrls WHERE 1=1";
    StorageHelper.doStatement(statement, [], true);
  },

  /**
   * Removes the top aCount tracked urls stored locally
   * @param aCount: the number of urls to be removed from the top of the table
   */
  removeTopTrackedUrls : function (aCount) {
    let statement =
      "DELETE FROM trackedUrls WHERE timestamp <= " +
        "(SELECT timestamp FROM trackedUrls ORDER BY timestamp " +
          "LIMIT "+ (aCount - 1) + " , 1)";

    StorageHelper.doStatement(statement, [], true);
  },

  /**
   * Returns the recent alerts stored locally
   * @param aUserId the user id to retrieve their recent alerts
   * @return the recent alerts stored locally
   */
  getRecentAlerts : function (aUserId) {
    let statement =
      "SELECT recentAlerts.* FROM recentAlerts WHERE " +
        "userId = ?1 ORDER BY timestamp ASC";
    let params = [aUserId];
    let rows = StorageHelper.doStatement(statement, params);

    let recentAlerts = [];
    for (let i = 0; i < rows.length; i++) {
      recentAlerts.push(this._recentAlertFromRow(rows[i]));
    }

    return recentAlerts;
  },

  /**
   * Removes the recent alert sent as parameter
   * @param aRecentAlert the recent alert to be removed
   */
  removeRecentAlert : function(aRecentAlert) {
    let statement =
      "DELETE FROM recentAlerts WHERE " +
        "userId == ?1 AND itemId == ?2";
    let params = [aRecentAlert.get("userId"),
                  aRecentAlert.get("itemId")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Deletes all the recent alerts by an user.
   * @param aUserId the user id.
   */
  deleteRecentAlertsByUser : function(aUserId) {
    let statement =
      "DELETE FROM recentAlerts WHERE userId == ?1";
    let params = [aUserId];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the recent bids stored locally
   * @param aUserId the user id to retrieve their recent bids
   * @return the recent bids stored locally
   */
  getRecentBids : function (aUserId) {
    let statement =
      "SELECT recentBids.* FROM recentBids WHERE " +
        "userId = ?1 ORDER BY timestamp ASC";
    let params = [aUserId];
    let rows = StorageHelper.doStatement(statement, params);

    let recentBids = [];
    for (let i = 0; i < rows.length; i++) {
      recentBids.push(this._recentBidFromRow(rows[i]));
    }

    return recentBids;
  },

  /**
   * Removes the recent bid sent as parameter
   * @param aRecentBid the recent bid to be removed
   */
  removeRecentBid : function(aRecentBid) {
    let statement =
      "DELETE FROM recentBids WHERE " +
        "userId == ?1 AND itemId == ?2";
    let params = [aRecentBid.get("userId"),
                  aRecentBid.get("itemId")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the recent searches stored locally
   * @param aUserId the user id to retrieve their recent searches
   * @return the recent searches stored locally
   */
  getRecentSearches : function (aUserId) {
    let statement =
      "SELECT recentSearches.* FROM recentSearches WHERE " +
        "userId = ?1 ORDER BY timestamp ASC";
    let params = [aUserId];
    let rows = StorageHelper.doStatement(statement, params);

    let recentSearches = [];
    for (let i = 0; i < rows.length; i++) {
      recentSearches.push(this._recentSearchFromRow(rows[i]));
    }

    return recentSearches;
  },

  /**
   * Removes the recent search sent as parameter
   * @param aRecentSearch the recent search to be removed
   */
  removeRecentSearch : function(aRecentSearch) {
    let statement =
      "DELETE FROM recentSearches WHERE " +
        "userId == ?1 AND searchString == ?2";
    let params = [aRecentSearch.get("userId"),
                  aRecentSearch.get("searchString")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Deletes all the recent searches by an user.
   * @param aUserId the user id.
   */
  deleteRecentSearchesByUser : function(aUserId) {
    let statement =
      "DELETE FROM recentSearches WHERE userId == ?1";
    let params = [aUserId];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the recent views stored locally
   * @param aUserId the user id to retrieve their recent views
   * @return the recent views stored locally
   */
  getRecentViews : function (aUserId) {
    let statement =
      "SELECT recentViews.* FROM recentViews WHERE " +
        "userId = ?1 ORDER BY timestamp ASC";
    let params = [aUserId];
    let rows = StorageHelper.doStatement(statement, params);

    let recentViews = [];
    for (let i = 0; i < rows.length; i++) {
      recentViews.push(this._recentViewFromRow(rows[i]));
    }

    return recentViews;
  },

  /**
   * Removes the recent view sent as parameter
   * @param aRecentView the recent view to be removed
   */
  removeRecentView : function(aRecentView) {
    let statement =
      "DELETE FROM recentViews WHERE " +
        "userId == ?1 AND itemId == ?2";
    let params = [aRecentView.get("userId"),
                  aRecentView.get("itemId")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Returns the recent watches stored locally
   * @param aUserId the user id to retrieve their recent watches
   * @return the recent watches stored locally
   */
  getRecentWatches : function (aUserId) {
    let statement =
      "SELECT recentWatches.* FROM recentWatches WHERE " +
        "userId = ?1 ORDER BY timestamp ASC";
    let params = [aUserId];
    let rows = StorageHelper.doStatement(statement, params);

    let recentWatches = [];
    for (let i = 0; i < rows.length; i++) {
      recentWatches.push(this._recentWatchFromRow(rows[i]));
    }

    return recentWatches;
  },

  /**
   * Removes the recent watch sent as parameter
   * @param aRecentWatch the recent watch to be removed
   */
  removeRecentWatch : function(aRecentWatch) {
    let statement =
      "DELETE FROM recentWatches WHERE " +
        "userId == ?1 AND itemId == ?2";
    let params = [aRecentWatch.get("userId"),
                  aRecentWatch.get("itemId")];
    StorageHelper.doStatement(statement, params, true);
  },

  /**
   * Creates an object from the given database row
   */
  _accountFromRow : function(row) {
    let account;
    try {
      account = new Account(row.userId, row.isSandboxAccount);
      let propertyTypes = Account.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        account.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create Account object from database row.",
                   Logger.DUMP_STACK);
    }
    return account;
  },

  /**
   * Creates an object from the given database row
   */
  _itemFromRow : function(row) {
    let item;
    try {
      item = new Item(row.itemId);
      let propertyTypes = Item.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        item.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create Item object from database row.",
                   Logger.DUMP_STACK);
    }
    return item;
  },

  /**
   * Creates an object from the given database row
   */
  _transactionFromRow : function(row) {
    let transaction;
    try {
      transaction = new Transaction(row.itemId, row.transactionId);
      let propertyTypes = Transaction.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        transaction.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create Transaction object from database row.",
                   Logger.DUMP_STACK);
    }
    return transaction;
  },

  /**
   * Creates an object from the given database row
   */
  _favoriteSellerFromRow : function(row) {
    let favoriteSeller;
    try {
      favoriteSeller =
        new FavoriteSeller(row.userId, row.isSandboxAccount, row.sellerId);
      let propertyTypes = FavoriteSeller.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        favoriteSeller.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create FavoriteSeller object from database row.",
                   Logger.DUMP_STACK);
    }
    return favoriteSeller;
  },

  /**
   * Creates an object from the given database row
   */
  _favoriteSearchFromRow : function(row) {
    let favoriteSearch;
    try {
      favoriteSearch =
        new FavoriteSearch(row.userId, row.isSandboxAccount, row.searchName);
      let propertyTypes = FavoriteSearch.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        favoriteSearch.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create FavoriteSearch object from database row.",
                   Logger.DUMP_STACK);
    }
    return favoriteSearch;
  },

  /**
   * Creates an object from the given database row
   */
  _trackingCodeFromRow : function(row) {
    let trackingCode;
    try {
      trackingCode = new TrackingCode(row.country, row.isRover, row.value);
      let propertyTypes = TrackingCode.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        trackingCode.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create TrackingCode object from database row.",
                   Logger.DUMP_STACK);
    }
    return trackingCode;
  },

  /**
   * Creates an object from the given database row
   */
  _systemMessageFromRow : function(row) {
    let systemMessage;
    try {
      systemMessage = new SystemMessage(row.messageId, row.userId);
      let propertyTypes = SystemMessage.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        systemMessage.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create SystemMessage object from database row.",
                   Logger.DUMP_STACK);
    }
    return systemMessage;
  },

  /**
   * Creates an object from the given database row
   */
  _DOMListUrlFromRow : function(row) {
    let domListUrl;
    try {
      domListUrl = new DOMListUrl(row.id);
      let propertyTypes = DOMListUrl.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        domListUrl.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create DOMListUrl object from database row." + e,
                   Logger.DUMP_STACK);
    }
    return domListUrl;
  },

  /**
   * Creates an object from the given database row
   */
  _whiteListUrlFromRow : function(row) {
    let whiteListUrl;
    try {
      whiteListUrl = new WhiteListUrl(row.urlId);
      let propertyTypes = WhiteListUrl.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        whiteListUrl.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create WhiteListUrl object from database row.",
                   Logger.DUMP_STACK);
    }
    return whiteListUrl;
  },

  /**
   * Creates an object from the given database row
   */
  _trackedUrlFromRow : function(row) {
    let trackedUrl;
    try {
      trackedUrl = new TrackedUrl();
      let propertyTypes = TrackedUrl.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        trackedUrl.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create TrackedUrl object from database row. ",
                   Logger.DUMP_STACK);
    }
    return trackedUrl;
  },

  /**
   * Creates an object from the given database row
   */
  _recentAlertFromRow : function(row) {
    let recentAlert;
    try {
      recentAlert = new RecentAlert(row.userId, row.itemId);
      let propertyTypes = RecentAlert.prototype.propertyTypes;
      let newAlert;
      let newItem;
      for (let [property,] in Iterator(propertyTypes)) {
        if (property == "alert") {
          newAlert = new Alert();
          newItem = new Item();
          //convert the string from the database to an Alert type object
          newAlert.updateQuietlyTo(JSON.parse(row[property]));
          //get the "object" in the alert, which is of type Item and convert the
          //object to Item
          newItem.updateQuietlyTo(newAlert.object);
          newAlert.object = newItem;
          recentAlert.set(property, newAlert);
        } else {
          recentAlert.set(property, row[property]);
        }

      }
    }
    catch (e) {
      Logger.error("Unable to create RecentAlert object from database row. ",
                   Logger.DUMP_STACK);
    }
    return recentAlert;
  },

  /**
   * Creates an object from the given database row
   */
  _recentBidFromRow : function(row) {
    let recentBid;
    try {
      recentBid = new RecentBid(row.userId, row.itemId);
      let propertyTypes = RecentBid.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        recentBid.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create RecentBid object from database row. ",
                   Logger.DUMP_STACK);
    }
    return recentBid;
  },

  /**
   * Creates an object from the given database row
   */
  _recentSearchFromRow : function(row) {
    let recentSearch;
    try {
      recentSearch = new RecentSearch(row.userId, row.searchString);
      let propertyTypes = RecentSearch.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        recentSearch.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create RecentSearch object from database row. ",
                   Logger.DUMP_STACK);
    }
    return recentSearch;
  },

  /**
   * Creates an object from the given database row
   */
  _recentViewFromRow : function(row) {
    let recentView;
    try {
      recentView = new RecentView(row.userId, row.itemId);
      let propertyTypes = RecentView.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        recentView.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create RecentView object from database row. ",
                   Logger.DUMP_STACK);
    }
    return recentView;
  },

  /**
   * Creates an object from the given database row
   */
  _recentWatchFromRow : function(row) {
    let recentWatch;
    try {
      recentWatch = new RecentWatch(row.userId, row.itemId);
      let propertyTypes = RecentWatch.prototype.propertyTypes;
      for (let [property,] in Iterator(propertyTypes)) {
        recentWatch.set(property, row[property]);
      }
    }
    catch (e) {
      Logger.error("Unable to create RecentWatch object from database row. ",
                   Logger.DUMP_STACK);
    }
    return recentWatch;
  },

  /**
   * Builds an SQL schema for a table that will hold the given object's
   * properties.
   * @param object The object (must contain propertyTypes)
   * @param primaryKeys Array of primary keys
   */
  _buildSchemaForObject : function(object, primaryKeys) {
    let schema = "";
    for (let [property, type] in Iterator(object.prototype.propertyTypes)) {
      schema += property + ",";
    }
    schema += "PRIMARY KEY(" + primaryKeys.join(",") + ")";

    return schema;
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(ObjectsStorage);
